/*-----------------------------------------------------------------------------

 Copyright 2017 Hopsan Group

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.


 The full license is available in the file LICENSE.
 For details about the 'Hopsan Group' or information about Authors and
 Contributors see the HOPSANGROUP and AUTHORS files that are located in
 the Hopsan source code root directory.

-----------------------------------------------------------------------------*/


//!
//! @file   SignalAcos.hpp
//! @author Petter Krus <petter.krus@liu.se>
//! @date   2011-10-18
//!
//! @brief Contains a signal acos function component
//!

#ifndef SIGNALACOS_HPP_INCLUDED
#define SIGNALACOS_HPP_INCLUDED

#include "ComponentEssentials.h"
#include <math.h>

namespace hopsan {

    //!
    //! @brief
    //! @ingroup SignalComponents
    //!
    class SignalAcos : public ComponentSignal
    {

    private:
        double *mpND_in, *mpND_out, *mpND_err, x;

    public:
        static Component *Creator()
        {
            return new SignalAcos();
        }

        void configure()
        {
            addInputVariable("in", "", "", 0.0, &mpND_in);
            addOutputVariable("out", "acos(in)","",&mpND_out);
            addOutputVariable("error", "error","",&mpND_err);
         }


        void initialize()
        {
            simulateOneTimestep();
        }


        void simulateOneTimestep()
        {
            x=(*mpND_in);
            if(x>1.)
            {
            (*mpND_out) = acos(1.);
            (*mpND_err)=1.;
            }
            else if(x<-1.)
            {
            (*mpND_out) = acos(-1.);
            (*mpND_err)=1.;
            }
            else
            {
            (*mpND_out) = acos(*mpND_in);
            (*mpND_err)=0.;
            }
        }
    };
}

#endif // SIGNALACOS_HPP_INCLUDED
