(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 8.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[     38295,        754]
NotebookOptionsPosition[     36860,        708]
NotebookOutlinePosition[     37420,        729]
CellTagsIndexPosition[     37377,        726]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{

Cell[CellGroupData[{
Cell["Piston", "Section",
 CellChangeTimes->{{3.4785828348125*^9, 3.478582896703125*^9}, {
  3.5159023652393904`*^9, 3.5159023668175154`*^9}, {3.6379219118975105`*^9, 
  3.6379219131195107`*^9}}],

Cell[BoxData[
 RowBox[{"<<", "C:\\\\Hopsan\\Compgen\\CompgenNG.mx"}]], "Input",
 CellChangeTimes->{{3.5772037650720577`*^9, 3.5772037650780582`*^9}, {
  3.611474457184202*^9, 3.611474523938877*^9}, {3.6114745588913717`*^9, 
  3.61147456057154*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"path", "=", 
   RowBox[{"ToFileName", "[", 
    RowBox[{"{", 
     RowBox[{
     "\"\<C:\>\"", ",", "\"\<HopsanTrunk\>\"", ",", 
      "\"\<ComponentLibraries\>\"", ",", "\"\<defaultLibrary\>\"", ",", 
      "\"\<Hydraulic\>\"", ",", "\"\<Special\>\""}], "}"}], "]"}]}], 
  ";"}]], "Input"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"ResetComponentVariables", "[", "]"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"domain", "=", "\"\<Hydraulic\>\""}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"displayName", "=", "\"\<Piston\>\""}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"author", "=", "\"\<Petter Krus <petter.krus@liu.se>\>\""}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"brief", "=", "\"\<Hydraulic piston with two connection\>\""}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"componentType", "=", "\"\<ComponentC\>\""}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
  "affiliation", " ", "=", " ", 
   "\"\<Division of Fluid and Mechatronic Systems, Link\[ODoubleDot]ping \
University\>\""}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"SetFilenames", "[", 
   RowBox[{"path", ",", "domain", ",", "displayName"}], "]"}], 
  ";"}], "\[IndentingNewLine]"}], "Input",
 CellChangeTimes->{
  3.5042704357554045`*^9, {3.5159023742081404`*^9, 3.5159024232706404`*^9}, {
   3.51670646940555*^9, 3.516706478608675*^9}, 3.5684505041258545`*^9, 
   3.571905458607195*^9, {3.571926377672677*^9, 3.5719263778442774`*^9}, {
   3.575772058707899*^9, 3.5757720838551435`*^9}, {3.6379191544250154`*^9, 
   3.63791915846542*^9}, {3.6379192049810705`*^9, 3.6379192116497374`*^9}}],

Cell[CellGroupData[{

Cell["Model", "Subsection",
 CellChangeTimes->{{3.5159024543018904`*^9, 3.5159024592393904`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"nodeConnections", "=", 
   RowBox[{"{", "\[IndentingNewLine]", "\t", 
    RowBox[{
     RowBox[{"HydraulicCnode", "[", 
      RowBox[{"1", ",", 
       RowBox[{"1.", "*", 
        RowBox[{"10", "^", "5"}]}], ",", "\"\<Upstream port\>\""}], "]"}], 
     ",", "\[IndentingNewLine]", "\t", 
     RowBox[{"HydraulicCnode", "[", 
      RowBox[{"2", ",", 
       RowBox[{"1.", "*", 
        RowBox[{"10", "^", "5"}]}], ",", "\"\<Downstream port\>\""}], "]"}], 
     ",", "\[IndentingNewLine]", "\t", 
     RowBox[{"MechanicCnode", "[", 
      RowBox[{"m1", ",", 
       RowBox[{"1.", "*", 
        RowBox[{"10", "^", "5"}]}], ",", "\"\<Mechanic port\>\""}], "]"}]}], 
    "}"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.4752292939381866`*^9, 3.4752293318441935`*^9}, {
   3.47808399728125*^9, 3.478083998921875*^9}, 3.4988885288475237`*^9, {
   3.50394176215625*^9, 3.503941811890625*^9}, {3.50427113076719*^9, 
   3.50427113339219*^9}, 3.50427121389219*^9, {3.51582103693931*^9, 
   3.5158210378611856`*^9}, {3.6379192372262955`*^9, 
   3.6379192574823203`*^9}, {3.6379194968652563`*^9, 3.6379195083544054`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"inputParameters", "=", 
   RowBox[{"{", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
      "A1", ",", "0.001", ",", "double", ",", "\"\<m2\>\"", ",", 
       "\"\<Piston area 1\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "A2", ",", "0.001", ",", "double", ",", "\"\<m2\>\"", ",", 
       "\"\<Piston area 2\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "V01", ",", "0.001", ",", "double", ",", "\"\<m3\>\"", ",", 
       "\"\<Dead Volume Piston 1\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "V02", ",", "0.001", ",", "double", ",", "\"\<m3\>\"", ",", 
       "\"\<Dead Volume Piston 2\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "SL", ",", "0.5", ",", "double", ",", "\"\<m\>\"", ",", 
       "\"\<Stroke\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"betae", ",", 
       RowBox[{"1.7", " ", 
        SuperscriptBox["10", "7"]}], ",", "double", ",", "\"\<Pa\>\"", ",", 
       "\"\<Bulk modulus\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "alpha", ",", "0.0", ",", "double", ",", "\"\<1/s \>\"", ",", 
       "\"\<Damp. factor\>\""}], "}"}]}], "\[IndentingNewLine]", "}"}]}], 
  ";"}]], "Input",
 CellGroupingRules->{GroupTogetherGrouping, 10000.},
 CellChangeTimes->{{3.4779981058619356`*^9, 3.4779981542683554`*^9}, 
   3.4780017782035847`*^9, {3.478002203430644*^9, 3.478002217153696*^9}, {
   3.478002277203772*^9, 3.4780023021490927`*^9}, {3.4780026209521627`*^9, 
   3.478002664340765*^9}, {3.478021796078125*^9, 3.478021842203125*^9}, {
   3.478021885296875*^9, 3.47802199571875*^9}, {3.47802539628125*^9, 
   3.478025479125*^9}, {3.478026054375*^9, 3.47802606034375*^9}, {
   3.47818056640625*^9, 3.47818064359375*^9}, {3.478184752578125*^9, 
   3.478184788859375*^9}, {3.4781848254375*^9, 3.47818487890625*^9}, {
   3.47818496809375*^9, 3.478184980140625*^9}, {3.478189354859375*^9, 
   3.478189379078125*^9}, {3.478582964046875*^9, 3.478583058015625*^9}, {
   3.47858310040625*^9, 3.478583226265625*^9}, {3.47858493834375*^9, 
   3.478584967203125*^9}, {3.478593422734375*^9, 3.47859344625*^9}, {
   3.478593494703125*^9, 3.478593496375*^9}, {3.47859361503125*^9, 
   3.478593622828125*^9}, {3.478593687*^9, 3.4785936875*^9}, {
   3.47860846046875*^9, 3.478608468515625*^9}, {3.478614677703125*^9, 
   3.478614712453125*^9}, {3.47861477634375*^9, 3.4786147769375*^9}, {
   3.47861500284375*^9, 3.478615012546875*^9}, 3.50427041823978*^9, {
   3.504271184564065*^9, 3.50427119129844*^9}, {3.504271245001565*^9, 
   3.50427125876719*^9}, 3.515900544738135*^9, {3.5166988869943166`*^9, 
   3.516698893900699*^9}, 3.516707238308592*^9, 3.571905380107857*^9, {
   3.637921974601511*^9, 3.637921989172511*^9}, {3.6379220752550793`*^9, 
   3.637922105832136*^9}, {3.6379222109736495`*^9, 3.63792223938249*^9}, {
   3.6379417521193933`*^9, 3.6379417546275373`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"initialExpressions", "=", 
   RowBox[{"{", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"c1", "==", "p1"}], ",", "\[IndentingNewLine]", 
     RowBox[{"c2", "==", "p2"}], ",", "\[IndentingNewLine]", 
     RowBox[{"c1f", "==", "p1"}], ",", "\[IndentingNewLine]", 
     RowBox[{"c2f", "==", "p2"}], ",", "\[IndentingNewLine]", 
     RowBox[{"cp1", "==", "p1"}], ",", "\[IndentingNewLine]", 
     RowBox[{"cp2", "==", "p2"}], ",", "\[IndentingNewLine]", 
     RowBox[{"cp1f", "==", "p1"}], ",", "\[IndentingNewLine]", 
     RowBox[{"cp2f", "==", "p2"}]}], "\[IndentingNewLine]", "}"}]}], 
  ";"}]], "Input",
 CellChangeTimes->{{3.51681842278125*^9, 3.516818476890625*^9}, 
   3.516818573890625*^9, {3.5719053860202675`*^9, 3.5719054030866976`*^9}, {
   3.6379278584783573`*^9, 3.6379278936098704`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"localExpressions", "=", 
   RowBox[{"{", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"V1", "\[Equal]", 
      RowBox[{
       RowBox[{
        RowBox[{"limit", "[", 
         RowBox[{
          RowBox[{"-", "xm1"}], ",", "0", ",", "SL"}], "]"}], " ", "A1"}], 
       "+", "V01"}]}], ",", "\[IndentingNewLine]", 
     RowBox[{"V2", "\[Equal]", 
      RowBox[{
       RowBox[{
        RowBox[{"limit", "[", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{"SL", "+", "xm1"}], ")"}], ",", "0", ",", "SL"}], "]"}], 
        " ", "A2"}], "+", "V02"}]}], ",", "\[IndentingNewLine]", 
     RowBox[{"Zc1", "==", 
      FractionBox[
       RowBox[{"mTimestep", " ", "betae"}], 
       RowBox[{"V1", 
        RowBox[{"(", 
         RowBox[{"1", "-", "alpha"}], ")"}]}]]}], ",", "\[IndentingNewLine]", 
     RowBox[{"Zc2", "==", 
      FractionBox[
       RowBox[{"mTimestep", " ", "betae"}], 
       RowBox[{"V2", " ", 
        RowBox[{"(", 
         RowBox[{"1", "-", "alpha"}], ")"}]}]]}], ",", "\[IndentingNewLine]", 
     "\[IndentingNewLine]", 
     RowBox[{"qp1", "\[Equal]", 
      RowBox[{"A1", " ", "vm1"}]}], ",", "\[IndentingNewLine]", 
     RowBox[{"qp2", "\[Equal]", 
      RowBox[{
       RowBox[{"-", "A2"}], " ", "vm1"}]}], ",", "\[IndentingNewLine]", 
     "\[IndentingNewLine]", 
     RowBox[{"c10", "==", 
      RowBox[{"cp1", "+", 
       RowBox[{"2", " ", "Zc1", " ", "qp1"}]}]}], ",", "\[IndentingNewLine]", 
     RowBox[{"c20", "==", 
      RowBox[{"cp2", "+", 
       RowBox[{"2", " ", "Zc2", " ", "qp2"}]}]}], ",", "\[IndentingNewLine]", 
     RowBox[{"cp10", "==", 
      RowBox[{"c1", "+", 
       RowBox[{"2", " ", "Zc1", " ", "q1"}]}]}], ",", "\[IndentingNewLine]", 
     RowBox[{"cp20", "==", 
      RowBox[{"c2", "+", 
       RowBox[{"2", " ", "Zc2", " ", "q2"}]}]}]}], "\[IndentingNewLine]", 
    "}"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.47858332371875*^9, 3.478583443984375*^9}, {
   3.478596512125*^9, 3.478596630296875*^9}, {3.47859673825*^9, 
   3.4785967756875*^9}, {3.478596964234375*^9, 3.478596992640625*^9}, 
   3.478597029796875*^9, {3.478600289875*^9, 3.47860030565625*^9}, {
   3.51582105284556*^9, 3.51582105678306*^9}, {3.5159004728175173`*^9, 
   3.515900475551874*^9}, {3.5159009659499717`*^9, 3.515900975043605*^9}, {
   3.5159024812862654`*^9, 3.5159025043018904`*^9}, {3.5167064051243*^9, 
   3.516706412796175*^9}, {3.516818175265625*^9, 3.51681819540625*^9}, {
   3.516818378453125*^9, 3.516818390015625*^9}, {3.5719054097791095`*^9, 
   3.5719054179223237`*^9}, {3.637920338944456*^9, 3.6379204078813486`*^9}, {
   3.637921727774511*^9, 3.6379218567835107`*^9}, {3.637922000556511*^9, 
   3.6379220014565105`*^9}, {3.6379220406576195`*^9, 
   3.6379220620867624`*^9}, {3.637922113202873*^9, 3.6379221984864006`*^9}, {
   3.6379222511636677`*^9, 3.637922289725524*^9}, {3.6379233495034914`*^9, 
   3.6379233523067713`*^9}, {3.637925412119732*^9, 3.637925414605981*^9}, {
   3.637925534541185*^9, 3.637925543801259*^9}, {3.6379255929273458`*^9, 
   3.6379255952321153`*^9}, {3.637925654925145*^9, 3.637925657337904*^9}, {
   3.637925807442081*^9, 3.637925809273081*^9}, {3.6379272088394003`*^9, 
   3.6379272215366697`*^9}, 3.6379276574962616`*^9, {3.637928217298236*^9, 
   3.6379282401395197`*^9}, {3.6379283342639313`*^9, 3.637928350841589*^9}, {
   3.6379290915076485`*^9, 3.637929146583155*^9}, {3.637929879900072*^9, 
   3.637929880966072*^9}, {3.6379299249600725`*^9, 3.637929942631072*^9}, {
   3.6379423251481695`*^9, 3.6379423343106933`*^9}, {3.637942398892387*^9, 
   3.637942418300497*^9}, {3.637942678387373*^9, 3.6379426865298386`*^9}, {
   3.63794282903399*^9, 3.6379428423557515`*^9}, {3.637942953646117*^9, 
   3.6379429584243903`*^9}, {3.6379433085964193`*^9, 
   3.6379433099574966`*^9}, {3.6379442243807993`*^9, 
   3.6379442493262258`*^9}, {3.637987263116647*^9, 3.6379872730346627`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"expressions", "=", 
   RowBox[{"{", "\[IndentingNewLine]", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"c1", "==", "c10"}], ",", "\[IndentingNewLine]", 
     RowBox[{"c2", "==", "c20"}], ",", "\[IndentingNewLine]", 
     "\[IndentingNewLine]", 
     RowBox[{"cm1", "\[Equal]", 
      RowBox[{
       RowBox[{"A1", " ", "cp1"}], "-", 
       RowBox[{"A2", " ", "cp2"}]}]}], ",", "\[IndentingNewLine]", 
     RowBox[{"Zcm1", "\[Equal]", 
      RowBox[{
       RowBox[{
        SuperscriptBox["A1", "2"], "Zc1"}], "+", 
       RowBox[{
        SuperscriptBox["A2", "2"], "Zc2"}]}]}]}], "\[IndentingNewLine]", 
    "}"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.47858332371875*^9, 3.478583443984375*^9}, {
   3.478596512125*^9, 3.478596630296875*^9}, {3.47859673825*^9, 
   3.4785967756875*^9}, {3.478596964234375*^9, 3.478596992640625*^9}, 
   3.478597029796875*^9, {3.478600289875*^9, 3.47860030565625*^9}, {
   3.51582105284556*^9, 3.51582105678306*^9}, {3.5159004728175173`*^9, 
   3.515900475551874*^9}, {3.5159009659499717`*^9, 3.515900975043605*^9}, {
   3.5159024812862654`*^9, 3.5159025043018904`*^9}, {3.5167064051243*^9, 
   3.516706412796175*^9}, {3.516818200453125*^9, 3.5168182011875*^9}, {
   3.516818238359375*^9, 3.5168182486875*^9}, {3.51681829715625*^9, 
   3.516818360484375*^9}, 3.516818397609375*^9, {3.5719054223839316`*^9, 
   3.5719054467979746`*^9}, {3.637919287139286*^9, 3.6379193986374345`*^9}, {
   3.6379194785414243`*^9, 3.6379195742739964`*^9}, {3.6379196078273516`*^9, 
   3.6379196182113895`*^9}, {3.6379196712466927`*^9, 
   3.6379196920307713`*^9}, {3.6379202636519275`*^9, 
   3.6379203329018517`*^9}, {3.637921994886511*^9, 3.637921997781511*^9}, 
   3.6379253877282934`*^9, 3.6379254742969494`*^9, {3.6379280397144794`*^9, 
   3.63792807522803*^9}, 3.6379282469381995`*^9, {3.637928279523458*^9, 
   3.63792830274778*^9}, {3.6379294446349573`*^9, 3.6379294580282965`*^9}, {
   3.6379424044447045`*^9, 3.6379424252788963`*^9}, {3.637942513934967*^9, 
   3.6379425181032057`*^9}, {3.6379425599656*^9, 3.6379425684010825`*^9}, 
   3.6379426591582737`*^9, {3.6379428612238307`*^9, 3.6379429070264506`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"expressions", "=", 
   RowBox[{"{", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"c1f", "==", 
      RowBox[{
       RowBox[{"alpha", " ", "c1f"}], " ", "+", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"1.0", "-", "alpha"}], ")"}], " ", "c10"}]}]}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"cp1f", "==", 
      RowBox[{
       RowBox[{"alpha", " ", "cp1f"}], " ", "+", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"1.0", "-", "alpha"}], ")"}], " ", "cp10"}]}]}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"c2f", "==", 
      RowBox[{
       RowBox[{"alpha", " ", "c2f"}], " ", "+", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"1.0", "-", "alpha"}], ")"}], " ", "c20"}]}]}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"cp2f", "==", 
      RowBox[{
       RowBox[{"alpha", " ", "cp2f"}], " ", "+", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"1.0", "-", "alpha"}], ")"}], " ", "cp20"}]}]}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"c1", "==", "c1f"}], ",", "\[IndentingNewLine]", 
     RowBox[{"cp1", "==", "cp1f"}], ",", "\[IndentingNewLine]", 
     RowBox[{"c2", "==", "c2f"}], ",", "\[IndentingNewLine]", 
     RowBox[{"cp2", "==", "cp2f"}], ",", "\[IndentingNewLine]", 
     RowBox[{"cm1", "\[Equal]", 
      RowBox[{
       RowBox[{"A1", " ", "cp1"}], "-", 
       RowBox[{"A2", " ", "cp2"}]}]}], ",", "\[IndentingNewLine]", 
     RowBox[{"Zcm1", "\[Equal]", 
      RowBox[{
       RowBox[{
        SuperscriptBox["A1", "2"], "Zc1"}], "+", 
       RowBox[{
        SuperscriptBox["A2", "2"], "Zc2"}]}]}]}], "\[IndentingNewLine]", 
    "}"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.47858332371875*^9, 3.478583443984375*^9}, {
   3.478596512125*^9, 3.478596630296875*^9}, {3.47859673825*^9, 
   3.4785967756875*^9}, {3.478596964234375*^9, 3.478596992640625*^9}, 
   3.478597029796875*^9, {3.478600289875*^9, 3.47860030565625*^9}, {
   3.51582105284556*^9, 3.51582105678306*^9}, {3.5159004728175173`*^9, 
   3.515900475551874*^9}, {3.5159009659499717`*^9, 3.515900975043605*^9}, {
   3.5159024812862654`*^9, 3.5159025043018904`*^9}, {3.5167064051243*^9, 
   3.516706412796175*^9}, {3.516818200453125*^9, 3.5168182011875*^9}, {
   3.516818238359375*^9, 3.5168182486875*^9}, {3.51681829715625*^9, 
   3.516818360484375*^9}, 3.516818397609375*^9, {3.5719054223839316`*^9, 
   3.5719054467979746`*^9}, {3.637919287139286*^9, 3.6379193986374345`*^9}, {
   3.6379194785414243`*^9, 3.6379195742739964`*^9}, {3.6379196078273516`*^9, 
   3.6379196182113895`*^9}, {3.6379196712466927`*^9, 
   3.6379196920307713`*^9}, {3.6379202636519275`*^9, 
   3.6379203329018517`*^9}, {3.637921994886511*^9, 3.637921997781511*^9}, 
   3.6379253877282934`*^9, 3.6379254742969494`*^9, {3.6379280397144794`*^9, 
   3.63792807522803*^9}, 3.6379282469381995`*^9, {3.637928279523458*^9, 
   3.63792830274778*^9}, {3.637929803376072*^9, 3.6379298051480722`*^9}, {
   3.6379299556270723`*^9, 3.637929956906072*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Compgen", "[", "file", "]"}]], "Input",
 CellChangeTimes->{3.5039025691670113`*^9}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"XMLElement", "::", "cntsList"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"\[NoBreak]\\!\\(XMLElement[\\(\\(\\\"modelobject\\\", \
\\(\\({\\(\\(\\\"typename\\\" \[Rule] \\\"HydraulicPiston\\\"\\)\\), \\(\\(\\\
\"displayname\\\" \[Rule] \\\"HydraulicPiston\\\"\\)\\)}\\)\\), \
\\(\\({\\(\\(XMLElement[\\(\\(\\\"icon\\\", \\(\\({\\(\\(\\\"isopath\\\" \
\[Rule] \\\"HydraulicPiston.svg\\\"\\)\\), \\(\\(\\\"iconrotation\\\" \[Rule] \
\\\"ON\\\"\\)\\), \\(\\(\\\"userpath\\\" \[Rule] \
\\\"HydraulicPiston.svg\\\"\\)\\)}\\)\\), \\(\\({}\\)\\)\\)\\)]\\)\\), \
\\(\\(XMLElement[\\(\\(\\\"portpositions\\\", \\(\\({}\\)\\), \\(\\(\
\[LeftSkeleton] 1 \[RightSkeleton]\\)\\)\\)\\)]\\)\\)}\\)\\)\\)\\)]\\)\
\[NoBreak] in \
\[NoBreak]\\!\\(XMLElement[\\(\\(\\\"hopsanobjectappearance\\\", \
\\(\\({\\\"version\\\" \[Rule] \\\"0.1\\\"}\\)\\), \
\\(\\(XMLElement[\\(\\(\\\"modelobject\\\", \\(\\({\\(\\(\\\"typename\\\" \
\[Rule] \\\"HydraulicPiston\\\"\\)\\), \\(\\(\\\"displayname\\\" \[Rule] \
\\\"HydraulicPiston\\\"\\)\\)}\\)\\), \
\\(\\({\\(\\(XMLElement[\\(\\(\\\"icon\\\", \\(\\({\\(\\(\\\"isopath\\\" \
\[Rule] \\\"HydraulicPiston.svg\\\"\\)\\), \\(\\(\\\"iconrotation\\\" \[Rule] \
\\\"ON\\\"\\)\\), \\(\\(\\\"userpath\\\" \[Rule] \
\\\"HydraulicPiston.svg\\\"\\)\\)}\\)\\), \\(\\({}\\)\\)\\)\\)]\\)\\), \\(\\(\
\[LeftSkeleton] 1 \[RightSkeleton]\\)\\)}\\)\\)\\)\\)]\\)\\)\\)\\)]\\)\
\[NoBreak] is not a list of contents. The third item in an XMLElement must be \
a list of contents, even if it is an empty list. \\!\\(\\*ButtonBox[\\\"\
\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", ButtonFrame->None, \
ButtonData:>\\\"paclet:ref/XMLElement\\\", ButtonNote -> \
\\\"XMLElement::cntsList\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{
  3.5719772217781134`*^9, 3.5756490990611706`*^9, 3.575651855006559*^9, 
   3.575716164804755*^9, 3.5757162192335043`*^9, 3.5757266100333047`*^9, 
   3.5757719989909945`*^9, 3.5757721856449223`*^9, 3.5757724057351866`*^9, 
   3.575772539240221*^9, 3.5757726373892403`*^9, 3.5758602632686234`*^9, 
   3.576214801682736*^9, {3.576214841338736*^9, 3.5762148665017357`*^9}, 
   3.5762154609532423`*^9, {3.5762155675078964`*^9, 3.5762155762397695`*^9}, 
   3.57621592198134*^9, 3.576216002397381*^9, 3.576216058336974*^9, 
   3.576216158149739*^9, 3.576216875232665*^9, 3.5762169607982206`*^9, 
   3.5762170162597666`*^9, 3.576217136553795*^9, {3.5762171980389423`*^9, 
   3.576217226269765*^9}, 3.5762412304325733`*^9, 3.5762413073629165`*^9, {
   3.5762413786668873`*^9, 3.5762414007097263`*^9}, 3.5762414761670585`*^9, 
   3.5762415543075957`*^9, 3.576921315167641*^9, 3.577611867184573*^9, 
   3.637922886716217*^9, 3.6379237506275997`*^9, 3.637925321021623*^9, {
   3.63792540620014*^9, 3.6379254461801376`*^9}, 3.6379254929178114`*^9, 
   3.6379255619534435`*^9, 3.6379256039092474`*^9, {3.6379256415244856`*^9, 
   3.637925713306307*^9}, 3.637925771148522*^9, 3.637925823063081*^9, 
   3.637927236560172*^9, {3.6379276618917007`*^9, 3.637927665452057*^9}, 
   3.6379283713266373`*^9, 3.637929157822279*^9, 3.637929534738967*^9, 
   3.637929808989072*^9, 3.637929986169072*^9, 3.637939978284936*^9, 
   3.63794031123298*^9, 3.6379429688979893`*^9, 3.637943337967099*^9, {
   3.637944228212018*^9, 3.637944253691476*^9}, 3.637987279662337*^9}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"Export", "::", "autofix"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"Malformed symbolic XML expression encountered. This may \
result in unexpected XML data. \\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", \
ButtonStyle->\\\"Link\\\", ButtonFrame->None, \
ButtonData:>\\\"paclet:ref/Export\\\", ButtonNote -> \\\"Export::autofix\\\"]\
\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{
  3.5719772217781134`*^9, 3.5756490990611706`*^9, 3.575651855006559*^9, 
   3.575716164804755*^9, 3.5757162192335043`*^9, 3.5757266100333047`*^9, 
   3.5757719989909945`*^9, 3.5757721856449223`*^9, 3.5757724057351866`*^9, 
   3.575772539240221*^9, 3.5757726373892403`*^9, 3.5758602632686234`*^9, 
   3.576214801682736*^9, {3.576214841338736*^9, 3.5762148665017357`*^9}, 
   3.5762154609532423`*^9, {3.5762155675078964`*^9, 3.5762155762397695`*^9}, 
   3.57621592198134*^9, 3.576216002397381*^9, 3.576216058336974*^9, 
   3.576216158149739*^9, 3.576216875232665*^9, 3.5762169607982206`*^9, 
   3.5762170162597666`*^9, 3.576217136553795*^9, {3.5762171980389423`*^9, 
   3.576217226269765*^9}, 3.5762412304325733`*^9, 3.5762413073629165`*^9, {
   3.5762413786668873`*^9, 3.5762414007097263`*^9}, 3.5762414761670585`*^9, 
   3.5762415543075957`*^9, 3.576921315167641*^9, 3.577611867184573*^9, 
   3.637922886716217*^9, 3.6379237506275997`*^9, 3.637925321021623*^9, {
   3.63792540620014*^9, 3.6379254461801376`*^9}, 3.6379254929178114`*^9, 
   3.6379255619534435`*^9, 3.6379256039092474`*^9, {3.6379256415244856`*^9, 
   3.637925713306307*^9}, 3.637925771148522*^9, 3.637925823063081*^9, 
   3.637927236560172*^9, {3.6379276618917007`*^9, 3.637927665452057*^9}, 
   3.6379283713266373`*^9, 3.637929157822279*^9, 3.637929534738967*^9, 
   3.637929808989072*^9, 3.637929986169072*^9, 3.637939978284936*^9, 
   3.63794031123298*^9, 3.6379429688979893`*^9, 3.637943337967099*^9, {
   3.637944228212018*^9, 3.637944253691476*^9}, 3.637987279667336*^9}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"XMLElement", "::", "attrhs"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"\[NoBreak]\\!\\(0.25`\\)\[NoBreak] in \
\[NoBreak]\\!\\(\\\"y\\\" \[Rule] 0.25`\\)\[NoBreak] is not a valid value for \
an attribute in an XMLElement. The value of the attribute must be a string. \
\\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", \
ButtonFrame->None, ButtonData:>\\\"paclet:ref/XMLElement\\\", ButtonNote -> \
\\\"XMLElement::attrhs\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{
  3.5719772217781134`*^9, 3.5756490990611706`*^9, 3.575651855006559*^9, 
   3.575716164804755*^9, 3.5757162192335043`*^9, 3.5757266100333047`*^9, 
   3.5757719989909945`*^9, 3.5757721856449223`*^9, 3.5757724057351866`*^9, 
   3.575772539240221*^9, 3.5757726373892403`*^9, 3.5758602632686234`*^9, 
   3.576214801682736*^9, {3.576214841338736*^9, 3.5762148665017357`*^9}, 
   3.5762154609532423`*^9, {3.5762155675078964`*^9, 3.5762155762397695`*^9}, 
   3.57621592198134*^9, 3.576216002397381*^9, 3.576216058336974*^9, 
   3.576216158149739*^9, 3.576216875232665*^9, 3.5762169607982206`*^9, 
   3.5762170162597666`*^9, 3.576217136553795*^9, {3.5762171980389423`*^9, 
   3.576217226269765*^9}, 3.5762412304325733`*^9, 3.5762413073629165`*^9, {
   3.5762413786668873`*^9, 3.5762414007097263`*^9}, 3.5762414761670585`*^9, 
   3.5762415543075957`*^9, 3.576921315167641*^9, 3.577611867184573*^9, 
   3.637922886716217*^9, 3.6379237506275997`*^9, 3.637925321021623*^9, {
   3.63792540620014*^9, 3.6379254461801376`*^9}, 3.6379254929178114`*^9, 
   3.6379255619534435`*^9, 3.6379256039092474`*^9, {3.6379256415244856`*^9, 
   3.637925713306307*^9}, 3.637925771148522*^9, 3.637925823063081*^9, 
   3.637927236560172*^9, {3.6379276618917007`*^9, 3.637927665452057*^9}, 
   3.6379283713266373`*^9, 3.637929157822279*^9, 3.637929534738967*^9, 
   3.637929808989072*^9, 3.637929986169072*^9, 3.637939978284936*^9, 
   3.63794031123298*^9, 3.6379429688979893`*^9, 3.637943337967099*^9, {
   3.637944228212018*^9, 3.637944253691476*^9}, 3.637987279668336*^9}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"Export", "::", "autofix"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"Malformed symbolic XML expression encountered. This may \
result in unexpected XML data. \\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", \
ButtonStyle->\\\"Link\\\", ButtonFrame->None, \
ButtonData:>\\\"paclet:ref/Export\\\", ButtonNote -> \\\"Export::autofix\\\"]\
\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{
  3.5719772217781134`*^9, 3.5756490990611706`*^9, 3.575651855006559*^9, 
   3.575716164804755*^9, 3.5757162192335043`*^9, 3.5757266100333047`*^9, 
   3.5757719989909945`*^9, 3.5757721856449223`*^9, 3.5757724057351866`*^9, 
   3.575772539240221*^9, 3.5757726373892403`*^9, 3.5758602632686234`*^9, 
   3.576214801682736*^9, {3.576214841338736*^9, 3.5762148665017357`*^9}, 
   3.5762154609532423`*^9, {3.5762155675078964`*^9, 3.5762155762397695`*^9}, 
   3.57621592198134*^9, 3.576216002397381*^9, 3.576216058336974*^9, 
   3.576216158149739*^9, 3.576216875232665*^9, 3.5762169607982206`*^9, 
   3.5762170162597666`*^9, 3.576217136553795*^9, {3.5762171980389423`*^9, 
   3.576217226269765*^9}, 3.5762412304325733`*^9, 3.5762413073629165`*^9, {
   3.5762413786668873`*^9, 3.5762414007097263`*^9}, 3.5762414761670585`*^9, 
   3.5762415543075957`*^9, 3.576921315167641*^9, 3.577611867184573*^9, 
   3.637922886716217*^9, 3.6379237506275997`*^9, 3.637925321021623*^9, {
   3.63792540620014*^9, 3.6379254461801376`*^9}, 3.6379254929178114`*^9, 
   3.6379255619534435`*^9, 3.6379256039092474`*^9, {3.6379256415244856`*^9, 
   3.637925713306307*^9}, 3.637925771148522*^9, 3.637925823063081*^9, 
   3.637927236560172*^9, {3.6379276618917007`*^9, 3.637927665452057*^9}, 
   3.6379283713266373`*^9, 3.637929157822279*^9, 3.637929534738967*^9, 
   3.637929808989072*^9, 3.637929986169072*^9, 3.637939978284936*^9, 
   3.63794031123298*^9, 3.6379429688979893`*^9, 3.637943337967099*^9, {
   3.637944228212018*^9, 3.637944253691476*^9}, 3.6379872796703353`*^9}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"XMLElement", "::", "attrhs"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"\[NoBreak]\\!\\(0.5`\\)\[NoBreak] in \[NoBreak]\\!\\(\\\"y\
\\\" \[Rule] 0.5`\\)\[NoBreak] is not a valid value for an attribute in an \
XMLElement. The value of the attribute must be a string. \
\\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", \
ButtonFrame->None, ButtonData:>\\\"paclet:ref/XMLElement\\\", ButtonNote -> \
\\\"XMLElement::attrhs\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{
  3.5719772217781134`*^9, 3.5756490990611706`*^9, 3.575651855006559*^9, 
   3.575716164804755*^9, 3.5757162192335043`*^9, 3.5757266100333047`*^9, 
   3.5757719989909945`*^9, 3.5757721856449223`*^9, 3.5757724057351866`*^9, 
   3.575772539240221*^9, 3.5757726373892403`*^9, 3.5758602632686234`*^9, 
   3.576214801682736*^9, {3.576214841338736*^9, 3.5762148665017357`*^9}, 
   3.5762154609532423`*^9, {3.5762155675078964`*^9, 3.5762155762397695`*^9}, 
   3.57621592198134*^9, 3.576216002397381*^9, 3.576216058336974*^9, 
   3.576216158149739*^9, 3.576216875232665*^9, 3.5762169607982206`*^9, 
   3.5762170162597666`*^9, 3.576217136553795*^9, {3.5762171980389423`*^9, 
   3.576217226269765*^9}, 3.5762412304325733`*^9, 3.5762413073629165`*^9, {
   3.5762413786668873`*^9, 3.5762414007097263`*^9}, 3.5762414761670585`*^9, 
   3.5762415543075957`*^9, 3.576921315167641*^9, 3.577611867184573*^9, 
   3.637922886716217*^9, 3.6379237506275997`*^9, 3.637925321021623*^9, {
   3.63792540620014*^9, 3.6379254461801376`*^9}, 3.6379254929178114`*^9, 
   3.6379255619534435`*^9, 3.6379256039092474`*^9, {3.6379256415244856`*^9, 
   3.637925713306307*^9}, 3.637925771148522*^9, 3.637925823063081*^9, 
   3.637927236560172*^9, {3.6379276618917007`*^9, 3.637927665452057*^9}, 
   3.6379283713266373`*^9, 3.637929157822279*^9, 3.637929534738967*^9, 
   3.637929808989072*^9, 3.637929986169072*^9, 3.637939978284936*^9, 
   3.63794031123298*^9, 3.6379429688979893`*^9, 3.637943337967099*^9, {
   3.637944228212018*^9, 3.637944253691476*^9}, 3.6379872796723347`*^9}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"Export", "::", "autofix"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"Malformed symbolic XML expression encountered. This may \
result in unexpected XML data. \\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", \
ButtonStyle->\\\"Link\\\", ButtonFrame->None, \
ButtonData:>\\\"paclet:ref/Export\\\", ButtonNote -> \\\"Export::autofix\\\"]\
\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{
  3.5719772217781134`*^9, 3.5756490990611706`*^9, 3.575651855006559*^9, 
   3.575716164804755*^9, 3.5757162192335043`*^9, 3.5757266100333047`*^9, 
   3.5757719989909945`*^9, 3.5757721856449223`*^9, 3.5757724057351866`*^9, 
   3.575772539240221*^9, 3.5757726373892403`*^9, 3.5758602632686234`*^9, 
   3.576214801682736*^9, {3.576214841338736*^9, 3.5762148665017357`*^9}, 
   3.5762154609532423`*^9, {3.5762155675078964`*^9, 3.5762155762397695`*^9}, 
   3.57621592198134*^9, 3.576216002397381*^9, 3.576216058336974*^9, 
   3.576216158149739*^9, 3.576216875232665*^9, 3.5762169607982206`*^9, 
   3.5762170162597666`*^9, 3.576217136553795*^9, {3.5762171980389423`*^9, 
   3.576217226269765*^9}, 3.5762412304325733`*^9, 3.5762413073629165`*^9, {
   3.5762413786668873`*^9, 3.5762414007097263`*^9}, 3.5762414761670585`*^9, 
   3.5762415543075957`*^9, 3.576921315167641*^9, 3.577611867184573*^9, 
   3.637922886716217*^9, 3.6379237506275997`*^9, 3.637925321021623*^9, {
   3.63792540620014*^9, 3.6379254461801376`*^9}, 3.6379254929178114`*^9, 
   3.6379255619534435`*^9, 3.6379256039092474`*^9, {3.6379256415244856`*^9, 
   3.637925713306307*^9}, 3.637925771148522*^9, 3.637925823063081*^9, 
   3.637927236560172*^9, {3.6379276618917007`*^9, 3.637927665452057*^9}, 
   3.6379283713266373`*^9, 3.637929157822279*^9, 3.637929534738967*^9, 
   3.637929808989072*^9, 3.637929986169072*^9, 3.637939978284936*^9, 
   3.63794031123298*^9, 3.6379429688979893`*^9, 3.637943337967099*^9, {
   3.637944228212018*^9, 3.637944253691476*^9}, 3.6379872796743345`*^9}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"General", "::", "stop"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"Further output of \[NoBreak]\\!\\(\\*StyleBox[\\(Export :: \
autofix\\), \\\"MessageName\\\"]\\)\[NoBreak] will be suppressed during this \
calculation. \\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", \
ButtonStyle->\\\"Link\\\", ButtonFrame->None, \
ButtonData:>\\\"paclet:ref/message/General/stop\\\", ButtonNote -> \
\\\"General::stop\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{
  3.5719772217781134`*^9, 3.5756490990611706`*^9, 3.575651855006559*^9, 
   3.575716164804755*^9, 3.5757162192335043`*^9, 3.5757266100333047`*^9, 
   3.5757719989909945`*^9, 3.5757721856449223`*^9, 3.5757724057351866`*^9, 
   3.575772539240221*^9, 3.5757726373892403`*^9, 3.5758602632686234`*^9, 
   3.576214801682736*^9, {3.576214841338736*^9, 3.5762148665017357`*^9}, 
   3.5762154609532423`*^9, {3.5762155675078964`*^9, 3.5762155762397695`*^9}, 
   3.57621592198134*^9, 3.576216002397381*^9, 3.576216058336974*^9, 
   3.576216158149739*^9, 3.576216875232665*^9, 3.5762169607982206`*^9, 
   3.5762170162597666`*^9, 3.576217136553795*^9, {3.5762171980389423`*^9, 
   3.576217226269765*^9}, 3.5762412304325733`*^9, 3.5762413073629165`*^9, {
   3.5762413786668873`*^9, 3.5762414007097263`*^9}, 3.5762414761670585`*^9, 
   3.5762415543075957`*^9, 3.576921315167641*^9, 3.577611867184573*^9, 
   3.637922886716217*^9, 3.6379237506275997`*^9, 3.637925321021623*^9, {
   3.63792540620014*^9, 3.6379254461801376`*^9}, 3.6379254929178114`*^9, 
   3.6379255619534435`*^9, 3.6379256039092474`*^9, {3.6379256415244856`*^9, 
   3.637925713306307*^9}, 3.637925771148522*^9, 3.637925823063081*^9, 
   3.637927236560172*^9, {3.6379276618917007`*^9, 3.637927665452057*^9}, 
   3.6379283713266373`*^9, 3.637929157822279*^9, 3.637929534738967*^9, 
   3.637929808989072*^9, 3.637929986169072*^9, 3.637939978284936*^9, 
   3.63794031123298*^9, 3.6379429688979893`*^9, 3.637943337967099*^9, {
   3.637944228212018*^9, 3.637944253691476*^9}, 3.6379872796753345`*^9}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"XMLElement", "::", "attrhs"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"\[NoBreak]\\!\\(0.75`\\)\[NoBreak] in \
\[NoBreak]\\!\\(\\\"y\\\" \[Rule] 0.75`\\)\[NoBreak] is not a valid value for \
an attribute in an XMLElement. The value of the attribute must be a string. \
\\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", \
ButtonFrame->None, ButtonData:>\\\"paclet:ref/XMLElement\\\", ButtonNote -> \
\\\"XMLElement::attrhs\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{
  3.5719772217781134`*^9, 3.5756490990611706`*^9, 3.575651855006559*^9, 
   3.575716164804755*^9, 3.5757162192335043`*^9, 3.5757266100333047`*^9, 
   3.5757719989909945`*^9, 3.5757721856449223`*^9, 3.5757724057351866`*^9, 
   3.575772539240221*^9, 3.5757726373892403`*^9, 3.5758602632686234`*^9, 
   3.576214801682736*^9, {3.576214841338736*^9, 3.5762148665017357`*^9}, 
   3.5762154609532423`*^9, {3.5762155675078964`*^9, 3.5762155762397695`*^9}, 
   3.57621592198134*^9, 3.576216002397381*^9, 3.576216058336974*^9, 
   3.576216158149739*^9, 3.576216875232665*^9, 3.5762169607982206`*^9, 
   3.5762170162597666`*^9, 3.576217136553795*^9, {3.5762171980389423`*^9, 
   3.576217226269765*^9}, 3.5762412304325733`*^9, 3.5762413073629165`*^9, {
   3.5762413786668873`*^9, 3.5762414007097263`*^9}, 3.5762414761670585`*^9, 
   3.5762415543075957`*^9, 3.576921315167641*^9, 3.577611867184573*^9, 
   3.637922886716217*^9, 3.6379237506275997`*^9, 3.637925321021623*^9, {
   3.63792540620014*^9, 3.6379254461801376`*^9}, 3.6379254929178114`*^9, 
   3.6379255619534435`*^9, 3.6379256039092474`*^9, {3.6379256415244856`*^9, 
   3.637925713306307*^9}, 3.637925771148522*^9, 3.637925823063081*^9, 
   3.637927236560172*^9, {3.6379276618917007`*^9, 3.637927665452057*^9}, 
   3.6379283713266373`*^9, 3.637929157822279*^9, 3.637929534738967*^9, 
   3.637929808989072*^9, 3.637929986169072*^9, 3.637939978284936*^9, 
   3.63794031123298*^9, 3.6379429688979893`*^9, 3.637943337967099*^9, {
   3.637944228212018*^9, 3.637944253691476*^9}, 3.637987279677334*^9}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"General", "::", "stop"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"Further output of \
\[NoBreak]\\!\\(\\*StyleBox[\\(XMLElement :: attrhs\\), \
\\\"MessageName\\\"]\\)\[NoBreak] will be suppressed during this calculation. \
\\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", \
ButtonFrame->None, ButtonData:>\\\"paclet:ref/message/General/stop\\\", \
ButtonNote -> \\\"General::stop\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{
  3.5719772217781134`*^9, 3.5756490990611706`*^9, 3.575651855006559*^9, 
   3.575716164804755*^9, 3.5757162192335043`*^9, 3.5757266100333047`*^9, 
   3.5757719989909945`*^9, 3.5757721856449223`*^9, 3.5757724057351866`*^9, 
   3.575772539240221*^9, 3.5757726373892403`*^9, 3.5758602632686234`*^9, 
   3.576214801682736*^9, {3.576214841338736*^9, 3.5762148665017357`*^9}, 
   3.5762154609532423`*^9, {3.5762155675078964`*^9, 3.5762155762397695`*^9}, 
   3.57621592198134*^9, 3.576216002397381*^9, 3.576216058336974*^9, 
   3.576216158149739*^9, 3.576216875232665*^9, 3.5762169607982206`*^9, 
   3.5762170162597666`*^9, 3.576217136553795*^9, {3.5762171980389423`*^9, 
   3.576217226269765*^9}, 3.5762412304325733`*^9, 3.5762413073629165`*^9, {
   3.5762413786668873`*^9, 3.5762414007097263`*^9}, 3.5762414761670585`*^9, 
   3.5762415543075957`*^9, 3.576921315167641*^9, 3.577611867184573*^9, 
   3.637922886716217*^9, 3.6379237506275997`*^9, 3.637925321021623*^9, {
   3.63792540620014*^9, 3.6379254461801376`*^9}, 3.6379254929178114`*^9, 
   3.6379255619534435`*^9, 3.6379256039092474`*^9, {3.6379256415244856`*^9, 
   3.637925713306307*^9}, 3.637925771148522*^9, 3.637925823063081*^9, 
   3.637927236560172*^9, {3.6379276618917007`*^9, 3.637927665452057*^9}, 
   3.6379283713266373`*^9, 3.637929157822279*^9, 3.637929534738967*^9, 
   3.637929808989072*^9, 3.637929986169072*^9, 3.637939978284936*^9, 
   3.63794031123298*^9, 3.6379429688979893`*^9, 3.637943337967099*^9, {
   3.637944228212018*^9, 3.637944253691476*^9}, 3.6379872796813335`*^9}]
}, Open  ]]
}, Open  ]]
}, Open  ]]
},
WindowSize->{889, 673},
WindowMargins->{{250, Automatic}, {Automatic, -32}},
PrintingCopies->1,
PrintingPageRange->{32000, 32000},
PrintingOptions->{"Magnification"->1.,
"PaperOrientation"->"Portrait",
"PaperSize"->{582.9599999999999, 829.0799999999999}},
Magnification->1.2000000476837158`,
FrontEndVersion->"8.0 for Microsoft Windows (64-bit) (October 6, 2011)",
StyleDefinitions->"Default.nb"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[CellGroupData[{
Cell[579, 22, 195, 3, 85, "Section"],
Cell[777, 27, 248, 4, 34, "Input"],
Cell[1028, 33, 325, 9, 34, "Input"],
Cell[1356, 44, 1345, 31, 195, "Input"],
Cell[CellGroupData[{
Cell[2726, 79, 97, 1, 43, "Subsection"],
Cell[2826, 82, 1142, 25, 95, "Input"],
Cell[3971, 109, 3058, 58, 206, "Input",
 CellGroupingRules->{GroupTogetherGrouping, 10000.}],
Cell[7032, 169, 838, 16, 215, "Input"],
Cell[7873, 187, 3912, 79, 361, "Input"],
Cell[11788, 268, 2180, 39, 183, "Input"],
Cell[13971, 309, 3000, 66, 263, "Input"],
Cell[CellGroupData[{
Cell[16996, 379, 107, 2, 34, "Input"],
Cell[17106, 383, 3340, 51, 26, "Message"],
Cell[20449, 436, 1980, 31, 26, "Message"],
Cell[22432, 469, 2097, 32, 26, "Message"],
Cell[24532, 503, 1982, 31, 26, "Message"],
Cell[26517, 536, 2097, 32, 26, "Message"],
Cell[28617, 570, 1982, 31, 26, "Message"],
Cell[30602, 603, 2056, 32, 26, "Message"],
Cell[32661, 637, 2097, 32, 26, "Message"],
Cell[34761, 671, 2059, 32, 26, "Message"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}
]
*)

(* End of internal cache information *)
