(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 8.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[     21963,        595]
NotebookOptionsPosition[     19846,        528]
NotebookOutlinePosition[     20189,        543]
CellTagsIndexPosition[     20146,        540]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{

Cell[CellGroupData[{
Cell[TextData[{
 StyleBox["PistonMload\n",
  FontFamily->"System"],
 StyleBox["Piston with an inertia load",
  FontSize->16]
}], "Section",
 Evaluatable->False,
 CellChangeTimes->{{3.5719259951024055`*^9, 3.571926013136037*^9}, {
  3.57192608862457*^9, 3.57192608876497*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"domain", "=", "\"\<Hydraulic\>\""}], ";"}], "\n", 
 RowBox[{
  RowBox[{"displayName", "=", "\"\<PistonMload\>\""}], ";"}], "\n", 
 RowBox[{
  RowBox[{"brief", "=", "\"\<This is piston with an inertia load\>\""}], 
  ";"}], "\n", 
 RowBox[{
  RowBox[{"componentType", "=", "\"\<ComponentQ\>\""}], ";"}], "\n", 
 RowBox[{
  RowBox[{"author", "=", "\"\<Petter Krus <petter.krus@liu.se>\>\""}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
  "affiliation", " ", "=", " ", 
   "\"\<Division of Fluid and Mechatronic Systems, Link\[ODoubleDot]ping \
University\>\""}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"SetFilenames", "[", 
   RowBox[{"path", ",", "domain", ",", "displayName"}], "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"ResetComponentVariables", "[", "]"}], ";"}]}], "Input",
 CellChangeTimes->{{3.5137112151875*^9, 3.513711217109375*^9}, {
   3.51378596903125*^9, 3.513785980671875*^9}, 3.515394539753217*^9, {
   3.515775693875*^9, 3.5157757050625*^9}, {3.517048046727305*^9, 
   3.51704806093043*^9}, 3.51707206725*^9, {3.5176530686834764`*^9, 
   3.51765311057437*^9}, {3.517673126703125*^9, 3.517673126796875*^9}, {
   3.5177214423487377`*^9, 3.517721453114225*^9}, {3.5177222281793466`*^9, 
   3.517722236008023*^9}, {3.5209603293125*^9, 3.520960329578125*^9}, 
   3.521782180515625*^9, {3.52178222675*^9, 3.521782271609375*^9}, {
   3.521782314984375*^9, 3.52178231528125*^9}, 3.575772117551203*^9}],

Cell[CellGroupData[{

Cell["Component description", "Subsection"],

Cell["Piston with an inertia load", "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Component equations", "Subsection"],

Cell["The name of the component  is stored in ComponentName.", "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Variables and parameters", "Subsection"],

Cell[BoxData[
 RowBox[{
  RowBox[{"inputParameters", "  ", "=", " ", 
   RowBox[{"{", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
      "A1", ",", "0.001", ",", "double", ",", "\"\<m2\>\"", ",", 
       "\"\<Piston area 1\>\""}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
      "A2", ",", "0.001", ",", "double", ",", "\"\<m2\>\"", ",", 
       "\"\<Piston area 2\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "SL", ",", "0.5", ",", "double", ",", "\"\<m\>\"", ",", 
       "\"\<Stroke\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "Cip", ",", "0.", ",", "double", ",", "\"\<m3/(s Pa)\>\"", ",", 
       "\"\<Leak coeff.\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "bp", ",", "0.", ",", "double", ",", "\"\<N/m/s\>\"", ",", 
       "\"\<Visc. friction coeff.\>\""}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
      "mL", ",", "1000.", ",", "double", ",", "\"\<kg\>\"", ",", 
       "\"\<Inertia\>\""}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
      "bL", ",", "0.", ",", "double", ",", "\"\<Ns/m\>\"", ",", 
       "\"\<Viscous friction coefficient of load\>\""}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
      "fc", ",", "10.", ",", "double", ",", "\"\<N\>\"", ",", 
       "\"\<Dry friction (+/-)\>\""}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
      "bfc", ",", "1.", ",", "double", ",", "\"\<\>\"", ",", 
       "\"\<Numerical friction factor.\>\""}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
      "xmin", ",", "0.", ",", "double", ",", "\"\<m\>\"", ",", 
       "\"\<Limitation on stroke\>\""}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
      "xmax", ",", "0.5", ",", "double", ",", "\"\<m\>\"", ",", 
       "\"\<Limitation on stroke\>\""}], "}"}]}], "\n", "\t", "}"}]}], 
  ";"}]], "Input",
 CellChangeTimes->{{3.524850755625*^9, 3.524850755640625*^9}, {
   3.633060752150016*^9, 3.633060808182619*^9}, 3.633061611957263*^9, {
   3.633061642712798*^9, 3.633061687746762*^9}, 3.63306483268377*^9, 
   3.6330667585349245`*^9, {3.633066835812377*^9, 3.6330668390900326`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"nodeConnections", "=", 
   RowBox[{"{", "\n", "    \t", 
    RowBox[{
     RowBox[{"HydraulicQnode", "[", 
      RowBox[{"1", ",", 
       RowBox[{"1.", "*", 
        RowBox[{"10", "^", "5"}]}], ",", "\"\<hydraulic node 1\>\""}], "]"}], 
     ",", "\n", "    \t", 
     RowBox[{"HydraulicQnode", "[", 
      RowBox[{"2", ",", 
       RowBox[{"1.", "*", 
        RowBox[{"10", "^", "5"}]}], ",", "\"\<hydraulic node 2\>\""}], "]"}], 
     ",", "\n", "    \t", 
     RowBox[{"MechanicQnode", "[", 
      RowBox[{"mp", ",", "0.", ",", "\"\<mechanical node\>\""}], "]"}]}], 
    "}"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.524850747078125*^9, 3.524850747078125*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["The system of equations", "Subsection"],

Cell[BoxData[
 RowBox[{"\t", 
  RowBox[{
   RowBox[{
    RowBox[{"p1e", " ", "=", " ", 
     RowBox[{"c1", " ", "+", " ", 
      RowBox[{"Zc1", " ", "q1"}]}]}], ";"}], "\n", "\t", 
   RowBox[{
    RowBox[{"p2e", " ", "=", " ", 
     RowBox[{"c2", " ", "+", " ", 
      RowBox[{"Zc2", " ", "q2"}]}]}], ";"}], "\n", "\t", 
   RowBox[{
    RowBox[{"fmpe", "=", " ", 
     RowBox[{"cmp", " ", "+", " ", 
      RowBox[{"Zcmp", " ", "vmp"}]}]}], ";"}]}]}]], "Input",
 CellChangeTimes->{{3.525533762265625*^9, 3.5255337993125*^9}}],

Cell["The generated piston force", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"fg", " ", "=", " ", 
   RowBox[{
    RowBox[{"A1", " ", "p1e"}], " ", "-", " ", 
    RowBox[{"A2", " ", "p2e"}]}]}], ";"}]], "Input",
 CellChangeTimes->{{3.52178209578125*^9, 3.52178209959375*^9}, {
  3.525533871796875*^9, 3.525533873453125*^9}}],

Cell["\<\
Dry friction is modelled with a small linear region for low speeds for \
numerical reasons\
\>", "Text",
 CellGroupingRules->{GroupTogetherGrouping, 10000.},
 CellChangeTimes->{{3.6326806773038225`*^9, 3.632680720320283*^9}, 
   3.6326809028107214`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Bf", "=", 
   RowBox[{"bfc", " ", 
    RowBox[{"mL", "/", "mTimestep"}]}]}], ";"}]], "Input",
 CellChangeTimes->{{3.632693130341366*^9, 3.632693147399341*^9}, {
   3.632693683040978*^9, 3.632693683406999*^9}, 3.6326938099692383`*^9, {
   3.632938375020041*^9, 3.632938375792085*^9}, {3.6329384152553425`*^9, 
   3.632938421068675*^9}, {3.63293920057626*^9, 3.632939201080289*^9}, 
   3.6330608651106114`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"fre", "=", 
   RowBox[{"limit", "[", 
    RowBox[{
     RowBox[{"Bf", " ", "vmp"}], ",", 
     RowBox[{"-", "fc"}], ",", "fc"}], "]"}]}], ";"}]], "Input",
 CellGroupingRules->{GroupTogetherGrouping, 10000.},
 CellChangeTimes->{{3.6326803828739824`*^9, 3.632680423655315*^9}, 
   3.6326809028107214`*^9, 3.632691667554699*^9, {3.632693062831504*^9, 
   3.6326930803705072`*^9}, {3.63269316205818*^9, 3.6326931700536366`*^9}, 
   3.633061731221661*^9}],

Cell["Differrential algebraic equations", "Text",
 CellChangeTimes->{{3.5719031558017125`*^9, 3.571903170964939*^9}}],

Cell[BoxData[
 RowBox[{"systemEquationsDA", " ", ":=", " ", 
  RowBox[{"{", "\n", "\t", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"mL", " ", 
       RowBox[{"der", "[", 
        RowBox[{"der", "[", "xmp", "]"}], "]"}]}], " ", "+", " ", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{"bp", "+", "bL"}], ")"}], " ", "vmp"}]}], " ", "==", " ", 
     RowBox[{"(", 
      RowBox[{"fg", " ", "-", " ", "fmpe"}], ")"}]}], ",", "\n", "\t", 
    RowBox[{
     RowBox[{
      RowBox[{"mL", " ", 
       RowBox[{"der", "[", "vmp", "]"}]}], " ", "+", " ", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{"bp", "+", "bL"}], ")"}], " ", "vmp"}]}], "  ", "==", " ", 
     RowBox[{"(", 
      RowBox[{"fg", " ", "-", " ", "fmpe"}], ")"}]}], ",", "\n", "\t", 
    RowBox[{"q1", " ", "\[Equal]", 
     RowBox[{"-", " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"A1", " ", "vmp"}], " ", "+", " ", 
        RowBox[{"Cip", "*", 
         RowBox[{"(", 
          RowBox[{"p1", "-", "p2"}], ")"}]}]}], ")"}]}]}], ",", "\n", "\t", 
    RowBox[{"q2", " ", "==", " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"A2", " ", "vmp"}], " ", "+", " ", 
       RowBox[{"Cip", "*", 
        RowBox[{"(", 
         RowBox[{"p1", "-", "p2"}], ")"}]}]}], ")"}]}]}], "\n", "\t", 
   "}"}]}]], "Input",
 CellChangeTimes->{{3.524850724265625*^9, 3.524850724265625*^9}, {
  3.5248862080873957`*^9, 3.5248862098842244`*^9}, {3.525533811359375*^9, 
  3.5255338209375*^9}, {3.525533863078125*^9, 3.52553386709375*^9}, {
  3.5719029473073463`*^9, 3.5719029663237796`*^9}, {3.633066779674152*^9, 
  3.6330667905603285`*^9}, {3.633068987371736*^9, 3.633068987382737*^9}}],

Cell["The boundarys", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"systemBoundaryEquations", " ", "=", " ", 
   RowBox[{"{", "\n", "\t", 
    RowBox[{
     RowBox[{"p1", " ", "==", " ", 
      RowBox[{"c1", " ", "+", " ", 
       RowBox[{"Zc1", " ", "q1"}]}]}], ",", "\n", "\t", 
     RowBox[{"p2", " ", "==", " ", 
      RowBox[{"c2", " ", "+", " ", 
       RowBox[{"Zc2", " ", "q2"}]}]}], ",", "\n", "\t", 
     RowBox[{"fmp", " ", "==", " ", 
      RowBox[{"cmp", " ", "+", " ", 
       RowBox[{"Zcmp", " ", "vmp"}]}]}]}], "\[IndentingNewLine]", "}"}]}], 
  ";"}]], "Input",
 CellChangeTimes->{{3.52485078196875*^9, 3.52485078196875*^9}, {
  3.5719029765417976`*^9, 3.571902992984226*^9}}],

Cell["The  vector of independent variables of the system are", "Text",
 CellChangeTimes->{{3.524850810625*^9, 3.52485083609375*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"systemVariables", "=", " ", 
   RowBox[{"{", 
    RowBox[{
    "xmp", ",", "vmp", ",", "q1", ",", "q2", ",", "p1", ",", "p2", ",", 
     "fmp"}], "}"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.524850716875*^9, 3.524850716875*^9}}],

Cell["Limitatons", "Text",
 CellChangeTimes->{{3.524850848515625*^9, 3.524850857984375*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"variable2Limits", "=", 
   RowBox[{"{", 
    RowBox[{"{", 
     RowBox[{"xmp", ",", "vmp", ",", "xmin", ",", "xmax"}], "}"}], "}"}]}], 
  ";"}]], "Input",
 CellChangeTimes->{{3.521781710359375*^9, 3.521781715921875*^9}, 
   3.521781911703125*^9, {3.521782089203125*^9, 3.521782090109375*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"variableLowLimits", "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"p1", ",", "0"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"p2", ",", "0"}], "}"}]}], "}"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.578505814681842*^9, 3.5785058445435495`*^9}, 
   3.578509359037567*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Compgen", "[", "file", "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"Part", "::", "partd"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"Part specification \[NoBreak]\\!\\(delayedPart \
\[LeftDoubleBracket] \\(\\(1, 1\\)\\) \[RightDoubleBracket]\\)\[NoBreak] is \
longer than depth of object. \\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", \
ButtonStyle->\\\"Link\\\", ButtonFrame->None, \
ButtonData:>\\\"paclet:ref/message/General/partd\\\", ButtonNote -> \
\\\"Part::partd\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{{3.6330668180128183`*^9, 3.6330668441650476`*^9}, 
   3.6330689911261115`*^9}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"Part", "::", "partd"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"Part specification \[NoBreak]\\!\\(delayedPart \
\[LeftDoubleBracket] \\(\\(1, 2\\)\\) \[RightDoubleBracket]\\)\[NoBreak] is \
longer than depth of object. \\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", \
ButtonStyle->\\\"Link\\\", ButtonFrame->None, \
ButtonData:>\\\"paclet:ref/message/General/partd\\\", ButtonNote -> \
\\\"Part::partd\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{{3.6330668180128183`*^9, 3.6330668441650476`*^9}, 
   3.633068991128112*^9}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"Part", "::", "partd"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"Part specification \[NoBreak]\\!\\(delayedPart \
\[LeftDoubleBracket] \\(\\(2, 1\\)\\) \[RightDoubleBracket]\\)\[NoBreak] is \
longer than depth of object. \\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", \
ButtonStyle->\\\"Link\\\", ButtonFrame->None, \
ButtonData:>\\\"paclet:ref/message/General/partd\\\", ButtonNote -> \
\\\"Part::partd\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{{3.6330668180128183`*^9, 3.6330668441650476`*^9}, 
   3.633068991140113*^9}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"General", "::", "stop"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"Further output of \[NoBreak]\\!\\(\\*StyleBox[\\(Part :: \
partd\\), \\\"MessageName\\\"]\\)\[NoBreak] will be suppressed during this \
calculation. \\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", \
ButtonStyle->\\\"Link\\\", ButtonFrame->None, \
ButtonData:>\\\"paclet:ref/message/General/stop\\\", ButtonNote -> \
\\\"General::stop\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{{3.6330668180128183`*^9, 3.6330668441650476`*^9}, 
   3.6330689911411133`*^9}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"XMLElement", "::", "cntsList"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"\[NoBreak]\\!\\(XMLElement[\\(\\(\\\"modelobject\\\", \
\\(\\({\\(\\(\\\"typename\\\" \[Rule] \\\"HydraulicPistonMload\\\"\\)\\), \\(\
\\(\\\"displayname\\\" \[Rule] \\\"HydraulicPistonMload\\\"\\)\\)}\\)\\), \\(\
\\({\\(\\(XMLElement[\\(\\(\\\"icon\\\", \\(\\({\\(\\(\\\"isopath\\\" \[Rule] \
\\\"HydraulicPistonMload.svg\\\"\\)\\), \\(\\(\\\"iconrotation\\\" \[Rule] \\\
\"ON\\\"\\)\\), \\(\\(\\\"userpath\\\" \[Rule] \\\"HydraulicPistonMload.svg\\\
\"\\)\\)}\\)\\), \\(\\({}\\)\\)\\)\\)]\\)\\), \
\\(\\(XMLElement[\\(\\(\\\"portpositions\\\", \\(\\({}\\)\\), \\(\\({\\(\
\[LeftSkeleton] 1 \[RightSkeleton]\\)}\\)\\)\\)\\)]\\)\\)}\\)\\)\\)\\)]\\)\
\[NoBreak] in \[NoBreak]\\!\\(\[LeftSkeleton] 1 \[RightSkeleton]\\)\[NoBreak] \
is not a list of contents. The third item in an XMLElement must be a list of \
contents, even if it is an empty list. \
\\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", \
ButtonFrame->None, ButtonData:>\\\"paclet:ref/XMLElement\\\", ButtonNote -> \
\\\"XMLElement::cntsList\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{{3.6330668180128183`*^9, 3.6330668441650476`*^9}, 
   3.6330689911971188`*^9}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"Export", "::", "autofix"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"Malformed symbolic XML expression encountered. This may \
result in unexpected XML data. \\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", \
ButtonStyle->\\\"Link\\\", ButtonFrame->None, \
ButtonData:>\\\"paclet:ref/Export\\\", ButtonNote -> \\\"Export::autofix\\\"]\
\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{{3.6330668180128183`*^9, 3.6330668441650476`*^9}, 
   3.6330689911981187`*^9}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"XMLElement", "::", "attrhs"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"\[NoBreak]\\!\\(0.25`\\)\[NoBreak] in \
\[NoBreak]\\!\\(\\\"y\\\" \[Rule] 0.25`\\)\[NoBreak] is not a valid value for \
an attribute in an XMLElement. The value of the attribute must be a string. \
\\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", \
ButtonFrame->None, ButtonData:>\\\"paclet:ref/XMLElement\\\", ButtonNote -> \
\\\"XMLElement::attrhs\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{{3.6330668180128183`*^9, 3.6330668441650476`*^9}, 
   3.6330689911991186`*^9}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"Export", "::", "autofix"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"Malformed symbolic XML expression encountered. This may \
result in unexpected XML data. \\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", \
ButtonStyle->\\\"Link\\\", ButtonFrame->None, \
ButtonData:>\\\"paclet:ref/Export\\\", ButtonNote -> \\\"Export::autofix\\\"]\
\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{{3.6330668180128183`*^9, 3.6330668441650476`*^9}, 
   3.633068991200119*^9}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"XMLElement", "::", "attrhs"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"\[NoBreak]\\!\\(0.5`\\)\[NoBreak] in \[NoBreak]\\!\\(\\\"y\
\\\" \[Rule] 0.5`\\)\[NoBreak] is not a valid value for an attribute in an \
XMLElement. The value of the attribute must be a string. \
\\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", \
ButtonFrame->None, ButtonData:>\\\"paclet:ref/XMLElement\\\", ButtonNote -> \
\\\"XMLElement::attrhs\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{{3.6330668180128183`*^9, 3.6330668441650476`*^9}, 
   3.633068991201119*^9}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"Export", "::", "autofix"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"Malformed symbolic XML expression encountered. This may \
result in unexpected XML data. \\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", \
ButtonStyle->\\\"Link\\\", ButtonFrame->None, \
ButtonData:>\\\"paclet:ref/Export\\\", ButtonNote -> \\\"Export::autofix\\\"]\
\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{{3.6330668180128183`*^9, 3.6330668441650476`*^9}, 
   3.633068991202119*^9}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"General", "::", "stop"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"Further output of \[NoBreak]\\!\\(\\*StyleBox[\\(Export :: \
autofix\\), \\\"MessageName\\\"]\\)\[NoBreak] will be suppressed during this \
calculation. \\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", \
ButtonStyle->\\\"Link\\\", ButtonFrame->None, \
ButtonData:>\\\"paclet:ref/message/General/stop\\\", ButtonNote -> \
\\\"General::stop\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{{3.6330668180128183`*^9, 3.6330668441650476`*^9}, 
   3.6330689912031193`*^9}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"XMLElement", "::", "attrhs"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"\[NoBreak]\\!\\(0.75`\\)\[NoBreak] in \
\[NoBreak]\\!\\(\\\"y\\\" \[Rule] 0.75`\\)\[NoBreak] is not a valid value for \
an attribute in an XMLElement. The value of the attribute must be a string. \
\\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", \
ButtonFrame->None, ButtonData:>\\\"paclet:ref/XMLElement\\\", ButtonNote -> \
\\\"XMLElement::attrhs\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{{3.6330668180128183`*^9, 3.6330668441650476`*^9}, 
   3.633068991204119*^9}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"General", "::", "stop"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"Further output of \
\[NoBreak]\\!\\(\\*StyleBox[\\(XMLElement :: attrhs\\), \
\\\"MessageName\\\"]\\)\[NoBreak] will be suppressed during this calculation. \
\\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", \
ButtonFrame->None, ButtonData:>\\\"paclet:ref/message/General/stop\\\", \
ButtonNote -> \\\"General::stop\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{{3.6330668180128183`*^9, 3.6330668441650476`*^9}, 
   3.6330689912061195`*^9}]
}, Open  ]],

Cell[BoxData["\[IndentingNewLine]"], "Input",
 CellChangeTimes->{3.6330689387478743`*^9}]
}, Open  ]]
}, Open  ]]
},
WindowSize->{707, 505},
WindowMargins->{{Automatic, 19}, {Automatic, 30}},
FrontEndVersion->"8.0 for Microsoft Windows (64-bit) (October 6, 2011)",
StyleDefinitions->"Default.nb"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[CellGroupData[{
Cell[579, 22, 275, 8, 92, "Section",
 Evaluatable->False],
Cell[857, 32, 1486, 32, 172, "Input"],
Cell[CellGroupData[{
Cell[2368, 68, 43, 0, 36, "Subsection"],
Cell[2414, 70, 43, 0, 29, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2494, 75, 41, 0, 36, "Subsection"],
Cell[2538, 77, 70, 0, 29, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2645, 82, 46, 0, 36, "Subsection"],
Cell[2694, 84, 2166, 53, 272, "Input"],
Cell[4863, 139, 701, 18, 92, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[5601, 162, 45, 0, 36, "Subsection"],
Cell[5649, 164, 524, 15, 72, "Input"],
Cell[6176, 181, 42, 0, 29, "Text"],
Cell[6221, 183, 281, 7, 31, "Input"],
Cell[6505, 192, 263, 6, 29, "Text",
 CellGroupingRules->{GroupTogetherGrouping, 10000.}],
Cell[6771, 200, 443, 9, 31, "Input"],
Cell[7217, 211, 485, 11, 31, "Input",
 CellGroupingRules->{GroupTogetherGrouping, 10000.}],
Cell[7705, 224, 117, 1, 29, "Text"],
Cell[7825, 227, 1666, 43, 132, "Input"],
Cell[9494, 272, 29, 0, 29, "Text"],
Cell[9526, 274, 660, 16, 112, "Input"],
Cell[10189, 292, 132, 1, 29, "Text"],
Cell[10324, 295, 266, 7, 31, "Input"],
Cell[10593, 304, 92, 1, 29, "Text"],
Cell[10688, 307, 327, 8, 31, "Input"],
Cell[11018, 317, 334, 10, 31, "Input"],
Cell[CellGroupData[{
Cell[11377, 331, 63, 1, 31, "Input"],
Cell[11443, 334, 588, 12, 21, "Message"],
Cell[12034, 348, 586, 12, 21, "Message"],
Cell[12623, 362, 586, 12, 21, "Message"],
Cell[13212, 376, 584, 12, 21, "Message"],
Cell[13799, 390, 1287, 21, 107, "Message"],
Cell[15089, 413, 514, 11, 21, "Message"],
Cell[15606, 426, 631, 12, 39, "Message"],
Cell[16240, 440, 512, 11, 21, "Message"],
Cell[16755, 453, 627, 12, 39, "Message"],
Cell[17385, 467, 512, 11, 21, "Message"],
Cell[17900, 480, 588, 12, 21, "Message"],
Cell[18491, 494, 629, 12, 39, "Message"],
Cell[19123, 508, 591, 12, 21, "Message"]
}, Open  ]],
Cell[19729, 523, 89, 1, 52, "Input"]
}, Open  ]]
}, Open  ]]
}
]
*)

(* End of internal cache information *)
