(* Content-type: application/mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 6.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       145,          7]
NotebookDataLength[    490230,      10675]
NotebookOptionsPosition[    476128,      10264]
NotebookOutlinePosition[    476858,      10290]
CellTagsIndexPosition[    476815,      10287]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{

Cell[CellGroupData[{
Cell["AeroAircraft6DOFSS", "Title",
 CellChangeTimes->{{3.6058538552194967`*^9, 3.6058538650474787`*^9}}],

Cell["\<\
Flight dynamics model of super-sonic aircraft\
\>", "Subtitle",
 CellChangeTimes->{{3.6058538375177264`*^9, 3.605853874265401*^9}}],

Cell[BoxData[
 RowBox[{"<<", "C:\\\\Hopsan\\Compgen\\CompgenNG.mx"}]], "Input",
 CellChangeTimes->{{3.5772037650720577`*^9, 3.5772037650780582`*^9}, {
  3.611474457184202*^9, 3.611474523938877*^9}, {3.6114745588913717`*^9, 
  3.61147456057154*^9}}],

Cell[BoxData[
 RowBox[{"Off", "[", 
  RowBox[{"General", "::", "\"\<spell1\>\""}], "]"}]], "Input",
 PageWidth->WindowWidth],

Cell[BoxData[
 RowBox[{
  RowBox[{"path", "=", 
   RowBox[{"ToFileName", "[", 
    RowBox[{"{", 
     RowBox[{
     "\"\<C:\>\"", ",", "\"\<HopsanTrunk\>\"", ",", 
      "\"\<ComponentLibraries\>\"", ",", "\"\<defaultLibrary\>\"", ",", 
      "\"\<Special\>\"", ",", "\"\<AeroComponents\>\""}], "}"}], "]"}]}], 
  ";"}]], "Input",
 CellChangeTimes->{
  3.5812370486543765`*^9, {3.58123709880517*^9, 3.5812371004744773`*^9}, {
   3.6058542560025706`*^9, 3.605854267307701*^9}}],

Cell[TextData[{
 "\nFlight dynamics simulation is used in a wide range of applications from \
aircraft design to operations evaluation and flight training. This means that \
the models are to be used over very different time scales. For aircraft \
design very detailed flight dynamics characteristics needs to be evaluated, \
while much simplified models can be used for simualation of missions and to \
represent other planes in pilot training (and computer games). In order to \
meat these ends of the spectra the simulation models must incorporate a lot \
of detail while on the other hand be robust so that they can be used also \
with large time steps. Here the symbolic math packages ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " is used. By using symbolic manipulation high level differential algebraic \
equations can be transformed into low level code ( C++) where highly robust \
numerical solvers are integrated into the model, for highly efficient robust \
simulation."
}], "Abstract",
 CellChangeTimes->{{3.6058539492108946`*^9, 3.605853978129786*^9}}],

Cell[BoxData[
 Graphics3DBox[{GraphicsComplex3DBox[CompressedData["
1:eJxtmnlYzd36xlMhs0yZUsiQlF5Eof2kiEgqM5UoJGWeRYkMSSpDSpoHIipJ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    "], {
     {EdgeForm[None], GraphicsGroup3DBox[Polygon3DBox[CompressedData["
1:eJxNm3fcFsW5hvd91ZjE2LsxCqigiIrYNUqxYQcL2AVBRcUuVkQQRAUrAlZU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        "]]]}, {}, {}, {}, {}},
    VertexNormals->CompressedData["
1:eJxNmnk41F0fxlsIkZS0KEWbSqHEgzTfUlpUlpItikpZ0mLLVkiFKJLsS8gW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     "]], 
   GraphicsComplex3DBox[CompressedData["
1:eJx12lmIlWUcgPFpKlrQi0LT0jH3JRsdlTJydCw1jFLMpNVEUqFEcQ9yQzGc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    "], {
     {EdgeForm[None], GraphicsGroup3DBox[Polygon3DBox[CompressedData["
1:eJxNmnecFEUChWtGFBNKZkEkiyJHzklYlhyXnHOQHCRJzggKSjIBKkEUFURA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        "]]]}, {}, {}, {}, {}},
    VertexNormals->CompressedData["
1:eJztm11sFFUYhlfFn4Qa2wol/VEgtpq2F1IuGuSiEwOoEYMJbYwSNP1hA4mt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     "]], GraphicsComplex3DBox[CompressedData["
1:eJxtnXdczt/7x5MZieyVkUJIlBXVyQjJzCbxsZWQrYyUyAgZZZS9ySoN6b5K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    "], {
     {EdgeForm[None], GraphicsGroup3DBox[Polygon3DBox[CompressedData["
1:eJxN3WWYLdXRBeC+Z84Ed3d3d7i4u7sE16CB4K4JIUgIBALhC5oECG5BggWX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        "]]]}, {}, {}, {}, {}},
    VertexNormals->CompressedData["
1:eJxUm3k4Vu339lGhlNIkyhChTIVSovtKkVkDFSWVSqQUlUyZk8w0mFKZStKA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     "]], 
   GraphicsComplex3DBox[CompressedData["
1:eJx1mG2IFlUUxx+10cLNl7VxlWRZbV3RhZREKTedq5YSBpHEGhEYhYoauhn0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    "], {
     {EdgeForm[None], GraphicsGroup3DBox[Polygon3DBox[CompressedData["
1:eJxNmQkYllMWx9+X7CJS2UIqiYhKKimUlIo2KZQWSlJZQilRaKGVpF1p0SaK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        "]]]}, {}, {}, {}, {}},
    VertexNormals->CompressedData["
1:eJztWmtQFFcaHQRkA1ERH4lrYI0W0UUxroiAbs8AZsHFRwJFcH2wGIVVk1j4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     "]], 
   GraphicsComplex3DBox[CompressedData["
1:eJx1WXtYzdka3iKZItdcDrq4PC6TJFForJWZETM9iDoiHY6US4NhKKbcqpkM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    "], {
     {EdgeForm[None], GraphicsGroup3DBox[Polygon3DBox[CompressedData["
1:eJxdmgnYV9MWh8+RKZRUROYkUopKigaVJCQlDVKalFQyZYhKIYooipIxdStD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        
        "]]]}, {}, {}, {}, {}},
    VertexNormals->CompressedData["
1:eJztmnlYj2nbxyNMY0n7QnalaRGJN8t0l5QlS2RpJ6JsCWWJZBmkUipZhiGM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     "]], GraphicsComplex3DBox[CompressedData["
1:eJxtmXk01O37x2cYY5ddlkr2bKWkxMO0kMo6qESrSJ6UhEdkS1narE+WVEII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    "], {
     {EdgeForm[None], GraphicsGroup3DBox[Polygon3DBox[CompressedData["
1:eJxNmQkYllMWx9+X7CJS2UIqiYhKKimUlIo2KZQWSlJZQilRaKGVpF1p0SaK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        "]]]}, {}, {}, {}, {}},
    VertexNormals->CompressedData["
1:eJztl8EJhEAMReNaiZXYw5YgeLYFO7OC7cUSvOxpJR40w/8L75/kEZLwmBlw
mJb3/IqITxfRx2/27YQseZZ9/H6s5tzMJ561PIudTzxruZlPPGt5FjufeNZy
M5941vIsdj7xrOVmPvGs5VnsfOJZy8184lnLs9j5bO1ZtSf11D+przrnre9F
43eJXKfs/Ve9q6pzXlSfRdWnyn+6D3OZy1zmMvfeXLd9Gv/vVO1DH/rQp6AP
HA6H/x0/APMaqsc=
     "]]},
  Axes->True,
  Boxed->False,
  ImageSize->{438.71222894706307`, 193.33333845491777`},
  PlotRange->All,
  PlotRangePadding->{
    Scaled[0.02], 
    Scaled[0.02], 
    Scaled[0.02]},
  ViewPoint->{-0.13180549570578584`, 2.4683142444828152`, 2.310855274954489},
  ViewVertical->{0.05097134140805026, 0.04714694109722245, 
   4.447624342078397}]], "Output",
 CellChangeTimes->{
  3.6087996136384964`*^9, 3.608799648458488*^9, 3.608799980542482*^9, 
   3.608800022941907*^9, {3.6124303544739027`*^9, 3.6124303611732855`*^9}, 
   3.6124304267810383`*^9, 3.612510122143033*^9, 3.6125171063723955`*^9, 
   3.61251714701972*^9, 3.612517201627844*^9, 3.612517751452292*^9, 
   3.6125177901735067`*^9, {3.6125214353810005`*^9, 3.6125214508038826`*^9}, 
   3.6125215081781645`*^9, 3.6125231654564576`*^9, 3.6125232706174726`*^9, 
   3.612523325896634*^9, 3.612523366803974*^9, 3.612523412551591*^9, 
   3.612523467178715*^9, 3.61252852752715*^9},
 ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"]],

Cell[BoxData[{
 RowBox[{
  RowBox[{"domain", "=", "\"\<Aero\>\""}], ";"}], "\n", 
 RowBox[{
  RowBox[{"displayName", "=", "\"\<Aircraft6DOFSS\>\""}], ";"}], "\n", 
 RowBox[{
  RowBox[{
  "brief", "=", "\"\<Flight dynamics model of super-sonic aircraft\>\""}], 
  ";"}], "\n", 
 RowBox[{
  RowBox[{"componentType", "=", "\"\<ComponentC\>\""}], ";"}], "\n", 
 RowBox[{
  RowBox[{"author", "=", "\"\<Petter Krus <petter.krus@liu.se>\>\""}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
  "affiliation", " ", "=", " ", 
   "\"\<Division of Fluid and Mechatronic Systems, Link\[ODoubleDot]ping \
University\>\""}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"SetFilenames", "[", 
   RowBox[{"path", ",", "domain", ",", "displayName"}], "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"ResetComponentVariables", "[", "]"}], ";"}]}], "Input",
 CellChangeTimes->{{3.503905077842648*^9, 3.503905079842648*^9}, {
   3.5039103076641607`*^9, 3.5039103985397425`*^9}, {3.5039164662834997`*^9, 
   3.5039164750178747`*^9}, {3.5039165143772497`*^9, 
   3.5039165189397497`*^9}, {3.5041051675625*^9, 3.504105226359375*^9}, {
   3.507570546125*^9, 3.507570569546875*^9}, {3.507570601046875*^9, 
   3.50757067696875*^9}, 3.507796401609375*^9, {3.507887022343302*^9, 
   3.507887057967846*^9}, {3.5078871702320337`*^9, 3.507887175028847*^9}, {
   3.5082235454098396`*^9, 3.5082235597692146`*^9}, 3.5082237305660896`*^9, {
   3.512921352515625*^9, 3.512921388578125*^9}, {3.513708407234375*^9, 
   3.51370842740625*^9}, 3.513708469203125*^9, {3.5137087323125*^9, 
   3.513708733765625*^9}, {3.513708789984375*^9, 3.5137088199375*^9}, {
   3.513709315953125*^9, 3.5137093166875*^9}, {3.51370934803125*^9, 
   3.513709386875*^9}, 3.515393228397682*^9, 3.515394538846967*^9, {
   3.518675928359375*^9, 3.518675955546875*^9}, {3.518676036671875*^9, 
   3.518676042765625*^9}, {3.5194107155625*^9, 3.519410716421875*^9}, {
   3.534158560625*^9, 3.53415858746875*^9}, 3.5556609494210105`*^9, {
   3.5562486448218775`*^9, 3.556248646009256*^9}, {3.567943950786781*^9, 
   3.5679439525278807`*^9}, 3.5763150828826694`*^9, 3.5763151129806786`*^9, {
   3.5785910949113007`*^9, 3.5785911284412184`*^9}, 3.581237056062117*^9, 
   3.5812371050611715`*^9, 3.6124577732491665`*^9}],

Cell[CellGroupData[{

Cell["Defining node variables", "Section"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"Tafin", "=", "torfin"}], ";", " ", 
  RowBox[{"defin", "=", "thetafin"}], ";", 
  RowBox[{"sdefin", "=", "wfin"}], ";", 
  RowBox[{"cxfin", "=", "cfin"}], ";", 
  RowBox[{"Zxfin", "=", "Zcfin"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Tal1", "=", "toral1"}], ";", " ", 
  RowBox[{"del1", "=", "thetaal1"}], ";", 
  RowBox[{"sdel1", "=", "wal1"}], ";", 
  RowBox[{"cxal1", "=", "cal1"}], ";", 
  RowBox[{"Zxal1", "=", "Zcal1"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Tar1", "=", "torar1"}], ";", " ", 
  RowBox[{"der1", "=", "thetaar1"}], ";", 
  RowBox[{"sder1", "=", "war1"}], ";", 
  RowBox[{"cxar1", "=", "car1"}], ";", 
  RowBox[{"Zxar1", "=", "Zcar1"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Tal12", "=", "toral12"}], ";", " ", 
  RowBox[{"del12", "=", "thetaal12"}], ";", 
  RowBox[{"sdel12", "=", "wal12"}], ";", 
  RowBox[{"cxal12", "=", "cal12"}], ";", 
  RowBox[{"Zxal12", "=", "Zcal12"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Tar12", "=", "torar12"}], ";", " ", 
  RowBox[{"der12", "=", "thetaar12"}], ";", 
  RowBox[{"sder12", "=", "war12"}], ";", 
  RowBox[{"cxar12", "=", "car12"}], ";", 
  RowBox[{"Zxar12", "=", "Zcar12"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Tal2", "=", "toral2"}], ";", " ", 
  RowBox[{"del2", "=", "thetaal2"}], ";", 
  RowBox[{"sdel2", "=", "wal2"}], ";", 
  RowBox[{"cxal2", "=", "cal2"}], ";", 
  RowBox[{"Zxal2", "=", "Zcal2"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Tar2", "=", "torar2"}], ";", " ", 
  RowBox[{"der2", "=", "thetaar2"}], ";", 
  RowBox[{"sder2", "=", "war2"}], ";", 
  RowBox[{"cxar2", "=", "car2"}], ";", 
  RowBox[{"Zxar2", "=", "Zcar2"}], ";"}]}], "Input",
 CellChangeTimes->{{3.51863778446875*^9, 3.518637882203125*^9}, {
  3.518675714921875*^9, 3.5186758090625*^9}, {3.519442425203125*^9, 
  3.519442437984375*^9}, {3.53416646871875*^9, 3.534166479875*^9}, {
  3.534166520890625*^9, 3.53416661540625*^9}, {3.53416664634375*^9, 
  3.5341667288125*^9}, {3.534167389859375*^9, 3.53416742271875*^9}, {
  3.534481455578125*^9, 3.534481467046875*^9}, {3.534481596953125*^9, 
  3.53448161109375*^9}, {3.534481748515625*^9, 3.534481764453125*^9}, {
  3.534481800265625*^9, 3.534481807890625*^9}, {3.534481851265625*^9, 
  3.534481854609375*^9}, {3.534481917046875*^9, 3.534481922234375*^9}, {
  3.534481964609375*^9, 3.534481981078125*^9}}]
}, Closed]],

Cell[BoxData[
 RowBox[{
  RowBox[{"MechanicCnode", "[", 
   RowBox[{"n_", ",", "position_", ",", "comment_"}], "]"}], ":=", 
  RowBox[{"{", 
   RowBox[{"n", ",", "\[IndentingNewLine]", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{
        RowBox[{"f", "[", "n", "]"}], ",", "0.", ",", "Real", ",", 
        "\"\<N\>\"", ",", "\"\<force\>\""}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"x", "[", "n", "]"}], ",", "position", ",", "Real", ",", 
        "\"\<m\>\"", ",", "\"\<position\>\""}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"sx", "[", "n", "]"}], ",", "0.", ",", "Real", ",", 
        "\"\<m/s\>\"", ",", "\"\<speed\>\""}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"cx", "[", "n", "]"}], ",", "0.", ",", "Real", ",", 
        "\"\<N\>\"", ",", "\"\<wave variable\>\""}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"Zx", "[", "n", "]"}], ",", "0.", ",", "Real", ",", 
        "\"\<N/m/s\>\"", ",", "\"\<Char. impedancs\>\""}], "}"}]}], "}"}], 
    ",", "\"\<mechanicCnodes\>\"", ",", "comment"}], "}"}]}]], "Input"],

Cell[CellGroupData[{

Cell["Loading Library", "Section"],

Cell[TextData[StyleBox["Derivative[1,0][Atan2][y_,x_]:=\nD1Atan2[y,x];",
 FontFamily->"Times"]], "Input",
 PageWidth->WindowWidth],

Cell[TextData[StyleBox["Derivative[1,0][Atan2L][y_,x_]:=\nD1Atan2L[y,x];",
 FontFamily->"Times"]], "Input",
 PageWidth->WindowWidth],

Cell[TextData[StyleBox["Derivative[0,1][Atan2][y_,x_]:=\nD2Atan2[y,x];",
 FontFamily->"Times"]], "Input",
 PageWidth->WindowWidth],

Cell[TextData[StyleBox["Derivative[0,1][Atan2L][y_,x_]:=\nD2Atan2L[y,x];",
 FontFamily->"Times"]], "Input",
 PageWidth->WindowWidth],

Cell[TextData[StyleBox["Derivative[1][ArcSinL][x_]:=\nDArcSinL[x];",
 FontFamily->"Times"]], "Input",
 PageWidth->WindowWidth],

Cell[TextData[StyleBox["Derivative[1][SecL][x_]:=\nDxSecL[x];\n\n\
Unprotect[Sec];\nSec[x_]:=SecL[x]",
 FontFamily->"Times"]], "Input",
 PageWidth->WindowWidth]
}, Open  ]],

Cell[CellGroupData[{

Cell["Component description", "Section",
 Evaluatable->False],

Cell["\<\
This model simulates a 3D model of an airplane.



\
\>", "Text",
 Evaluatable->False],

Cell[CellGroupData[{

Cell["Parameters and variables used in this component.", "Subsection",
 Evaluatable->False,
 PageBreakAbove->True],

Cell[CellGroupData[{

Cell["Auxilary functions", "Subsubsection"],

Cell[BoxData[
 RowBox[{
  RowBox[{"RestoreComments", "[", 
   RowBox[{"localParameters_", ",", "localParametersb_"}], "]"}], ":=", 
  "\[IndentingNewLine]", 
  RowBox[{"Block", "[", 
   RowBox[{
    RowBox[{"{", "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"n", "=", 
      RowBox[{"Length", "[", "localParameters", "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Print", "[", "n", "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"newPar", "=", 
      RowBox[{"Array", "[", 
       RowBox[{"a", ",", 
        RowBox[{"{", 
         RowBox[{"n", ",", "5"}], "}"}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Do", "[", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"newPar", "[", 
          RowBox[{"[", 
           RowBox[{"i", ",", "1"}], "]"}], "]"}], "=", 
         RowBox[{"localParameters", "[", 
          RowBox[{"[", 
           RowBox[{"i", ",", "1"}], " ", "]"}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"newPar", "[", 
          RowBox[{"[", 
           RowBox[{"i", ",", "2"}], "]"}], "]"}], "=", 
         RowBox[{"localParameters", "[", 
          RowBox[{"[", 
           RowBox[{"i", ",", "2"}], " ", "]"}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"newPar", "[", 
          RowBox[{"[", 
           RowBox[{"i", ",", "3"}], "]"}], "]"}], "=", "\"\<real\>\""}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"newPar", "[", 
          RowBox[{"[", 
           RowBox[{"i", ",", "4"}], "]"}], "]"}], "=", 
         RowBox[{"localParametersb", "[", 
          RowBox[{"[", 
           RowBox[{"i", ",", "4"}], " ", "]"}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"newPar", "[", 
          RowBox[{"[", 
           RowBox[{"i", ",", "5"}], "]"}], "]"}], "=", 
         RowBox[{"localParametersb", "[", 
          RowBox[{"[", 
           RowBox[{"i", ",", "5"}], " ", "]"}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"Print", "[", 
         RowBox[{"newPar", "[", 
          RowBox[{"[", "i", "]"}], "]"}], "]"}]}], "\[IndentingNewLine]", ",", 
       RowBox[{"{", 
        RowBox[{"i", ",", "1", ",", "n"}], "}"}]}], "]"}], ";", 
     RowBox[{"Return", "[", "newPar", "]"}]}]}], "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"AddComments1", "[", 
   RowBox[{"localParameters_", ",", "localParametersb_"}], "]"}], ":=", 
  "\[IndentingNewLine]", 
  RowBox[{"Block", "[", 
   RowBox[{
    RowBox[{"{", "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"n", "=", 
      RowBox[{"Length", "[", "localParameters", "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Print", "[", "n", "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"newPar", "=", 
      RowBox[{"Array", "[", 
       RowBox[{"a", ",", 
        RowBox[{"{", 
         RowBox[{"n", ",", "5"}], "}"}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Do", "[", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"newPar", "[", 
          RowBox[{"[", 
           RowBox[{"i", ",", "1"}], "]"}], "]"}], "=", 
         RowBox[{"localParameters", "[", 
          RowBox[{"[", 
           RowBox[{"i", ",", "1"}], " ", "]"}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"newPar", "[", 
          RowBox[{"[", 
           RowBox[{"i", ",", "2"}], "]"}], "]"}], "=", 
         RowBox[{"localParameters", "[", 
          RowBox[{"[", 
           RowBox[{"i", ",", "2"}], " ", "]"}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"newPar", "[", 
          RowBox[{"[", 
           RowBox[{"i", ",", "3"}], "]"}], "]"}], "=", "\"\<real\>\""}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"newPar", "[", 
          RowBox[{"[", 
           RowBox[{"i", ",", "4"}], "]"}], "]"}], "=", "\"\<\>\""}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"newPar", "[", 
          RowBox[{"[", 
           RowBox[{"i", ",", "5"}], "]"}], "]"}], "=", 
         RowBox[{"localParametersb", "[", 
          RowBox[{"[", 
           RowBox[{"i", ",", "2"}], " ", "]"}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"Print", "[", 
         RowBox[{"newPar", "[", 
          RowBox[{"[", "i", "]"}], "]"}], "]"}]}], "\[IndentingNewLine]", ",", 
       RowBox[{"{", 
        RowBox[{"i", ",", "1", ",", "n"}], "}"}]}], "]"}], ";", 
     RowBox[{"Return", "[", "newPar", "]"}]}]}], "]"}]}]], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["Declarations", "Subsubsection"],

Cell["Alias for some variables", "Text"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"\[Theta]", "=", "Thetao"}], ";"}], "\n", 
 RowBox[{
  RowBox[{"\[Psi]", "=", "Psi"}], ";"}], "\n", 
 RowBox[{
  RowBox[{"\[Phi]", "=", "Phi"}], ";"}], "\n", 
 RowBox[{
  RowBox[{"\[Alpha]", "=", "Alpha"}], ";"}], "\n", 
 RowBox[{
  RowBox[{"\[Beta]", "=", "Beta"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"\[CapitalLambda]", "=", "Lambda1"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"\[CapitalGamma]", "=", "Gamma1"}], ";"}]}], "Input"],

Cell[CellGroupData[{

Cell[BoxData["\[Theta]"], "Input",
 FontWeight->"Plain"],

Cell[BoxData["Thetao"], "Output",
 CellChangeTimes->{
  3.51867792809375*^9, {3.51867800696875*^9, 3.518678031921875*^9}, 
   3.518678539328125*^9, 3.518680167046875*^9, 3.518680425984375*^9, 
   3.518680612703125*^9, 3.518680706265625*^9, 3.518862356390625*^9, 
   3.518862399546875*^9, 3.51886280528125*^9, 3.518863329625*^9, 
   3.51940482484375*^9, 3.519404898609375*^9, 3.519410721859375*^9, 
   3.519442228203125*^9, 3.519442354125*^9, 3.519442494328125*^9, 
   3.519442621578125*^9, 3.51944280178125*^9, 3.519443090390625*^9, 
   3.51944315284375*^9, 3.519443634875*^9, 3.519443693875*^9, 
   3.519443760296875*^9, {3.519444184234375*^9, 3.51944420440625*^9}, 
   3.5194447166875*^9, 3.519543648453125*^9, 3.519805575765625*^9, 
   3.51980594821875*^9, 3.51981395771875*^9, 3.519847787875*^9, 
   3.522042509182646*^9, 3.522042662603814*^9, 3.5220426943998756`*^9, 
   3.522042863801789*^9, 3.5220786068125*^9, 3.531823402651251*^9, 
   3.531851270359375*^9, 3.531851339125*^9, 3.53185142375*^9, 
   3.531851789484375*^9, 3.531852603953125*^9, 3.531852667484375*^9, 
   3.531852740203125*^9, 3.531852829453125*^9, 3.531852887203125*^9, 
   3.53185294984375*^9, 3.53185308728125*^9, 3.5318531418125*^9, 
   3.531853279171875*^9, 3.531853354578125*^9, 3.5318534209375*^9, 
   3.53185347196875*^9, 3.53185353065625*^9, 3.531853674796875*^9, 
   3.53185378165625*^9, 3.531853993703125*^9, {3.531854130578125*^9, 
   3.531854158265625*^9}, 3.53185424525*^9, 3.53185428428125*^9, 
   3.5318543705625*^9, 3.53185447746875*^9, 3.531854540765625*^9, 
   3.531855031125*^9, 3.53185704271875*^9, 3.532106950375*^9, 
   3.534158612703125*^9, {3.5341674966875*^9, 3.534167518859375*^9}, 
   3.53416762903125*^9, 3.5341678143125*^9, 3.53416789228125*^9, 
   3.53417433315625*^9, 3.5342185536875*^9, 3.53421859665625*^9, 
   3.534218780796875*^9, 3.534229533078125*^9, 3.53423035390625*^9, 
   3.534230699640625*^9, 3.534415266875*^9, 3.534419005140625*^9, 
   3.534419510921875*^9, 3.53443739446875*^9, 3.5344376803125*^9, 
   3.534438097203125*^9, 3.534485685359375*^9, 3.534486079484375*^9, 
   3.534486147046875*^9, 3.538214450454418*^9, 3.5382200243968697`*^9, 
   3.5382215888845243`*^9, 3.538221882662014*^9, {3.538222189361213*^9, 
   3.53822219468927*^9}, 3.538222505369074*^9, 3.53822287399687*^9, 
   3.538223233384595*^9, 3.538223624067094*^9, 3.538236865453125*^9, 
   3.538237042140625*^9, 3.5382817596865*^9, 3.538283344194621*^9, 
   3.5382841943789606`*^9, 3.5382843075188613`*^9, 3.5382850862482524`*^9, 
   3.5382882703334093`*^9, 3.5382890614122314`*^9, 3.538289948572404*^9, 
   3.5383951371227465`*^9, 3.538395395589842*^9, 3.5383955958229356`*^9, 
   3.538480325473518*^9, 3.538503420390625*^9, 3.5385036878125*^9, 
   3.538503779890625*^9, 3.5387212281415777`*^9, 3.538722694978875*^9, 
   3.5388035642302165`*^9, 3.538805919417714*^9, 3.5388061685098696`*^9, 
   3.5388296370981054`*^9, 3.5388318569673147`*^9, 3.538930126984375*^9, 
   3.5389775449739065`*^9, 3.5389930335904565`*^9, 3.539011390983625*^9, 
   3.539012263343*^9, 3.539016048483625*^9, 3.5411477836482797`*^9, 
   3.541147896131215*^9, 3.541154665061887*^9, 3.541157080537939*^9, 
   3.5483863429482765`*^9, 3.548386383265421*^9, {3.55479790315625*^9, 
   3.55479791846875*^9}, 3.5556619274152503`*^9, 3.5562489699292088`*^9, 
   3.5562493066216793`*^9, 3.5562495510124593`*^9, 3.5562764695899897`*^9, 
   3.5562765890088077`*^9, 3.556296176859375*^9, 3.556296285640625*^9, 
   3.567943965007594*^9, 3.568001770127531*^9, 3.5680020166231785`*^9, 
   3.568002882587572*^9, 3.5680031024865723`*^9, 3.5680284831133375`*^9, 
   3.5680300168236933`*^9, 3.5763155150718837`*^9, 3.576411383832909*^9, 
   3.576425136426154*^9, 3.576425347965254*^9, 3.5764262082124567`*^9, 
   3.576993496537633*^9, 3.576993669537633*^9, 3.576993734795633*^9, 
   3.578504372821372*^9, 3.578591504292716*^9, 3.5786616376259623`*^9, 
   3.5812368929265766`*^9, 3.581237061417653*^9, 3.581237108774209*^9, 
   3.5812371476743574`*^9, 3.5812372809674115`*^9, 3.5812373481078854`*^9, 
   3.5813519968632708`*^9, 3.581746214176471*^9, 3.581746398609913*^9, 
   3.5817465988109307`*^9, 3.581755007600159*^9, 3.5817808892265034`*^9, 
   3.581826859219659*^9, 3.582351329282127*^9, 3.584025089368529*^9, 
   3.5840252604753156`*^9, 3.584025379080099*^9, 3.584025421178507*^9, 
   3.584025456945553*^9, 3.585372035742384*^9, 3.5853721192907376`*^9, 
   3.58537224322713*^9, 3.612457827225254*^9, 3.61245809675867*^9, 
   3.6124581784173408`*^9, {3.612458308937806*^9, 3.6124583146031303`*^9}, 
   3.613193605376475*^9, 3.6131936493348703`*^9, 3.613193687813718*^9, 
   3.613193844668189*^9, 3.620712355001048*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["M", 
    OverscriptBox[
     RowBox[{"\[EmptySet]", " "}], "."]], "=", "Mdvphi"}], ";"}]], "Text"],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   SubscriptBox["M", 
    OverscriptBox["\[Theta]", "."]], " ", "=", "Mdvtheta"}], ";"}], "\n", 
 RowBox[{
  RowBox[{
   SubscriptBox["M", 
    OverscriptBox["\[Psi]", "."]], "=", "Mdvpsi"}], ";"}], "\n", 
 RowBox[{
  RowBox[{
   SubscriptBox["M", 
    OverscriptBox[
     RowBox[{"\[EmptySet]", " "}], "."]], "=."}], ";"}]}], "Input"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"q", " ", "=", " ", "qpress"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"q0", "=."}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"q1", "=."}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"q2", "=."}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"q3", "=."}], ";"}]}], "Input"],

Cell[BoxData[""], "Input"],

Cell[CellGroupData[{

Cell["Declarations of variables", "Subsubsection"],

Cell["\<\
The global output variables are those variables that are of interst outside \
the component.\
\>", "Text",
 Evaluatable->False],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"outputVariables", "=", 
    RowBox[{"{", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{
       "xcg", ",", "0", ",", "double", ",", "\"\<m\>\"", ",", 
        "\"\<Horizontal position 1\>\""}], "}"}], ",", "\[IndentingNewLine]", 
      
      RowBox[{"{", 
       RowBox[{
       "ycg", ",", "0", ",", "double", ",", "\"\<m\>\"", ",", 
        "\"\<Horizontal position 2\>\""}], "}"}], ",", "\[IndentingNewLine]", 
      
      RowBox[{"{", 
       RowBox[{
       "zcg", ",", "0", ",", "double", ",", "\"\<m\>\"", ",", 
        "\"\<Vertical position\>\""}], "}"}], ",", "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{
       "vx", ",", "0", ",", "double", ",", "\"\<m\>\"", ",", 
        "\"\<Horizontal speed 1\>\""}], "}"}], ",", "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{
       "vy", ",", "0", ",", "double", ",", "\"\<m\>\"", ",", 
        "\"\<Horizontal speed 2\>\""}], "}"}], ",", "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{
       "vz", ",", "0", ",", "double", ",", "\"\<m\>\"", ",", 
        "\"\<Vertical speed\>\""}], "}"}], ",", "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{
       "\[Psi]", ",", "0", ",", "double", ",", "\"\<rad\>\"", ",", 
        "\"\<Azimuth angle\>\""}], "}"}], ",", "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{
       "\[Theta]", ",", "0", ",", "double", ",", "\"\<rad\>\"", ",", 
        "\"\<Elevation angle\>\""}], "}"}], ",", "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{
       "\[Phi]", ",", "0", ",", "double", ",", "\"\<rad\>\"", ",", 
        "\"\<Bank angle\>\""}], "}"}], ",", "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{
       "Ub", ",", "100", ",", "double", ",", "\"\<m/s\>\"", ",", 
        "\"\<Speed xb-axis\>\""}], "}"}], ",", "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{
       "Vb", ",", "0", ",", "double", ",", "\"\<m/s\>\"", ",", 
        "\"\<Speed yb-axis\>\""}], "}"}], ",", "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{
       "Wb", ",", "0", ",", "double", ",", "\"\<m/s\>\"", ",", 
        "\"\<Speed zb-axis\>\""}], "}"}], ",", "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{
       "Pb", ",", "0", ",", "double", ",", "\"\<rad/s\>\"", ",", 
        "\"\<Angular velocity\>\""}], "}"}], ",", "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{
       "Qb", ",", "0", ",", "double", ",", "\"\<rad/s\>\"", ",", 
        "\"\<Angular velocity\>\""}], "}"}], ",", "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{
       "Rb", ",", "0", ",", "double", ",", "\"\<rad/s\>\"", ",", 
        "\"\<Angular velocity\>\""}], "}"}], ",", "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{
       "q0", ",", "0", ",", "double", ",", "\"\<\>\"", ",", 
        "\"\<quartenion 0\>\""}], "}"}], ",", "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{
       "q1", ",", "0", ",", "double", ",", "\"\<\>\"", ",", 
        "\"\<quartenion 1\>\""}], "}"}], ",", "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{
       "q2", ",", "0", ",", "double", ",", "\"\<\>\"", ",", 
        "\"\<quartenion 2\>\""}], "}"}], ",", "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{
       "q3", ",", "0", ",", "double", ",", "\"\<\>\"", ",", 
        "\"\<quartenion 3\>\""}], "}"}], ",", "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{
       "AlphaAttack", ",", "0", ",", "double", ",", "\"\<rad\>\"", ",", 
        "\"\<Angle of atack\>\""}], "}"}], ",", "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{
       "BetaSlip", ",", "0", ",", "double", ",", "\"\<rad/s\>\"", ",", 
        "\"\<Sideslip angle\>\""}], "}"}], ",", "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{
       "altitude", ",", "0", ",", "double", ",", "\"\<m\>\"", ",", 
        "\"\<altitude\>\""}], "}"}], ",", "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{
       "gfx", ",", "0", ",", "double", ",", "\"\<m/s^2\>\"", ",", 
        "\"\<g-force in x\>\""}], "}"}], ",", "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{
       "gfy", ",", "0", ",", "double", ",", "\"\<m/s^2\>\"", ",", 
        "\"\<g-force in y\>\""}], "}"}], ",", "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{
       "gfz", ",", "0", ",", "double", ",", "\"\<m/s^2\>\"", ",", 
        "\"\<g-force in z\>\""}], "}"}], ",", "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{
       "CL1", ",", "0", ",", "double", ",", "\"\<\>\"", ",", 
        "\"\<Lift coeff. wing 1\>\""}], "}"}], ",", "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{
       "Cd1", ",", "0", ",", "double", ",", "\"\<\>\"", ",", 
        "\"\<Drag coeff. wing 1\>\""}], "}"}], ",", "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{
       "Fax", ",", "0", ",", "double", ",", "\"\<N\>\"", ",", 
        "\"\<Aero force in z\>\""}], "}"}], ",", "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{
       "Faz", ",", "0", ",", "double", ",", "\"\<N\>\"", ",", 
        "\"\<Aero force in x\>\""}], "}"}]}], "\[IndentingNewLine]", "}"}]}], 
   ";"}], "\n"}]], "Input",
 CellChangeTimes->{{3.518677965015625*^9, 3.518677965015625*^9}, {
  3.518859664453125*^9, 3.518859695546875*^9}, {3.51885990190625*^9, 
  3.518859910359375*^9}, {3.534218733859375*^9, 3.5342187765*^9}, {
  3.538221842021909*^9, 3.538221871552781*^9}, {3.5388296092073016`*^9, 
  3.5388296270199156`*^9}, {3.5764191068929653`*^9, 3.576419120121722*^9}}],

Cell["\<\
The input variables are those variables that are inputs to the component but  \
are not passed through the nodes..\
\>", "Text",
 Evaluatable->False],

Cell[BoxData[
 RowBox[{
  RowBox[{"inputVariables", "=", 
   RowBox[{"{", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
      "thrustl", ",", "0.", ",", "double", ",", "\"\<N\>\"", ",", 
       "\"\<Engine thrust\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "thrustr", ",", "0.", ",", "double", ",", "\"\<N\>\"", ",", 
       "\"\<Engine thrust\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "dezthrustl", ",", "0.", ",", "double", ",", "\"\<rad\>\"", ",", 
       "\"\<Thrust angle\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "dezthrustr", ",", "0.", ",", "double", ",", "\"\<rad\>\"", ",", 
       "\"\<Thrust angle\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "deythrustl", ",", "0.", ",", "double", ",", "\"\<rad\>\"", ",", 
       "\"\<Thrust angle\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "deythrustr", ",", "0.", ",", "double", ",", "\"\<rad\>\"", ",", 
       "\"\<Thrust angle\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "Mfuel", ",", "0.", ",", "double", ",", "\"\<kg\>\"", ",", 
       "\"\<Fuel weight\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "Mcargo", ",", "0.", ",", "double", ",", "\"\<kg\>\"", ",", 
       "\"\<Cargo weight\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "rho", ",", "1.25", ",", "double", ",", "\"\<kg/m3\>\"", ",", 
       "\"\<Air density\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "vM", ",", "340.", ",", "double", ",", "\"\<m/s\>\"", ",", 
       "\"\<Speed of sound\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "vturbx", ",", "0.", ",", "double", ",", "\"\<m/s\>\"", ",", 
       "\"\<air turbulence x\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "vturby", ",", "0.", ",", "double", ",", "\"\<m/s\>\"", ",", 
       "\"\<air turbulence y\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "vturbz", ",", "0.", ",", "double", ",", "\"\<m/s\>\"", ",", 
       "\"\<air turbulence z\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "wturbx", ",", "0.", ",", "double", ",", "\"\<rad/s\>\"", ",", 
       "\"\<air turbulence x\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "wturby", ",", "0.", ",", "double", ",", "\"\<rad/s\>\"", ",", 
       "\"\<air turbulence y\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "wturbz", ",", "0.", ",", "double", ",", "\"\<rad/s\>\"", ",", 
       "\"\<air turbulence z\>\""}], "}"}]}], "}"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.518678097375*^9, 3.518678097375*^9}, {
  3.518859701390625*^9, 3.51885971253125*^9}, {3.51885991175*^9, 
  3.51885991428125*^9}, {3.5220457101110153`*^9, 3.5220457138604493`*^9}, {
  3.522045777956208*^9, 3.522045787615116*^9}, {3.5678802618306117`*^9, 
  3.5678802891861763`*^9}, {3.58174440079202*^9, 3.58174442225602*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"nodeConnections", "=", 
   RowBox[{"{", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"MechanicRotCnode", "[", 
      RowBox[{
      "al1", ",", "0.", ",", "0.", ",", 
       "\"\<mechanical node left airleron 1\>\""}], "]"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"MechanicRotCnode", "[", 
      RowBox[{
      "ar1", ",", "0.", ",", "0.", ",", 
       "\"\<mechanical node right airleron 1\>\""}], "]"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"MechanicRotCnode", "[", 
      RowBox[{
      "al12", ",", "0.", ",", "0.", ",", 
       "\"\<mechanical node left flaperon 1\>\""}], "]"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"MechanicRotCnode", "[", 
      RowBox[{
      "ar12", ",", "0.", ",", "0.", ",", 
       "\"\<mechanical node right flaperon 1\>\""}], "]"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"MechanicRotCnode", "[", 
      RowBox[{
      "al2", ",", "0.", ",", "0.", ",", 
       "\"\<mechanical node left airleraon 2\>\""}], "]"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"MechanicRotCnode", "[", 
      RowBox[{
      "ar2", ",", "0.", ",", "0.", ",", 
       "\"\<mechanical node right airleraon 2\>\""}], "]"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"MechanicRotCnode", "[", 
      RowBox[{
      "fin", ",", "0.", ",", "0.", ",", "\"\<mechanical node fin\>\""}], 
      "]"}]}], "}"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.518678117875*^9, 3.518678117875*^9}, {
   3.51885998453125*^9, 3.518859990515625*^9}, {3.519319018984375*^9, 
   3.519319030734375*^9}, {3.534157702359375*^9, 3.53415777759375*^9}, 
   3.534166849125*^9, {3.5341673530625*^9, 3.5341673644375*^9}, 
   3.53448127625*^9, 3.534481376453125*^9, {3.53448148078125*^9, 
   3.534481481796875*^9}, {3.5344816129375*^9, 3.534481614015625*^9}, 
   3.5344817658125*^9, 3.534481809015625*^9}],

Cell["\<\
Global parameters are parameters that are normally used throughout the whole \
system.\
\>", "Text",
 Evaluatable->False],

Cell["\<\
The local parameters are the component specific parameters. Default \
parameters are based on the F-16.\
\>", "Text",
 Evaluatable->False,
 CellChangeTimes->{{3.578591480572359*^9, 3.5785914918810062`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"inputParameters", "=", 
   RowBox[{"{", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
      "afin", ",", "0.3", ",", "double", ",", "\"\<rad\>\"", ",", 
       "\"\<break angle 1\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "an1", ",", "0.6", ",", "double", ",", "\"\<rad\>\"", ",", 
       "\"\<Neg. break angle 1\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "an2", ",", "0.6", ",", "double", ",", "\"\<rad\>\"", ",", 
       "\"\<Neg. break angle 2\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "ap1", ",", "0.9", ",", "double", ",", "\"\<rad\>\"", ",", 
       "\"\<Pos. break angle 1\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "ap2", ",", "0.7", ",", "double", ",", "\"\<rad\>\"", ",", 
       "\"\<Pos. break angle 2\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "AR1", ",", "3.62", ",", "double", ",", "\"\<\>\"", ",", 
       "\"\<Aspect ratio 1\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "AR2", ",", "3.62", ",", "double", ",", "\"\<\>\"", ",", 
       "\"\<Aspect ratio 2\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "ARfin", ",", "1.5", ",", "double", ",", "\"\<\>\"", ",", 
       "\"\<Aspect ratio fin+body\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "Cd0", ",", "0.021", ",", "double", ",", "\"\<\>\"", ",", 
       "\"\<Parasitic Drag coef\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "CdW0", ",", "0.023", ",", "double", ",", "\"\<\>\"", ",", 
       "\"\<Wave drag coef\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "CLalpha1", ",", "2.1", ",", "double", ",", "\"\<\>\"", ",", 
       "\"\<L. slope coef. wing 1+body\>\""}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "CLalpha2", ",", "2.2", ",", "double", ",", "\"\<\>\"", ",", 
       "\"\<L. slope coef. wing 2\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "CLalphafin", ",", "0.80", ",", "double", ",", "\"\<\>\"", ",", 
       "\"\<L. sl. c. fin and body\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     
     RowBox[{"{", 
      RowBox[{
      "CLde1", ",", "0.1", ",", "double", ",", "\"\<\>\"", ",", 
       "\"\<Ctrl surface coef 1\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "CLde12", ",", "0.2", ",", "double", ",", "\"\<\>\"", ",", 
       "\"\<Flap rudder coef 1\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "Cdide1", ",", "0.", ",", "double", ",", "\"\<\>\"", ",", 
       "\"\<Flap rudder drag coef 1\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "Cdide12", ",", "0.", ",", "double", ",", "\"\<\>\"", ",", 
       "\"\<Flap rudder drag coef 1\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "Cdide112", ",", "0.", ",", "double", ",", "\"\<\>\"", ",", 
       "\"\<Flap rudder cross drag coef 1\>\""}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "de10", ",", "0.01", ",", "double", ",", "\"\<\>\"", ",", 
       "\"\<rudder min drag angle 1\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "de120", ",", "0.01", ",", "double", ",", "\"\<\>\"", ",", 
       "\"\<Flap min drag angle 1\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "dM", ",", "0.1", ",", "double", ",", "\"\<\>\"", ",", 
       "\"\<Width of transonic region (rel. Mach)\>\""}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"Cm01", ",", 
       RowBox[{"-", "0.1"}], ",", "double", ",", "\"\<\>\"", ",", 
       "\"\<Mom coeff. wing 1\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"Cmfs1", ",", 
       RowBox[{"-", "0.5"}], ",", "double", ",", "\"\<\>\"", ",", 
       "\"\<Mom coeff.1, fully separated\>\""}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "Cmde1", ",", "0.02", ",", "double", ",", "\"\<\>\"", ",", 
       "\"\<Mom slop coeff 1\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "Cmde12", ",", "0.1", ",", "double", ",", "\"\<\>\"", ",", 
       "\"\<Flap Mom slop coeff 1\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "CLdefin", ",", "0.0827084`", ",", "double", ",", "\"\<\>\"", ",", 
       "\"\<Rudder coef 1\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "Cydeelev", ",", "0.1", ",", "double", ",", "\"\<\>\"", ",", 
       "\"\<elevator side force coef\>\""}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "dah1", ",", "1.0", ",", "double", ",", "\"\<\>\"", ",", 
       "\"\<down wash effect on 1\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "dah2", ",", "0.6", ",", "double", ",", "\"\<\>\"", ",", 
       "\"\<down wash effect on 2\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "en", ",", "2.71828", ",", "double", ",", "\"\<\>\"", ",", 
       "\"\<e\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "e1", ",", "0.95", ",", "double", ",", "\"\<\>\"", ",", 
       "\"\<Osw. effic. factor 1\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "e2", ",", "0.95", ",", "double", ",", "\"\<\>\"", ",", 
       "\"\<Osw. effic. factor 21\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "efin", ",", "0.95", ",", "double", ",", "\"\<\>\"", ",", 
       "\"\<Osw. eff. f. fin\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "epsM", ",", "0.3", ",", "double", ",", "\"\<\>\"", ",", 
       "\"\<Mach smoothening factor\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "awfin", ",", ".2", ",", "double", ",", "\"\<\>\"", ",", 
       "\"\<CL exponent fin\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "awn1", ",", ".2", ",", "double", ",", "\"\<\>\"", ",", 
       "\"\<CL exponent neg. 1\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "awn2", ",", ".2", ",", "double", ",", "\"\<\>\"", ",", 
       "\"\<CL exponent neg. 2\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "awp1", ",", ".2", ",", "double", ",", "\"\<\>\"", ",", 
       "\"\<CL exponent pos 1\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "awp2", ",", ".2", ",", "double", ",", "\"\<\>\"", ",", 
       "\"\<CL exponent pos 2\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"gamma1", ",", 
       RowBox[{"-", "0.0872665`"}], ",", "double", ",", "\"\<rad\>\"", ",", 
       "\"\<dehidral 1\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"gamma2", ",", 
       RowBox[{"-", "0.0872665`"}], ",", "double", ",", "\"\<rad\>\"", ",", 
       "\"\<dehidral 2\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "hthrust0", ",", "0.", ",", "double", ",", "\"\<\>\"", ",", 
       "\"\<engine vert. pos\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "ia1", ",", "0.", ",", "double", ",", "\"\<rad\>\"", ",", 
       "\"\<incidence angle 1\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"ia2", ",", "0.02", ",", "double", ",", 
       RowBox[{"rad", "\"\<\>\""}], ",", "\"\<incidence angle 1\>\""}], "}"}],
      ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "Ix0", ",", "0.0147", ",", "double", ",", "\"\< \>\"", ",", 
       "\"\<Norm. Inertia moment Ix/(Me S1)\>\""}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "Ixz0", ",", "0.0055", ",", "double", ",", "\"\< \>\"", ",", 
       "\"\<Norm. Inertia moment\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "Iy0", ",", "1.131", ",", "double", ",", "\"\< \>\"", ",", 
       "\"\<Norm. Inertia moment\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "Iz0", ",", "1.279", ",", "double", ",", "\"\< \>\"", ",", 
       "\"\<Inertia moment\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "lambda1", ",", "0.436332", ",", "double", ",", "\"\<rad\>\"", ",", 
       "\"\<sweep 1\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "lambda2", ",", "0.436332", ",", "double", ",", "\"\<rad\>\"", ",", 
       "\"\<sweep 2\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "lambdafin", ",", "0.785398", ",", "double", ",", "\"\<rad\>\"", ",", 
       "\"\<sweep fin\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "lc10", ",", "0.01", ",", "double", ",", "\"\<\>\"", ",", 
       "\"\<norm. ctrl surf. 1 ac fr hinge lc1/sqrt(AR1 S1)\>\""}], "}"}], 
     ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "lc20", ",", "0.05", ",", "double", ",", "\"\<\>\"", ",", 
       "\"\<norm. ctrl surf. 2 ac fr hinge lc1/sqrt(AR1 S1)\>\""}], "}"}], 
     ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "lc120", ",", "0.01", ",", "double", ",", "\"\<\>\"", ",", 
       "\"\<norm. flap 1 ac fr hinge\>\""}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "lcfin0", ",", "0.01", ",", "double", ",", "\"\<\>\"", ",", 
       "\"\<ctrl s. fin ac fr hinge\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "Me", ",", "8700.", ",", "double", ",", "\"\<kg\>\"", ",", 
       "\"\<Empty weight\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "mac0", ",", "1.", ",", "double", ",", "\"\<\>\"", ",", 
       "\"\<mean aerodynamic cord/Sqrt(S1/b1)\>\""}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "rc10", ",", "0.25", ",", "double", ",", "\"\<\>\"", ",", 
       "\"\<norm. ctrl surface 1 mom. arm\>\""}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "rc120", ",", "0.25", ",", "double", ",", "\"\<\>\"", ",", 
       "\"\<norm. ctrl surface 12 mom. arm\>\""}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "rc20", ",", "0.15", ",", "double", ",", "\"\<\>\"", ",", 
       "\"\<norm. ctrl surface 1 mom. arm\>\""}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "rcfin0", ",", ".1", ",", "double", ",", "\"\<\>\"", ",", 
       "\"\<norm. ctrl surf. fin mom. arm\>\""}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "S1", ",", "27.", ",", "double", ",", "\"\<m2\>\"", ",", 
       "\"\<reference wing area 1\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "S20", ",", "0.36", ",", "double", ",", "\"\<\>\"", ",", 
       "\"\<norm. wing area 2\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "Sbh0", ",", "0.2", ",", "double", ",", "\"\<\>\"", ",", 
       "\"\<norm. hor. proj. area\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "Sbv0", ",", "0.1", ",", "double", ",", "\"\<\>\"", ",", 
       "\"\<norm.body vert. proj. area\>\""}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "Sfin0", ",", "0.17", ",", "double", ",", "\"\<\>\"", ",", 
       "\"\<norm. fin area\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "xbcge0", ",", "3", ",", "double", ",", "\"\< \>\"", ",", 
       "\"\<norm. body cg\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "xcargo0", ",", "3", ",", "double", ",", "\"\< \>\"", ",", 
       "\"\<norm. cargo pos.\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "xfuel0", ",", "3", ",", "double", ",", "\"\< \>\"", ",", "\"\<\>\""}], 
      "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "xw10", ",", "3", ",", "double", ",", "\"\< \>\"", ",", 
       "\"\<norm. wing1  position\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "xw20", ",", "4.8", ",", "double", ",", "\"\< \>\"", ",", 
       "\"\<norm. wing 2 position\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "xwfin0", ",", "4.8", ",", "double", ",", "\"\<\>\"", ",", 
       "\"\<norm. fin position\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "xeng0", ",", "4.8", ",", "double", ",", "\"\<\>\"", ",", 
       "\"\<norm. fin position\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "yeng0", ",", "0.", ",", "double", ",", "\"\<\>\"", ",", 
       "\"\<engines off. from center\>\""}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "g0", ",", "9.81", ",", "double", ",", "\"\<m/s^2\>\"", ",", 
       "\"\<Gravity acceleration\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "kground", ",", "10000.", ",", "double", ",", "\"\<N/m\>\"", ",", 
       "\"\<Ground stiffness (for limitiation)\>\""}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "cground", ",", "1000.", ",", "double", ",", "\"\<Ns/m\>\"", ",", 
       "\"\<Ground damping (for limitiation)\>\""}], "}"}]}], "}"}]}], 
  ";"}]], "Input",
 CellChangeTimes->{{3.468168905578125*^9, 3.4681689365*^9}, {
   3.5183287998405657`*^9, 3.5183288007467923`*^9}, {3.51885971821875*^9, 
   3.5188598484375*^9}, {3.518859934609375*^9, 3.51885996934375*^9}, {
   3.519319065046875*^9, 3.51931906684375*^9}, {3.519404881671875*^9, 
   3.51940489046875*^9}, {3.5194432470625*^9, 3.5194433024375*^9}, {
   3.53415801028125*^9, 3.534158041234375*^9}, {3.53416638509375*^9, 
   3.53416639525*^9}, {3.534167284*^9, 3.534167336078125*^9}, {
   3.534225103421875*^9, 3.53422516034375*^9}, {3.534225190671875*^9, 
   3.53422521971875*^9}, {3.53422904878125*^9, 3.534229069421875*^9}, {
   3.534229331609375*^9, 3.53422935996875*^9}, 3.5342294155*^9, {
   3.53422945175*^9, 3.53422945690625*^9}, {3.534229490578125*^9, 
   3.53422950471875*^9}, {3.534229906984375*^9, 3.534229968125*^9}, {
   3.5342305748125*^9, 3.53423060728125*^9}, {3.534485850390625*^9, 
   3.534485965140625*^9}, 3.53448614121875*^9, {3.5388143586297503`*^9, 
   3.5388143599110003`*^9}, {3.5388315138862495`*^9, 
   3.5388315611831245`*^9}, {3.53883166832436*^9, 3.538831750325325*^9}, {
   3.5389298875625*^9, 3.538929925953125*^9}, {3.53892997175*^9, 
   3.538929974984375*^9}, {3.538930008453125*^9, 3.5389300180625*^9}, {
   3.538930105828125*^9, 3.538930109953125*^9}, {3.5412367205852985`*^9, 
   3.5412367461798677`*^9}, {3.54123886388357*^9, 3.5412389185238457`*^9}, 
   3.5412389622579403`*^9, {3.5556604876710105`*^9, 3.5556606532647605`*^9}, 
   3.5556606864366355`*^9, {3.5556607194835105`*^9, 3.5556608648272605`*^9}, {
   3.5556643290013204`*^9, 3.555664364798196*^9}, {3.5560150728577294`*^9, 
   3.5560151053864813`*^9}, {3.556015152195386*^9, 3.5560152019571886`*^9}, {
   3.556015248375499*^9, 3.5560154101755714`*^9}, {3.556015488825781*^9, 
   3.556015642073084*^9}, {3.5560157609195766`*^9, 3.5560158196176057`*^9}, {
   3.5560162376077776`*^9, 3.55601624360701*^9}, {3.5560162803991747`*^9, 
   3.5560163011308956`*^9}, {3.5560170023080587`*^9, 3.556017016291109*^9}, {
   3.556017294563049*^9, 3.5560173265289345`*^9}, {3.556017356979331*^9, 
   3.556017412240004*^9}, {3.556017469500498*^9, 3.5560175234331656`*^9}, {
   3.556043814671875*^9, 3.556043858109375*^9}, {3.556043895984375*^9, 
   3.556043917234375*^9}, {3.556297419109375*^9, 3.556297515078125*^9}, {
   3.5563349457633634`*^9, 3.5563350425833035`*^9}, {3.5563351742779994`*^9, 
   3.5563352195251036`*^9}, {3.56802828821885*^9, 3.5680282982698545`*^9}, {
   3.578591247893051*^9, 3.5785912838201056`*^9}, {3.5785913426244693`*^9, 
   3.57859135837637*^9}, {3.5785913909212313`*^9, 3.5785914477004795`*^9}, {
   3.5786726451355553`*^9, 3.5786726626835594`*^9}, {3.578737963419788*^9, 
   3.578737966651111*^9}, {3.579582807504713*^9, 3.5795828536253242`*^9}, {
   3.5817434884538507`*^9, 3.5817435768855333`*^9}, {3.581745609986058*^9, 
   3.5817456513181915`*^9}, {3.5817461449055448`*^9, 3.5817461805181055`*^9}, 
   3.5823525182896132`*^9, {3.5840197180923095`*^9, 3.5840197438637834`*^9}, {
   3.584024837531124*^9, 3.584024882890719*^9}, 3.5853719655433645`*^9, 
   3.612461945033779*^9, {3.6124620818656054`*^9, 3.6124620823406324`*^9}, {
   3.612537050709649*^9, 3.612537058166075*^9}, {3.612704434653322*^9, 
   3.6127045011574163`*^9}, {3.613191294972512*^9, 3.613191358523512*^9}, {
   3.6150462419902234`*^9, 3.615046356714785*^9}, {3.615046420655442*^9, 
   3.615046432418115*^9}, {3.615046467696133*^9, 3.6150464865592117`*^9}}],

Cell["The Qnodes are the connections to other components", "Text",
 Evaluatable->False],

Cell["Qnodes = {};", "Input",
 PageWidth->WindowWidth],

Cell["\<\
There are also a constants. Noiter is the number of iterations performed in \
each time step\
\>", "Text",
 Evaluatable->False]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Nomenclature", "Subsection"],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   SubscriptBox["L", "BV"], TraditionalForm]]],
 "         :Transformation matrix, body coordinates to vehicle-carried \
vertical frame"
}], "Text"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["Component equations",
 FontFamily->"Times"]], "Section",
 Evaluatable->False],

Cell[CellGroupData[{

Cell["Definitions", "Subsection",
 Evaluatable->False,
 PageBreakAbove->True],

Cell[BoxData[
 RowBox[{"\[Zeta]", ":=", 
  TagBox[
   RowBox[{"(", GridBox[{
      {"1", 
       RowBox[{
        RowBox[{"Sin", "[", "\[Phi]", "]"}], 
        RowBox[{"Tan", "[", "\[Theta]", "]"}]}], 
       RowBox[{
        RowBox[{"Cos", "[", "\[Phi]", "]"}], 
        RowBox[{"Tan", "[", "\[Theta]", "]"}]}]},
      {"0", 
       RowBox[{"Cos", "[", "\[Phi]", "]"}], 
       RowBox[{"-", 
        RowBox[{"Sin", "[", "\[Phi]", "]"}]}]},
      {"0", 
       RowBox[{
        RowBox[{"Sin", "[", "\[Phi]", "]"}], 
        RowBox[{"SecL", "[", "\[Theta]", "]"}]}], 
       RowBox[{
        RowBox[{"Cos", "[", "\[Phi]", "]"}], 
        RowBox[{"SecL", "[", "\[Theta]", "]"}]}]}
     }], ")"}],
   MatrixForm[#]& ]}]], "Input"],

Cell["\<\
The transformation matrices for transformation from body to earth axis are \
(Etkin (4.5,2)\
\>", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["L", "x"], "[", "\[Phi]_", "]"}], ":=", 
  TagBox[
   RowBox[{"(", GridBox[{
      {"1", "0", "0"},
      {"0", 
       RowBox[{"Cos", "[", "\[Phi]", "]"}], 
       RowBox[{"Sin", "[", "\[Phi]", "]"}]},
      {"0", 
       RowBox[{"-", 
        RowBox[{"Sin", "[", "\[Phi]", "]"}]}], 
       RowBox[{"Cos", "[", "\[Phi]", "]"}]}
     }], ")"}],
   MatrixForm[#]& ]}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["L", "y"], "[", "\[Theta]_", "]"}], " ", ":=", 
  TagBox[
   RowBox[{"(", GridBox[{
      {
       RowBox[{"Cos", "[", "\[Theta]", "]"}], "0", 
       RowBox[{"-", 
        RowBox[{"Sin", "[", "\[Theta]", "]"}]}]},
      {"0", "1", "0"},
      {
       RowBox[{"Sin", "[", "\[Theta]", "]"}], "0", 
       RowBox[{"Cos", "[", "\[Theta]", "]"}]}
     }], ")"}],
   MatrixForm[#]& ]}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["L", "z"], "[", "\[Psi]_", "]"}], ":=", 
  TagBox[
   RowBox[{"(", GridBox[{
      {
       RowBox[{"Cos", "[", "\[Psi]", "]"}], 
       RowBox[{"Sin", "[", "\[Psi]", "]"}], "0"},
      {
       RowBox[{"-", 
        RowBox[{"Sin", "[", "\[Psi]", "]"}]}], 
       RowBox[{"Cos", "[", "\[Psi]", "]"}], "0"},
      {"0", "0", "1"}
     }], ")"}],
   MatrixForm[#]& ]}]], "Input"],

Cell["\<\
Transformation from vehicle-carried vertical frame to body coordinates can be \
expressed as (Etkin 4.5,3)\
\>", "Text"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["L", "BV"], ":=", 
  RowBox[{
   RowBox[{
    SubscriptBox["L", "x"], "[", "\[Phi]", "]"}], ".", 
   RowBox[{
    SubscriptBox["L", "y"], "[", "\[Theta]", "]"}], ".", 
   RowBox[{
    SubscriptBox["L", "z"], "[", "\[Psi]", "]"}]}]}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"MatrixForm", "[", 
  SubscriptBox["L", "BV"], " ", "]"}]], "Input"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      RowBox[{
       RowBox[{"Cos", "[", "Psi", "]"}], " ", 
       RowBox[{"Cos", "[", "Thetao", "]"}]}], 
      RowBox[{
       RowBox[{"Cos", "[", "Thetao", "]"}], " ", 
       RowBox[{"Sin", "[", "Psi", "]"}]}], 
      RowBox[{"-", 
       RowBox[{"Sin", "[", "Thetao", "]"}]}]},
     {
      RowBox[{
       RowBox[{
        RowBox[{"-", 
         RowBox[{"Cos", "[", "Phi", "]"}]}], " ", 
        RowBox[{"Sin", "[", "Psi", "]"}]}], "+", 
       RowBox[{
        RowBox[{"Cos", "[", "Psi", "]"}], " ", 
        RowBox[{"Sin", "[", "Phi", "]"}], " ", 
        RowBox[{"Sin", "[", "Thetao", "]"}]}]}], 
      RowBox[{
       RowBox[{
        RowBox[{"Cos", "[", "Phi", "]"}], " ", 
        RowBox[{"Cos", "[", "Psi", "]"}]}], "+", 
       RowBox[{
        RowBox[{"Sin", "[", "Phi", "]"}], " ", 
        RowBox[{"Sin", "[", "Psi", "]"}], " ", 
        RowBox[{"Sin", "[", "Thetao", "]"}]}]}], 
      RowBox[{
       RowBox[{"Cos", "[", "Thetao", "]"}], " ", 
       RowBox[{"Sin", "[", "Phi", "]"}]}]},
     {
      RowBox[{
       RowBox[{
        RowBox[{"Sin", "[", "Phi", "]"}], " ", 
        RowBox[{"Sin", "[", "Psi", "]"}]}], "+", 
       RowBox[{
        RowBox[{"Cos", "[", "Phi", "]"}], " ", 
        RowBox[{"Cos", "[", "Psi", "]"}], " ", 
        RowBox[{"Sin", "[", "Thetao", "]"}]}]}], 
      RowBox[{
       RowBox[{
        RowBox[{"-", 
         RowBox[{"Cos", "[", "Psi", "]"}]}], " ", 
        RowBox[{"Sin", "[", "Phi", "]"}]}], "+", 
       RowBox[{
        RowBox[{"Cos", "[", "Phi", "]"}], " ", 
        RowBox[{"Sin", "[", "Psi", "]"}], " ", 
        RowBox[{"Sin", "[", "Thetao", "]"}]}]}], 
      RowBox[{
       RowBox[{"Cos", "[", "Phi", "]"}], " ", 
       RowBox[{"Cos", "[", "Thetao", "]"}]}]}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{
  3.518675986296875*^9, 3.518676205828125*^9, 3.5186762839375*^9, {
   3.51867639475*^9, 3.51867639975*^9}, 3.5186764465625*^9, {
   3.518677207796875*^9, 3.51867724528125*^9}, 3.518677292828125*^9, 
   3.5186775239375*^9, 3.518677600171875*^9, 3.5186778015625*^9, {
   3.518678007109375*^9, 3.5186780320625*^9}, 3.518678540359375*^9, 
   3.518680167234375*^9, 3.518680426203125*^9, 3.51868061290625*^9, 
   3.51868070646875*^9, 3.518862356734375*^9, 3.518862399734375*^9, 
   3.5188628054375*^9, 3.518863329796875*^9, 3.5194048250625*^9, 
   3.51940489875*^9, 3.519410721984375*^9, 3.51944222840625*^9, 
   3.51944235428125*^9, 3.5194424945625*^9, 3.519442621953125*^9, 
   3.5194428019375*^9, 3.519443090515625*^9, 3.519443152984375*^9, 
   3.5194436350625*^9, 3.51944369409375*^9, 3.519443760546875*^9, {
   3.51944418440625*^9, 3.519444204609375*^9}, 3.519444716859375*^9, 
   3.519543649859375*^9, 3.5198055760625*^9, 3.51980594834375*^9, 
   3.51981395784375*^9, 3.519847788296875*^9, 3.522042509448269*^9, 
   3.5220426628381834`*^9, 3.522042694571746*^9, 3.5220428640517826`*^9, 
   3.522078607078125*^9, 3.531823402948124*^9, 3.53185127046875*^9, 
   3.53185133925*^9, 3.531851423875*^9, 3.531851789609375*^9, 
   3.531852604078125*^9, 3.53185266759375*^9, 3.531852740328125*^9, 
   3.5318528295625*^9, 3.531852887328125*^9, 3.531852949953125*^9, 
   3.53185308740625*^9, 3.531853141953125*^9, 3.53185327928125*^9, 
   3.5318533546875*^9, 3.531853421125*^9, 3.531853472078125*^9, 
   3.531853530765625*^9, 3.53185367490625*^9, 3.531853781984375*^9, 
   3.5318539938125*^9, {3.531854130703125*^9, 3.531854158390625*^9}, 
   3.531854245359375*^9, 3.531854284421875*^9, 3.5318543706875*^9, 
   3.53185447759375*^9, 3.531854540890625*^9, 3.531855031234375*^9, 
   3.531857042859375*^9, 3.532106951046875*^9, 3.5341586136875*^9, {
   3.534167497609375*^9, 3.534167519546875*^9}, 3.53416762975*^9, 
   3.534167815171875*^9, 3.534167893109375*^9, 3.534174333953125*^9, 
   3.534218554765625*^9, 3.5342185975625*^9, 3.534218781671875*^9, 
   3.534229534859375*^9, 3.534230354765625*^9, 3.53423070103125*^9, 
   3.534415267609375*^9, 3.534419006015625*^9, 3.53441951196875*^9, 
   3.5344373965*^9, 3.534437681484375*^9, 3.53443809840625*^9, 
   3.534485686453125*^9, 3.5344860804375*^9, 3.53448614796875*^9, 
   3.538214450657544*^9, 3.5382200245531178`*^9, 3.5382215890407724`*^9, 
   3.5382218827713876`*^9, {3.538222189470587*^9, 3.538222194845518*^9}, 
   3.5382225054940777`*^9, 3.538222874137494*^9, 3.538223233525218*^9, 
   3.538223624207717*^9, 3.538236866078125*^9, 3.53823704275*^9, 
   3.538281759858375*^9, 3.5382833443508797`*^9, 3.5382841945039597`*^9, 
   3.538284307690735*^9, 3.538285086388877*^9, 3.5382882705052834`*^9, 
   3.5382890615841055`*^9, 3.538289948775527*^9, 3.538395137482119*^9, 
   3.538395395746091*^9, 3.5383955959948096`*^9, 3.5384803257077327`*^9, 
   3.538503420671875*^9, 3.538503688*^9, 3.538503780046875*^9, 
   3.538721228485338*^9, 3.538722695260125*^9, 3.5388035646520996`*^9, 
   3.538805919558338*^9, 3.538806168712993*^9, 3.5388296374418573`*^9, 
   3.5388318571079416`*^9, 3.538930128296875*^9, 3.5389775451145325`*^9, 
   3.5389930345435815`*^9, 3.539011393014875*^9, 3.539012265343*^9, 
   3.539016050296125*^9, 3.541147783945151*^9, 3.541147896271838*^9, 
   3.541154665249383*^9, 3.541157080662964*^9, 3.5483863431357985`*^9, 
   3.548386383406062*^9, {3.554797903375*^9, 3.554797918609375*^9}, 
   3.555661927743376*^9, 3.556248970944729*^9, 3.5562493071368628`*^9, 
   3.556249551246735*^9, 3.55627646979311*^9, 3.556276589196303*^9, 
   3.556296177*^9, 3.5562962858125*^9, 3.567943965206606*^9, 
   3.5680017703695555`*^9, 3.5680020168351994`*^9, 3.568002882919572*^9, 
   3.568003102667572*^9, 3.568028483258352*^9, 3.568030016985709*^9, 
   3.5763155152519016`*^9, 3.576411384004919*^9, 3.576425136582163*^9, 
   3.5764253481212626`*^9, 3.576426208372466*^9, 3.5769934967316327`*^9, 
   3.576993669726633*^9, 3.5769937351666327`*^9, 3.578504373013383*^9, 
   3.578591504499728*^9, 3.5786616378089724`*^9, 3.5812368931176147`*^9, 
   3.581237061581669*^9, 3.581237108930219*^9, 3.5812371478615694`*^9, 
   3.581237281139023*^9, 3.5812373482794905`*^9, 3.5813519970592823`*^9, 
   3.5817462143714905`*^9, 3.5817463987859306`*^9, 3.5817465989929495`*^9, 
   3.581755007805171*^9, 3.5817808894115143`*^9, 3.5818268594286594`*^9, 
   3.5823513295371017`*^9, 3.584025089540539*^9, 3.584025260692328*^9, 
   3.5840253792781105`*^9, 3.5840254213595176`*^9, 3.584025457124563*^9, 
   3.5853720362204313`*^9, 3.5853721197117796`*^9, 3.585372243487156*^9, 
   3.6124578273742623`*^9, 3.612458096910679*^9, 3.61245817857335*^9, {
   3.6124583090998154`*^9, 3.6124583147621393`*^9}, 3.6131936056104984`*^9, 
   3.6131936494808846`*^9, 3.613193687966733*^9, 3.613193844804189*^9, 
   3.6207123551720657`*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{"(", "\[NoBreak]", GridBox[{
    {
     RowBox[{
      RowBox[{"Cos", "[", "Psi", "]"}], " ", 
      RowBox[{"Cos", "[", "Thetao", "]"}]}], 
     RowBox[{
      RowBox[{"Cos", "[", "Thetao", "]"}], " ", 
      RowBox[{"Sin", "[", "Psi", "]"}]}], 
     RowBox[{"-", 
      RowBox[{"Sin", "[", "Thetao", "]"}]}]},
    {
     RowBox[{
      RowBox[{
       RowBox[{"-", 
        RowBox[{"Cos", "[", "Phi", "]"}]}], " ", 
       RowBox[{"Sin", "[", "Psi", "]"}]}], "+", 
      RowBox[{
       RowBox[{"Cos", "[", "Psi", "]"}], " ", 
       RowBox[{"Sin", "[", "Phi", "]"}], " ", 
       RowBox[{"Sin", "[", "Thetao", "]"}]}]}], 
     RowBox[{
      RowBox[{
       RowBox[{"Cos", "[", "Phi", "]"}], " ", 
       RowBox[{"Cos", "[", "Psi", "]"}]}], "+", 
      RowBox[{
       RowBox[{"Sin", "[", "Phi", "]"}], " ", 
       RowBox[{"Sin", "[", "Psi", "]"}], " ", 
       RowBox[{"Sin", "[", "Thetao", "]"}]}]}], 
     RowBox[{
      RowBox[{"Cos", "[", "Thetao", "]"}], " ", 
      RowBox[{"Sin", "[", "Phi", "]"}]}]},
    {
     RowBox[{
      RowBox[{
       RowBox[{"Sin", "[", "Phi", "]"}], " ", 
       RowBox[{"Sin", "[", "Psi", "]"}]}], "+", 
      RowBox[{
       RowBox[{"Cos", "[", "Phi", "]"}], " ", 
       RowBox[{"Cos", "[", "Psi", "]"}], " ", 
       RowBox[{"Sin", "[", "Thetao", "]"}]}]}], 
     RowBox[{
      RowBox[{
       RowBox[{"-", 
        RowBox[{"Cos", "[", "Psi", "]"}]}], " ", 
       RowBox[{"Sin", "[", "Phi", "]"}]}], "+", 
      RowBox[{
       RowBox[{"Cos", "[", "Phi", "]"}], " ", 
       RowBox[{"Sin", "[", "Psi", "]"}], " ", 
       RowBox[{"Sin", "[", "Thetao", "]"}]}]}], 
     RowBox[{
      RowBox[{"Cos", "[", "Phi", "]"}], " ", 
      RowBox[{"Cos", "[", "Thetao", "]"}]}]}
   }], "\[NoBreak]", ")"}]], "DisplayFormula",
 FontSize->7],

Cell["This can also be expressed using quartenions", "Text"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["L", "BV"], ":=", 
  TagBox[
   RowBox[{"(", "\[NoBreak]", GridBox[{
      {
       RowBox[{
        SuperscriptBox["q0", "2"], "+", 
        SuperscriptBox["q1", "2"], "-", 
        SuperscriptBox["q2", "2"], "-", 
        SuperscriptBox["q3", "2"]}], 
       RowBox[{"2", " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"q1", " ", "q2"}], "+", 
          RowBox[{"q0", " ", "q3"}]}], ")"}]}], 
       RowBox[{"2", " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{"-", "q0"}], " ", "q2"}], "+", 
          RowBox[{"q1", " ", "q3"}]}], ")"}]}]},
      {
       RowBox[{"2", " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"q1", " ", "q2"}], "-", 
          RowBox[{"q0", " ", "q3"}]}], ")"}]}], 
       RowBox[{
        SuperscriptBox["q0", "2"], "-", 
        SuperscriptBox["q1", "2"], "+", 
        SuperscriptBox["q2", "2"], "-", 
        SuperscriptBox["q3", "2"]}], 
       RowBox[{"2", " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"q0", " ", "q1"}], "+", 
          RowBox[{"q2", " ", "q3"}]}], ")"}]}]},
      {
       RowBox[{"2", " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"q0", " ", "q2"}], "+", 
          RowBox[{"q1", " ", "q3"}]}], ")"}]}], 
       RowBox[{"2", " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{"-", "q0"}], " ", "q1"}], "+", 
          RowBox[{"q2", " ", "q3"}]}], ")"}]}], 
       RowBox[{
        SuperscriptBox["q0", "2"], "-", 
        SuperscriptBox["q1", "2"], "-", 
        SuperscriptBox["q2", "2"], "+", 
        SuperscriptBox["q3", "2"]}]}
     }], "\[NoBreak]", ")"}],
   MatrixForm[#]& ]}]], "Input"],

Cell["\<\
where the quartenions can be expressed as below. The inital values of the \
quartenions are caluclated using these expressions\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"initialExpressions", 
  StyleBox["=",
   FontFamily->"Times"], 
  RowBox[{
   StyleBox["{",
    FontFamily->"Times"], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"q0", ",", 
      RowBox[{
       RowBox[{
        RowBox[{"Cos", "[", 
         FractionBox["\[Phi]", "2"], "]"}], 
        RowBox[{"Cos", "[", 
         FractionBox["\[Theta]", "2"], "]"}], 
        RowBox[{"Cos", "[", 
         FractionBox["\[Psi]", "2"], "]"}]}], "+", 
       RowBox[{
        RowBox[{"Sin", "[", 
         FractionBox["\[Phi]", "2"], "]"}], 
        RowBox[{"Sin", "[", 
         FractionBox["\[Theta]", "2"], "]"}], 
        RowBox[{"Sin", "[", 
         FractionBox["\[Psi]", "2"], "]"}]}]}]}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{"{", 
     RowBox[{"q1", ",", 
      RowBox[{
       RowBox[{
        RowBox[{"Sin", "[", 
         FractionBox["\[Phi]", "2"], "]"}], 
        RowBox[{"Cos", "[", 
         FractionBox["\[Theta]", "2"], "]"}], 
        RowBox[{"Cos", "[", 
         FractionBox["\[Psi]", "2"], "]"}]}], "-", 
       RowBox[{
        RowBox[{"Cos", "[", 
         FractionBox["\[Phi]", "2"], "]"}], 
        RowBox[{"Sin", "[", 
         FractionBox["\[Theta]", "2"], "]"}], 
        RowBox[{"Sin", "[", 
         FractionBox["\[Psi]", "2"], "]"}]}]}]}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{"{", 
     RowBox[{"q2", ",", 
      RowBox[{
       RowBox[{
        RowBox[{"Cos", "[", 
         FractionBox["\[Phi]", "2"], "]"}], 
        RowBox[{"Sin", "[", 
         FractionBox["\[Theta]", "2"], "]"}], 
        RowBox[{"Cos", "[", 
         FractionBox["\[Psi]", "2"], "]"}]}], "+", 
       RowBox[{
        RowBox[{"Sin", "[", 
         FractionBox["\[Phi]", "2"], "]"}], 
        RowBox[{"Cos", "[", 
         FractionBox["\[Theta]", "2"], "]"}], 
        RowBox[{"Sin", "[", 
         FractionBox["\[Psi]", "2"], "]"}]}]}]}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{"{", 
     RowBox[{"q3", ",", 
      RowBox[{
       RowBox[{
        RowBox[{"Cos", "[", 
         FractionBox["\[Phi]", "2"], "]"}], 
        RowBox[{"Cos", "[", 
         FractionBox["\[Theta]", "2"], "]"}], 
        RowBox[{"Sin", "[", 
         FractionBox["\[Psi]", "2"], "]"}]}], "-", 
       RowBox[{
        RowBox[{"Sin", "[", 
         FractionBox["\[Phi]", "2"], "]"}], 
        RowBox[{"Sin", "[", 
         FractionBox["\[Theta]", "2"], "]"}], 
        RowBox[{"Cos", "[", 
         FractionBox["\[Psi]", "2"], "]"}]}]}]}], "}"}]}], "}"}]}]], "Input",
 CellChangeTimes->{{3.51984772459375*^9, 3.51984776115625*^9}, {
  3.556249439413949*^9, 3.556249462394224*^9}, {3.556274250560241*^9, 
  3.5562742520134315`*^9}, {3.556276577524727*^9, 3.5562765819933624`*^9}, {
  3.556296109875*^9, 3.55629611140625*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"q0", ",", 
     RowBox[{
      RowBox[{
       RowBox[{"Cos", "[", 
        FractionBox["Phi", "2"], "]"}], " ", 
       RowBox[{"Cos", "[", 
        FractionBox["Psi", "2"], "]"}], " ", 
       RowBox[{"Cos", "[", 
        FractionBox["Thetao", "2"], "]"}]}], "+", 
      RowBox[{
       RowBox[{"Sin", "[", 
        FractionBox["Phi", "2"], "]"}], " ", 
       RowBox[{"Sin", "[", 
        FractionBox["Psi", "2"], "]"}], " ", 
       RowBox[{"Sin", "[", 
        FractionBox["Thetao", "2"], "]"}]}]}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"q1", ",", 
     RowBox[{
      RowBox[{
       RowBox[{"Cos", "[", 
        FractionBox["Psi", "2"], "]"}], " ", 
       RowBox[{"Cos", "[", 
        FractionBox["Thetao", "2"], "]"}], " ", 
       RowBox[{"Sin", "[", 
        FractionBox["Phi", "2"], "]"}]}], "-", 
      RowBox[{
       RowBox[{"Cos", "[", 
        FractionBox["Phi", "2"], "]"}], " ", 
       RowBox[{"Sin", "[", 
        FractionBox["Psi", "2"], "]"}], " ", 
       RowBox[{"Sin", "[", 
        FractionBox["Thetao", "2"], "]"}]}]}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"q2", ",", 
     RowBox[{
      RowBox[{
       RowBox[{"Cos", "[", 
        FractionBox["Thetao", "2"], "]"}], " ", 
       RowBox[{"Sin", "[", 
        FractionBox["Phi", "2"], "]"}], " ", 
       RowBox[{"Sin", "[", 
        FractionBox["Psi", "2"], "]"}]}], "+", 
      RowBox[{
       RowBox[{"Cos", "[", 
        FractionBox["Phi", "2"], "]"}], " ", 
       RowBox[{"Cos", "[", 
        FractionBox["Psi", "2"], "]"}], " ", 
       RowBox[{"Sin", "[", 
        FractionBox["Thetao", "2"], "]"}]}]}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"q3", ",", 
     RowBox[{
      RowBox[{
       RowBox[{"Cos", "[", 
        FractionBox["Phi", "2"], "]"}], " ", 
       RowBox[{"Cos", "[", 
        FractionBox["Thetao", "2"], "]"}], " ", 
       RowBox[{"Sin", "[", 
        FractionBox["Psi", "2"], "]"}]}], "-", 
      RowBox[{
       RowBox[{"Cos", "[", 
        FractionBox["Psi", "2"], "]"}], " ", 
       RowBox[{"Sin", "[", 
        FractionBox["Phi", "2"], "]"}], " ", 
       RowBox[{"Sin", "[", 
        FractionBox["Thetao", "2"], "]"}]}]}]}], "}"}]}], "}"}]], "Output",
 CellChangeTimes->{
  3.556249466390794*^9, 3.5562495512935905`*^9, 3.556276469886857*^9, 
   3.5562765892431765`*^9, 3.556296177046875*^9, 3.556296285875*^9, 
   3.567943965239607*^9, 3.568001770421561*^9, 3.568002016879204*^9, 
   3.5680028831115723`*^9, 3.568003102714572*^9, 3.5680284832973557`*^9, 
   3.5680300170197124`*^9, 3.576315515293906*^9, 3.5764113840459213`*^9, 
   3.5764251366241655`*^9, 3.5764253481642647`*^9, 3.5764262084214687`*^9, 
   3.576993496771633*^9, 3.5769936697676325`*^9, 3.576993735207633*^9, 
   3.5785043730723867`*^9, 3.5785915045387297`*^9, 3.5786616378469744`*^9, 
   3.5812368931696253`*^9, 3.581237061619673*^9, 3.581237108977022*^9, 
   3.5812371478927712`*^9, 3.5812372811858253`*^9, 3.581237348310692*^9, 
   3.581351997103285*^9, 3.5817462144094944`*^9, 3.5817463988209343`*^9, 
   3.581746599031953*^9, 3.581755007844173*^9, 3.5817808894535165`*^9, 
   3.5818268594666595`*^9, 3.582351329586097*^9, 3.584025089576541*^9, 
   3.5840252607363305`*^9, 3.584025379322113*^9, 3.5840254214055204`*^9, 
   3.584025457170566*^9, 3.5853720362734365`*^9, 3.585372119762785*^9, 
   3.585372243549162*^9, 3.612457827408264*^9, 3.612458096948681*^9, 
   3.612458178611352*^9, {3.6124583091398177`*^9, 3.6124583147991414`*^9}, 
   3.613193605658503*^9, 3.613193649513888*^9, 3.6131936880017366`*^9, 
   3.613193844840189*^9, 3.6207123552140694`*^9}]
}, Open  ]],

Cell["\<\
Conversly the total transformation matrix from body coordinates to \
vehicle-carried vertical frame is thus:\
\>", "Text"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["L", "VB"], " ", ":=", 
  RowBox[{"Transpose", "[", 
   SubscriptBox["L", "BV"], "]"}]}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"MatrixForm", "[", 
  SubscriptBox["L", "VB"], " ", "]"}]], "Input"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      RowBox[{
       SuperscriptBox["q0", "2"], "+", 
       SuperscriptBox["q1", "2"], "-", 
       SuperscriptBox["q2", "2"], "-", 
       SuperscriptBox["q3", "2"]}], 
      RowBox[{"2", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"q1", " ", "q2"}], "-", 
         RowBox[{"q0", " ", "q3"}]}], ")"}]}], 
      RowBox[{"2", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"q0", " ", "q2"}], "+", 
         RowBox[{"q1", " ", "q3"}]}], ")"}]}]},
     {
      RowBox[{"2", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"q1", " ", "q2"}], "+", 
         RowBox[{"q0", " ", "q3"}]}], ")"}]}], 
      RowBox[{
       SuperscriptBox["q0", "2"], "-", 
       SuperscriptBox["q1", "2"], "+", 
       SuperscriptBox["q2", "2"], "-", 
       SuperscriptBox["q3", "2"]}], 
      RowBox[{"2", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"-", "q0"}], " ", "q1"}], "+", 
         RowBox[{"q2", " ", "q3"}]}], ")"}]}]},
     {
      RowBox[{"2", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"-", "q0"}], " ", "q2"}], "+", 
         RowBox[{"q1", " ", "q3"}]}], ")"}]}], 
      RowBox[{"2", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"q0", " ", "q1"}], "+", 
         RowBox[{"q2", " ", "q3"}]}], ")"}]}], 
      RowBox[{
       SuperscriptBox["q0", "2"], "-", 
       SuperscriptBox["q1", "2"], "-", 
       SuperscriptBox["q2", "2"], "+", 
       SuperscriptBox["q3", "2"]}]}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{
  3.5186759864375*^9, 3.518676205921875*^9, 3.518676284*^9, {
   3.51867639484375*^9, 3.5186763998125*^9}, 3.518676446640625*^9, {
   3.518677207875*^9, 3.51867724534375*^9}, 3.518677292875*^9, 3.518677524*^9,
    3.518677600421875*^9, 3.518677801640625*^9, {3.51867800715625*^9, 
   3.51867803209375*^9}, 3.5186785405625*^9, 3.518680167296875*^9, 
   3.518680426265625*^9, 3.51868061296875*^9, 3.51868070653125*^9, 
   3.51886235678125*^9, 3.51886239978125*^9, 3.518862805484375*^9, 
   3.518863329859375*^9, 3.519404825109375*^9, 3.51940489878125*^9, 
   3.51941072203125*^9, 3.519442228453125*^9, 3.519442354328125*^9, 
   3.51944249465625*^9, 3.519442622140625*^9, 3.519442801984375*^9, 
   3.519443090546875*^9, 3.51944315303125*^9, 3.519443635109375*^9, 
   3.51944369415625*^9, 3.51944376059375*^9, {3.519444184453125*^9, 
   3.51944420465625*^9}, 3.519444716890625*^9, 3.5195436499375*^9, 
   3.5198055763125*^9, 3.519805948375*^9, 3.519813957875*^9, 
   3.519847788390625*^9, 3.522042509510769*^9, 3.5220426628850574`*^9, 
   3.52204269461862*^9, 3.5220428640986557`*^9, 3.522078607359375*^9, 
   3.531823403276247*^9, 3.5318512705*^9, 3.53185133928125*^9, 
   3.53185142390625*^9, 3.531851789640625*^9, 3.531852604125*^9, 
   3.531852667625*^9, 3.531852740359375*^9, 3.53185282959375*^9, 
   3.531852887359375*^9, 3.531852949984375*^9, 3.5318530874375*^9, 
   3.53185314196875*^9, 3.531853279375*^9, 3.53185335471875*^9, 
   3.53185342115625*^9, 3.531853472125*^9, 3.53185353078125*^9, 
   3.5318536749375*^9, 3.531853782015625*^9, 3.53185399384375*^9, {
   3.53185413071875*^9, 3.531854158421875*^9}, 3.531854245390625*^9, 
   3.531854284453125*^9, 3.53185437071875*^9, 3.531854477625*^9, 
   3.531854540921875*^9, 3.531855031265625*^9, 3.531857042890625*^9, 
   3.532106951203125*^9, 3.534158613859375*^9, {3.534167497796875*^9, 
   3.534167519703125*^9}, 3.53416762990625*^9, 3.53416781534375*^9, 
   3.53416789328125*^9, 3.534174334109375*^9, 3.534218555140625*^9, 
   3.534218597796875*^9, 3.534218781828125*^9, 3.53422953503125*^9, 
   3.53423035490625*^9, 3.534230701640625*^9, 3.5344152679375*^9, 
   3.53441900615625*^9, 3.53441951215625*^9, 3.534437396828125*^9, 
   3.534437682125*^9, 3.5344380990625*^9, 3.534485686625*^9, 
   3.5344860805625*^9, 3.534486148109375*^9, 3.538214450720045*^9, 
   3.538220024693741*^9, 3.5382215890876465`*^9, 3.538221882802637*^9, {
   3.5382221895018363`*^9, 3.538222194892392*^9}, 3.538222505525329*^9, 
   3.538222874184369*^9, 3.5382232335564675`*^9, 3.538223624254592*^9, 
   3.53823686628125*^9, 3.53823704315625*^9, 3.53828175990525*^9, 
   3.5382833443977566`*^9, 3.5382841945508347`*^9, 3.53828430773761*^9, 
   3.538285086435751*^9, 3.538288270552158*^9, 3.5382890616466045`*^9, 
   3.538289948822402*^9, 3.5383951375289936`*^9, 3.538395395777341*^9, 
   3.538395596026059*^9, 3.5384803257545757`*^9, 3.53850342071875*^9, 
   3.538503688046875*^9, 3.53850378009375*^9, 3.5387212287822227`*^9, 
   3.53872269533825*^9, 3.5388035647302265`*^9, 3.5388059195895877`*^9, 
   3.5388061687598677`*^9, 3.5388296374731073`*^9, 3.538831857139192*^9, 
   3.538930129015625*^9, 3.5389775451457825`*^9, 3.5389930346842065`*^9, 
   3.539011393233625*^9, 3.539012265546125*^9, 3.5390160505305*^9, 
   3.5411477840388994`*^9, 3.541147896318712*^9, 3.541154665311882*^9, 
   3.5411570807098484`*^9, 3.548386343198305*^9, 3.548386383468569*^9, {
   3.554797903421875*^9, 3.554797918671875*^9}, 3.5556619278527503`*^9, 
   3.556248971007223*^9, 3.556249307199309*^9, 3.5562495513404455`*^9, 
   3.5562764699493556`*^9, 3.556276589305675*^9, 3.556296177109375*^9, 
   3.55629628590625*^9, 3.5679439652796097`*^9, 3.568001770459564*^9, 
   3.5680020169482107`*^9, 3.568002883173572*^9, 3.568003102757572*^9, 
   3.5680284833353596`*^9, 3.5680300170537157`*^9, 3.57631551533391*^9, 
   3.5764113840919237`*^9, 3.5764251366621675`*^9, 3.5764253482062674`*^9, 
   3.5764262084564705`*^9, 3.576993496813633*^9, 3.5769936698056326`*^9, 
   3.576993735248633*^9, 3.5785043731263895`*^9, 3.5785915045747323`*^9, 
   3.578661637884977*^9, 3.5812368932066326`*^9, 3.5812370616576767`*^9, 
   3.581237109008224*^9, 3.5812371479395742`*^9, 3.5812372812170277`*^9, 
   3.5812373483418927`*^9, 3.581351997139287*^9, 3.5817462144454985`*^9, 
   3.5817463988559375`*^9, 3.5817465990709567`*^9, 3.5817550079561796`*^9, 
   3.5817808894915185`*^9, 3.581826859505659*^9, 3.582351329774078*^9, 
   3.584025089612543*^9, 3.584025260785333*^9, 3.584025379372116*^9, 
   3.5840254214605236`*^9, 3.5840254572175684`*^9, 3.585372036328442*^9, 
   3.5853721198257914`*^9, 3.5853722438611937`*^9, 3.6124578274422665`*^9, 
   3.612458096984683*^9, 3.612458178649354*^9, {3.6124583091788197`*^9, 
   3.612458314837144*^9}, 3.6131936057025075`*^9, 3.6131936495478916`*^9, 
   3.61319368803874*^9, 3.613193844876189*^9, 3.6207123552540736`*^9}]
}, Open  ]],

Cell[TextData[{
 "When Euler angles are used, the state vector ",
 StyleBox["stateVariables",
  FontSlant->"Italic"],
 " of the system is defined as:"
}], "Text",
 Evaluatable->False],

Cell[BoxData[
 RowBox[{
  RowBox[{"stateVariables", "=", 
   RowBox[{"{", 
    RowBox[{
    "Ub", ",", "Vb", ",", "Wb", ",", "Pb", ",", "Qb", ",", "Rb", ",", 
     "\[Phi]", ",", "\[Theta]", ",", "\[Psi]", ",", "xcg", ",", "ycg", ",", 
     "zcg"}], "}"}]}], ";"}]], "DisplayFormula"],

Cell["When quartenions are used it instead becomes:", "Text"],

Cell["\<\
systemVariables = {Ub,Vb,Wb,Pb,Qb,Rb,q0,q1,q2,q3,xcg,ycg,zcg};\
\>", "Input",
 PageWidth->WindowWidth,
 CellChangeTimes->{{3.51867773915625*^9, 3.518677740046875*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"variableLowLimits", "=", 
  RowBox[{"{", 
   RowBox[{"xcg", ",", "0."}], "}"}]}]], "Input",
 CellChangeTimes->{{3.460036081859375*^9, 3.460036120125*^9}, {
  3.578504354003296*^9, 3.5785043581795344`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"xcg", ",", "0.`"}], "}"}]], "Output",
 CellChangeTimes->{
  3.51867598646875*^9, 3.51867620596875*^9, 3.518676284015625*^9, {
   3.518676394875*^9, 3.51867639984375*^9}, 3.518676446671875*^9, {
   3.518677207921875*^9, 3.51867724540625*^9}, 3.518677292890625*^9, 
   3.518677524046875*^9, 3.518677600453125*^9, 3.518677801671875*^9, {
   3.5186780071875*^9, 3.518678032125*^9}, 3.5186785406875*^9, 
   3.518680167328125*^9, 3.5186804263125*^9, 3.518680613*^9, 
   3.5186807065625*^9, 3.5188623568125*^9, 3.5188623998125*^9, 
   3.518862805515625*^9, 3.518863329890625*^9, 3.51940482515625*^9, 
   3.5194048988125*^9, 3.5194107220625*^9, 3.519442228484375*^9, 
   3.519442354359375*^9, 3.51944249484375*^9, 3.5194426221875*^9, 
   3.519442802015625*^9, 3.51944309059375*^9, 3.519443153046875*^9, 
   3.519443635140625*^9, 3.5194436941875*^9, 3.519443760640625*^9, {
   3.519444184484375*^9, 3.519444204703125*^9}, 3.519444716921875*^9, 
   3.51954365*^9, 3.519805576328125*^9, 3.51980594840625*^9, 
   3.51981395790625*^9, 3.519847788453125*^9, 3.522042509542019*^9, 
   3.5220426629163065`*^9, 3.522042694649869*^9, 3.5220428641455297`*^9, 
   3.52207860740625*^9, 3.531823403307496*^9, 3.531851270515625*^9, 
   3.531851339296875*^9, 3.531851423921875*^9, 3.53185178965625*^9, 
   3.531852604140625*^9, 3.531852667640625*^9, 3.531852740375*^9, 
   3.531852829609375*^9, 3.531852887375*^9, 3.53185295*^9, 
   3.531853087453125*^9, 3.531853141984375*^9, 3.531853279390625*^9, 
   3.531853354734375*^9, 3.531853421171875*^9, 3.531853472140625*^9, 
   3.531853530796875*^9, 3.531853674953125*^9, 3.53185378203125*^9, 
   3.531853993859375*^9, {3.53185413075*^9, 3.531854158453125*^9}, 
   3.53185424540625*^9, 3.53185428446875*^9, 3.531854370734375*^9, 
   3.531854477640625*^9, 3.5318545409375*^9, 3.53185503128125*^9, 
   3.531857042921875*^9, 3.5321069513125*^9, 3.53415861396875*^9, {
   3.534167498046875*^9, 3.534167519796875*^9}, 3.53416763*^9, 
   3.534167815640625*^9, 3.534167893390625*^9, 3.5341743341875*^9, 
   3.534218555234375*^9, 3.534218599890625*^9, 3.53421878190625*^9, 
   3.534229535125*^9, 3.534230354984375*^9, 3.53423070178125*^9, 
   3.5344152680625*^9, 3.53441900625*^9, 3.534419512265625*^9, 
   3.534437396984375*^9, 3.534437682296875*^9, 3.534438099234375*^9, 
   3.53448568675*^9, 3.534486080640625*^9, 3.5344861481875*^9, 
   3.538214450751295*^9, 3.538220024740616*^9, 3.5382215891345215`*^9, 
   3.538221882818262*^9, {3.5382221895174613`*^9, 3.538222194923642*^9}, 
   3.53822250555658*^9, 3.5382228742156186`*^9, 3.538223233587717*^9, 
   3.5382236242702165`*^9, 3.538236866390625*^9, 3.53823704328125*^9, 
   3.5382817599365*^9, 3.5382833444133825`*^9, 3.538284194582084*^9, 
   3.53828430776886*^9, 3.538285086451376*^9, 3.5382882705834074`*^9, 
   3.5382890616778545`*^9, 3.5382899488536515`*^9, 3.5383951375446186`*^9, 
   3.538395395808591*^9, 3.538395596057309*^9, 3.538480325785804*^9, 
   3.538503420765625*^9, 3.538503688078125*^9, 3.538503780109375*^9, 
   3.5387212288134737`*^9, 3.538722695385125*^9, 3.5388035648864794`*^9, 
   3.5388059196208377`*^9, 3.5388061687754927`*^9, 3.5388296374887323`*^9, 
   3.5388318571704426`*^9, 3.538930129125*^9, 3.5389775451770325`*^9, 
   3.5389930347467065`*^9, 3.539011394014875*^9, 3.539012266264875*^9, 
   3.5390160506555*^9, 3.5411477840545244`*^9, 3.541147896334337*^9, 
   3.541154665327507*^9, 3.541157080741104*^9, 3.548386343229559*^9, 
   3.5483863834998226`*^9, {3.55479790353125*^9, 3.55479791871875*^9}, 
   3.555661927899626*^9, 3.5562489710384693`*^9, 3.5562493072305317`*^9, 
   3.5562495513873005`*^9, 3.556276469980605*^9, 3.5562765893369246`*^9, 
   3.556296177140625*^9, 3.5562962859375*^9, 3.567943965320612*^9, 
   3.5680017704995685`*^9, 3.5680020170282187`*^9, 3.568002883232572*^9, 
   3.568003102807572*^9, 3.568028483371363*^9, 3.56803001709372*^9, 
   3.576315515373914*^9, 3.5764113841289263`*^9, 3.57642513670117*^9, 
   3.5764253482462697`*^9, 3.5764262084914727`*^9, 3.576993497038633*^9, 
   3.5769936698456326`*^9, 3.576993735287633*^9, 3.5785043731793923`*^9, 
   3.5785915046077337`*^9, 3.5786616379209785`*^9, 3.5812368932446404`*^9, 
   3.5812370616926804`*^9, 3.581237109039426*^9, 3.581237147970776*^9, 
   3.5812372812482295`*^9, 3.5812373483730936`*^9, 3.581351997175289*^9, 
   3.5817462144815016`*^9, 3.5817463988959417`*^9, 3.5817465991079607`*^9, 
   3.5817550079961815`*^9, 3.581780889534521*^9, 3.581826859540659*^9, 
   3.582351329861069*^9, 3.5840250896635456`*^9, 3.5840252608393364`*^9, 
   3.5840253794251194`*^9, 3.584025421502526*^9, 3.5840254572715716`*^9, 
   3.585372036381448*^9, 3.5853721198837967`*^9, 3.5853722439251995`*^9, 
   3.612457827475268*^9, 3.612458097022685*^9, 3.6124581786863565`*^9, {
   3.612458309215822*^9, 3.6124583148741455`*^9}, 3.6131936057435117`*^9, 
   3.613193649579895*^9, 3.6131936880727434`*^9, 3.613193844911189*^9, 
   3.6207123552950773`*^9}]
}, Open  ]],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   SubscriptBox["\[CapitalOmega]", "B"], TraditionalForm]]],
 " is from Stevensson and Lewis (1.5-2), "
}], "Text"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["\[CapitalOmega]", "B"], ":=", 
  TagBox[
   RowBox[{"(", "\[NoBreak]", GridBox[{
      {"0", 
       RowBox[{"-", "Rb"}], "Qb"},
      {"Rb", "0", 
       RowBox[{"-", "Pb"}]},
      {
       RowBox[{"-", "Qb"}], "Pb", "0"}
     }], "\[NoBreak]", ")"}],
   MatrixForm[#]& ]}]], "Input"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["\[CapitalOmega]", "q"], ":=", 
  TagBox[
   RowBox[{"(", "\[NoBreak]", GridBox[{
      {"0", "Pb", "Qb", "Rb"},
      {
       RowBox[{"-", "Pb"}], "0", 
       RowBox[{"-", "Rb"}], "Qb"},
      {
       RowBox[{"-", "Qb"}], "Rb", "0", 
       RowBox[{"-", "Pb"}]},
      {
       RowBox[{"-", "Rb"}], 
       RowBox[{"-", "Qb"}], "Pb", "0"}
     }], "\[NoBreak]", ")"}],
   MatrixForm[#]& ]}]], "Input"],

Cell[BoxData[
 RowBox[{"J", ":=", 
  TagBox[
   RowBox[{"(", "\[NoBreak]", GridBox[{
      {"Ix", "0", 
       RowBox[{"-", "Ixz"}]},
      {"0", "Iy", "0"},
      {
       RowBox[{"-", "Ixz"}], "0", "Iz"}
     }], "\[NoBreak]", ")"}],
   MatrixForm[#]& ]}]], "Input"],

Cell["The invers of the inertia matrix is", "Text"],

Cell[BoxData[
 RowBox[{"Jinv", ":=", 
  RowBox[{"Simplify", " ", "[", 
   RowBox[{"Inverse", "[", "J", "]"}], "]"}]}]], "Input"],

Cell["The speed vector in body coordinates", "Text"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["V", "body"], ":=", 
  RowBox[{"{", 
   RowBox[{"Ub", ",", "Vb", ",", "Wb"}], "}"}]}]], "Input"],

Cell["The speed vector in earth coordinates", "Text"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["V", "NED"], ":=", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"s", " ", "xcg"}], ",", " ", 
    RowBox[{"s", " ", "ycg"}], ",", " ", 
    RowBox[{"s", " ", "zcg"}]}], "}"}]}]], "Input"],

Cell["The force vector in body coordinates", "Text"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["F", "b"], " ", ":=", " ", 
  RowBox[{"{", 
   RowBox[{"Fx", ",", "Fy", ",", "Fz"}], "}"}]}]], "Input"],

Cell["The gravitational vector is defined as (earth coordinates):", "Text"],

Cell[BoxData[
 RowBox[{" ", 
  RowBox[{
   RowBox[{
    SubscriptBox["G", "0"], ":=", 
    RowBox[{"{", 
     RowBox[{"0", ",", "0", ",", "g0"}], "}"}]}], "\[IndentingNewLine]", 
   RowBox[{"Fground", ":=", 
    RowBox[{"{", 
     RowBox[{"0", ",", "0", ",", 
      RowBox[{
       RowBox[{"-", 
        RowBox[{"onPositive", "[", "zcg", "]"}]}], "kground", " ", "zcg"}]}], 
     "}"}]}]}]}]], "Input",
 CellChangeTimes->{{3.555056841403258*^9, 3.5550568708249445`*^9}, {
  3.5550569212621217`*^9, 3.555056940511998*^9}}],

Cell["The vector of Euler angles is:", "Text"],

Cell[BoxData[
 RowBox[{"\[CapitalPhi]", " ", ":=", 
  RowBox[{"{", 
   RowBox[{"\[Phi]", ",", " ", "\[Theta]", ",", "\[Psi]"}], 
   "}"}]}]], "DisplayFormula"],

Cell["The quartenion vector is:", "Text"],

Cell[BoxData[
 RowBox[{"quart", ":=", 
  RowBox[{"{", 
   RowBox[{"q0", ",", "q1", ",", "q2", ",", "q3"}], "}"}]}]], "Input"],

Cell["The angular rates in body coordinates", "Text"],

Cell[BoxData[
 RowBox[{" ", 
  RowBox[{
   SubscriptBox["\[Omega]", "b"], ":=", 
   RowBox[{"{", 
    RowBox[{"Pb", ",", "Qb", ",", "Rb"}], "}"}]}]}]], "Input"],

Cell["The vector of moments in body coordinates", "Text"],

Cell[BoxData[
 RowBox[{
  SubscriptBox["T", "b"], ":=", 
  RowBox[{"{", 
   RowBox[{"Lb", ",", "Mb", ",", "Nb"}], "}"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["The system equations", "Subsection"],

Cell[TextData[StyleBox["Force Equations",
 FontSlant->"Italic"]], "Text"],

Cell[BoxData[
 RowBox[{"forceEquation", ":=", 
  RowBox[{
   RowBox[{"s", " ", 
    SubscriptBox["V", "body"]}], "-", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      RowBox[{
       SubscriptBox["\[CapitalOmega]", "B"], ".", 
       SubscriptBox["V", "body"]}]}], "+", 
     RowBox[{
      SubscriptBox["L", "BV"], " ", ".", "  ", 
      RowBox[{"(", 
       RowBox[{
        SubscriptBox["G", "0"], "+", 
        FractionBox["Fground", "mass"]}], ")"}]}], "+", 
     FractionBox[
      SubscriptBox["F", "b"], "mass"]}], ")"}]}]}]], "Input",
 CellChangeTimes->{{3.5550568095597115`*^9, 3.555056811856572*^9}, {
  3.555056866246849*^9, 3.5550568965747795`*^9}, {3.5550569356839046`*^9, 
  3.555056936949521*^9}}],

Cell[TextData[StyleBox["Kinematic Equations",
 FontSlant->"Italic"]], "Text"],

Cell["(when Euler angles are used)", "Text"],

Cell[BoxData[
 RowBox[{"kinematicEquation", ":=", 
  RowBox[{
   RowBox[{"s", " ", "\[CapitalPhi]"}], "-", 
   RowBox[{"\[Zeta]", ".", " ", 
    SubscriptBox["\[Omega]", "b"]}]}]}]], "DisplayFormula"],

Cell["(when quartenions are used)", "Text"],

Cell[BoxData[
 RowBox[{"kinematicEquation", ":=", 
  RowBox[{
   RowBox[{"s", " ", "quart"}], " ", "-", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      FractionBox["1", "2"]}], " ", 
     RowBox[{
      SubscriptBox["\[CapitalOmega]", "q"], ".", "quart"}]}], 
    ")"}]}]}]], "Input"],

Cell[TextData[StyleBox["Moment Equations",
 FontSlant->"Italic"]], "Text"],

Cell[BoxData[
 RowBox[{"momentEquation", ":=", 
  RowBox[{"Simplify", "[", 
   RowBox[{
    RowBox[{"s", " ", 
     SubscriptBox["\[Omega]", "b"]}], "-", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", 
       RowBox[{"Jinv", " ", ".", 
        SubscriptBox["\[CapitalOmega]", "B"], " ", ".", "J", " ", ".", " ", 
        SubscriptBox["\[Omega]", "b"]}]}], "+", 
      RowBox[{"Jinv", ".", " ", 
       SubscriptBox["T", "b"]}]}], ")"}]}], "]"}]}]], "Input"],

Cell[TextData[StyleBox["Naviagation Equations",
 FontSlant->"Italic"]], "Text"],

Cell[BoxData[
 RowBox[{"navigationEquation", ":=", 
  RowBox[{
   SubscriptBox["V", "NED"], "-", 
   RowBox[{
    SubscriptBox["L", "VB"], ".", 
    SubscriptBox["V", "body"]}]}]}]], "Input"],

Cell[TextData[{
 "Tranformation between quartenions and Euler angles can be done in a \
straightforward way by simply adding this set of equation to the ",
 StyleBox["systemEquationDa",
  FontSlant->"Italic"],
 "."
}], "Text"],

Cell[BoxData[
 RowBox[{"quartenion2Euler", ":=", 
  RowBox[{"{", "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"q1", "-", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"Sin", "[", 
         FractionBox["\[Phi]", "2"], "]"}], 
        RowBox[{"Cos", "[", 
         FractionBox["\[Theta]", "2"], "]"}], 
        RowBox[{"Cos", "[", 
         FractionBox["\[Psi]", "2"], "]"}]}], "-", 
       RowBox[{
        RowBox[{"Cos", "[", 
         FractionBox["\[Phi]", "2"], "]"}], 
        RowBox[{"Sin", "[", 
         FractionBox["\[Theta]", "2"], "]"}], 
        RowBox[{"Sin", "[", 
         FractionBox["\[Psi]", "2"], "]"}]}]}], ")"}]}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{"q2", "-", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"Cos", "[", 
         FractionBox["\[Phi]", "2"], "]"}], 
        RowBox[{"Sin", "[", 
         FractionBox["\[Theta]", "2"], "]"}], 
        RowBox[{"Cos", "[", 
         FractionBox["\[Psi]", "2"], "]"}]}], "+", 
       RowBox[{
        RowBox[{"Sin", "[", 
         FractionBox["\[Phi]", "2"], "]"}], 
        RowBox[{"Cos", "[", 
         FractionBox["\[Theta]", "2"], "]"}], 
        RowBox[{"Sin", "[", 
         FractionBox["\[Psi]", "2"], "]"}]}]}], ")"}]}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{"q3", "-", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"Cos", "[", 
         FractionBox["\[Phi]", "2"], "]"}], 
        RowBox[{"Cos", "[", 
         FractionBox["\[Theta]", "2"], "]"}], 
        RowBox[{"Sin", "[", 
         FractionBox["\[Psi]", "2"], "]"}]}], "-", 
       RowBox[{
        RowBox[{"Sin", "[", 
         FractionBox["\[Phi]", "2"], "]"}], 
        RowBox[{"Sin", "[", 
         FractionBox["\[Theta]", "2"], "]"}], 
        RowBox[{"Cos", "[", 
         FractionBox["\[Psi]", "2"], "]"}]}]}], ")"}]}]}], 
   "\[IndentingNewLine]", "}"}]}]], "Input"],

Cell["\<\
A (marginally) more computationally efficient way, however, is to calculate \
them explicitly (which is used here).\
\>", "Text"],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   SubscriptBox["\[Phi]", "expr"], "=", 
   RowBox[{"Atan2L", "[", 
    RowBox[{
     RowBox[{"2", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"q2", " ", "q3"}], "+", 
        RowBox[{"q0", " ", "q1"}]}], ")"}]}], ",", 
     RowBox[{
      SuperscriptBox["q0", "2"], "-", 
      SuperscriptBox["q1", "2"], "-", 
      SuperscriptBox["q2", "2"], "+", 
      SuperscriptBox["q3", "2"]}]}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   SubscriptBox["\[Theta]", "expr"], "=", 
   RowBox[{"ArcSinL", "[", 
    RowBox[{"2", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"q0", " ", "q2"}], "-", 
       RowBox[{"q1", " ", "q3"}]}], ")"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   SubscriptBox["\[Psi]", "expr"], "=", 
   RowBox[{"Atan2L", "[", 
    RowBox[{
     RowBox[{"2", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"q1", " ", "q2"}], "+", 
        RowBox[{"q0", " ", "q3"}]}], ")"}]}], ",", 
     RowBox[{
      SuperscriptBox["q0", "2"], "+", 
      SuperscriptBox["q1", "2"], "-", 
      SuperscriptBox["q2", "2"], "-", 
      SuperscriptBox["q3", "2"]}]}], "]"}]}], ";"}]}], "Input"],

Cell["\<\
To solve the system of equations all equations are equal to zero.\
\>", "Text"]
}, Open  ]],

Cell["\<\
Another instance of the navigation equation is needed to obtain the speeds i \
earth coordinates.\
\>", "Text"],

Cell[BoxData[
 RowBox[{"expressionVE", ":=", 
  RowBox[{"Transpose", "[", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"vx", ",", "vy", ",", "vz"}], "}"}], ",", 
     RowBox[{
      SubscriptBox["L", "VB"], ".", 
      SubscriptBox["V", "body"]}]}], "}"}], "]"}]}]], "Input",
 CellChangeTimes->{3.541153908526613*^9}],

Cell[CellGroupData[{

Cell["Assembling the system of equations", "Subsection"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"forceEquation", "//", "TableForm"}]], "Input"],

Cell[BoxData[
 TagBox[
  TagBox[GridBox[{
     {
      RowBox[{
       RowBox[{"-", 
        FractionBox["Fx", "mass"]}], "+", 
       RowBox[{"s", " ", "Ub"}], "-", 
       RowBox[{"Rb", " ", "Vb"}], "+", 
       RowBox[{"Qb", " ", "Wb"}], "-", 
       RowBox[{"2", " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{"-", "q0"}], " ", "q2"}], "+", 
          RowBox[{"q1", " ", "q3"}]}], ")"}], " ", 
        RowBox[{"(", 
         RowBox[{"g0", "-", 
          FractionBox[
           RowBox[{"kground", " ", "zcg", " ", 
            RowBox[{"onPositive", "[", "zcg", "]"}]}], "mass"]}], ")"}]}]}]},
     {
      RowBox[{
       RowBox[{"-", 
        FractionBox["Fy", "mass"]}], "+", 
       RowBox[{"Rb", " ", "Ub"}], "+", 
       RowBox[{"s", " ", "Vb"}], "-", 
       RowBox[{"Pb", " ", "Wb"}], "-", 
       RowBox[{"2", " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"q0", " ", "q1"}], "+", 
          RowBox[{"q2", " ", "q3"}]}], ")"}], " ", 
        RowBox[{"(", 
         RowBox[{"g0", "-", 
          FractionBox[
           RowBox[{"kground", " ", "zcg", " ", 
            RowBox[{"onPositive", "[", "zcg", "]"}]}], "mass"]}], ")"}]}]}]},
     {
      RowBox[{
       RowBox[{"-", 
        FractionBox["Fz", "mass"]}], "-", 
       RowBox[{"Qb", " ", "Ub"}], "+", 
       RowBox[{"Pb", " ", "Vb"}], "+", 
       RowBox[{"s", " ", "Wb"}], "-", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox["q0", "2"], "-", 
          SuperscriptBox["q1", "2"], "-", 
          SuperscriptBox["q2", "2"], "+", 
          SuperscriptBox["q3", "2"]}], ")"}], " ", 
        RowBox[{"(", 
         RowBox[{"g0", "-", 
          FractionBox[
           RowBox[{"kground", " ", "zcg", " ", 
            RowBox[{"onPositive", "[", "zcg", "]"}]}], "mass"]}], ")"}]}]}]}
    },
    GridBoxAlignment->{
     "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.5599999999999999]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}],
   Column],
  Function[BoxForm`e$, 
   TableForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{
  3.518675986671875*^9, 3.518676206125*^9, 3.518676284234375*^9, {
   3.518676395015625*^9, 3.518676399984375*^9}, 3.518676446828125*^9, {
   3.51867720825*^9, 3.518677245546875*^9}, 3.518677293*^9, 
   3.518677524203125*^9, 3.51867760059375*^9, 3.5186778018125*^9, {
   3.5186780073125*^9, 3.518678032234375*^9}, 3.51867854153125*^9, 
   3.51868016746875*^9, 3.51868042653125*^9, 3.51868061315625*^9, 
   3.51868070671875*^9, 3.518862356953125*^9, 3.518862399921875*^9, 
   3.518862805671875*^9, 3.51886333003125*^9, 3.519404825453125*^9, 
   3.519404898921875*^9, 3.519410722203125*^9, 3.519442228625*^9, 
   3.5194423545*^9, 3.519442495109375*^9, 3.51944262234375*^9, 
   3.519442802140625*^9, 3.5194430906875*^9, 3.51944315315625*^9, 
   3.51944363528125*^9, 3.5194436945*^9, 3.519443760953125*^9, {
   3.519444184578125*^9, 3.5194442048125*^9}, 3.5194447170625*^9, 
   3.519543650484375*^9, 3.519805576484375*^9, 3.5198059485*^9, 
   3.519813958*^9, 3.519847788890625*^9, 3.522042509792017*^9, 
   3.522042663088177*^9, 3.522042694774866*^9, 3.522042864286151*^9, 
   3.52207860759375*^9, 3.5318234034637456`*^9, 3.53185127059375*^9, 
   3.531851339375*^9, 3.531851424*^9, 3.531851789765625*^9, 
   3.531852604234375*^9, 3.53185266771875*^9, 3.531852740453125*^9, 
   3.5318528296875*^9, 3.531852887453125*^9, 3.53185295009375*^9, 
   3.53185308753125*^9, 3.531853142078125*^9, 3.53185327946875*^9, 
   3.531853354828125*^9, 3.53185342125*^9, 3.531853472234375*^9, 
   3.531853530875*^9, 3.531853675015625*^9, 3.531853782109375*^9, 
   3.5318539939375*^9, {3.5318541308125*^9, 3.53185415853125*^9}, 
   3.531854245484375*^9, 3.531854284578125*^9, 3.5318543708125*^9, 
   3.53185447771875*^9, 3.531854541015625*^9, 3.531855031359375*^9, 
   3.531857043015625*^9, 3.53210695225*^9, 3.534158614703125*^9, {
   3.534167498609375*^9, 3.53416752028125*^9}, 3.5341676305*^9, 
   3.534167816140625*^9, 3.534167893890625*^9, 3.5341743348125*^9, 
   3.534218555671875*^9, 3.534218602328125*^9, 3.534218782609375*^9, 
   3.534229535921875*^9, 3.53423035571875*^9, 3.534230702765625*^9, 
   3.534415268578125*^9, 3.534419006890625*^9, 3.53441951296875*^9, 
   3.534437398515625*^9, 3.534437683640625*^9, 3.534438100609375*^9, 
   3.5344856876875*^9, 3.53448608153125*^9, 3.53448614896875*^9, 
   3.538214450923171*^9, 3.538220024896864*^9, 3.538221589368893*^9, 
   3.538221882912011*^9, {3.53822218961121*^9, 3.5382221950486403`*^9}, 
   3.538222505650333*^9, 3.538222874340618*^9, 3.538223233697091*^9, 
   3.538223624395215*^9, 3.538236867125*^9, 3.53823704371875*^9, 
   3.538281760077125*^9, 3.5382833445540147`*^9, 3.5382841946758337`*^9, 
   3.538284307893859*^9, 3.5382850865451255`*^9, 3.5382882707084064`*^9, 
   3.5382890618184786`*^9, 3.538289948963026*^9, 3.5383951377164927`*^9, 
   3.53839539593359*^9, 3.538395596182308*^9, 3.5384803259263325`*^9, 
   3.538503420890625*^9, 3.538503688390625*^9, 3.538503780234375*^9, 
   3.5387212289853544`*^9, 3.538722695541375*^9, 3.538803565136484*^9, 
   3.5388059197145867`*^9, 3.5388061689317417`*^9, 3.5388296376137333`*^9, 
   3.5388318572798185`*^9, 3.538930129953125*^9, 3.5389775455676603`*^9, 
   3.5389930352310815`*^9, 3.53901139524925*^9, 3.539012267452375*^9, 
   3.5390160519055*^9, 3.5411477841795225`*^9, 3.541147896443711*^9, 
   3.541154665468129*^9, 3.5411570809286413`*^9, 3.548386343604602*^9, 
   3.5483863836717176`*^9, {3.554797903640625*^9, 3.554797918890625*^9}, 
   3.5556619280715003`*^9, 3.556248971288444*^9, 3.556249307386648*^9, 
   3.556249551637195*^9, 3.5562764703712196`*^9, 3.5562765894306717`*^9, 
   3.556296177265625*^9, 3.55629628603125*^9, 3.5679439655786266`*^9, 
   3.5680017708206005`*^9, 3.5680020176832843`*^9, 3.568002883646572*^9, 
   3.5680031031895723`*^9, 3.568028483572383*^9, 3.56803001729574*^9, 
   3.5763155156899457`*^9, 3.5764113844359436`*^9, 3.576425137018188*^9, 
   3.576425348440281*^9, 3.5764262087764893`*^9, 3.5769934972576327`*^9, 
   3.576993670243633*^9, 3.5769937354966326`*^9, 3.5785043734394073`*^9, 
   3.578591504850748*^9, 3.578661638205995*^9, 3.581236893618715*^9, 
   3.5812370618937006`*^9, 3.581237109226638*^9, 3.581237148282796*^9, 
   3.5812372815758505`*^9, 3.581237348685104*^9, 3.581351997475306*^9, 
   3.581746214734527*^9, 3.581746399206972*^9, 3.58174659930098*^9, 
   3.581755008212194*^9, 3.581780889814537*^9, 3.5818268598316593`*^9, 
   3.582351330173038*^9, 3.5840250900105658`*^9, 3.5840252611893563`*^9, 
   3.5840253797161355`*^9, 3.5840254218165436`*^9, 3.584025457572589*^9, 
   3.585372036854495*^9, 3.58537212031584*^9, 3.585372244236231*^9, 
   3.61245782768228*^9, 3.6124580972276974`*^9, 3.612458178890368*^9, {
   3.6124583094288344`*^9, 3.612458315085158*^9}, 3.613193605967534*^9, 
   3.613193649782915*^9, 3.613193688286765*^9, 3.6131938451061893`*^9, 
   3.620712355497098*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"momentEquation", "//", "TableForm"}]], "Input"],

Cell[BoxData[
 TagBox[
  TagBox[GridBox[{
     {
      FractionBox[
       RowBox[{
        RowBox[{
         RowBox[{"-", 
          SuperscriptBox["Iz", "2"]}], " ", "Qb", " ", "Rb"}], "+", 
        RowBox[{"Iz", " ", 
         RowBox[{"(", 
          RowBox[{"Lb", "+", 
           RowBox[{"Ixz", " ", "Pb", " ", "Qb"}], "+", 
           RowBox[{"Iy", " ", "Qb", " ", "Rb"}], "-", 
           RowBox[{"Ix", " ", "Pb", " ", "s"}]}], ")"}]}], "+", 
        RowBox[{"Ixz", " ", 
         RowBox[{"(", 
          RowBox[{"Nb", "+", 
           RowBox[{"Ix", " ", "Pb", " ", "Qb"}], "-", 
           RowBox[{"Iy", " ", "Pb", " ", "Qb"}], "-", 
           RowBox[{"Ixz", " ", "Qb", " ", "Rb"}], "+", 
           RowBox[{"Ixz", " ", "Pb", " ", "s"}]}], ")"}]}]}], 
       RowBox[{
        SuperscriptBox["Ixz", "2"], "-", 
        RowBox[{"Ix", " ", "Iz"}]}]]},
     {
      FractionBox[
       RowBox[{
        RowBox[{"-", "Mb"}], "+", 
        RowBox[{"Ix", " ", "Pb", " ", "Rb"}], "-", 
        RowBox[{"Iz", " ", "Pb", " ", "Rb"}], "+", 
        RowBox[{"Ixz", " ", 
         RowBox[{"(", 
          RowBox[{
           SuperscriptBox["Pb", "2"], "-", 
           SuperscriptBox["Rb", "2"]}], ")"}]}], "+", 
        RowBox[{"Iy", " ", "Qb", " ", "s"}]}], "Iy"]},
     {
      FractionBox[
       RowBox[{
        RowBox[{"Ixz", " ", 
         RowBox[{"(", 
          RowBox[{"Lb", "-", 
           RowBox[{
            RowBox[{"(", 
             RowBox[{"Ix", "-", "Iy", "+", "Iz"}], ")"}], " ", "Qb", " ", 
            "Rb"}]}], ")"}]}], "+", 
        RowBox[{
         SuperscriptBox["Ixz", "2"], " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"Pb", " ", "Qb"}], "+", 
           RowBox[{"Rb", " ", "s"}]}], ")"}]}], "+", 
        RowBox[{"Ix", " ", 
         RowBox[{"(", 
          RowBox[{"Nb", "+", 
           RowBox[{"Ix", " ", "Pb", " ", "Qb"}], "-", 
           RowBox[{"Iy", " ", "Pb", " ", "Qb"}], "-", 
           RowBox[{"Iz", " ", "Rb", " ", "s"}]}], ")"}]}]}], 
       RowBox[{
        SuperscriptBox["Ixz", "2"], "-", 
        RowBox[{"Ix", " ", "Iz"}]}]]}
    },
    GridBoxAlignment->{
     "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.5599999999999999]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}],
   Column],
  Function[BoxForm`e$, 
   TableForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{
  3.518675987*^9, 3.518676206234375*^9, 3.518676284328125*^9, {
   3.518676395078125*^9, 3.5186764000625*^9}, 3.518676446890625*^9, {
   3.5186772083125*^9, 3.518677246875*^9}, 3.518677293015625*^9, 
   3.518677524234375*^9, 3.518677600734375*^9, 3.518677803421875*^9, {
   3.51867800871875*^9, 3.51867803365625*^9}, 3.518678542984375*^9, 
   3.518680168890625*^9, 3.518680428109375*^9, 3.5186806145625*^9, 
   3.518680708171875*^9, 3.518862357609375*^9, 3.51886240178125*^9, 
   3.518862807078125*^9, 3.518863331484375*^9, 3.519404825828125*^9, 
   3.519404900546875*^9, 3.519410723625*^9, 3.519442230046875*^9, 
   3.519442356*^9, 3.519442496578125*^9, 3.519442622515625*^9, 
   3.519442803859375*^9, 3.519443092125*^9, 3.519443154578125*^9, 
   3.519443635421875*^9, 3.51944369621875*^9, 3.519443761109375*^9, {
   3.51944418525*^9, 3.519444205296875*^9}, 3.5194447175625*^9, 
   3.51954365090625*^9, 3.519805576859375*^9, 3.51980595096875*^9, 
   3.519813960078125*^9, 3.5198477910625*^9, 3.522042509948266*^9, 
   3.522042663822533*^9, 3.5220426953061028`*^9, 3.522042864426773*^9, 
   3.522078607859375*^9, 3.531823403854368*^9, 3.531851270828125*^9, 
   3.531851339625*^9, 3.53185142421875*^9, 3.531851790015625*^9, 
   3.5318526045*^9, 3.531852667953125*^9, 3.531852740703125*^9, 
   3.53185282990625*^9, 3.5318528876875*^9, 3.531852950328125*^9, 
   3.531853087765625*^9, 3.531853142328125*^9, 3.531853279703125*^9, 
   3.531853355078125*^9, 3.531853421484375*^9, 3.53185347225*^9, 
   3.531853530875*^9, 3.53185367503125*^9, 3.531853782234375*^9, 
   3.5318539941875*^9, {3.531854131046875*^9, 3.53185415878125*^9}, 
   3.531854245703125*^9, 3.531854284828125*^9, 3.53185437103125*^9, 
   3.53185447796875*^9, 3.531854541265625*^9, 3.531855031578125*^9, 
   3.531857043265625*^9, 3.53210695325*^9, 3.534158615765625*^9, {
   3.534167499234375*^9, 3.534167520390625*^9}, 3.534167631640625*^9, 
   3.534167817234375*^9, 3.534167895015625*^9, 3.534174335796875*^9, 
   3.534218556671875*^9, 3.534218603375*^9, 3.53421878365625*^9, 
   3.534229537078125*^9, 3.53423035671875*^9, 3.534230704125*^9, 
   3.534415269625*^9, 3.5344190079375*^9, 3.534419513984375*^9, 
   3.534437399671875*^9, 3.5344376846875*^9, 3.5344381016875*^9, 
   3.534485688703125*^9, 3.53448608253125*^9, 3.534486150125*^9, 
   3.538214451048172*^9, 3.538220025068736*^9, 3.5382215894782667`*^9, 
   3.5382218830213842`*^9, {3.538222189720584*^9, 3.5382221951580143`*^9}, 
   3.5382225057597113`*^9, 3.538222874449992*^9, 3.538223233806464*^9, 
   3.538223624504588*^9, 3.538236867734375*^9, 3.5382370443125*^9, 
   3.538281760202125*^9, 3.5382833446633954`*^9, 3.5382841947852077`*^9, 
   3.538284308003233*^9, 3.538285086670125*^9, 3.538288270833406*^9, 
   3.538289061943478*^9, 3.5382899490724*^9, 3.5383951378258667`*^9, 
   3.5383953960429645`*^9, 3.5383955962916827`*^9, 3.538480326035633*^9, 
   3.53850342103125*^9, 3.538503688515625*^9, 3.53850378034375*^9, 
   3.538721229110358*^9, 3.538722695682*^9, 3.5388035653083625`*^9, 
   3.5388059198239613`*^9, 3.538806169041116*^9, 3.5388296377387342`*^9, 
   3.5388318574048204`*^9, 3.5389301308125*^9, 3.5389775456770363`*^9, 
   3.5389930354967065`*^9, 3.539011395858625*^9, 3.539012267952375*^9, 
   3.539016052421125*^9, 3.541147784632642*^9, 3.5411478965530844`*^9, 
   3.5411546656087513`*^9, 3.541157081053666*^9, 3.54838634410466*^9, 
   3.548386383781105*^9, {3.55479790384375*^9, 3.554797919*^9}, 
   3.5556619281965003`*^9, 3.5562489721477313`*^9, 3.5562493075895987`*^9, 
   3.5562495517777605`*^9, 3.556276470480592*^9, 3.556276589540044*^9, 
   3.55629617740625*^9, 3.556296286140625*^9, 3.5679439656786327`*^9, 
   3.5680017709106092`*^9, 3.5680020177822943`*^9, 3.568002883725572*^9, 
   3.568003103258572*^9, 3.5680284836383896`*^9, 3.568030017357746*^9, 
   3.576315515786955*^9, 3.5764113845059476`*^9, 3.576425137088192*^9, 
   3.5764253485092845`*^9, 3.576426208839493*^9, 3.5769934973436327`*^9, 
   3.576993670341633*^9, 3.576993735596633*^9, 3.578504373539413*^9, 
   3.578591504916752*^9, 3.5786616382990007`*^9, 3.581236893704732*^9, 
   3.5812370619647074`*^9, 3.581237109289042*^9, 3.581237148407604*^9, 
   3.5812372816694565`*^9, 3.5812373487787066`*^9, 3.581351997576312*^9, 
   3.5817462148365374`*^9, 3.581746399304982*^9, 3.5817465993659863`*^9, 
   3.5817550083162003`*^9, 3.5817808898845415`*^9, 3.581826859925659*^9, 
   3.582351330298025*^9, 3.584025090110571*^9, 3.584025261283362*^9, 
   3.58402537979014*^9, 3.584025421891548*^9, 3.5840254576445932`*^9, 
   3.58537203720453*^9, 3.585372120564865*^9, 3.5853722443142385`*^9, 
   3.6124578277412834`*^9, 3.612458097292701*^9, 3.612458178955372*^9, {
   3.6124583094948378`*^9, 3.6124583151501617`*^9}, 3.613193606067544*^9, 
   3.613193649843921*^9, 3.6131936883497715`*^9, 3.613193845173189*^9, 
   3.6207123555631046`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"kinematicEquation", "//", "TableForm"}]], "Input"],

Cell[BoxData[
 TagBox[
  TagBox[GridBox[{
     {
      RowBox[{
       RowBox[{
        FractionBox["1", "2"], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"Pb", " ", "q1"}], "+", 
          RowBox[{"q2", " ", "Qb"}], "+", 
          RowBox[{"q3", " ", "Rb"}]}], ")"}]}], "+", 
       RowBox[{"q0", " ", "s"}]}]},
     {
      RowBox[{
       RowBox[{
        FractionBox["1", "2"], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{"-", "Pb"}], " ", "q0"}], "+", 
          RowBox[{"q3", " ", "Qb"}], "-", 
          RowBox[{"q2", " ", "Rb"}]}], ")"}]}], "+", 
       RowBox[{"q1", " ", "s"}]}]},
     {
      RowBox[{
       RowBox[{
        FractionBox["1", "2"], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{"-", "Pb"}], " ", "q3"}], "-", 
          RowBox[{"q0", " ", "Qb"}], "+", 
          RowBox[{"q1", " ", "Rb"}]}], ")"}]}], "+", 
       RowBox[{"q2", " ", "s"}]}]},
     {
      RowBox[{
       RowBox[{
        FractionBox["1", "2"], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"Pb", " ", "q2"}], "-", 
          RowBox[{"q1", " ", "Qb"}], "-", 
          RowBox[{"q0", " ", "Rb"}]}], ")"}]}], "+", 
       RowBox[{"q3", " ", "s"}]}]}
    },
    GridBoxAlignment->{
     "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.5599999999999999]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}],
   Column],
  Function[BoxForm`e$, 
   TableForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{
  3.51867598715625*^9, 3.51867620628125*^9, 3.51867628434375*^9, {
   3.518676395203125*^9, 3.5186764001875*^9}, 3.51867644703125*^9, {
   3.51867720834375*^9, 3.518677247109375*^9}, 3.51867729303125*^9, 
   3.51867752428125*^9, 3.51867760075*^9, 3.518677803625*^9, {
   3.518678008921875*^9, 3.518678033859375*^9}, 3.51867854303125*^9, 
   3.518680169125*^9, 3.51868042815625*^9, 3.51868061478125*^9, 
   3.51868070840625*^9, 3.51886235778125*^9, 3.518862401984375*^9, 
   3.518862807234375*^9, 3.518863331640625*^9, 3.519404825875*^9, 
   3.519404900796875*^9, 3.51941072378125*^9, 3.519442230140625*^9, 
   3.519442356109375*^9, 3.519442496734375*^9, 3.5194426225625*^9, 
   3.519442804*^9, 3.519443092296875*^9, 3.519443154765625*^9, 
   3.519443635625*^9, 3.51944369625*^9, 3.519443761140625*^9, {
   3.519444185375*^9, 3.519444205421875*^9}, 3.5194447176875*^9, 
   3.519543650953125*^9, 3.519805576875*^9, 3.519805951125*^9, 
   3.51981396025*^9, 3.519847791125*^9, 3.522042509979516*^9, 
   3.5220426638537827`*^9, 3.52204269539985*^9, 3.5220428645673943`*^9, 
   3.52207860790625*^9, 3.5318234038856177`*^9, 3.53185127084375*^9, 
   3.531851339640625*^9, 3.531851424234375*^9, 3.53185179003125*^9, 
   3.531852604515625*^9, 3.53185266796875*^9, 3.53185274071875*^9, 
   3.531852829921875*^9, 3.531852887703125*^9, 3.53185295034375*^9, 
   3.53185308778125*^9, 3.53185314234375*^9, 3.53185327971875*^9, 
   3.531853355078125*^9, 3.5318534215*^9, 3.531853472265625*^9, 
   3.531853530890625*^9, 3.531853675046875*^9, 3.531853782515625*^9, 
   3.531853994203125*^9, {3.5318541310625*^9, 3.531854158796875*^9}, 
   3.53185424571875*^9, 3.53185428484375*^9, 3.531854371046875*^9, 
   3.531854477984375*^9, 3.53185454128125*^9, 3.53185503159375*^9, 
   3.53185704328125*^9, 3.532106953359375*^9, 3.534158615859375*^9, {
   3.534167499515625*^9, 3.53416752046875*^9}, 3.53416763184375*^9, 
   3.534167817484375*^9, 3.534167895125*^9, 3.534174335890625*^9, 
   3.534218556953125*^9, 3.53421860346875*^9, 3.53421878375*^9, 
   3.5342295371875*^9, 3.534230356796875*^9, 3.534230704625*^9, 
   3.534415269859375*^9, 3.53441900803125*^9, 3.534419514078125*^9, 
   3.534437399859375*^9, 3.5344376848125*^9, 3.5344381018125*^9, 
   3.534485688796875*^9, 3.534486082609375*^9, 3.53448615021875*^9, 
   3.538214451173173*^9, 3.538220025115611*^9, 3.5382215895095167`*^9, 
   3.5382218830370092`*^9, {3.538222189751833*^9, 3.5382221951892633`*^9}, 
   3.538222505775337*^9, 3.538222874465617*^9, 3.538223233822089*^9, 
   3.538223624535838*^9, 3.53823686784375*^9, 3.538237044609375*^9, 
   3.538281760233375*^9, 3.538283344694647*^9, 3.5382841948164577`*^9, 
   3.538284308034483*^9, 3.5382850866857495`*^9, 3.5382882708646555`*^9, 
   3.5382890619591026`*^9, 3.53828994910365*^9, 3.5383951378727417`*^9, 
   3.538395396074214*^9, 3.5383955963073072`*^9, 3.538480326051247*^9, 
   3.53850342121875*^9, 3.538503688546875*^9, 3.538503780359375*^9, 
   3.5387212291416097`*^9, 3.538722695947625*^9, 3.538803565339613*^9, 
   3.5388059198552113`*^9, 3.5388061690879908`*^9, 3.5388296377699842`*^9, 
   3.5388318574204454`*^9, 3.53893013140625*^9, 3.5389775457082863`*^9, 
   3.5389930356685815`*^9, 3.539011396577375*^9, 3.539012268077375*^9, 
   3.539016052514875*^9, 3.541147784773265*^9, 3.5411478965687094`*^9, 
   3.541154665640001*^9, 3.5411570810692945`*^9, 3.5483863441359134`*^9, 
   3.548386383796732*^9, {3.554797903875*^9, 3.55479791903125*^9}, 
   3.5556619282902503`*^9, 3.556248972194601*^9, 3.556249307620822*^9, 
   3.556249551824616*^9, 3.5562764705118413`*^9, 3.556276589571293*^9, 
   3.55629617746875*^9, 3.556296286171875*^9, 3.5679439657016335`*^9, 
   3.5680017709426126`*^9, 3.568002017848301*^9, 3.5680028837705717`*^9, 
   3.568003103291572*^9, 3.5680284836663923`*^9, 3.568030017383749*^9, 
   3.5763155158169584`*^9, 3.576411384531949*^9, 3.5764251371171937`*^9, 
   3.5764253486652937`*^9, 3.5764262088644943`*^9, 3.576993497373633*^9, 
   3.5769936703716326`*^9, 3.5769937356266327`*^9, 3.5785043735824156`*^9, 
   3.5785915049417534`*^9, 3.578661638327002*^9, 3.581236893730737*^9, 
   3.5812370621347246`*^9, 3.581237109320244*^9, 3.581237148438806*^9, 
   3.581237281700659*^9, 3.581237348809908*^9, 3.5813519975993133`*^9, 
   3.5817462148625402`*^9, 3.5817463993289847`*^9, 3.581746599466996*^9, 
   3.5817550083422017`*^9, 3.581780889911543*^9, 3.581826859953659*^9, 
   3.5823513303550196`*^9, 3.5840250901385727`*^9, 3.5840252613103633`*^9, 
   3.584025379825142*^9, 3.58402542192455*^9, 3.5840254576765947`*^9, 
   3.585372037238533*^9, 3.585372121203929*^9, 3.585372244356243*^9, 
   3.612457827766285*^9, 3.612458097319702*^9, 3.612458178983373*^9, {
   3.61245830952484*^9, 3.612458315179163*^9}, 3.613193606098547*^9, 
   3.6131936498689237`*^9, 3.613193688376774*^9, 3.6131938452021894`*^9, 
   3.620712355597108*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"navigationEquation", "//", "TableForm"}]], "Input"],

Cell[BoxData[
 TagBox[
  TagBox[GridBox[{
     {
      RowBox[{
       RowBox[{
        RowBox[{"-", 
         RowBox[{"(", 
          RowBox[{
           SuperscriptBox["q0", "2"], "+", 
           SuperscriptBox["q1", "2"], "-", 
           SuperscriptBox["q2", "2"], "-", 
           SuperscriptBox["q3", "2"]}], ")"}]}], " ", "Ub"}], "-", 
       RowBox[{"2", " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"q1", " ", "q2"}], "-", 
          RowBox[{"q0", " ", "q3"}]}], ")"}], " ", "Vb"}], "-", 
       RowBox[{"2", " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"q0", " ", "q2"}], "+", 
          RowBox[{"q1", " ", "q3"}]}], ")"}], " ", "Wb"}], "+", 
       RowBox[{"s", " ", "xcg"}]}]},
     {
      RowBox[{
       RowBox[{
        RowBox[{"-", "2"}], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"q1", " ", "q2"}], "+", 
          RowBox[{"q0", " ", "q3"}]}], ")"}], " ", "Ub"}], "-", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox["q0", "2"], "-", 
          SuperscriptBox["q1", "2"], "+", 
          SuperscriptBox["q2", "2"], "-", 
          SuperscriptBox["q3", "2"]}], ")"}], " ", "Vb"}], "-", 
       RowBox[{"2", " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{"-", "q0"}], " ", "q1"}], "+", 
          RowBox[{"q2", " ", "q3"}]}], ")"}], " ", "Wb"}], "+", 
       RowBox[{"s", " ", "ycg"}]}]},
     {
      RowBox[{
       RowBox[{
        RowBox[{"-", "2"}], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{"-", "q0"}], " ", "q2"}], "+", 
          RowBox[{"q1", " ", "q3"}]}], ")"}], " ", "Ub"}], "-", 
       RowBox[{"2", " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"q0", " ", "q1"}], "+", 
          RowBox[{"q2", " ", "q3"}]}], ")"}], " ", "Vb"}], "-", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox["q0", "2"], "-", 
          SuperscriptBox["q1", "2"], "-", 
          SuperscriptBox["q2", "2"], "+", 
          SuperscriptBox["q3", "2"]}], ")"}], " ", "Wb"}], "+", 
       RowBox[{"s", " ", "zcg"}]}]}
    },
    GridBoxAlignment->{
     "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.5599999999999999]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}],
   Column],
  Function[BoxForm`e$, 
   TableForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{
  3.5186759871875*^9, 3.518676206328125*^9, 3.518676284359375*^9, {
   3.51867639521875*^9, 3.518676400203125*^9}, 3.5186764470625*^9, {
   3.518677208359375*^9, 3.518677247125*^9}, 3.518677293046875*^9, 
   3.5186775243125*^9, 3.51867760078125*^9, 3.51867780365625*^9, {
   3.518678008953125*^9, 3.518678033890625*^9}, 3.518678543078125*^9, 
   3.51868016915625*^9, 3.5186804281875*^9, 3.5186806148125*^9, 
   3.5186807084375*^9, 3.518862357796875*^9, 3.518862402015625*^9, 
   3.518862807265625*^9, 3.518863331671875*^9, 3.51940482590625*^9, 
   3.51940490084375*^9, 3.5194107238125*^9, 3.519442230171875*^9, 
   3.51944235615625*^9, 3.51944249678125*^9, 3.519442622609375*^9, 
   3.51944280403125*^9, 3.519443092328125*^9, 3.51944315478125*^9, 
   3.519443635640625*^9, 3.51944369628125*^9, 3.51944376115625*^9, {
   3.51944418540625*^9, 3.51944420546875*^9}, 3.51944471771875*^9, 
   3.519543650984375*^9, 3.519805576890625*^9, 3.519805951140625*^9, 
   3.519813960265625*^9, 3.519847791171875*^9, 3.5220425100107656`*^9, 
   3.5220426638850317`*^9, 3.5220426954154744`*^9, 3.5220428646142683`*^9, 
   3.522078607953125*^9, 3.5318234039324923`*^9, 3.531851270859375*^9, 
   3.53185133965625*^9, 3.53185142425*^9, 3.531851790046875*^9, 
   3.531852604703125*^9, 3.531852667984375*^9, 3.531852740734375*^9, 
   3.5318528299375*^9, 3.53185288771875*^9, 3.531852950359375*^9, 
   3.531853087796875*^9, 3.531853142359375*^9, 3.531853279734375*^9, 
   3.53185335509375*^9, 3.531853421515625*^9, 3.53185347228125*^9, 
   3.53185353090625*^9, 3.5318536750625*^9, 3.53185378253125*^9, 
   3.53185399421875*^9, {3.531854131078125*^9, 3.5318541588125*^9}, 
   3.531854245734375*^9, 3.531854284875*^9, 3.531854371078125*^9, 
   3.531854478*^9, 3.531854541296875*^9, 3.531855031609375*^9, 
   3.531857043296875*^9, 3.5321069534375*^9, 3.534158615953125*^9, {
   3.534167499609375*^9, 3.5341675205625*^9}, 3.534167631953125*^9, 
   3.534167817578125*^9, 3.53416789521875*^9, 3.53417433596875*^9, 
   3.53421855703125*^9, 3.5342186035625*^9, 3.5342187838125*^9, 
   3.53422953728125*^9, 3.534230356875*^9, 3.53423070471875*^9, 
   3.534415269953125*^9, 3.534419008109375*^9, 3.53441951415625*^9, 
   3.5344374*^9, 3.534437684921875*^9, 3.534438101921875*^9, 
   3.534485688875*^9, 3.5344860826875*^9, 3.53448615028125*^9, 
   3.538214451204423*^9, 3.53822002514686*^9, 3.538221589540766*^9, 
   3.5382218830526342`*^9, {3.538222189767458*^9, 3.5382221952205133`*^9}, 
   3.538222505790962*^9, 3.5382228744968667`*^9, 3.5382232338533387`*^9, 
   3.538223624551463*^9, 3.5382368679375*^9, 3.5382370446875*^9, 
   3.538281760249*^9, 3.538283344710273*^9, 3.5382841948320827`*^9, 
   3.538284308050108*^9, 3.5382850867013745`*^9, 3.5382882708802805`*^9, 
   3.5382890620059776`*^9, 3.538289949119275*^9, 3.5383951378883667`*^9, 
   3.538395396089839*^9, 3.5383955963229322`*^9, 3.5384803260824757`*^9, 
   3.53850342125*^9, 3.538503688578125*^9, 3.538503780390625*^9, 
   3.53872122917286*^9, 3.53872269596325*^9, 3.538803565370864*^9, 
   3.5388059198708363`*^9, 3.5388061691192408`*^9, 3.5388296377856092`*^9, 
   3.538831857436071*^9, 3.538930131515625*^9, 3.5389775457239113`*^9, 
   3.5389930357467065`*^9, 3.539011396671125*^9, 3.539012268171125*^9, 
   3.539016052639875*^9, 3.541147784804515*^9, 3.541147896584334*^9, 
   3.5411546656556253`*^9, 3.5411570811005507`*^9, 3.5483863441671667`*^9, 
   3.5483863838279853`*^9, {3.55479790390625*^9, 3.5547979190625*^9}, 
   3.5556619283215003`*^9, 3.5562489722414713`*^9, 3.5562493076520452`*^9, 
   3.556249551855853*^9, 3.556276470558715*^9, 3.556276589758788*^9, 
   3.55629617753125*^9, 3.556296286203125*^9, 3.567943965726635*^9, 
   3.5680017709786167`*^9, 3.5680020179183083`*^9, 3.568002883817572*^9, 
   3.568003103320572*^9, 3.5680284836973953`*^9, 3.5680300174107513`*^9, 
   3.5763155158479614`*^9, 3.5764113845639505`*^9, 3.5764251371441956`*^9, 
   3.5764253486982956`*^9, 3.576426208889496*^9, 3.576993497403633*^9, 
   3.576993670401633*^9, 3.576993735655633*^9, 3.578504373623418*^9, 
   3.578591504975755*^9, 3.5786616383550034`*^9, 3.581236893755742*^9, 
   3.58123706219073*^9, 3.581237109445052*^9, 3.581237148454407*^9, 
   3.58123728171626*^9, 3.581237348809908*^9, 3.5813519976243143`*^9, 
   3.5817462148875427`*^9, 3.581746399354987*^9, 3.5817465994969997`*^9, 
   3.5817550083712034`*^9, 3.5817808899425445`*^9, 3.581826859984659*^9, 
   3.5823513304040146`*^9, 3.584025090172575*^9, 3.584025261339365*^9, 
   3.5840253798671446`*^9, 3.584025421960552*^9, 3.5840254577085967`*^9, 
   3.5853720372895384`*^9, 3.585372121234932*^9, 3.585372244403248*^9, 
   3.6124578277892857`*^9, 3.612458097347704*^9, 3.6124581790113745`*^9, {
   3.6124583095538416`*^9, 3.6124583152081647`*^9}, 3.6131936061305504`*^9, 
   3.6131936498939257`*^9, 3.6131936884017763`*^9, 3.6131938452301893`*^9, 
   3.6207123556601143`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["The assembled set of equations", "Subsubsection"],

Cell[BoxData[
 RowBox[{"systemEquationsDa", ":=", 
  RowBox[{"Flatten", "[", 
   RowBox[{"{", 
    RowBox[{
    "forceEquation", ",", "momentEquation", ",", "kinematicEquation", ",", 
     "navigationEquation"}], "}"}], "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"boundaryEquations", "=", 
   RowBox[{"{", "}"}]}], ";"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"systemEquationsDa", "//", "TableForm"}]], "Input"],

Cell[BoxData[
 TagBox[
  TagBox[GridBox[{
     {
      RowBox[{
       RowBox[{"-", 
        FractionBox["Fx", "mass"]}], "+", 
       RowBox[{"s", " ", "Ub"}], "-", 
       RowBox[{"Rb", " ", "Vb"}], "+", 
       RowBox[{"Qb", " ", "Wb"}], "-", 
       RowBox[{"2", " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{"-", "q0"}], " ", "q2"}], "+", 
          RowBox[{"q1", " ", "q3"}]}], ")"}], " ", 
        RowBox[{"(", 
         RowBox[{"g0", "-", 
          FractionBox[
           RowBox[{"kground", " ", "zcg", " ", 
            RowBox[{"onPositive", "[", "zcg", "]"}]}], "mass"]}], ")"}]}]}]},
     {
      RowBox[{
       RowBox[{"-", 
        FractionBox["Fy", "mass"]}], "+", 
       RowBox[{"Rb", " ", "Ub"}], "+", 
       RowBox[{"s", " ", "Vb"}], "-", 
       RowBox[{"Pb", " ", "Wb"}], "-", 
       RowBox[{"2", " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"q0", " ", "q1"}], "+", 
          RowBox[{"q2", " ", "q3"}]}], ")"}], " ", 
        RowBox[{"(", 
         RowBox[{"g0", "-", 
          FractionBox[
           RowBox[{"kground", " ", "zcg", " ", 
            RowBox[{"onPositive", "[", "zcg", "]"}]}], "mass"]}], ")"}]}]}]},
     {
      RowBox[{
       RowBox[{"-", 
        FractionBox["Fz", "mass"]}], "-", 
       RowBox[{"Qb", " ", "Ub"}], "+", 
       RowBox[{"Pb", " ", "Vb"}], "+", 
       RowBox[{"s", " ", "Wb"}], "-", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox["q0", "2"], "-", 
          SuperscriptBox["q1", "2"], "-", 
          SuperscriptBox["q2", "2"], "+", 
          SuperscriptBox["q3", "2"]}], ")"}], " ", 
        RowBox[{"(", 
         RowBox[{"g0", "-", 
          FractionBox[
           RowBox[{"kground", " ", "zcg", " ", 
            RowBox[{"onPositive", "[", "zcg", "]"}]}], "mass"]}], ")"}]}]}]},
     {
      FractionBox[
       RowBox[{
        RowBox[{
         RowBox[{"-", 
          SuperscriptBox["Iz", "2"]}], " ", "Qb", " ", "Rb"}], "+", 
        RowBox[{"Iz", " ", 
         RowBox[{"(", 
          RowBox[{"Lb", "+", 
           RowBox[{"Ixz", " ", "Pb", " ", "Qb"}], "+", 
           RowBox[{"Iy", " ", "Qb", " ", "Rb"}], "-", 
           RowBox[{"Ix", " ", "Pb", " ", "s"}]}], ")"}]}], "+", 
        RowBox[{"Ixz", " ", 
         RowBox[{"(", 
          RowBox[{"Nb", "+", 
           RowBox[{"Ix", " ", "Pb", " ", "Qb"}], "-", 
           RowBox[{"Iy", " ", "Pb", " ", "Qb"}], "-", 
           RowBox[{"Ixz", " ", "Qb", " ", "Rb"}], "+", 
           RowBox[{"Ixz", " ", "Pb", " ", "s"}]}], ")"}]}]}], 
       RowBox[{
        SuperscriptBox["Ixz", "2"], "-", 
        RowBox[{"Ix", " ", "Iz"}]}]]},
     {
      FractionBox[
       RowBox[{
        RowBox[{"-", "Mb"}], "+", 
        RowBox[{"Ix", " ", "Pb", " ", "Rb"}], "-", 
        RowBox[{"Iz", " ", "Pb", " ", "Rb"}], "+", 
        RowBox[{"Ixz", " ", 
         RowBox[{"(", 
          RowBox[{
           SuperscriptBox["Pb", "2"], "-", 
           SuperscriptBox["Rb", "2"]}], ")"}]}], "+", 
        RowBox[{"Iy", " ", "Qb", " ", "s"}]}], "Iy"]},
     {
      FractionBox[
       RowBox[{
        RowBox[{"Ixz", " ", 
         RowBox[{"(", 
          RowBox[{"Lb", "-", 
           RowBox[{
            RowBox[{"(", 
             RowBox[{"Ix", "-", "Iy", "+", "Iz"}], ")"}], " ", "Qb", " ", 
            "Rb"}]}], ")"}]}], "+", 
        RowBox[{
         SuperscriptBox["Ixz", "2"], " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"Pb", " ", "Qb"}], "+", 
           RowBox[{"Rb", " ", "s"}]}], ")"}]}], "+", 
        RowBox[{"Ix", " ", 
         RowBox[{"(", 
          RowBox[{"Nb", "+", 
           RowBox[{"Ix", " ", "Pb", " ", "Qb"}], "-", 
           RowBox[{"Iy", " ", "Pb", " ", "Qb"}], "-", 
           RowBox[{"Iz", " ", "Rb", " ", "s"}]}], ")"}]}]}], 
       RowBox[{
        SuperscriptBox["Ixz", "2"], "-", 
        RowBox[{"Ix", " ", "Iz"}]}]]},
     {
      RowBox[{
       RowBox[{
        FractionBox["1", "2"], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"Pb", " ", "q1"}], "+", 
          RowBox[{"q2", " ", "Qb"}], "+", 
          RowBox[{"q3", " ", "Rb"}]}], ")"}]}], "+", 
       RowBox[{"q0", " ", "s"}]}]},
     {
      RowBox[{
       RowBox[{
        FractionBox["1", "2"], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{"-", "Pb"}], " ", "q0"}], "+", 
          RowBox[{"q3", " ", "Qb"}], "-", 
          RowBox[{"q2", " ", "Rb"}]}], ")"}]}], "+", 
       RowBox[{"q1", " ", "s"}]}]},
     {
      RowBox[{
       RowBox[{
        FractionBox["1", "2"], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{"-", "Pb"}], " ", "q3"}], "-", 
          RowBox[{"q0", " ", "Qb"}], "+", 
          RowBox[{"q1", " ", "Rb"}]}], ")"}]}], "+", 
       RowBox[{"q2", " ", "s"}]}]},
     {
      RowBox[{
       RowBox[{
        FractionBox["1", "2"], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"Pb", " ", "q2"}], "-", 
          RowBox[{"q1", " ", "Qb"}], "-", 
          RowBox[{"q0", " ", "Rb"}]}], ")"}]}], "+", 
       RowBox[{"q3", " ", "s"}]}]},
     {
      RowBox[{
       RowBox[{
        RowBox[{"-", 
         RowBox[{"(", 
          RowBox[{
           SuperscriptBox["q0", "2"], "+", 
           SuperscriptBox["q1", "2"], "-", 
           SuperscriptBox["q2", "2"], "-", 
           SuperscriptBox["q3", "2"]}], ")"}]}], " ", "Ub"}], "-", 
       RowBox[{"2", " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"q1", " ", "q2"}], "-", 
          RowBox[{"q0", " ", "q3"}]}], ")"}], " ", "Vb"}], "-", 
       RowBox[{"2", " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"q0", " ", "q2"}], "+", 
          RowBox[{"q1", " ", "q3"}]}], ")"}], " ", "Wb"}], "+", 
       RowBox[{"s", " ", "xcg"}]}]},
     {
      RowBox[{
       RowBox[{
        RowBox[{"-", "2"}], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"q1", " ", "q2"}], "+", 
          RowBox[{"q0", " ", "q3"}]}], ")"}], " ", "Ub"}], "-", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox["q0", "2"], "-", 
          SuperscriptBox["q1", "2"], "+", 
          SuperscriptBox["q2", "2"], "-", 
          SuperscriptBox["q3", "2"]}], ")"}], " ", "Vb"}], "-", 
       RowBox[{"2", " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{"-", "q0"}], " ", "q1"}], "+", 
          RowBox[{"q2", " ", "q3"}]}], ")"}], " ", "Wb"}], "+", 
       RowBox[{"s", " ", "ycg"}]}]},
     {
      RowBox[{
       RowBox[{
        RowBox[{"-", "2"}], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{"-", "q0"}], " ", "q2"}], "+", 
          RowBox[{"q1", " ", "q3"}]}], ")"}], " ", "Ub"}], "-", 
       RowBox[{"2", " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"q0", " ", "q1"}], "+", 
          RowBox[{"q2", " ", "q3"}]}], ")"}], " ", "Vb"}], "-", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox["q0", "2"], "-", 
          SuperscriptBox["q1", "2"], "-", 
          SuperscriptBox["q2", "2"], "+", 
          SuperscriptBox["q3", "2"]}], ")"}], " ", "Wb"}], "+", 
       RowBox[{"s", " ", "zcg"}]}]}
    },
    GridBoxAlignment->{
     "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.5599999999999999]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}],
   Column],
  Function[BoxForm`e$, 
   TableForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{
  3.518675987234375*^9, 3.518676206390625*^9, 3.518676284375*^9, {
   3.518676395265625*^9, 3.51867640025*^9}, 3.518676447109375*^9, {
   3.51867720840625*^9, 3.51867724715625*^9}, 3.51867729309375*^9, 
   3.518677524359375*^9, 3.518677600828125*^9, 3.518677803703125*^9, {
   3.518678008984375*^9, 3.518678033921875*^9}, 3.51867854315625*^9, 
   3.51868016921875*^9, 3.518680428296875*^9, 3.518680614859375*^9, 
   3.518680708484375*^9, 3.51886235784375*^9, 3.5188624020625*^9, 
   3.518862807296875*^9, 3.51886333171875*^9, 3.519404825953125*^9, 
   3.51940490090625*^9, 3.519410723859375*^9, 3.519442230234375*^9, 
   3.519442356203125*^9, 3.519442496859375*^9, 3.519442622671875*^9, 
   3.5194428040625*^9, 3.519443092375*^9, 3.519443154828125*^9, 
   3.5194436356875*^9, 3.519443696328125*^9, 3.519443761203125*^9, {
   3.519444185484375*^9, 3.51944420553125*^9}, 3.519444717765625*^9, 
   3.519543651046875*^9, 3.5198055769375*^9, 3.519805951171875*^9, 
   3.5198139603125*^9, 3.51984779125*^9, 3.5220425100420156`*^9, 
   3.522042663963155*^9, 3.5220426954623485`*^9, 3.5220428646767664`*^9, 
   3.522078608*^9, 3.5318234040574913`*^9, 3.531851270890625*^9, 
   3.53185133971875*^9, 3.53185142428125*^9, 3.531851790078125*^9, 
   3.53185260475*^9, 3.5318526680625*^9, 3.531852740765625*^9, 
   3.53185282996875*^9, 3.53185288775*^9, 3.531852950390625*^9, 
   3.531853087828125*^9, 3.531853142390625*^9, 3.531853279765625*^9, 
   3.531853355125*^9, 3.531853421546875*^9, 3.531853472296875*^9, 
   3.5318535309375*^9, 3.53185367509375*^9, 3.5318537825625*^9, 
   3.53185399425*^9, {3.531854131109375*^9, 3.53185415884375*^9}, 
   3.531854245765625*^9, 3.53185428490625*^9, 3.53185437109375*^9, 
   3.53185447803125*^9, 3.53185454134375*^9, 3.531855031640625*^9, 
   3.53185704334375*^9, 3.53210695359375*^9, 3.534158616140625*^9, {
   3.53416749978125*^9, 3.5341675206875*^9}, 3.534167632140625*^9, 
   3.534167817703125*^9, 3.534167895359375*^9, 3.53417433609375*^9, 
   3.534218557171875*^9, 3.53421860371875*^9, 3.534218783953125*^9, 
   3.534229537484375*^9, 3.53423035703125*^9, 3.53423070490625*^9, 
   3.53441527021875*^9, 3.53441900825*^9, 3.534419514296875*^9, 
   3.534437400296875*^9, 3.534437685125*^9, 3.53443810209375*^9, 
   3.534485689015625*^9, 3.5344860828125*^9, 3.534486150421875*^9, 
   3.538214451251298*^9, 3.5382200252093596`*^9, 3.5382215896657643`*^9, 
   3.5382218830838833`*^9, {3.5382221897987075`*^9, 3.5382221952673874`*^9}, 
   3.5382225058222136`*^9, 3.5382228745281167`*^9, 3.5382232338845882`*^9, 
   3.538223624582712*^9, 3.53823686809375*^9, 3.5382370448125*^9, 
   3.538281760295875*^9, 3.5382833447415247`*^9, 3.5382841948633327`*^9, 
   3.538284308096983*^9, 3.538285086732624*^9, 3.5382882709115305`*^9, 
   3.538289062037227*^9, 3.538289949259899*^9, 3.5383951379508657`*^9, 
   3.538395396121089*^9, 3.5383955963698072`*^9, 3.5384803261137047`*^9, 
   3.5385034213125*^9, 3.538503688640625*^9, 3.538503780421875*^9, 
   3.538721229219737*^9, 3.538722696010125*^9, 3.538803565402114*^9, 
   3.5388059199020853`*^9, 3.538806169353614*^9, 3.5388296378168592`*^9, 
   3.5388318574829464`*^9, 3.53893013165625*^9, 3.5389775457707863`*^9, 
   3.5389930358717065`*^9, 3.53901139687425*^9, 3.539012268452375*^9, 
   3.539016053514875*^9, 3.541147784851389*^9, 3.5411478966155834`*^9, 
   3.5411546657024994`*^9, 3.5411570812568316`*^9, 3.5483863442765546`*^9, 
   3.5483863838748655`*^9, {3.554797903953125*^9, 3.55479791909375*^9}, 
   3.5556619283840003`*^9, 3.5562489723352118`*^9, 3.55624930769888*^9, 
   3.556249551902708*^9, 3.556276470605589*^9, 3.556276589805662*^9, 
   3.556296177796875*^9, 3.55629628625*^9, 3.567943965788639*^9, 
   3.568001771043623*^9, 3.56800201804432*^9, 3.568002883901572*^9, 
   3.568003103389572*^9, 3.568028483861412*^9, 3.5680300175757685`*^9, 
   3.5763155158989663`*^9, 3.576411384619954*^9, 3.5764251371961985`*^9, 
   3.5764253487532988`*^9, 3.576426208932498*^9, 3.5769934974546328`*^9, 
   3.576993670450633*^9, 3.5769937357046328`*^9, 3.578504373678421*^9, 
   3.5785915050277576`*^9, 3.578661638402006*^9, 3.5812368938047523`*^9, 
   3.5812370622467356`*^9, 3.581237109523057*^9, 3.58123714850121*^9, 
   3.5812372817630625`*^9, 3.5812373488567095`*^9, 3.5813519976773176`*^9, 
   3.5817462149305468`*^9, 3.581746399399992*^9, 3.581746599543004*^9, 
   3.581755008420206*^9, 3.5817808899955473`*^9, 3.5818268600326595`*^9, 
   3.5823513305250025`*^9, 3.5840250902225776`*^9, 3.5840252613883677`*^9, 
   3.584025379927148*^9, 3.584025422012555*^9, 3.5840254577676*^9, 
   3.585372037383548*^9, 3.5853721222880373`*^9, 3.5853722444932566`*^9, 
   3.6124578278392887`*^9, 3.612458097402707*^9, 3.612458179067378*^9, {
   3.6124583096078444`*^9, 3.612458315264168*^9}, 3.6131936061815553`*^9, 
   3.61319364993793*^9, 3.613193688447781*^9, 3.6131938452751894`*^9, 
   3.620712355739122*^9}]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["The Forces and Moments Acting on the Aircraft", "Subsection"],

Cell["\<\
The forces and moments acting on the aircraft are aerodynamic forces and \
moments and forces and moments from the engine\
\>", "Text"],

Cell["\<\
aircraft`F := aircraft`aero`F + engine`F + ground`F;
aircraft`T := aircraft`aero`T + engine`T;\
\>", "Input",
 PageWidth->WindowWidth,
 CellChangeTimes->{{3.460037191578125*^9, 3.46003721184375*^9}, {
  3.53924679321875*^9, 3.539246810921875*^9}}],

Cell["torque on x-axis", "Text"],

Cell["Lb := aircraft`T[[1]][[1]]", "Text",
 CellChangeTimes->{{3.519444644671875*^9, 3.519444679109375*^9}, 
   3.5220428413961124`*^9}],

Cell["\<\
The aerodynamic forces acting on the airplane (expressed in wind coordinates) \
can be expressed as:\
\>", "Text",
 Evaluatable->False],

Cell[BoxData[
 RowBox[{"aircraft`aero`Fw", ":=", 
  RowBox[{"wing1`aero`F", "+", "wing2`aero`F", "+", "fin`aero`F"}]}]], "Input",\

 CellChangeTimes->{3.615039368654091*^9}],

Cell["Transformed into body coordinates", "Text"],

Cell[BoxData[
 RowBox[{"aircraft`aero`F", ":=", 
  RowBox[{
   SubscriptBox["L", "BW"], ".", "aircraft`aero`Fw"}]}]], "Input"],

Cell["\<\
where the transformation matrix from wind to body coordinates is:\
\>", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["L", "BW"], " ", "=", " ", 
   RowBox[{
    RowBox[{
     SubscriptBox["L", "y"], "[", "\[Alpha]", "]"}], ".", 
    RowBox[{
     SubscriptBox["L", "z"], "[", 
     RowBox[{"-", "\[Beta]"}], "]"}]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{"aircraft`aero`Tw", ":=", 
  RowBox[{"wing1`aero`T", "+", "wing2`aero`T", "+", "fin`aero`T"}]}]], "Input",\

 CellChangeTimes->{3.568149962321068*^9, 3.6150393730463424`*^9}],

Cell["Moments in body coordinates", "Text"],

Cell[BoxData[
 RowBox[{"aircraft`aero`T", ":=", 
  RowBox[{
   SubscriptBox["L", "BW"], ".", "aircraft`aero`Tw"}]}]], "Input",
 CellChangeTimes->{{3.568149971406587*^9, 3.5681499832482643`*^9}}],

Cell[BoxData[
 RowBox[{"aircraft`aero`T", ":=", "aircraft`aero`Tw"}]], "Input",
 CellChangeTimes->{3.5764109484100046`*^9}],

Cell[CellGroupData[{

Cell["Wing", "Subsubsection",
 FontSlant->"Italic"],

Cell["The contributions from the wing are", "Text"],

Cell[BoxData[
 RowBox[{"wing1`aero`F", ":=", 
  RowBox[{"(", GridBox[{
     {
      RowBox[{
       RowBox[{"-", "Dragl1"}], "-", "Dragr1"}]},
     {"0"},
     {
      RowBox[{
       RowBox[{"-", "Liftl1"}], "-", "Liftr1"}]}
    }], ")"}]}]], "Input",
 CellChangeTimes->{{3.5344253985625*^9, 3.534425399078125*^9}, {
  3.534425447328125*^9, 3.534425532*^9}, {3.53448127696875*^9, 
  3.5344812775625*^9}, {3.53448137740625*^9, 3.53448137828125*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"wing1`aero`T", ":=", 
   RowBox[{"(", GridBox[{
      {
       RowBox[{
        RowBox[{
         RowBox[{"-", 
          SubscriptBox["M", 
           OverscriptBox[
            RowBox[{"\[EmptySet]", " "}], "."]]}], " ", "Pbe"}], "+", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{"(", 
             RowBox[{"Liftl1", "-", "Liftr1"}], ")"}], 
            RowBox[{"Cos", "[", "\[Alpha]", "]"}]}], "-", 
           RowBox[{
            RowBox[{"(", 
             RowBox[{"Dragl1", "-", "Dragr1"}], ")"}], " ", 
            RowBox[{"Sin", "[", "\[Alpha]", "]"}]}]}], ")"}], "rc2", " ", 
         "rc1"}]}]},
      {
       RowBox[{
        RowBox[{
         RowBox[{"(", 
          RowBox[{"xbcg", "-", "xw1"}], ")"}], " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{"(", 
             RowBox[{"Liftl1", "+", "Liftr1"}], ")"}], " ", 
            RowBox[{"Cos", "[", "\[Alpha]", "]"}]}], "+", 
           RowBox[{
            RowBox[{"(", 
             RowBox[{"Dragl1", "+", "Dragr1"}], ")"}], " ", 
            RowBox[{"Sin", "[", "\[Alpha]", "]"}]}]}], ")"}]}], "+", 
        "Moment1"}]},
      {
       RowBox[{"rc1", " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             RowBox[{"-", "Dragl1"}], "+", "Dragr1"}], ")"}], " ", 
           RowBox[{"Cos", "[", "\[Alpha]", "]"}]}], "+", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             RowBox[{"-", "Liftl1"}], "+", "Liftr1"}], ")"}], " ", 
           RowBox[{"Sin", "[", "\[Alpha]", "]"}]}]}], ")"}]}]}
     }], ")"}]}], "\[IndentingNewLine]"}]], "Input",
 CellChangeTimes->{{3.51944410128125*^9, 3.519444135703125*^9}, 
   3.5220455159009824`*^9, {3.534425316421875*^9, 3.53442532525*^9}, {
   3.53442554415625*^9, 3.53442558225*^9}, {3.5344808701875*^9, 
   3.53448089928125*^9}, 3.53448096390625*^9, {3.5344812799375*^9, 
   3.534481282453125*^9}, {3.534481379125*^9, 3.534481381859375*^9}, 
   3.539011217671125*^9, 3.539012056608625*^9, {3.576410891443746*^9, 
   3.5764109043904867`*^9}}],

Cell["\<\
Calculating the rudder hinge forces and aerodynamic stiffness\
\>", "Text"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"kal1", ":=", 
   RowBox[{"CLde1", " ", "S1", " ", "q", " ", "lc1"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"kar1", ":=", 
   RowBox[{"CLde1", " ", "S1", " ", "q", " ", "lc1"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"kal12", ":=", 
   RowBox[{"CLde12", " ", "S1", " ", "q", " ", "lc12"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"kar12", ":=", 
   RowBox[{"CLde12", " ", "S1", " ", "q", " ", "lc12"}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.5220785874375*^9, 3.522078589*^9}, {
   3.523893724671875*^9, 3.523893728484375*^9}, {3.534157901265625*^9, 
   3.534157914703125*^9}, {3.5341670091875*^9, 3.534167082234375*^9}, 
   3.534481283640625*^9, 3.534481383203125*^9, 3.534481487984375*^9, 
   3.534481615125*^9, 3.534481769640625*^9, 3.5344818101875*^9, {
   3.538289666760149*^9, 3.538289696760149*^9}, {3.5764261576225634`*^9, 
   3.5764261628508625`*^9}}],

Cell["Rudder forces (wave variables for connection nodes)", "Text",
 CellChangeTimes->{{3.4468796269280376`*^9, 3.4468796487544866`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"exprW1", "=", 
   RowBox[{"{", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"cxal1", ",", 
       RowBox[{"kal1", " ", 
        RowBox[{"(", 
         RowBox[{"del1", "+", "\[Alpha]"}], ")"}]}]}], "}"}], " ", ",", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"cxar1", ",", 
       RowBox[{"kar1", 
        RowBox[{"(", 
         RowBox[{"der1", "+", "\[Alpha]"}], ")"}]}]}], "}"}], " ", ",", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"cxal12", ",", 
       RowBox[{"kal12", " ", 
        RowBox[{"(", 
         RowBox[{"del12", "+", "\[Alpha]"}], ")"}]}]}], "}"}], " ", ",", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"cxar12", ",", 
       RowBox[{"kar12", 
        RowBox[{"(", 
         RowBox[{"der12", "+", "\[Alpha]"}], ")"}]}]}], "}"}]}], "}"}]}], 
  ";"}]], "Input",
 CellChangeTimes->{{3.534157932328125*^9, 3.534157956078125*^9}, {
  3.5341668865625*^9, 3.534166913015625*^9}, {3.5341670903125*^9, 
  3.534167112046875*^9}, {3.534481284171875*^9, 3.5344812855625*^9}, {
  3.534481384578125*^9, 3.5344813856875*^9}, {3.534481489234375*^9, 
  3.53448149259375*^9}, {3.534481615953125*^9, 3.534481617453125*^9}, {
  3.534481770609375*^9, 3.53448177215625*^9}, {3.534481811328125*^9, 
  3.5344818129375*^9}, {3.5764250883074017`*^9, 3.576425091752599*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Horizontal tail", "Subsubsection",
 FontSlant->"Italic"],

Cell["The contributions from the horizontal tail are", "Text"],

Cell["Forces", "Text"],

Cell[BoxData[
 RowBox[{"wing2`aero`F", ":=", 
  RowBox[{"(", GridBox[{
     {
      RowBox[{
       RowBox[{"-", "Dragl2"}], "-", "Dragr2"}]},
     {
      RowBox[{"-", "Cy2"}]},
     {
      RowBox[{
       RowBox[{"-", "Liftl2"}], "-", "Liftr2"}]}
    }], ")"}]}]], "Input",
 CellChangeTimes->{{3.534425643453125*^9, 3.534425665515625*^9}, {
  3.534481494140625*^9, 3.53448149671875*^9}, {3.53448161915625*^9, 
  3.534481620296875*^9}, {3.613191467483512*^9, 3.6131915180675116`*^9}, {
  3.613234750298771*^9, 3.6132347530179267`*^9}, {3.6132349392335777`*^9, 
  3.61323494329381*^9}}],

Cell["Moments", "Text"],

Cell[BoxData[
 RowBox[{"wing2`aero`T", ":=", 
  RowBox[{"(", GridBox[{
     {
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"(", 
           RowBox[{"Liftl2", "-", "Liftr2"}], ")"}], " ", 
          RowBox[{"Cos", "[", "\[Alpha]", "]"}]}], "-", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{"Dragl2", "-", "Dragr2"}], ")"}], " ", 
          RowBox[{"Sin", "[", "\[Alpha]", "]"}]}]}], ")"}], "rc2"}]},
     {
      RowBox[{
       RowBox[{
        RowBox[{"-", 
         SubscriptBox["M", 
          OverscriptBox["\[Theta]", 
           RowBox[{".", " "}]]]}], " ", "Qbe"}], "+", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{"(", 
            RowBox[{"Liftl2", "+", "Liftr2"}], ")"}], " ", 
           RowBox[{"Cos", "[", "\[Alpha]", "]"}]}], "-", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{"Dragl2", "+", "Dragr2"}], ")"}], " ", 
           RowBox[{"Sin", "[", "\[Alpha]", "]"}]}], "-", "Cl2", "-", "Cr2"}], 
         ")"}], " ", 
        RowBox[{"(", 
         RowBox[{"xbcg", "-", "xw2"}], ")"}]}]}]},
     {
      RowBox[{"rc2", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            RowBox[{"-", "Dragl2"}], "+", "Dragr2"}], ")"}], " ", 
          RowBox[{"Cos", "[", "\[Alpha]", "]"}]}], "+", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            RowBox[{"-", "Liftl2"}], "+", "Liftr2"}], ")"}], " ", 
          RowBox[{"Sin", "[", "\[Alpha]", "]"}]}]}], ")"}]}]}
    }], ")"}]}]], "Input",
 CellChangeTimes->{{3.446878866368227*^9, 3.4468788788835316`*^9}, 
   3.5220454949230537`*^9, {3.5344257330625*^9, 3.534425775515625*^9}, {
   3.5344379266875*^9, 3.534437947171875*^9}, 3.534437993765625*^9, 
   3.53443804259375*^9, {3.53448149865625*^9, 3.5344815021875*^9}, {
   3.53448162225*^9, 3.534481628671875*^9}, {3.57641050862185*^9, 
   3.5764105319281826`*^9}, 3.5764107724709415`*^9, {3.5764108081339808`*^9, 
   3.5764108200376616`*^9}, {3.6131915582577834`*^9, 3.6131915842923865`*^9}}],

Cell["\<\
Rudder hinge forces and aerodynamic stiffness
\
\>", "Text"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"kal2", ":=", 
   RowBox[{"CLalpha2e", " ", "S2", " ", "q", " ", "lc2"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"kar2", ":=", 
   RowBox[{"CLalpha2e", " ", "S2", " ", "q", " ", "lc2"}]}], ";"}]}], "Input",\

 CellChangeTimes->{{3.52207857765625*^9, 3.52207858290625*^9}, {
   3.523893603328125*^9, 3.5238936369375*^9}, {3.52389370346875*^9, 
   3.52389370778125*^9}, 3.5344815035*^9, 3.5344816296875*^9, {
   3.576426101213337*^9, 3.5764261089177775`*^9}, {3.5786615984007187`*^9, 
   3.5786616130895586`*^9}, {3.5840152655066357`*^9, 3.5840152669147167`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"exprW2", "=", 
   RowBox[{"{", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"cxal2", ",", 
       RowBox[{"kal2", " ", 
        RowBox[{"(", 
         RowBox[{"del2", "+", "\[Alpha]"}], ")"}]}]}], "}"}], " ", ",", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"cxar2", ",", 
       RowBox[{"kar2", 
        RowBox[{"(", 
         RowBox[{"der2", "+", "\[Alpha]"}], ")"}]}]}], "}"}]}], " ", "}"}]}], 
  ";"}]], "Input",
 CellChangeTimes->{{3.534481504890625*^9, 3.53448150671875*^9}, {
  3.534481630515625*^9, 3.5344816318125*^9}, {3.576425096174852*^9, 
  3.576425096956897*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Fin", "Subsubsection",
 FontSlant->"Italic"],

Cell["The contributions from the vertical tail are", "Text"],

Cell["Forces", "Text"],

Cell[BoxData[
 RowBox[{"fin`aero`F", ":=", 
  RowBox[{"(", GridBox[{
     {
      RowBox[{"-", "Dragfin"}]},
     {
      RowBox[{"-", "Cfin"}]},
     {"0"}
    }], ")"}]}]], "Input",
 CellChangeTimes->{
  3.5344769524375*^9, {3.534476999921875*^9, 3.53447701875*^9}}],

Cell["Moments", "Text"],

Cell[BoxData[
 RowBox[{"fin`aero`T", ":=", 
  RowBox[{"(", GridBox[{
     {"0"},
     {"0"},
     {
      RowBox[{
       RowBox[{
        RowBox[{"-", "Cfin"}], " ", 
        RowBox[{"(", 
         RowBox[{"xwfin", "-", "xbcg"}], ")"}]}], "-", 
       RowBox[{"Rbe", " ", 
        SubscriptBox["M", 
         OverscriptBox["\[Psi]", "."]]}]}]}
    }], ")"}]}]], "Input",
 CellChangeTimes->{
  3.522045497297324*^9, {3.53447696490625*^9, 3.534476990671875*^9}}],

Cell["Rudder hinge forces and aerodynamic stiffness", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"kafin", ":=", 
   RowBox[{"CLdefin", " ", "Sfin", " ", "q", " ", "lcfin"}]}], ";"}]], "Input",\

 CellChangeTimes->{
  3.5220785980625*^9, 3.523893597703125*^9, {3.523893642671875*^9, 
   3.52389364409375*^9}, {3.523893697140625*^9, 3.523893697765625*^9}, {
   3.5764261358563185`*^9, 3.5764261360363283`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"exprFin", "=", 
   RowBox[{"{", "\[IndentingNewLine]", 
    RowBox[{"{", 
     RowBox[{"cxfin", ",", 
      RowBox[{"kafin", " ", 
       RowBox[{"(", 
        RowBox[{"defin", "+", "\[Beta]"}], ")"}]}]}], "}"}], 
    "\[IndentingNewLine]", "}"}]}], ";"}]], "Input",
 CellChangeTimes->{3.576411191923932*^9, 3.5764251086875677`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["Propulsion",
 FontSlant->"Italic"]], "Subsubsection"],

Cell["The thrust vector in body coordinates is", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"engine`T", ":=", 
   RowBox[{"{", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"yeng", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"thrustl", " ", 
         RowBox[{"Sin", "[", "dezthrustl", "]"}], " ", 
         RowBox[{"Cos", "[", "deythrustl", "]"}]}], "-", 
        RowBox[{"thrustr", " ", 
         RowBox[{"Sin", "[", "dezthrustr", "]"}], " ", 
         RowBox[{"Cos", "[", "deythrustr", "]"}]}]}], ")"}]}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"-", 
       RowBox[{"(", 
        RowBox[{"xeng", "-", "xbcg"}], ")"}]}], 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"thrustl", " ", 
         RowBox[{"Sin", "[", "dezthrustl", "]"}], 
         RowBox[{"Cos", "[", "deythrustl", "]"}]}], "+", 
        RowBox[{"thrustr", " ", 
         RowBox[{"Sin", "[", "dezthrustr", "]"}], 
         RowBox[{"Cos", "[", "deythrustr", "]"}]}]}], ")"}]}], " ", ",", 
     "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"-", 
       RowBox[{"(", 
        RowBox[{"xeng", "-", "xbcg"}], ")"}]}], 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"thrustl", " ", 
         RowBox[{"Cos", "[", "dezthrustl", "]"}], " ", 
         RowBox[{"Sin", "[", "deythrustl", "]"}]}], " ", "+", 
        RowBox[{"thrustr", " ", 
         RowBox[{"Cos", "[", "dezthrustr", "]"}], " ", 
         RowBox[{"Sin", "[", "deythrustr", "]"}]}]}], ")"}]}]}], "  ", 
    "\[IndentingNewLine]", "}"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.567880405111807*^9, 3.567880430046233*^9}, 
   3.567880589933378*^9, {3.5678806300296717`*^9, 3.56788064346444*^9}, {
   3.5678806941553392`*^9, 3.5678807311564555`*^9}, {3.567880768684602*^9, 
   3.5678808267689247`*^9}, {3.567880877287814*^9, 3.5678810170858097`*^9}, {
   3.5678811350795584`*^9, 3.567881191522787*^9}, 3.567943793873806*^9, {
   3.5679438574544425`*^9, 3.5679438582744894`*^9}, 3.5680030649205723`*^9, {
   3.5680283597420015`*^9, 3.5680283862856555`*^9}, 3.568028439984025*^9}],

Cell[BoxData[
 RowBox[{"engine`F", " ", ":=", " ", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     RowBox[{"thrustl", " ", 
      RowBox[{"Cos", "[", "deythrustl", "]"}], 
      RowBox[{"Cos", "[", "dezthrustr", "]"}]}], "+", 
     RowBox[{"thrustr", " ", 
      RowBox[{"Cos", "[", "deythrustr", "]"}], 
      RowBox[{"Cos", "[", "dezthrustr", "]"}]}]}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{
      RowBox[{"-", "thrustl"}], " ", 
      RowBox[{"Sin", "[", "deythrustl", "]"}], 
      RowBox[{"Cos", "[", "dezthrustr", "]"}]}], "-", 
     RowBox[{"thrustr", " ", 
      RowBox[{"Sin", "[", "deythrustr", "]"}], 
      RowBox[{"Cos", "[", "dezthrustr", "]"}]}]}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{
      RowBox[{"-", "thrustl"}], " ", 
      RowBox[{"Cos", "[", "deythrustl", "]"}], 
      RowBox[{"Sin", "[", "dezthrustr", "]"}]}], "-", 
     RowBox[{"thrustr", " ", 
      RowBox[{"Cos", "[", "deythrustr", "]"}], 
      RowBox[{"Sin", "[", "dezthrustr", "]"}]}]}]}], "\[IndentingNewLine]", 
   "}"}]}]], "Input",
 CellChangeTimes->{{3.5680024386973815`*^9, 3.5680026409596057`*^9}, {
   3.568002758067572*^9, 3.568002776969572*^9}, {3.5680028518355722`*^9, 
   3.568002874338572*^9}, 3.568003071606572*^9, {3.5877430966711063`*^9, 
   3.587743106380661*^9}}],

Cell["\<\
The total force an moment vectors in body coordinates are then\
\>", "Text",
 CellChangeTimes->{{3.567880379910365*^9, 3.5678803823515053`*^9}, 
   3.568003087731572*^9}],

Cell[BoxData[""], "Input"]
}, Open  ]]
}, Open  ]],

Cell[BoxData[
 RowBox[{" ", 
  RowBox[{"ground`F", ":=", 
   RowBox[{"(", GridBox[{
      {"0"},
      {"0"},
      {"0"}
     }], ")"}]}]}]], "Input",
 CellChangeTimes->{{3.460037253*^9, 3.4600373400625*^9}, {
   3.518863135171875*^9, 3.518863187140625*^9}, 3.518863232328125*^9, {
   3.518863272328125*^9, 3.518863323953125*^9}, 3.519443222*^9, {
   3.538220000709673*^9, 3.538220002444026*^9}, {3.5389929952154565`*^9, 
   3.5389929974654565`*^9}, {3.5411476588373775`*^9, 3.541147663759189*^9}, 
   3.5550567684193497`*^9}],

Cell[BoxData[
 RowBox[{" ", 
  RowBox[{"ground`F", ":=", 
   RowBox[{"(", GridBox[{
      {"0"},
      {"0"},
      {
       RowBox[{
        RowBox[{"-", 
         RowBox[{"onPositive", "[", "zcg", "]"}]}], "kground", " ", "zcg"}]}
     }], ")"}]}]}]], "Input",
 CellChangeTimes->{{3.460037253*^9, 3.4600373400625*^9}, {
   3.518863135171875*^9, 3.518863187140625*^9}, 3.518863232328125*^9, {
   3.518863272328125*^9, 3.518863323953125*^9}, 3.519443222*^9, {
   3.538220000709673*^9, 3.538220002444026*^9}, {3.5389929952154565`*^9, 
   3.5389929974654565`*^9}, {3.5411476588373775`*^9, 3.541147663759189*^9}}],

Cell[CellGroupData[{

Cell["Display forces and moments", "Subsection"],

Cell["The inverse of the inertia matrix", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Jinv", "//", "MatrixForm"}]], "Input"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      FractionBox["Iz", 
       RowBox[{
        RowBox[{"-", 
         SuperscriptBox["Ixz", "2"]}], "+", 
        RowBox[{"Ix", " ", "Iz"}]}]], "0", 
      FractionBox["Ixz", 
       RowBox[{
        RowBox[{"-", 
         SuperscriptBox["Ixz", "2"]}], "+", 
        RowBox[{"Ix", " ", "Iz"}]}]]},
     {"0", 
      FractionBox["1", "Iy"], "0"},
     {
      FractionBox["Ixz", 
       RowBox[{
        RowBox[{"-", 
         SuperscriptBox["Ixz", "2"]}], "+", 
        RowBox[{"Ix", " ", "Iz"}]}]], "0", 
      FractionBox["Ix", 
       RowBox[{
        RowBox[{"-", 
         SuperscriptBox["Ixz", "2"]}], "+", 
        RowBox[{"Ix", " ", "Iz"}]}]]}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{
  3.576426209215514*^9, 3.576993497755633*^9, 3.5769936707316327`*^9, 
   3.5769937361586328`*^9, 3.578504373949436*^9, 3.5785915053047743`*^9, 
   3.5786616386740217`*^9, 3.581236894108813*^9, 3.581237062574768*^9, 
   3.581237109772673*^9, 3.581237148766427*^9, 3.581237282090684*^9, 
   3.5812373491687193`*^9, 3.5813519979723344`*^9, 3.5817462151805716`*^9, 
   3.5817463996540174`*^9, 3.581746599825032*^9, 3.5817550087032223`*^9, 
   3.581780890267563*^9, 3.581826860300659*^9, 3.5823513310749474`*^9, 
   3.5840250905175943`*^9, 3.584025261655383*^9, 3.584025380225165*^9, 
   3.5840254223005714`*^9, 3.584025458043616*^9, 3.585372038147624*^9, 
   3.585372123003109*^9, 3.5853722452633333`*^9, 3.6124578282393117`*^9, 
   3.61245809780273*^9, 3.612458179468401*^9, {3.612458309998867*^9, 
   3.6124583156701913`*^9}, 3.6131936065165887`*^9, 3.6131936502629633`*^9, 
   3.6131936887818146`*^9, 3.6131938455321894`*^9, 3.6207123561111593`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"wing1`aero`F", "//", "MatrixForm"}]], "Input"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      RowBox[{
       RowBox[{"-", "Dragl1"}], "-", "Dragr1"}]},
     {"0"},
     {
      RowBox[{
       RowBox[{"-", "Liftl1"}], "-", "Liftr1"}]}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{
  3.5764262092395153`*^9, 3.5769934977906327`*^9, 3.5769936707606325`*^9, 
   3.5769937361916327`*^9, 3.5785043739874387`*^9, 3.5785915053327756`*^9, 
   3.578661638703024*^9, 3.5812368941338177`*^9, 3.5812370626017714`*^9, 
   3.581237109803875*^9, 3.5812371487976294`*^9, 3.581237282106285*^9, 
   3.5812373491999207`*^9, 3.581351997998336*^9, 3.5817462152085743`*^9, 
   3.58174639968002*^9, 3.5817465998570356`*^9, 3.581755008735224*^9, 
   3.5817808902965646`*^9, 3.581826860327659*^9, 3.5823513311169434`*^9, 
   3.5840250905435963`*^9, 3.5840252616853848`*^9, 3.5840253802641673`*^9, 
   3.5840254223365736`*^9, 3.584025458075618*^9, 3.58537203820863*^9, 
   3.585372123056114*^9, 3.5853722453123384`*^9, 3.6124578282663136`*^9, 
   3.612458097834732*^9, 3.6124581794984026`*^9, {3.6124583100298686`*^9, 
   3.6124583157021933`*^9}, 3.613193606549592*^9, 3.613193650287966*^9, 
   3.6131936888078175`*^9, 3.613193845611189*^9, 3.6207123561441627`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"wing1`aero`T", "//", "MatrixForm"}]], "Input"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      RowBox[{
       RowBox[{"rc1", " ", "rc2", " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{"(", 
            RowBox[{"Liftl1", "-", "Liftr1"}], ")"}], " ", 
           RowBox[{"Cos", "[", "Alpha", "]"}]}], "-", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{"Dragl1", "-", "Dragr1"}], ")"}], " ", 
           RowBox[{"Sin", "[", "Alpha", "]"}]}]}], ")"}]}], "-", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"Pb", "+", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             SuperscriptBox["q0", "2"], "+", 
             SuperscriptBox["q1", "2"], "-", 
             SuperscriptBox["q2", "2"], "-", 
             SuperscriptBox["q3", "2"]}], ")"}], " ", "wturbx"}], "+", 
          RowBox[{"2", " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"q1", " ", "q2"}], "-", 
             RowBox[{"q0", " ", "q3"}]}], ")"}], " ", "wturby"}], "+", 
          RowBox[{"2", " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"q0", " ", "q2"}], "+", 
             RowBox[{"q1", " ", "q3"}]}], ")"}], " ", "wturbz"}]}], ")"}], 
        " ", 
        SubscriptBox["M", 
         OverscriptBox["\[EmptySet]", "."]]}]}]},
     {
      RowBox[{
       RowBox[{"qpress", " ", "S1", " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{"-", "Cmde1"}], " ", "thetaal1"}], "-", 
          RowBox[{"Cmde12", " ", "thetaal12"}], "-", 
          RowBox[{"Cmde1", " ", "thetaar1"}], "-", 
          RowBox[{"Cmde12", " ", "thetaar12"}], "-", 
          FractionBox[
           RowBox[{"smc", " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{"CLde1", " ", "thetaal1"}], "+", 
              RowBox[{"CLde12", " ", "thetaal12"}], "+", 
              RowBox[{
               FractionBox["1", "2"], " ", 
               RowBox[{"CLift", "[", 
                RowBox[{
                 RowBox[{
                  RowBox[{"Alpha", " ", "dah1"}], "-", "ia1"}], ",", 
                 "CLalpha1e", ",", "ap1", ",", "an1", ",", "awp1", ",", 
                 "awn1"}], "]"}]}]}], ")"}]}], 
           RowBox[{"4", " ", 
            RowBox[{"(", 
             RowBox[{"1", "+", 
              SuperscriptBox["en", 
               RowBox[{"-", 
                FractionBox[
                 RowBox[{"2", " ", 
                  RowBox[{"(", 
                   RowBox[{"v", "-", "vM"}], ")"}]}], 
                 RowBox[{"dM", " ", "vM"}]]}]]}], ")"}]}]], "-", 
          FractionBox[
           RowBox[{"smc", " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{"CLde1", " ", "thetaar1"}], "+", 
              RowBox[{"CLde12", " ", "thetaar12"}], "+", 
              RowBox[{
               FractionBox["1", "2"], " ", 
               RowBox[{"CLift", "[", 
                RowBox[{
                 RowBox[{
                  RowBox[{"Alpha", " ", "dah1"}], "-", "ia1"}], ",", 
                 "CLalpha1e", ",", "ap1", ",", "an1", ",", "awp1", ",", 
                 "awn1"}], "]"}]}]}], ")"}]}], 
           RowBox[{"4", " ", 
            RowBox[{"(", 
             RowBox[{"1", "+", 
              SuperscriptBox["en", 
               RowBox[{"-", 
                FractionBox[
                 RowBox[{"2", " ", 
                  RowBox[{"(", 
                   RowBox[{"v", "-", "vM"}], ")"}]}], 
                 RowBox[{"dM", " ", "vM"}]]}]]}], ")"}]}]], "+", 
          RowBox[{"CMoment", "[", 
           RowBox[{
            RowBox[{
             RowBox[{"Alpha", " ", "dah1"}], "-", "ia1"}], ",", "Cm01", ",", 
            "Cmfs1", ",", "ap1", ",", "an1", ",", "awp1", ",", "awn1"}], 
           "]"}]}], ")"}]}], "+", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"xbcg", "-", "xw1"}], ")"}], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{"(", 
            RowBox[{"Liftl1", "+", "Liftr1"}], ")"}], " ", 
           RowBox[{"Cos", "[", "Alpha", "]"}]}], "+", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{"Dragl1", "+", "Dragr1"}], ")"}], " ", 
           RowBox[{"Sin", "[", "Alpha", "]"}]}]}], ")"}]}]}]},
     {
      RowBox[{"rc1", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            RowBox[{"-", "Dragl1"}], "+", "Dragr1"}], ")"}], " ", 
          RowBox[{"Cos", "[", "Alpha", "]"}]}], "+", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            RowBox[{"-", "Liftl1"}], "+", "Liftr1"}], ")"}], " ", 
          RowBox[{"Sin", "[", "Alpha", "]"}]}]}], ")"}]}]}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{
  3.576426209264517*^9, 3.576993497820633*^9, 3.5769936707906327`*^9, 
   3.576993736222633*^9, 3.578504374028441*^9, 3.578591505360777*^9, 
   3.578661638731025*^9, 3.581236894158823*^9, 3.5812370626277733`*^9, 
   3.581237109819476*^9, 3.58123714881323*^9, 3.5812372821374865`*^9, 
   3.581237349215521*^9, 3.5813519980233374`*^9, 3.581746215238578*^9, 
   3.5817463997060223`*^9, 3.5817465998850384`*^9, 3.5817550087682257`*^9, 
   3.5817808903285666`*^9, 3.5818268603556595`*^9, 3.5823513311479406`*^9, 
   3.5840250905715976`*^9, 3.5840252617183867`*^9, 3.584025380298169*^9, 
   3.584025422377576*^9, 3.5840254581076193`*^9, 3.5853720382486343`*^9, 
   3.58537212311012*^9, 3.5853722453683443`*^9, 3.612457828291315*^9, 
   3.6124580978637333`*^9, 3.6124581795294046`*^9, {3.61245831005787*^9, 
   3.612458315730195*^9}, 3.613193606580595*^9, 3.6131936503129683`*^9, 
   3.6131936888338194`*^9, 3.6131938456431894`*^9, 3.6207123561761656`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"wing2`aero`F", "//", "MatrixForm"}]], "Input"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      RowBox[{
       RowBox[{"-", "Dragl2"}], "-", "Dragr2"}]},
     {
      RowBox[{
       RowBox[{"-", "Cydeelev"}], " ", "qpress", " ", "S2", " ", 
       RowBox[{"(", 
        RowBox[{"thetaal2", "-", "thetaar2"}], ")"}]}]},
     {
      RowBox[{
       RowBox[{"-", "Liftl2"}], "-", "Liftr2"}]}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.6207123562121696`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"wing2`aero`T", "//", "MatrixForm"}]], "Input"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      RowBox[{"rc2", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"(", 
           RowBox[{"Liftl2", "-", "Liftr2"}], ")"}], " ", 
          RowBox[{"Cos", "[", "Alpha", "]"}]}], "-", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{"Dragl2", "-", "Dragr2"}], ")"}], " ", 
          RowBox[{"Sin", "[", "Alpha", "]"}]}]}], ")"}]}]},
     {
      RowBox[{
       RowBox[{
        RowBox[{"-", "Mdvtheta"}], " ", 
        RowBox[{"(", 
         RowBox[{"Qb", "+", 
          RowBox[{"2", " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"q1", " ", "q2"}], "+", 
             RowBox[{"q0", " ", "q3"}]}], ")"}], " ", "wturbx"}], "+", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             SuperscriptBox["q0", "2"], "-", 
             SuperscriptBox["q1", "2"], "+", 
             SuperscriptBox["q2", "2"], "-", 
             SuperscriptBox["q3", "2"]}], ")"}], " ", "wturby"}], "+", 
          RowBox[{"2", " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{
              RowBox[{"-", "q0"}], " ", "q1"}], "+", 
             RowBox[{"q2", " ", "q3"}]}], ")"}], " ", "wturbz"}]}], ")"}]}], 
       "+", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"xbcg", "-", "xw2"}], ")"}], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"-", "Cl2"}], "-", "Cr2", "+", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{"Liftl2", "+", "Liftr2"}], ")"}], " ", 
           RowBox[{"Cos", "[", "Alpha", "]"}]}], "-", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{"Dragl2", "+", "Dragr2"}], ")"}], " ", 
           RowBox[{"Sin", "[", "Alpha", "]"}]}]}], ")"}]}]}]},
     {
      RowBox[{"rc2", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            RowBox[{"-", "Dragl2"}], "+", "Dragr2"}], ")"}], " ", 
          RowBox[{"Cos", "[", "Alpha", "]"}]}], "+", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            RowBox[{"-", "Liftl2"}], "+", "Liftr2"}], ")"}], " ", 
          RowBox[{"Sin", "[", "Alpha", "]"}]}]}], ")"}]}]}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{
  3.57642620931452*^9, 3.5769934980696325`*^9, 3.5769936708496327`*^9, 
   3.576993736280633*^9, 3.5785043741074457`*^9, 3.578591505428781*^9, 
   3.5786616387870283`*^9, 3.5812368942088327`*^9, 3.5812370626827793`*^9, 
   3.581237109866279*^9, 3.581237148860033*^9, 3.58123728218429*^9, 
   3.5812373492467217`*^9, 3.581351998089341*^9, 3.581746215301584*^9, 
   3.5817463997580276`*^9, 3.5817465999430437`*^9, 3.58175500884023*^9, 
   3.58178089039257*^9, 3.5818268604216595`*^9, 3.5823513312179337`*^9, 
   3.5840250906296005`*^9, 3.5840252617853904`*^9, 3.584025380362173*^9, 
   3.5840254224405794`*^9, 3.584025458173623*^9, 3.585372038395649*^9, 
   3.5853721231851273`*^9, 3.585372245460353*^9, 3.6124578283413177`*^9, 
   3.612458097920737*^9, 3.612458179589408*^9, {3.6124583101158733`*^9, 
   3.612458315787198*^9}, 3.6131936066436014`*^9, 3.613193650362973*^9, 
   3.6131936888848248`*^9, 3.6131938456981893`*^9, 3.6207123562431726`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"fin`aero`F", "//", "MatrixForm"}]], "Input"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      RowBox[{"-", "Dragfin"}]},
     {
      RowBox[{"-", "Cfin"}]},
     {"0"}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{
  3.5764262093395214`*^9, 3.576993498111633*^9, 3.5769936708786325`*^9, 
   3.576993736308633*^9, 3.578504374148448*^9, 3.5785915054597826`*^9, 
   3.57866163881503*^9, 3.5812368942328377`*^9, 3.5812370627087817`*^9, 
   3.581237109897481*^9, 3.5812371488912354`*^9, 3.5812372822154913`*^9, 
   3.5812373492623224`*^9, 3.5813519981263433`*^9, 3.581746215336587*^9, 
   3.58174639978403*^9, 3.581746599974047*^9, 3.581755008868232*^9, 
   3.581780890426572*^9, 3.5818268604546595`*^9, 3.58235133125193*^9, 
   3.584025090663603*^9, 3.5840252618183928`*^9, 3.5840253803941746`*^9, 
   3.5840254224735813`*^9, 3.5840254582056255`*^9, 3.5853720384506545`*^9, 
   3.5853721232261314`*^9, 3.5853722455113583`*^9, 3.612457828366319*^9, 
   3.6124580979497385`*^9, 3.6124581796204095`*^9, {3.6124583101448755`*^9, 
   3.6124583158161993`*^9}, 3.6131936066766047`*^9, 3.6131936503879757`*^9, 
   3.6131936889108276`*^9, 3.6131938457281895`*^9, 3.6207123562781763`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"fin`aero`T", "//", "MatrixForm"}]], "Input"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"0"},
     {"0"},
     {
      RowBox[{
       RowBox[{
        RowBox[{"-", "Mdvpsi"}], " ", 
        RowBox[{"(", 
         RowBox[{"Rb", "+", 
          RowBox[{"2", " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{
              RowBox[{"-", "q0"}], " ", "q2"}], "+", 
             RowBox[{"q1", " ", "q3"}]}], ")"}], " ", "wturbx"}], "+", 
          RowBox[{"2", " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"q0", " ", "q1"}], "+", 
             RowBox[{"q2", " ", "q3"}]}], ")"}], " ", "wturby"}], "+", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             SuperscriptBox["q0", "2"], "-", 
             SuperscriptBox["q1", "2"], "-", 
             SuperscriptBox["q2", "2"], "+", 
             SuperscriptBox["q3", "2"]}], ")"}], " ", "wturbz"}]}], ")"}]}], 
       "-", 
       RowBox[{"Cfin", " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"-", "xbcg"}], "+", "xwfin"}], ")"}]}]}]}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{
  3.576426209364523*^9, 3.576993498155633*^9, 3.576993670907633*^9, 
   3.576993736337633*^9, 3.57850437418545*^9, 3.578591505493785*^9, 
   3.578661638843032*^9, 3.581236894257843*^9, 3.581237062736785*^9, 
   3.5812371099286833`*^9, 3.581237148922437*^9, 3.5812372822466936`*^9, 
   3.5812373492779226`*^9, 3.581351998160345*^9, 3.5817462153705907`*^9, 
   3.581746399809033*^9, 3.58174660000105*^9, 3.5817550088992333`*^9, 
   3.581780890460574*^9, 3.581826860490659*^9, 3.5823513313059244`*^9, 
   3.5840250906916046`*^9, 3.584025261852394*^9, 3.5840253804291763`*^9, 
   3.584025422507583*^9, 3.584025458237627*^9, 3.5853720385006595`*^9, 
   3.5853721232571344`*^9, 3.5853722455703645`*^9, 3.6124578283903203`*^9, 
   3.6124580979777403`*^9, 3.6124581796544113`*^9, {3.612458310172877*^9, 
   3.6124583158452015`*^9}, 3.6131936067146087`*^9, 3.613193650411978*^9, 
   3.61319368893583*^9, 3.6131938457561893`*^9, 3.620712356310179*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"body`aero`F", "//", "MatrixForm"}]], "Input"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", 
   TagBox[GridBox[{
      {
       RowBox[{"-", "Dragb"}]},
      {
       RowBox[{"CLalphabv", " ", "qpress", " ", "Sbv", " ", 
        RowBox[{"Sin", "[", "Beta", "]"}]}]},
      {
       RowBox[{"-", "Liftb"}]}
     },
     GridBoxAlignment->{
      "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}},
        "RowsIndexed" -> {}},
     GridBoxSpacings->{"Columns" -> {
         Offset[0.27999999999999997`], {
          Offset[0.5599999999999999]}, 
         Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
         Offset[0.2], {
          Offset[0.4]}, 
         Offset[0.2]}, "RowsIndexed" -> {}}],
    Column], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{
  3.576426209389524*^9, 3.5769934981986327`*^9, 3.576993671117633*^9, 
   3.576993736365633*^9, 3.578504374222452*^9, 3.5785915055257864`*^9, 
   3.5786616388700333`*^9, 3.581236894281848*^9, 3.5812370627657876`*^9, 
   3.581237109944284*^9, 3.581237148938038*^9, 3.581237282262295*^9, 
   3.581237349309124*^9, 3.5813519981903467`*^9, 3.581746215403594*^9, 
   3.581746399834035*^9, 3.5817466000310526`*^9, 3.581755008994239*^9, 
   3.5817808904925756`*^9, 3.5818268605226593`*^9, 3.582351331337921*^9, 
   3.584025090719606*^9, 3.584025261885396*^9, 3.5840253804631786`*^9, 
   3.5840254225405855`*^9, 3.584025458271629*^9, 3.585372038538663*^9, 
   3.585372123323141*^9, 3.585372245615369*^9, 3.6124578284153223`*^9, 
   3.6124580980067415`*^9, 3.6124581796934137`*^9, {3.6124583102018785`*^9, 
   3.6124583158742027`*^9}, 3.613193606747612*^9, 3.6131936504399805`*^9, 
   3.6131936889608326`*^9, 3.613193845785189*^9, 3.6207123563401823`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"body`aero`T", "//", "MatrixForm"}]], "Input"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", 
   TagBox[GridBox[{
      {"0"},
      {"0"},
      {"0"}
     },
     GridBoxAlignment->{
      "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}},
        "RowsIndexed" -> {}},
     GridBoxSpacings->{"Columns" -> {
         Offset[0.27999999999999997`], {
          Offset[0.5599999999999999]}, 
         Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
         Offset[0.2], {
          Offset[0.4]}, 
         Offset[0.2]}, "RowsIndexed" -> {}}],
    Column], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{
  3.5764262094125257`*^9, 3.5769934982376328`*^9, 3.576993671207633*^9, 
   3.576993736394633*^9, 3.578504374260454*^9, 3.5785915055597887`*^9, 
   3.578661638898035*^9, 3.5812368943068523`*^9, 3.58123706279179*^9, 
   3.581237109975486*^9, 3.58123714896924*^9, 3.581237282293497*^9, 
   3.581237349340325*^9, 3.581351998224349*^9, 3.581746215436597*^9, 
   3.5817463998590374`*^9, 3.581746600058056*^9, 3.5817550090332413`*^9, 
   3.5817808905255775`*^9, 3.581826860554659*^9, 3.5823513313749175`*^9, 
   3.5840250907476077`*^9, 3.584025261918398*^9, 3.58402538049518*^9, 
   3.584025422572587*^9, 3.5840254583076305`*^9, 3.5853720386146708`*^9, 
   3.585372123351144*^9, 3.5853722456903763`*^9, 3.6124578284393234`*^9, 
   3.6124580980347433`*^9, 3.6124581797314157`*^9, {3.6124583102298803`*^9, 
   3.612458315903205*^9}, 3.6131936067846155`*^9, 3.613193650464983*^9, 
   3.613193688985835*^9, 3.613193845812189*^9, 3.620712356410189*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"engine`F", "//", "MatrixForm"}]], "Input"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", 
   TagBox[GridBox[{
      {
       RowBox[{
        RowBox[{"thrustl", " ", 
         RowBox[{"Cos", "[", "deythrustl", "]"}], " ", 
         RowBox[{"Cos", "[", "dezthrustr", "]"}]}], "+", 
        RowBox[{"thrustr", " ", 
         RowBox[{"Cos", "[", "deythrustr", "]"}], " ", 
         RowBox[{"Cos", "[", "dezthrustr", "]"}]}]}]},
      {
       RowBox[{
        RowBox[{
         RowBox[{"-", "thrustl"}], " ", 
         RowBox[{"Cos", "[", "dezthrustr", "]"}], " ", 
         RowBox[{"Sin", "[", "deythrustl", "]"}]}], "-", 
        RowBox[{"thrustr", " ", 
         RowBox[{"Cos", "[", "dezthrustr", "]"}], " ", 
         RowBox[{"Sin", "[", "deythrustr", "]"}]}]}]},
      {
       RowBox[{
        RowBox[{
         RowBox[{"-", "thrustl"}], " ", 
         RowBox[{"Cos", "[", "deythrustl", "]"}], " ", 
         RowBox[{"Sin", "[", "dezthrustr", "]"}]}], "-", 
        RowBox[{"thrustr", " ", 
         RowBox[{"Cos", "[", "deythrustr", "]"}], " ", 
         RowBox[{"Sin", "[", "dezthrustr", "]"}]}]}]}
     },
     GridBoxAlignment->{
      "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}},
        "RowsIndexed" -> {}},
     GridBoxSpacings->{"Columns" -> {
         Offset[0.27999999999999997`], {
          Offset[0.5599999999999999]}, 
         Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
         Offset[0.2], {
          Offset[0.4]}, 
         Offset[0.2]}, "RowsIndexed" -> {}}],
    Column], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{
  3.5764262094375267`*^9, 3.5769934982836328`*^9, 3.5769936712706327`*^9, 
   3.5769937364226327`*^9, 3.5785043742974567`*^9, 3.5785915055927906`*^9, 
   3.5786616389250364`*^9, 3.5812368943318577`*^9, 3.581237062818793*^9, 
   3.581237109991087*^9, 3.5812371489848413`*^9, 3.5812372823246984`*^9, 
   3.581237349355925*^9, 3.5813519982563505`*^9, 3.5817462154696007`*^9, 
   3.58174639988404*^9, 3.581746600085058*^9, 3.5817550090692434`*^9, 
   3.5817808905605793`*^9, 3.581826860590659*^9, 3.582351331402915*^9, 
   3.584025090774609*^9, 3.5840252620434055`*^9, 3.5840253806031866`*^9, 
   3.584025422605589*^9, 3.5840254583436327`*^9, 3.585372038684678*^9, 
   3.5853721234241505`*^9, 3.585372245745382*^9, 3.612457828464325*^9, 
   3.6124580980637445`*^9, 3.6124581797614174`*^9, {3.6124583102578816`*^9, 
   3.612458315932206*^9}, 3.613193606817619*^9, 3.613193650490986*^9, 
   3.6131936890128374`*^9, 3.613193845840189*^9, 3.620712356449193*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"engine`T", "//", "MatrixForm"}]], "Input"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", 
   TagBox[GridBox[{
      {
       RowBox[{"yeng", " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"thrustl", " ", 
           RowBox[{"Cos", "[", "deythrustl", "]"}], " ", 
           RowBox[{"Sin", "[", "dezthrustl", "]"}]}], "-", 
          RowBox[{"thrustr", " ", 
           RowBox[{"Cos", "[", "deythrustr", "]"}], " ", 
           RowBox[{"Sin", "[", "dezthrustr", "]"}]}]}], ")"}]}]},
      {
       RowBox[{
        RowBox[{"(", 
         RowBox[{"xbcg", "-", "xeng"}], ")"}], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"thrustl", " ", 
           RowBox[{"Cos", "[", "deythrustl", "]"}], " ", 
           RowBox[{"Sin", "[", "dezthrustl", "]"}]}], "+", 
          RowBox[{"thrustr", " ", 
           RowBox[{"Cos", "[", "deythrustr", "]"}], " ", 
           RowBox[{"Sin", "[", "dezthrustr", "]"}]}]}], ")"}]}]},
      {
       RowBox[{
        RowBox[{"(", 
         RowBox[{"xbcg", "-", "xeng"}], ")"}], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"thrustl", " ", 
           RowBox[{"Cos", "[", "dezthrustl", "]"}], " ", 
           RowBox[{"Sin", "[", "deythrustl", "]"}]}], "+", 
          RowBox[{"thrustr", " ", 
           RowBox[{"Cos", "[", "dezthrustr", "]"}], " ", 
           RowBox[{"Sin", "[", "deythrustr", "]"}]}]}], ")"}]}]}
     },
     GridBoxAlignment->{
      "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}},
        "RowsIndexed" -> {}},
     GridBoxSpacings->{"Columns" -> {
         Offset[0.27999999999999997`], {
          Offset[0.5599999999999999]}, 
         Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
         Offset[0.2], {
          Offset[0.4]}, 
         Offset[0.2]}, "RowsIndexed" -> {}}],
    Column], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{
  3.5764262094615283`*^9, 3.576993498322633*^9, 3.5769936713076324`*^9, 
   3.576993736450633*^9, 3.5785043743344584`*^9, 3.5785915056287923`*^9, 
   3.578661638953038*^9, 3.581236894356862*^9, 3.5812370628457956`*^9, 
   3.5812371100222893`*^9, 3.581237149016043*^9, 3.5812372823402996`*^9, 
   3.581237349402727*^9, 3.5813519983803577`*^9, 3.5817462155076046`*^9, 
   3.581746399909043*^9, 3.581746600116061*^9, 3.581755009099245*^9, 
   3.5817808905935817`*^9, 3.581826860622659*^9, 3.5823513314369116`*^9, 
   3.584025090802611*^9, 3.5840252620854073`*^9, 3.5840253806411886`*^9, 
   3.5840254227155952`*^9, 3.584025458377635*^9, 3.5853720388526945`*^9, 
   3.5853721234721556`*^9, 3.5853722458053875`*^9, 3.612457828489326*^9, 
   3.6124580980927467`*^9, 3.6124581797914195`*^9, {3.6124583102868834`*^9, 
   3.612458315960208*^9}, 3.613193606855623*^9, 3.613193650514988*^9, 
   3.6131936890398407`*^9, 3.6131938458691893`*^9, 3.620712356490197*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"aircraft`F", "//", "MatrixForm"}]], "Input"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{"-", "Dragfin"}], "-", "Dragl1", "-", "Dragl2", "-", 
          "Dragr1", "-", "Dragr2"}], ")"}], " ", 
        RowBox[{"Cos", "[", "Alpha", "]"}], " ", 
        RowBox[{"Cos", "[", "Beta", "]"}]}], "+", 
       RowBox[{"thrustl", " ", 
        RowBox[{"Cos", "[", "deythrustl", "]"}], " ", 
        RowBox[{"Cos", "[", "dezthrustr", "]"}]}], "+", 
       RowBox[{"thrustr", " ", 
        RowBox[{"Cos", "[", "deythrustr", "]"}], " ", 
        RowBox[{"Cos", "[", "dezthrustr", "]"}]}], "-", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{"-", "Liftl1"}], "-", "Liftl2", "-", "Liftr1", "-", 
          "Liftr2"}], ")"}], " ", 
        RowBox[{"Sin", "[", "Alpha", "]"}]}], "-", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{"-", "Cfin"}], "-", 
          RowBox[{"Cydeelev", " ", "qpress", " ", "S2", " ", 
           RowBox[{"(", 
            RowBox[{"thetaal2", "-", "thetaar2"}], ")"}]}]}], ")"}], " ", 
        RowBox[{"Cos", "[", "Alpha", "]"}], " ", 
        RowBox[{"Sin", "[", "Beta", "]"}]}]}]},
     {
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{"-", "Cfin"}], "-", 
          RowBox[{"Cydeelev", " ", "qpress", " ", "S2", " ", 
           RowBox[{"(", 
            RowBox[{"thetaal2", "-", "thetaar2"}], ")"}]}]}], ")"}], " ", 
        RowBox[{"Cos", "[", "Beta", "]"}]}], "+", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{"-", "Dragfin"}], "-", "Dragl1", "-", "Dragl2", "-", 
          "Dragr1", "-", "Dragr2"}], ")"}], " ", 
        RowBox[{"Sin", "[", "Beta", "]"}]}], "-", 
       RowBox[{"thrustl", " ", 
        RowBox[{"Cos", "[", "dezthrustr", "]"}], " ", 
        RowBox[{"Sin", "[", "deythrustl", "]"}]}], "-", 
       RowBox[{"thrustr", " ", 
        RowBox[{"Cos", "[", "dezthrustr", "]"}], " ", 
        RowBox[{"Sin", "[", "deythrustr", "]"}]}]}]},
     {
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{"-", "Liftl1"}], "-", "Liftl2", "-", "Liftr1", "-", 
          "Liftr2"}], ")"}], " ", 
        RowBox[{"Cos", "[", "Alpha", "]"}]}], "-", 
       RowBox[{"kground", " ", "zcg", " ", 
        RowBox[{"onPositive", "[", "zcg", "]"}]}], "+", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{"-", "Dragfin"}], "-", "Dragl1", "-", "Dragl2", "-", 
          "Dragr1", "-", "Dragr2"}], ")"}], " ", 
        RowBox[{"Cos", "[", "Beta", "]"}], " ", 
        RowBox[{"Sin", "[", "Alpha", "]"}]}], "-", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{"-", "Cfin"}], "-", 
          RowBox[{"Cydeelev", " ", "qpress", " ", "S2", " ", 
           RowBox[{"(", 
            RowBox[{"thetaal2", "-", "thetaar2"}], ")"}]}]}], ")"}], " ", 
        RowBox[{"Sin", "[", "Alpha", "]"}], " ", 
        RowBox[{"Sin", "[", "Beta", "]"}]}], "-", 
       RowBox[{"thrustl", " ", 
        RowBox[{"Cos", "[", "deythrustl", "]"}], " ", 
        RowBox[{"Sin", "[", "dezthrustr", "]"}]}], "-", 
       RowBox[{"thrustr", " ", 
        RowBox[{"Cos", "[", "deythrustr", "]"}], " ", 
        RowBox[{"Sin", "[", "dezthrustr", "]"}]}]}]}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{
  3.5764262094865294`*^9, 3.5769934983626328`*^9, 3.5769936713386326`*^9, 
   3.576993736480633*^9, 3.5785043744244633`*^9, 3.5785915056607943`*^9, 
   3.5786616389810395`*^9, 3.581236894382868*^9, 3.581237062871798*^9, 
   3.581237110053491*^9, 3.581237149031644*^9, 3.581237282371502*^9, 
   3.5812373494183273`*^9, 3.5813519984133596`*^9, 3.5817462155386076`*^9, 
   3.5817463999350452`*^9, 3.5817466001450644`*^9, 3.5817550091352468`*^9, 
   3.5817808906235833`*^9, 3.5818268606576595`*^9, 3.582351331470908*^9, 
   3.5840250908296127`*^9, 3.5840252621244097`*^9, 3.5840253806781907`*^9, 
   3.5840254227615976`*^9, 3.584025458410637*^9, 3.5853720389327025`*^9, 
   3.585372123527161*^9, 3.585372245852392*^9, 3.6124578285143275`*^9, 
   3.612458098121748*^9, 3.612458179824421*^9, {3.612458310315885*^9, 
   3.6124583159892097`*^9}, 3.613193606889626*^9, 3.6131936505419908`*^9, 
   3.6131936890658426`*^9, 3.613193845897189*^9, 3.620712356525201*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"aircraft`T", "//", "MatrixForm"}]], "Input"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      RowBox[{
       RowBox[{"rc1", " ", "rc2", " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{"(", 
            RowBox[{"Liftl1", "-", "Liftr1"}], ")"}], " ", 
           RowBox[{"Cos", "[", "Alpha", "]"}]}], "-", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{"Dragl1", "-", "Dragr1"}], ")"}], " ", 
           RowBox[{"Sin", "[", "Alpha", "]"}]}]}], ")"}]}], "+", 
       RowBox[{"rc2", " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{"(", 
            RowBox[{"Liftl2", "-", "Liftr2"}], ")"}], " ", 
           RowBox[{"Cos", "[", "Alpha", "]"}]}], "-", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{"Dragl2", "-", "Dragr2"}], ")"}], " ", 
           RowBox[{"Sin", "[", "Alpha", "]"}]}]}], ")"}]}], "+", 
       RowBox[{"yeng", " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"thrustl", " ", 
           RowBox[{"Cos", "[", "deythrustl", "]"}], " ", 
           RowBox[{"Sin", "[", "dezthrustl", "]"}]}], "-", 
          RowBox[{"thrustr", " ", 
           RowBox[{"Cos", "[", "deythrustr", "]"}], " ", 
           RowBox[{"Sin", "[", "dezthrustr", "]"}]}]}], ")"}]}], "-", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"Pb", "+", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             SuperscriptBox["q0", "2"], "+", 
             SuperscriptBox["q1", "2"], "-", 
             SuperscriptBox["q2", "2"], "-", 
             SuperscriptBox["q3", "2"]}], ")"}], " ", "wturbx"}], "+", 
          RowBox[{"2", " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"q1", " ", "q2"}], "-", 
             RowBox[{"q0", " ", "q3"}]}], ")"}], " ", "wturby"}], "+", 
          RowBox[{"2", " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"q0", " ", "q2"}], "+", 
             RowBox[{"q1", " ", "q3"}]}], ")"}], " ", "wturbz"}]}], ")"}], 
        " ", 
        SubscriptBox["M", 
         OverscriptBox["\[EmptySet]", "."]]}]}]},
     {
      RowBox[{
       RowBox[{
        RowBox[{"-", "Mdvtheta"}], " ", 
        RowBox[{"(", 
         RowBox[{"Qb", "+", 
          RowBox[{"2", " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"q1", " ", "q2"}], "+", 
             RowBox[{"q0", " ", "q3"}]}], ")"}], " ", "wturbx"}], "+", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             SuperscriptBox["q0", "2"], "-", 
             SuperscriptBox["q1", "2"], "+", 
             SuperscriptBox["q2", "2"], "-", 
             SuperscriptBox["q3", "2"]}], ")"}], " ", "wturby"}], "+", 
          RowBox[{"2", " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{
              RowBox[{"-", "q0"}], " ", "q1"}], "+", 
             RowBox[{"q2", " ", "q3"}]}], ")"}], " ", "wturbz"}]}], ")"}]}], 
       "+", 
       RowBox[{"qpress", " ", "S1", " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{"-", "Cmde1"}], " ", "thetaal1"}], "-", 
          RowBox[{"Cmde12", " ", "thetaal12"}], "-", 
          RowBox[{"Cmde1", " ", "thetaar1"}], "-", 
          RowBox[{"Cmde12", " ", "thetaar12"}], "-", 
          FractionBox[
           RowBox[{"smc", " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{"CLde1", " ", "thetaal1"}], "+", 
              RowBox[{"CLde12", " ", "thetaal12"}], "+", 
              RowBox[{
               FractionBox["1", "2"], " ", 
               RowBox[{"CLift", "[", 
                RowBox[{
                 RowBox[{
                  RowBox[{"Alpha", " ", "dah1"}], "-", "ia1"}], ",", 
                 "CLalpha1e", ",", "ap1", ",", "an1", ",", "awp1", ",", 
                 "awn1"}], "]"}]}]}], ")"}]}], 
           RowBox[{"4", " ", 
            RowBox[{"(", 
             RowBox[{"1", "+", 
              SuperscriptBox["en", 
               RowBox[{"-", 
                FractionBox[
                 RowBox[{"2", " ", 
                  RowBox[{"(", 
                   RowBox[{"v", "-", "vM"}], ")"}]}], 
                 RowBox[{"dM", " ", "vM"}]]}]]}], ")"}]}]], "-", 
          FractionBox[
           RowBox[{"smc", " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{"CLde1", " ", "thetaar1"}], "+", 
              RowBox[{"CLde12", " ", "thetaar12"}], "+", 
              RowBox[{
               FractionBox["1", "2"], " ", 
               RowBox[{"CLift", "[", 
                RowBox[{
                 RowBox[{
                  RowBox[{"Alpha", " ", "dah1"}], "-", "ia1"}], ",", 
                 "CLalpha1e", ",", "ap1", ",", "an1", ",", "awp1", ",", 
                 "awn1"}], "]"}]}]}], ")"}]}], 
           RowBox[{"4", " ", 
            RowBox[{"(", 
             RowBox[{"1", "+", 
              SuperscriptBox["en", 
               RowBox[{"-", 
                FractionBox[
                 RowBox[{"2", " ", 
                  RowBox[{"(", 
                   RowBox[{"v", "-", "vM"}], ")"}]}], 
                 RowBox[{"dM", " ", "vM"}]]}]]}], ")"}]}]], "+", 
          RowBox[{"CMoment", "[", 
           RowBox[{
            RowBox[{
             RowBox[{"Alpha", " ", "dah1"}], "-", "ia1"}], ",", "Cm01", ",", 
            "Cmfs1", ",", "ap1", ",", "an1", ",", "awp1", ",", "awn1"}], 
           "]"}]}], ")"}]}], "+", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"xbcg", "-", "xw1"}], ")"}], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{"(", 
            RowBox[{"Liftl1", "+", "Liftr1"}], ")"}], " ", 
           RowBox[{"Cos", "[", "Alpha", "]"}]}], "+", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{"Dragl1", "+", "Dragr1"}], ")"}], " ", 
           RowBox[{"Sin", "[", "Alpha", "]"}]}]}], ")"}]}], "+", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"xbcg", "-", "xw2"}], ")"}], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"-", "Cl2"}], "-", "Cr2", "+", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{"Liftl2", "+", "Liftr2"}], ")"}], " ", 
           RowBox[{"Cos", "[", "Alpha", "]"}]}], "-", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{"Dragl2", "+", "Dragr2"}], ")"}], " ", 
           RowBox[{"Sin", "[", "Alpha", "]"}]}]}], ")"}]}], "+", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"xbcg", "-", "xeng"}], ")"}], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"thrustl", " ", 
           RowBox[{"Cos", "[", "deythrustl", "]"}], " ", 
           RowBox[{"Sin", "[", "dezthrustl", "]"}]}], "+", 
          RowBox[{"thrustr", " ", 
           RowBox[{"Cos", "[", "deythrustr", "]"}], " ", 
           RowBox[{"Sin", "[", "dezthrustr", "]"}]}]}], ")"}]}]}]},
     {
      RowBox[{
       RowBox[{
        RowBox[{"-", "Mdvpsi"}], " ", 
        RowBox[{"(", 
         RowBox[{"Rb", "+", 
          RowBox[{"2", " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{
              RowBox[{"-", "q0"}], " ", "q2"}], "+", 
             RowBox[{"q1", " ", "q3"}]}], ")"}], " ", "wturbx"}], "+", 
          RowBox[{"2", " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"q0", " ", "q1"}], "+", 
             RowBox[{"q2", " ", "q3"}]}], ")"}], " ", "wturby"}], "+", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             SuperscriptBox["q0", "2"], "-", 
             SuperscriptBox["q1", "2"], "-", 
             SuperscriptBox["q2", "2"], "+", 
             SuperscriptBox["q3", "2"]}], ")"}], " ", "wturbz"}]}], ")"}]}], 
       "-", 
       RowBox[{"Cfin", " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"-", "xbcg"}], "+", "xwfin"}], ")"}]}], "+", 
       RowBox[{"rc1", " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             RowBox[{"-", "Dragl1"}], "+", "Dragr1"}], ")"}], " ", 
           RowBox[{"Cos", "[", "Alpha", "]"}]}], "+", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             RowBox[{"-", "Liftl1"}], "+", "Liftr1"}], ")"}], " ", 
           RowBox[{"Sin", "[", "Alpha", "]"}]}]}], ")"}]}], "+", 
       RowBox[{"rc2", " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             RowBox[{"-", "Dragl2"}], "+", "Dragr2"}], ")"}], " ", 
           RowBox[{"Cos", "[", "Alpha", "]"}]}], "+", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             RowBox[{"-", "Liftl2"}], "+", "Liftr2"}], ")"}], " ", 
           RowBox[{"Sin", "[", "Alpha", "]"}]}]}], ")"}]}], "+", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"xbcg", "-", "xeng"}], ")"}], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"thrustl", " ", 
           RowBox[{"Cos", "[", "dezthrustl", "]"}], " ", 
           RowBox[{"Sin", "[", "deythrustl", "]"}]}], "+", 
          RowBox[{"thrustr", " ", 
           RowBox[{"Cos", "[", "dezthrustr", "]"}], " ", 
           RowBox[{"Sin", "[", "deythrustr", "]"}]}]}], ")"}]}]}]}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{
  3.5764262095125313`*^9, 3.5769934983996325`*^9, 3.576993671368633*^9, 
   3.5769937365106325`*^9, 3.578504374466466*^9, 3.578591505694796*^9, 
   3.578661639010041*^9, 3.581236894523896*^9, 3.581237062898801*^9, 
   3.581237110069092*^9, 3.581237149062846*^9, 3.5812372824027033`*^9, 
   3.5812373494495287`*^9, 3.5813519984433613`*^9, 3.5817462155716105`*^9, 
   3.5817463999600477`*^9, 3.5817466001770678`*^9, 3.581755009166249*^9, 
   3.5817808906595855`*^9, 3.581826860780659*^9, 3.5823513315389013`*^9, 
   3.584025090863614*^9, 3.5840252621554117`*^9, 3.584025380711193*^9, 
   3.5840254227976*^9, 3.5840254584456387`*^9, 3.5853720390897183`*^9, 
   3.585372123556164*^9, 3.5853722459284*^9, 3.612457828540329*^9, 
   3.61245809815075*^9, 3.6124581798574233`*^9, {3.6124583103458867`*^9, 
   3.612458316020211*^9}, 3.613193606921629*^9, 3.6131936505669937`*^9, 
   3.613193689092846*^9, 3.613193845927189*^9, 3.620712356568205*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Assembling all the rudder forces and aerodynamic stiffnesses", \
"Subsubsection"],

Cell[BoxData[
 RowBox[{
  RowBox[{"expression", " ", ":=", " ", 
   RowBox[{"Flatten", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"exprW1", ",", "exprW2", ",", "exprFin"}], "}"}], ",", "1"}], 
    "]"}]}], ";"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"expression", " ", "//", "TableForm"}]], "Input"],

Cell[BoxData[
 TagBox[GridBox[{
    {"cal1", 
     RowBox[{"CLde1", " ", "lc1", " ", "qpress", " ", "S1", " ", 
      RowBox[{"(", 
       RowBox[{"Alpha", "+", "thetaal1"}], ")"}]}]},
    {"car1", 
     RowBox[{"CLde1", " ", "lc1", " ", "qpress", " ", "S1", " ", 
      RowBox[{"(", 
       RowBox[{"Alpha", "+", "thetaar1"}], ")"}]}]},
    {"cal12", 
     RowBox[{"CLde12", " ", "lc12", " ", "qpress", " ", "S1", " ", 
      RowBox[{"(", 
       RowBox[{"Alpha", "+", "thetaal12"}], ")"}]}]},
    {"car12", 
     RowBox[{"CLde12", " ", "lc12", " ", "qpress", " ", "S1", " ", 
      RowBox[{"(", 
       RowBox[{"Alpha", "+", "thetaar12"}], ")"}]}]},
    {"cal2", 
     RowBox[{"CLalpha2e", " ", "lc2", " ", "qpress", " ", "S2", " ", 
      RowBox[{"(", 
       RowBox[{"Alpha", "+", "thetaal2"}], ")"}]}]},
    {"car2", 
     RowBox[{"CLalpha2e", " ", "lc2", " ", "qpress", " ", "S2", " ", 
      RowBox[{"(", 
       RowBox[{"Alpha", "+", "thetaar2"}], ")"}]}]},
    {"cfin", 
     RowBox[{"CLdefin", " ", "lcfin", " ", "qpress", " ", "Sfin", " ", 
      RowBox[{"(", 
       RowBox[{"Beta", "+", "thetafin"}], ")"}]}]}
   },
   GridBoxAlignment->{
    "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
     "RowsIndexed" -> {}},
   GridBoxSpacings->{"Columns" -> {
       Offset[0.27999999999999997`], {
        Offset[2.0999999999999996`]}, 
       Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
       Offset[0.2], {
        Offset[0.4]}, 
       Offset[0.2]}, "RowsIndexed" -> {}}],
  Function[BoxForm`e$, 
   TableForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{
  3.576426209546533*^9, 3.576993498457633*^9, 3.576993671407633*^9, 
   3.576993736565633*^9, 3.5785043745134687`*^9, 3.5785915057377987`*^9, 
   3.5786616390550437`*^9, 3.581236894621916*^9, 3.5812370629358044`*^9, 
   3.581237110100294*^9, 3.581237149094048*^9, 3.5812372825431128`*^9, 
   3.581237349480729*^9, 3.581351998478363*^9, 3.581746215613615*^9, 
   3.581746399994051*^9, 3.581746600218072*^9, 3.581755009207251*^9, 
   3.5817808907775917`*^9, 3.5818268608246593`*^9, 3.5823513316118937`*^9, 
   3.584025090979621*^9, 3.5840252621994143`*^9, 3.5840253807571955`*^9, 
   3.5840254228396025`*^9, 3.5840254585656457`*^9, 3.585372039167726*^9, 
   3.5853721236441727`*^9, 3.5853722460124083`*^9, 3.6124578285843315`*^9, 
   3.612458098195752*^9, 3.612458179905426*^9, {3.612458310390889*^9, 
   3.612458316068214*^9}, 3.613193606967634*^9, 3.613193650604997*^9, 
   3.613193689128849*^9, 3.6131938459641895`*^9, 3.6207123566262107`*^9}]
}, Open  ]],

Cell["The wing spans can be calculated as:", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"b1", "=", 
   SqrtBox[
    RowBox[{"S1", " ", "AR1"}]]}], ";"}]], "Text",
 CellChangeTimes->{3.6124576154081383`*^9}],

Cell["and", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"b2", "=", 
   SqrtBox[
    RowBox[{"S2", " ", "AR2"}]]}], ";"}]], "Text",
 CellChangeTimes->{3.6124576177992754`*^9}],

Cell["Roll damping term", "Text"],

Cell[BoxData[
 RowBox[{
  SubscriptBox[
   SubscriptBox["M", 
    OverscriptBox[
     RowBox[{"\[EmptySet]", " "}], "."]], "expr"], " ", ":=", " ", 
  RowBox[{"q", " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"S1", "  ", "CLalpha1e", " ", 
      SuperscriptBox["b1", "2"]}], "+", 
     RowBox[{"S2", "  ", "CLalpha2e", " ", 
      SuperscriptBox["b2", "2"]}]}], ")"}], 
   RowBox[{"(", 
    FractionBox[
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"0.7", "/", "2"}], ")"}], "2"], 
     RowBox[{"v", "+", "0.1"}]], ")"}]}]}]], "Input",
 CellChangeTimes->{3.5840152882869387`*^9, 3.58401534896441*^9}],

Cell[BoxData[
 RowBox[{
  SubscriptBox["M", 
   OverscriptBox[
    RowBox[{"\[EmptySet]", " "}], "."]], " ", ":=", " ", 
  RowBox[{"q", " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"S1", "  ", "CLalpha1e", " ", 
      SuperscriptBox["b1", "2"]}], "+", 
     RowBox[{"S2", "  ", "CLalpha2e", " ", 
      SuperscriptBox["b2", "2"]}]}], ")"}], 
   RowBox[{"(", 
    FractionBox[
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"0.7", "/", "2"}], ")"}], "2"], 
     RowBox[{"v", "+", "0.1"}]], ")"}]}]}]], "Input",
 CellChangeTimes->{3.5840152891749897`*^9, 3.5840153477223387`*^9}],

Cell[BoxData[
 RowBox[{
  SubscriptBox["M1", 
   OverscriptBox[
    RowBox[{"\[EmptySet]", " "}], "."]], " ", ":=", " ", 
  RowBox[{"q", " ", 
   RowBox[{"(", 
    RowBox[{"S1", "  ", "CLalpha1e", " ", 
     SuperscriptBox["b1", "2"]}], ")"}], 
   RowBox[{"(", 
    FractionBox[
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"0.7", "/", "2"}], ")"}], "2"], 
     RowBox[{"v", "+", "0.1"}]], ")"}]}]}]], "Input",
 CellChangeTimes->{{3.53444083075*^9, 3.53444083896875*^9}, 
   3.5840152929602065`*^9}],

Cell[BoxData[
 RowBox[{
  SubscriptBox["M2", 
   OverscriptBox[
    RowBox[{"\[EmptySet]", " "}], "."]], " ", ":=", " ", 
  RowBox[{"q", " ", 
   RowBox[{"(", 
    RowBox[{"S2", "  ", "CLalpha2e", " ", 
     SuperscriptBox["b2", "2"]}], ")"}], 
   RowBox[{"(", 
    FractionBox[
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"0.7", "/", "2"}], ")"}], "2"], 
     RowBox[{"v", "+", "0.1"}]], ")"}]}]}]], "Input",
 CellChangeTimes->{{3.534440832171875*^9, 3.534440843859375*^9}, 
   3.5840153430530715`*^9}],

Cell["Pitch damping term", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox[
    SubscriptBox["M1", 
     OverscriptBox["\[Theta]", 
      RowBox[{".", " "}]]], "expr"], ":=", " ", 
   RowBox[{"q", " ", "S1", " ", "CLalpha2e", " ", 
    FractionBox[
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"xw1", "-", "xbcg"}], ")"}], "2"], 
     RowBox[{"v", "+", "0.1"}]]}]}], ";"}]], "Input",
 CellChangeTimes->{{3.534440811328125*^9, 3.53444081909375*^9}, 
   3.5840153391768494`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox[
    SubscriptBox["M2", 
     OverscriptBox["\[Theta]", 
      RowBox[{".", " "}]]], "expr"], ":=", " ", 
   RowBox[{"q", " ", "S2", " ", "CLalpha2e", " ", 
    FractionBox[
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"xw2", "-", "xbcg"}], ")"}], "2"], 
     RowBox[{"v", "+", "0.1"}]]}]}], ";"}]], "Input",
 CellChangeTimes->{3.534440809515625*^9, 3.584015336296685*^9}],

Cell["Yaw damping term", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox[
    SubscriptBox["M", 
     OverscriptBox["\[Psi]", "."]], 
    RowBox[{" ", "expr"}]], ":=", " ", 
   RowBox[{"q", " ", "Sfin", " ", "CLalphafin", " ", 
    FractionBox[
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"xwfin", "-", "xbcg"}], ")"}], "2"], 
     RowBox[{"v", "+", "0.1"}]]}]}], ";"}]], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Calculation of aerodynamic forces", "Subsection"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"Dragl1expr", ":=", 
   RowBox[{"Cdl1", " ", "S1", " ", "q"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Dragr1expr", ":=", 
   RowBox[{"Cdr1", " ", "S1", " ", "q"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"Liftl1expr", ":=", 
   RowBox[{"CLl1", " ", "S1", " ", "q"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"Liftr1expr", ":=", 
   RowBox[{"CLr1", " ", "S1", " ", "q"}]}], ";"}], "\n", 
 RowBox[{"Moment1", ":=", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{"Cml1", "+", "Cmr1"}], ")"}], "S1", " ", 
   "q"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Momentx1", ":=", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"(", 
           RowBox[{"CLl1", "-", "CLr1"}], ")"}], 
          RowBox[{"Cos", "[", "alpha", "]"}]}], "+", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{"Cdl1", "-", "Cdr1"}], ")"}], 
          RowBox[{"Sin", "[", "alpha", "]"}]}]}], ")"}], "rc2"}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"CLde12", " ", "del12"}], "-", 
         RowBox[{"CLde12", " ", "del12"}]}], " ", ")"}], 
       RowBox[{"(", 
        RowBox[{"rc12", "-", "rc2"}], ")"}]}]}], ")"}], " ", "S2", " ", 
    "q"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Dragl2expr", ":=", 
   RowBox[{"Cdl2", " ", "S2", " ", "q"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Dragr2expr", ":=", 
   RowBox[{"Cdr2", " ", "S2", " ", "q"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Liftl2expr", ":=", 
   RowBox[{"CLl2", " ", "S2", " ", "q"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Liftr2expr", ":=", 
   RowBox[{"CLr2", " ", "S2", " ", "q"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Cl2expr", ":=", 
   RowBox[{"CLl2", " ", "S2", " ", "q"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Cr2expr", ":=", 
   RowBox[{
    RowBox[{"-", "CLr2"}], " ", "S2", " ", "q"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Momentx2", ":=", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{"CLl2", "-", "CLr2"}], ")"}], 
       RowBox[{"Cos", "[", "alpha", "]"}]}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{"Cdl2", "-", "Cdr2"}], ")"}], 
       RowBox[{"Sin", "[", "alpha", "]"}]}]}], ")"}], "rc2", "  ", "S2", " ", 
    "q"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Dragfinexpr", ":=", 
   RowBox[{"Cd0fin", " ", "Sfin", " ", "q"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Cfinexpr", ":=", 
   RowBox[{"CLfin", " ", "Sfin", " ", "q"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Dragbexpr", ":=", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"Cd0b", " ", "Sbh"}], " ", "+", 
      RowBox[{"CdW", " ", "S1"}]}], ")"}], "q"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"Liftbexpr", ":=", 
   RowBox[{"CLbh", " ", "Sbh", " ", "q"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"Cy2", ":=", 
   RowBox[{"Cydeelev", " ", 
    RowBox[{"(", 
     RowBox[{"del2", "-", "der2"}], ")"}], "S2", " ", "q"}]}], 
  ";"}]}], "Input",
 CellChangeTimes->{{3.518678260109375*^9, 3.518678260109375*^9}, {
   3.53210692278125*^9, 3.532106936453125*^9}, {3.532107825921875*^9, 
   3.532107951296875*^9}, 3.534320678484375*^9, {3.53448101940625*^9, 
   3.534481026546875*^9}, {3.5344813126875*^9, 3.534481315359375*^9}, {
   3.5344814064375*^9, 3.534481408640625*^9}, {3.53448152559375*^9, 
   3.53448153346875*^9}, {3.534481648609375*^9, 3.53448165109375*^9}, 
   3.538223075698704*^9, {3.538288944537979*^9, 3.538288968912823*^9}, {
   3.5382890143656573`*^9, 3.538289015318776*^9}, {3.5390158761555*^9, 
   3.53901589337425*^9}, {3.539015924639875*^9, 3.539015941343*^9}, 
   3.5411478835688753`*^9, {3.554797755390625*^9, 3.554797819609375*^9}, {
   3.55479785471875*^9, 3.554797857546875*^9}, {3.58174456145402*^9, 
   3.5817445743860197`*^9}, {3.581745543475408*^9, 3.5817455593829985`*^9}, {
   3.6131916803879952`*^9, 3.613191726620618*^9}, {3.6131917909810534`*^9, 
   3.6131917915501103`*^9}, {3.613191838430798*^9, 3.613191874287383*^9}, {
   3.6131919447794313`*^9, 3.6131919667696304`*^9}, 3.6131920220361567`*^9, {
   3.6131920651404667`*^9, 3.613192137129665*^9}, {3.613192341640114*^9, 
   3.613192363485298*^9}, {3.61319258823077*^9, 3.613192629408888*^9}, 
   3.613192680898036*^9, {3.613193168051747*^9, 3.613193369454885*^9}, {
   3.6132347840237*^9, 3.613234892190887*^9}, {3.615045962051212*^9, 
   3.6150459627142496`*^9}}],

Cell["The dynamic pressure is calculated as", "Text",
 Evaluatable->False],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["q", "expr"], " ", ":=", " ", 
   RowBox[{"rho", " ", 
    FractionBox[
     SuperscriptBox["v", "2"], "2"]}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"epsv", "=", "0.0001"}], ";"}]], "Input"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"Ube", "=", 
   RowBox[{"Ub", "+", "vturbx"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Vbe", "=", 
   RowBox[{"Vb", "+", "vturby"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Wbe", "=", 
   RowBox[{"Wb", "+", "vturbz"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Pbe", "=", 
   RowBox[{"Pb", "+", "wturbx"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Qbe", "=", 
   RowBox[{"Qb", "+", "wturby"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Rbe", "=", 
   RowBox[{"Rb", "+", "wturbz"}]}], ";"}]}], "Input",
 CellChangeTimes->{
  3.5194423393125*^9, {3.522045582708578*^9, 3.5220456211974216`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["v", "expr"], "=", 
   SqrtBox[
    RowBox[{
     SuperscriptBox["Ube", "2"], "+", 
     SuperscriptBox["Vbe", "2"], "+", 
     SuperscriptBox["Wbe", "2"], "+", "epsv"}]]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["\[Alpha]", "expr"], " ", ":=", " ", 
   RowBox[{"ArcSin", "[", 
    RowBox[{"Wbe", ",", 
     SubscriptBox["v", "expr"]}], "]"}]}], ";"}]], "Text",
 CellChangeTimes->{{3.518678491515625*^9, 3.518678514734375*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["\[Alpha]", "expr"], " ", ":=", " ", 
   RowBox[{"Atan2L", "[", 
    RowBox[{"Wbe", ",", 
     RowBox[{"Ube", "+", "epsv"}]}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["\[Beta]", "expr"], " ", ":=", " ", 
   RowBox[{"Atan2L", "[", 
    RowBox[{"Vbe", ",", 
     SqrtBox[
      RowBox[{
       SuperscriptBox["Ube", "2"], "+", 
       SuperscriptBox["Wbe", "2"], "+", "epsv"}]]}], " ", "]"}]}], 
  ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SubscriptBox["\[Beta]", "expr"], " ", ":=", " ", 
   RowBox[{"ArcSin", "[", 
    RowBox[{"Vb", "/", 
     SubscriptBox["v", "expr"]}], "]"}]}], ";", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{"Etkin", " ", 
     RowBox[{"(", 
      RowBox[{"4.3", ",", "3"}], ")"}]}], ",", " ", 
    RowBox[{"and", " ", "Stevens", " ", "and", " ", "Lewis", " ", 
     RowBox[{"(", 
      RowBox[{"2.3", "-", "6"}], ")"}]}]}], ")"}]}]], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"LogisticFunc", "[", "x_", "]"}], ":=", 
   FractionBox["1", 
    RowBox[{"1", "+", 
     SuperscriptBox["en", 
      RowBox[{"-", " ", "x"}]]}]]}], ";"}]], "Input",
 CellChangeTimes->{
  3.581743397796723*^9, {3.5817434602882185`*^9, 3.5817434672806168`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"transM", "[", 
    RowBox[{"v_", ",", "vM_", ",", "dM_"}], "]"}], ":=", 
   RowBox[{"LogisticFunc", "[", 
    RowBox[{"2", 
     FractionBox[
      RowBox[{"v", "-", "vM"}], 
      RowBox[{"vM", " ", "dM"}]]}], "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.581743838275891*^9, 3.581743901041167*^9}, {
   3.581743965727635*^9, 3.581743976835746*^9}, 3.58174461825602*^9, 
   3.58174465101502*^9, 3.581745588028863*^9, {3.584015575419362*^9, 
   3.5840155775164824`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"CdW", "=", 
   RowBox[{"CdW0", " ", 
    RowBox[{"transM", "[", 
     RowBox[{"v", ",", "vM", ",", "dM"}], "]"}]}]}], ";"}]], "Input",
 CellChangeTimes->{{3.58174462447202*^9, 3.58174463683502*^9}, {
   3.5817465665257025`*^9, 3.581746591020152*^9}, 3.615046201942933*^9}],

Cell["The lifts and drag are calculated from:", "Text",
 Evaluatable->False],

Cell[BoxData[{
 RowBox[{
  RowBox[{"CDflat", "=", "1."}], ";"}], "\n", 
 RowBox[{
  RowBox[{"CDfinflat", "=", "1."}], ";"}]}], "Input"],

Cell["\<\
The lift coefficients. This model assumes all moving tailplane that is used \
on supersonic aircraft.\
\>", "Text",
 CellChangeTimes->{
  3.534477326125*^9, {3.5785875232390127`*^9, 3.578587563121294*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"CLl1", "=", 
   RowBox[{
    RowBox[{
     RowBox[{"CLift", "[", 
      RowBox[{
      "Alpha1", ",", "CLalpha1e", ",", "ap1", ",", "an1", ",", "awp1", ",", 
       "awn1"}], "]"}], "/", "2"}], "+", 
    RowBox[{"CLde1", " ", "del1"}], "+", 
    RowBox[{"CLde12", " ", "del12"}]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"CLr1", "=", 
   RowBox[{
    RowBox[{
     RowBox[{"CLift", "[", 
      RowBox[{
      "Alpha1", ",", "CLalpha1e", ",", "ap1", ",", "an1", ",", "awp1", ",", 
       "awn1"}], "]"}], "/", "2"}], "+", 
    RowBox[{"CLde1", " ", "der1"}], "+", 
    RowBox[{"CLde12", " ", "der12"}]}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"CLl2", "=", 
   RowBox[{
    RowBox[{"CLift", "[", 
     RowBox[{
      RowBox[{"Alpha2", "+", "del2"}], ",", "CLalpha2e", ",", "ap2", ",", 
      "an2", ",", "awp2", ",", "awn2"}], "]"}], "/", "2"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"CLr2", "=", 
   RowBox[{
    RowBox[{"CLift", "[", 
     RowBox[{
      RowBox[{"Alpha2", "+", "der2"}], ",", "CLalpha2e", ",", "ap2", ",", 
      "an2", ",", "awp2", ",", "awn2"}], "]"}], "/", "2"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"Cdil1", "=", 
   RowBox[{
    RowBox[{
     RowBox[{"CDragInd", "[", 
      RowBox[{
      "Alpha1", ",", "AR1", ",", "e1e", ",", "CLalpha1e", ",", "ap1", ",", 
       "an1", ",", "awp1", ",", "awn1"}], "]"}], "/", "2"}], "+", 
    RowBox[{"Cdide1", " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"del1", "-", "de10"}], ")"}], "2"]}], "+", 
    RowBox[{"Cdide12", "  ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"del12", "-", "de120"}], ")"}], "2"]}], "-", 
    RowBox[{"Cdide112", " ", 
     RowBox[{"(", 
      RowBox[{"del1", "-", "de10"}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{"del12", "-", "de120"}], ")"}]}]}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Cdir1", "=", 
   RowBox[{
    RowBox[{
     RowBox[{"CDragInd", "[", 
      RowBox[{
      "Alpha1", ",", "AR1", ",", "e1e", ",", "CLalpha1e", ",", "ap1", ",", 
       "an1", ",", "awp1", ",", "awn1"}], "]"}], "/", "2"}], "+", 
    RowBox[{"Cdide1", " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"der1", "-", "de10"}], ")"}], "2"]}], "+", 
    RowBox[{"Cdide12", "  ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"der12", "-", "de120"}], ")"}], "2"]}], "-", 
    RowBox[{"Cdide112", " ", 
     RowBox[{"(", 
      RowBox[{"der1", "-", "de10"}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{"der12", "-", "de120"}], ")"}]}]}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"Cdil2", "=", 
   RowBox[{
    RowBox[{"CDragInd", "[", 
     RowBox[{
      RowBox[{"Alpha2", "+", "del2"}], ",", "AR2", ",", "e2e", ",", 
      "CLalpha2e", ",", "ap2", ",", "an2", ",", "awp2", ",", "awn2"}], "]"}], 
    "/", "2"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Cdir2", "=", 
   RowBox[{
    RowBox[{"CDragInd", "[", 
     RowBox[{
      RowBox[{"Alpha2", "+", "der2"}], ",", "AR2", ",", "e2e", ",", 
      "CLalpha2e", ",", "ap2", ",", "an2", ",", "awp2", ",", "awn2"}], "]"}], 
    "/", "2"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"CLfin", " ", "=", " ", 
    RowBox[{"CLift", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"-", "\[Beta]"}], " ", "-", " ", 
       RowBox[{"CLdefin", " ", 
        RowBox[{"defin", "/", "CLalphafin"}]}]}], ",", " ", "CLalphafin", ",",
       " ", "afin", ",", " ", "afin", ",", " ", "awfin", ",", " ", "awfin"}], 
     "]"}]}], ";"}], "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Cmr1", "=", 
   RowBox[{
    RowBox[{
     RowBox[{"CMoment", "[", 
      RowBox[{
      "Alpha1", ",", "Cm01", ",", "Cmfs1", ",", "ap1", ",", "an1", ",", 
       "awp1", ",", "awn1"}], "]"}], "/", "2"}], "-", 
    RowBox[{"Cmde1", " ", "der1"}], "-", 
    RowBox[{"Cmde12", " ", "der12"}], "-", 
    RowBox[{"CLr1", " ", 
     FractionBox["smc", "4"], " ", 
     RowBox[{"transM", "[", 
      RowBox[{"v", ",", "vM", ",", "dM"}], "]"}]}]}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"Cml1", "=", 
   RowBox[{
    RowBox[{
     RowBox[{"CMoment", "[", 
      RowBox[{
      "Alpha1", ",", "Cm01", ",", "Cmfs1", ",", "ap1", ",", "an1", ",", 
       "awp1", ",", "awn1"}], "]"}], "/", "2"}], "-", 
    RowBox[{"Cmde1", " ", "del1"}], "-", 
    RowBox[{"Cmde12", " ", "del12"}], "-", 
    RowBox[{"CLl1", " ", 
     FractionBox["smc", "4"], " ", 
     RowBox[{"transM", "[", 
      RowBox[{"v", ",", "vM", ",", "dM"}], "]"}]}]}]}], 
  ";"}], "\[IndentingNewLine]"}], "Input",
 CellChangeTimes->{{3.518678322859375*^9, 3.5186783538125*^9}, {
   3.534157342875*^9, 3.53415741446875*^9}, {3.53416716728125*^9, 
   3.534167197015625*^9}, {3.5341677299375*^9, 3.53416773265625*^9}, 
   3.534174289359375*^9, {3.534218084953125*^9, 3.534218114515625*^9}, {
   3.53421832103125*^9, 3.5342183315*^9}, {3.53421843934375*^9, 
   3.53421845734375*^9}, {3.5342185058125*^9, 3.5342185401875*^9}, {
   3.534220781015625*^9, 3.534220822203125*^9}, {3.53422107496875*^9, 
   3.534221128125*^9}, {3.53422493534375*^9, 3.534224996921875*^9}, {
   3.5342296561875*^9, 3.53422977696875*^9}, {3.5342298265*^9, 
   3.5342298719375*^9}, 3.534439902421875*^9, {3.53448100075*^9, 
   3.534481009796875*^9}, {3.534481041734375*^9, 3.534481147125*^9}, {
   3.534481319359375*^9, 3.534481324859375*^9}, {3.534481415546875*^9, 
   3.534481424734375*^9}, {3.53448154534375*^9, 3.534481563875*^9}, {
   3.53448165503125*^9, 3.534481664640625*^9}, {3.53448177309375*^9, 
   3.534481775609375*^9}, {3.53448181396875*^9, 3.53448181721875*^9}, {
   3.5381986429649677`*^9, 3.5381986897498107`*^9}, {3.538276360140625*^9, 
   3.538276385390625*^9}, {3.5388035282451506`*^9, 3.5388035417297845`*^9}, {
   3.5388051211667347`*^9, 3.5388051239167347`*^9}, {3.5388058568561125`*^9, 
   3.538805861871641*^9}, {3.538806392822195*^9, 3.53880641155657*^9}, {
   3.5388146117078753`*^9, 3.5388146180516253`*^9}, {3.5388316068239665`*^9, 
   3.538831621714687*^9}, {3.538929760515625*^9, 3.538929833890625*^9}, {
   3.578587332113081*^9, 3.5785873611487417`*^9}, {3.578587397553824*^9, 
   3.578587402173088*^9}, {3.5785874548411007`*^9, 3.578587457615259*^9}, {
   3.5817374867846437`*^9, 3.5817374938256435`*^9}, {3.5817385281089087`*^9, 
   3.581738557939891*^9}, {3.58178066572672*^9, 3.5817807516456347`*^9}, {
   3.5817808123151045`*^9, 3.581780835070406*^9}, {3.5817808714474864`*^9, 
   3.581780875052693*^9}, {3.581820866255924*^9, 3.581820875305924*^9}, {
   3.5840152953123407`*^9, 3.584015328255225*^9}, {3.5840245657565804`*^9, 
   3.584024573262009*^9}, {3.589527894355874*^9, 3.589527920863613*^9}}]
}, Open  ]],

Cell[BoxData[{
 RowBox[{
  RowBox[{"CL1expr", " ", "=", " ", 
   RowBox[{"(", 
    RowBox[{"CLl1", " ", "+", " ", "CLr1"}], ")"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"CL2expr", " ", "=", " ", 
   RowBox[{"(", 
    RowBox[{"CLl2", " ", "+", " ", "CLr2"}], ")"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"Cd1expr", " ", "=", " ", 
   RowBox[{"(", 
    RowBox[{"Cdil1", " ", "+", " ", "Cdir1"}], ")"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"Cd2expr", " ", "=", " ", 
   RowBox[{"(", 
    RowBox[{"Cdil2", " ", "+", " ", "Cdir2"}], ")"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"CLtotexpr", "=", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"CLl1", " ", "+", " ", "CLr1"}], ")"}], "+", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"CLl2", " ", "+", " ", "CLr2"}], ")"}], 
     RowBox[{"S2", "/", "S1"}]}]}]}], " ", ";"}], "\n", 
 RowBox[{
  RowBox[{
   RowBox[{"CDtotexpr", "=", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"Cdil1", " ", "+", " ", "Cdir1"}], ")"}], "+", "CdW", "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"Cdl2", " ", "+", " ", "Cdr2"}], ")"}], 
      RowBox[{"S2", "/", "S1"}]}]}]}], ";"}], "\n"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Alpha1", " ", "=", " ", 
   RowBox[{
    RowBox[{"\[Alpha]", " ", "dah1"}], "-", " ", "ia1"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"Alpha2", " ", "=", " ", 
   RowBox[{
    RowBox[{"\[Alpha]", " ", "dah2"}], " ", "-", " ", "ia2"}]}], 
  ";"}]}], "Input",
 CellChangeTimes->{{3.538806418478445*^9, 3.538806436415945*^9}, {
   3.538831622699068*^9, 3.5388316237928247`*^9}, 3.548386592684738*^9, {
   3.5769929901985025`*^9, 3.576993029541436*^9}, {3.576993077060188*^9, 
   3.576993081415623*^9}, {3.5769931136298447`*^9, 3.5769931792664075`*^9}, 
   3.5769933563741164`*^9, 3.576993482176633*^9, 3.576993722367633*^9, 
   3.581737485333644*^9, {3.581738549799077*^9, 3.5817385806421614`*^9}, {
   3.581745489745035*^9, 3.58174549188925*^9}, 3.5817549615575256`*^9, 
   3.582351257274501*^9, 3.582351291254899*^9, {3.6150397569473*^9, 
   3.6150397777434893`*^9}, 3.6150458091844683`*^9, {3.615045859787363*^9, 
   3.615045886188873*^9}}],

Cell[CellGroupData[{

Cell["Weight and balance", "Subsection"],

Cell[BoxData[
 RowBox[{
  RowBox[{"massexpr", "=", 
   RowBox[{"Me", "+", "Mfuel", "+", "Mcargo"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"xbcgexpr", " ", "=", " ", 
   RowBox[{
    FractionBox["1", "mass"], 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"Me", " ", "xbcge"}], " ", "+", " ", 
      RowBox[{"Mfuel", " ", "xfuel"}], "+", 
      RowBox[{"Mcargo", " ", "xcargo"}]}], ")"}]}]}], ";"}]], "Input"]
}, Open  ]],

Cell["LocalExpressions", "Subsection",
 Evaluatable->False],

Cell[BoxData[
 RowBox[{
  RowBox[{"initialExpressions", " ", "=", " ", 
   RowBox[{"{", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"b1", ",", 
       SqrtBox[
        RowBox[{"S1", " ", "AR1"}]], ",", "\"\<m\>\"", ",", 
       "\"\<Wing span 1\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"b2", ",", 
       SqrtBox[
        RowBox[{"S2", " ", "AR2"}]], ",", "\"\<m\>\"", ",", 
       "\"\<Wing span 2\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"smc", ",", 
       SqrtBox[
        RowBox[{"S1", " ", "/", "AR1"}]], ",", "\"\<m\>\"", ",", 
       "\"\<Standard mean chord\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"hthrust", ",", 
       RowBox[{"hthrust0", " ", "smc"}], ",", "double", ",", "\"\<\>\"", ",", 
       "\"\<engine vert. pos\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"Ix", ",", 
       RowBox[{"Ix0", " ", "Me", " ", "S1", " ", "AR1"}], ",", "double", ",", 
       "\"\< \>\"", ",", "\"\<Inertia moment Ix/(Me S1 AR1)\>\""}], "}"}], 
     ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"Ixz", ",", 
       RowBox[{"Ixz0", " ", "Me", " ", "S1"}], ",", "double", ",", 
       "\"\< \>\"", ",", "\"\<Inertia moment\>\""}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"Iy", ",", 
       RowBox[{"Iy0", " ", "Me", " ", 
        RowBox[{"S1", "/", "AR1"}]}], ",", "double", ",", "\"\< \>\"", ",", 
       "\"\<Inertia moment Iy/(Me S1/AR1)\>\""}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"Iz", ",", 
       RowBox[{"Iz0", " ", "Me", " ", 
        RowBox[{"S1", " ", "/", "AR1"}]}], ",", "double", ",", "\"\< \>\"", 
       ",", "\"\<Inertia moment Iy/(Me S1/AR1)\>\""}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"lc1", ",", 
       RowBox[{"lc10", " ", "smc"}], ",", "double", ",", "\"\<\>\"", ",", 
       "\"\<norm. ctrl surf. 1 ac fr hinge lc1/sqrt(AR1 S1)\>\""}], "}"}], 
     ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"lc2", ",", 
       RowBox[{"lc20", " ", "smc"}], ",", "double", ",", "\"\<\>\"", ",", 
       "\"\<norm. ctrl surf. 2 ac fr hinge lc1/sqrt(AR1 S1)\>\""}], "}"}], 
     ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"lc12", ",", 
       RowBox[{"lc120", " ", "smc"}], ",", "double", ",", "\"\<\>\"", ",", 
       "\"\<norm. flap 1 ac fr hinge\>\""}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"lcfin", ",", 
       RowBox[{"lcfin0", " ", "smc"}], ",", "double", ",", "\"\<\>\"", ",", 
       "\"\<ctrl s. fin ac fr hinge\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"rc1", ",", 
       RowBox[{"rc10", "  ", "b1"}], ",", "double", ",", "\"\<m\>\"", ",", 
       "\"\<norm. ctrl surface 1 mom. arm\>\""}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"rc2", ",", 
       RowBox[{"rc20", "  ", "b1"}], ",", "double", ",", "\"\<m\>\"", ",", 
       "\"\<norm. ctrl surface 1 mom. arm\>\""}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"rcfin", ",", 
       RowBox[{"rcfin0", " ", "smc"}], ",", "double", ",", "\"\<m\>\"", ",", 
       "\"\<norm. ctrl surf. fin mom. arm\>\""}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"S2", ",", 
       RowBox[{"S20", " ", "S1"}], ",", "double", ",", "\"\<\>\"", ",", 
       "\"\<norm. wing area 2\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"Sbh", ",", 
       RowBox[{"Sbh0", " ", "S1"}], ",", "double", ",", "\"\<\>\"", ",", 
       "\"\<norm. hor. proj. area\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"Sbv", ",", 
       RowBox[{"Sbv0", " ", "S1"}], ",", "double", ",", "\"\<\>\"", ",", 
       "\"\<norm.body vert. proj. area\>\""}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"Sfin", ",", 
       RowBox[{"Sfin0", " ", "S1"}], ",", "double", ",", "\"\<\>\"", ",", 
       "\"\<norm. fin area\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"xbach", ",", 
       RowBox[{"xbach0", " ", "smc"}], ",", "double", ",", "\"\< \>\"", ",", 
       "\"\<body ac. hor.\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"xbacv", ",", 
       RowBox[{"xbacv0", " ", "smc"}], ",", "double", ",", "\"\< \>\"", ",", 
       "\"\<body ac vert.\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"xbcge", ",", 
       RowBox[{"xbcge0", " ", "smc"}], ",", "double", ",", "\"\< \>\"", ",", 
       "\"\<body cg\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"xcargo", ",", 
       RowBox[{"xcargo0", " ", "smc"}], ",", "double", ",", "\"\< \>\"", ",", 
       "\"\<cargo pos.\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"xfuel", ",", 
       RowBox[{"xfuel0", " ", "smc"}], ",", "double", ",", "\"\< \>\"", ",", 
       "\"\<\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"xw1", ",", 
       RowBox[{"xw10", " ", "smc"}], ",", "double", ",", "\"\< \>\"", ",", 
       "\"\<wing1  position\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"xw2", ",", 
       RowBox[{"xw20", " ", "smc"}], ",", "double", ",", "\"\< \>\"", ",", 
       "\"\<wing 2 position\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"xwfin", ",", 
       RowBox[{"xwfin0", " ", "smc"}], ",", "double", ",", "\"\<\>\"", ",", 
       "\"\<\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"xeng", ",", 
       RowBox[{"xeng0", "  ", "smc"}], ",", "double", ",", "\"\<m\>\"", ",", 
       "\"\<engines x-pos\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"yeng", ",", 
       RowBox[{"yeng0", "  ", "b1"}], ",", "double", ",", "\"\<m\>\"", ",", 
       "\"\<engines off. from center\>\""}], "}"}]}], "}"}]}], ";"}]], "Text",\

 CellChangeTimes->{
  3.446886190291441*^9, 3.518678404671875*^9, {3.518859856015625*^9, 
   3.518859872328125*^9}, {3.519443135515625*^9, 3.51944313903125*^9}, {
   3.522041884674801*^9, 3.5220418970179977`*^9}, {3.53448132590625*^9, 
   3.53448132765625*^9}, {3.534481426109375*^9, 3.53448142834375*^9}, {
   3.53448156546875*^9, 3.534481573671875*^9}, {3.534481665765625*^9, 
   3.53448166796875*^9}, {3.5344857835625*^9, 3.53448579675*^9}, {
   3.5382217251171556`*^9, 3.538221797569353*^9}, 3.538285079529546*^9, {
   3.5382882328492737`*^9, 3.538288239583606*^9}, 3.5547974635625*^9, {
   3.556274268232908*^9, 3.556274269732975*^9}, {3.556274362877773*^9, 
   3.556274364299711*^9}, {3.556296130140625*^9, 3.55629613940625*^9}, {
   3.5563351520294237`*^9, 3.5563351536386957`*^9}, {3.56802995458747*^9, 
   3.5680299590599174`*^9}, 3.5795823425412207`*^9, {3.5795824880617714`*^9, 
   3.579582488833849*^9}, {3.5840153974071803`*^9, 3.5840155285066786`*^9}, {
   3.5840156380089417`*^9, 3.5840156668645926`*^9}, 3.5840196864745007`*^9, {
   3.5840246108241577`*^9, 3.584024684242357*^9}, {3.5840247534843173`*^9, 
   3.5840248011400433`*^9}, {3.5840253028747406`*^9, 
   3.5840253052538767`*^9}, {3.5840253686585035`*^9, 3.584025370492608*^9}, {
   3.584025403639504*^9, 3.584025414189108*^9}, 3.584025453866377*^9, {
   3.5853716679487457`*^9, 3.58537173001336*^9}, {3.585371813974209*^9, 
   3.5853718795737686`*^9}, {3.5853719451133213`*^9, 
   3.5853719452133317`*^9}, {3.5853722332881365`*^9, 
   3.5853722367544827`*^9}, {3.5895277449014916`*^9, 3.58952788157575*^9}, {
   3.589527951387344*^9, 3.589527999608221*^9}, {3.5895288326556234`*^9, 
   3.5895288353855305`*^9}, {3.589528941420594*^9, 3.589528960435693*^9}, 
   3.608801991890525*^9, {3.612457641212614*^9, 3.61245769393563*^9}, {
   3.612458053641204*^9, 3.612458054550256*^9}, 3.6124580876921515`*^9, 
   3.612458300988352*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"localExpressions", " ", "=", " ", 
   RowBox[{"{", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"betaM", ",", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{"1", "-", 
             SuperscriptBox[
              RowBox[{"(", 
               FractionBox["v", "vM"], ")"}], "2"]}], ")"}], "2"], "+", 
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{"epsM", " ", 
             FractionBox["v", "vM"]}], ")"}], "2"]}], ")"}], 
        RowBox[{"1", "/", "4"}]], ",", "double", ",", 
       "\"\<Mach effect on lift\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"CLalpha1e", ",", 
       RowBox[{"CLalpha1", "/", "betaM"}], ",", "double", ",", 
       "\"\<Effective lift sloop\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"CLalpha2e", ",", 
       RowBox[{"CLalpha2", "/", "betaM"}], ",", "double", ",", 
       "\"\<Effective lift sloop\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"e1e", ",", 
       RowBox[{"e1", " ", "-", 
        RowBox[{"e1", 
         RowBox[{"(", 
          RowBox[{"1", "-", 
           FractionBox["1", "AR1"]}], ")"}], " ", 
         RowBox[{"transM", "[", 
          RowBox[{"v", ",", "vM", ",", "dM"}], "]"}]}]}], ",", "double", ",", 
       "\"\<Effective oswald efficieny 1\>\""}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"e2e", ",", 
       RowBox[{"e2", " ", "-", 
        RowBox[{"e2", 
         RowBox[{"(", 
          RowBox[{"1", "-", 
           FractionBox["1", "AR2"]}], ")"}], " ", 
         RowBox[{"transM", "[", 
          RowBox[{"v", ",", "vM", ",", "dM"}], "]"}]}]}], ",", "double", ",", 
       "\"\<Effective oswald efficieny 2\>\""}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"v", ",", 
       SubscriptBox["v", "expr"], ",", "double", ",", 
       "\"\<Abs. value of speed\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"\[Alpha]", ",", 
       SubscriptBox["\[Alpha]", "expr"], ",", "double", ",", 
       "\"\<Angle of attack\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"q", ",", 
       SubscriptBox["q", "expr"], ",", "double", ",", 
       "\"\<Dynamic pressure\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"\[Beta]", ",", 
       SubscriptBox["\[Beta]", "expr"], ",", "double", ",", 
       "\"\<Slip angle\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "mass", " ", ",", "massexpr", ",", "double", ",", 
       "\"\<total AC-weight\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "xbcg", ",", " ", "xbcgexpr", ",", "double", ",", "\"\<AC-cg\>\""}], 
      "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "Dragl1", ",", "Dragl1expr", ",", "double", ",", 
       "\"\<Drag from wing 1\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "Dragr1", ",", "Dragr1expr", ",", "double", ",", 
       "\"\<Drag from wing 1\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "Liftl1", ",", "Liftl1expr", ",", "double", ",", 
       "\"\<Lift from wing 1\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "Liftr1", ",", "Liftr1expr", ",", "double", ",", 
       "\"\<Lift from wing 1\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "Dragl2", ",", "Dragl2expr", ",", "double", ",", 
       "\"\<Drag from wing 2\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "Dragr2", ",", "Dragr2expr", ",", "double", ",", 
       "\"\<Drag from wing 2\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "Liftl2", ",", "Liftl2expr", ",", "double", ",", 
       "\"\<Lift from wing 2\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "Liftr2", ",", "Liftr2expr", ",", "double", ",", 
       "\"\<Lift from wing 2\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "Liftb", ",", "Liftbexpr", ",", "double", ",", 
       "\"\<Lift from body\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "Dragb", ",", "Dragbexpr", ",", "double", ",", 
       "\"\<Drag from body\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "Cfin", ",", "Cfinexpr", ",", "double", ",", 
       "\"\<Force from vertical tail\>\""}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "Dragfin", ",", "Dragfinexpr", ",", "double", ",", 
       "\"\<Drag from body\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
       SubscriptBox["M", 
        OverscriptBox["\[Theta]", "."]], " ", ",", 
       SubscriptBox[
        SubscriptBox["M", 
         OverscriptBox["\[Theta]", 
          RowBox[{".", " "}]]], "expr"], ",", "double", ",", 
       "\"\<Damping term in pitch\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
       SubscriptBox["M", 
        OverscriptBox["\[Psi]", "."]], ",", 
       SubscriptBox[
        SubscriptBox["M", 
         OverscriptBox["\[Psi]", "."]], 
        RowBox[{" ", "expr"}]], ",", "double", ",", 
       "\"\<Damping term in yaw\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"Fx", ",", 
       RowBox[{
        RowBox[{"aircraft`F", "[", 
         RowBox[{"[", "1", "]"}], "]"}], "[", 
        RowBox[{"[", "1", "]"}], "]"}], ",", "double", ",", 
       "\"\<Force in x\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"Fy", ",", 
       RowBox[{
        RowBox[{"aircraft`F", "[", 
         RowBox[{"[", "2", "]"}], "]"}], "[", 
        RowBox[{"[", "1", "]"}], "]"}], ",", "double", ",", 
       "\"\<Force in y\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"Fz", ",", 
       RowBox[{
        RowBox[{"aircraft`F", "[", 
         RowBox[{"[", "3", "]"}], "]"}], "[", 
        RowBox[{"[", "1", "]"}], "]"}], ",", "double", ",", 
       "\"\<Force in z\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"Lb", ",", 
       RowBox[{
        RowBox[{"aircraft`T", "[", 
         RowBox[{"[", "1", "]"}], "]"}], "[", 
        RowBox[{"[", "1", "]"}], "]"}], ",", "double", ",", 
       "\"\<moment on x-axis\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"Mb", ",", 
       RowBox[{
        RowBox[{"aircraft`T", "[", 
         RowBox[{"[", "2", "]"}], "]"}], "[", 
        RowBox[{"[", "1", "]"}], "]"}], ",", "double", ",", 
       "\"\<moment on y-axis\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"Nb", ",", 
       RowBox[{
        RowBox[{"aircraft`T", "[", 
         RowBox[{"[", "3", "]"}], "]"}], "[", 
        RowBox[{"[", "1", "]"}], "]"}], ",", "double", ",", 
       "\"\<moment on z-axis\>\""}], "}"}]}], "}"}]}], ";"}]], "Text",
 CellChangeTimes->{
  3.446886190291441*^9, 3.518678404671875*^9, {3.518859856015625*^9, 
   3.518859872328125*^9}, {3.519443135515625*^9, 3.51944313903125*^9}, {
   3.522041884674801*^9, 3.5220418970179977`*^9}, {3.53448132590625*^9, 
   3.53448132765625*^9}, {3.534481426109375*^9, 3.53448142834375*^9}, {
   3.53448156546875*^9, 3.534481573671875*^9}, {3.534481665765625*^9, 
   3.53448166796875*^9}, {3.5344857835625*^9, 3.53448579675*^9}, {
   3.5382217251171556`*^9, 3.538221797569353*^9}, 3.538285079529546*^9, {
   3.5382882328492737`*^9, 3.538288239583606*^9}, 3.5547974635625*^9, {
   3.556274268232908*^9, 3.556274269732975*^9}, {3.556274362877773*^9, 
   3.556274364299711*^9}, {3.556296130140625*^9, 3.55629613940625*^9}, {
   3.5563351520294237`*^9, 3.5563351536386957`*^9}, {3.56802995458747*^9, 
   3.5680299590599174`*^9}, 3.5795823425412207`*^9, {3.5795824880617714`*^9, 
   3.579582488833849*^9}, {3.5840153974071803`*^9, 3.5840155285066786`*^9}, {
   3.5840156380089417`*^9, 3.5840156668645926`*^9}, 3.5840196864745007`*^9, {
   3.5840246108241577`*^9, 3.584024684242357*^9}, {3.5840247534843173`*^9, 
   3.5840248011400433`*^9}, {3.5840253028747406`*^9, 
   3.5840253052538767`*^9}, {3.5840253686585035`*^9, 3.584025370492608*^9}, {
   3.584025403639504*^9, 3.584025414189108*^9}, 3.584025453866377*^9, {
   3.5853716679487457`*^9, 3.58537173001336*^9}, {3.585371813974209*^9, 
   3.5853718795737686`*^9}, {3.5853719451133213`*^9, 
   3.5853719452133317`*^9}, {3.5853722332881365`*^9, 
   3.5853722367544827`*^9}, {3.5895277449014916`*^9, 3.58952788157575*^9}, {
   3.589527951387344*^9, 3.589527999608221*^9}, {3.5895288326556234`*^9, 
   3.5895288353855305`*^9}, {3.589528941420594*^9, 3.589528960435693*^9}, 
   3.608801991890525*^9, {3.612457641212614*^9, 3.61245769393563*^9}, 
   3.6124581733370504`*^9, 3.61245830376351*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"localExpressions", " ", "=", " ", 
   RowBox[{"{", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"b1", ",", 
       SqrtBox[
        RowBox[{"S1", " ", "AR1"}]], ",", "\"\<m\>\"", ",", 
       "\"\<Wing span 1\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"b2", ",", 
       SqrtBox[
        RowBox[{"S2", " ", "AR2"}]], ",", "\"\<m\>\"", ",", 
       "\"\<Wing span 2\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"smc", ",", 
       SqrtBox[
        RowBox[{"S1", " ", "/", "AR1"}]], ",", "\"\<m\>\"", ",", 
       "\"\<Standard aerodynamic chord\>\""}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"mac", ",", 
       RowBox[{"mac0", 
        SqrtBox[
         RowBox[{"S1", " ", "/", "AR1"}]]}], ",", "\"\<m\>\"", ",", 
       "\"\<Mean aerodynamic chord\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     
     RowBox[{"{", 
      RowBox[{"hthrust", ",", 
       RowBox[{"hthrust0", " ", "b1"}], ",", "double", ",", "\"\<\>\"", ",", 
       "\"\<engine vert. pos\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"Ix", ",", 
       RowBox[{"Ix0", " ", "Me", " ", "S1", " ", "AR1"}], ",", "double", ",", 
       "\"\< \>\"", ",", "\"\<Inertia moment Ix/(Me S1 AR1)\>\""}], "}"}], 
     ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"Ixz", ",", 
       RowBox[{"Ixz0", " ", "Me", " ", "S1"}], ",", "double", ",", 
       "\"\< \>\"", ",", "\"\<Inertia moment\>\""}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"Iy", ",", 
       RowBox[{"Iy0", " ", "Me", " ", 
        RowBox[{"S1", "/", "AR1"}]}], ",", "double", ",", "\"\< \>\"", ",", 
       "\"\<Inertia moment Iy/(Me S1/AR1)\>\""}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"Iz", ",", 
       RowBox[{"Iz0", " ", "Me", " ", 
        RowBox[{"S1", " ", "/", "AR1"}]}], ",", "double", ",", "\"\< \>\"", 
       ",", "\"\<Inertia moment Iy/(Me S1/AR1)\>\""}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"lc1", ",", 
       RowBox[{"lc10", " ", "mac"}], ",", "double", ",", "\"\<\>\"", ",", 
       "\"\<norm. ctrl surf. 1 ac fr hinge lc1/sqrt(AR1 S1)\>\""}], "}"}], 
     ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"lc2", ",", 
       RowBox[{"lc20", " ", "mac"}], ",", "double", ",", "\"\<\>\"", ",", 
       "\"\<norm. ctrl surf. 2 ac fr hinge lc1/sqrt(AR1 S1)\>\""}], "}"}], 
     ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"lc12", ",", 
       RowBox[{"lc120", " ", "mac"}], ",", "double", ",", "\"\<\>\"", ",", 
       "\"\<norm. flap 1 ac fr hinge\>\""}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"lcfin", ",", 
       RowBox[{"lcfin0", " ", "mac"}], ",", "double", ",", "\"\<\>\"", ",", 
       "\"\<ctrl s. fin ac fr hinge\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"rc1", ",", 
       RowBox[{"rc10", "  ", "b1"}], ",", "double", ",", "\"\<m\>\"", ",", 
       "\"\<norm. ctrl surface 1 mom. arm\>\""}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"rc12", ",", 
       RowBox[{"rc120", "  ", "b1"}], ",", "double", ",", "\"\<m\>\"", ",", 
       "\"\<norm. ctrl surface 12 mom. arm\>\""}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"rc2", ",", 
       RowBox[{"rc20", "  ", "b1"}], ",", "double", ",", "\"\<m\>\"", ",", 
       "\"\<norm. ctrl surface 1 mom. arm\>\""}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"rcfin", ",", 
       RowBox[{"rcfin0", " ", "mac"}], ",", "double", ",", "\"\<m\>\"", ",", 
       "\"\<norm. ctrl surf. fin mom. arm\>\""}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"S2", ",", 
       RowBox[{"S20", " ", "S1"}], ",", "double", ",", "\"\<\>\"", ",", 
       "\"\<norm. wing area 2\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"Sbh", ",", 
       RowBox[{"Sbh0", " ", "S1"}], ",", "double", ",", "\"\<\>\"", ",", 
       "\"\<norm. hor. proj. area\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"Sbv", ",", 
       RowBox[{"Sbv0", " ", "S1"}], ",", "double", ",", "\"\<\>\"", ",", 
       "\"\<norm.body vert. proj. area\>\""}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"Sfin", ",", 
       RowBox[{"Sfin0", " ", "S1"}], ",", "double", ",", "\"\<\>\"", ",", 
       "\"\<norm. fin area\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"xbach", ",", 
       RowBox[{"xbach0", " ", "mac"}], ",", "double", ",", "\"\< \>\"", ",", 
       "\"\<body ac. hor.\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"xbacv", ",", 
       RowBox[{"xbacv0", " ", "mac"}], ",", "double", ",", "\"\< \>\"", ",", 
       "\"\<body ac vert.\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"xbcge", ",", 
       RowBox[{"xbcge0", " ", "mac"}], ",", "double", ",", "\"\< \>\"", ",", 
       "\"\<body cg\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"xcargo", ",", 
       RowBox[{"xcargo0", " ", "mac"}], ",", "double", ",", "\"\< \>\"", ",", 
       "\"\<cargo pos.\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"xfuel", ",", 
       RowBox[{"xfuel0", " ", "mac"}], ",", "double", ",", "\"\< \>\"", ",", 
       "\"\<\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"xw1", ",", 
       RowBox[{"xw10", " ", "mac"}], ",", "double", ",", "\"\< \>\"", ",", 
       "\"\<wing1  position\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"xw2", ",", 
       RowBox[{"xw20", " ", "mac"}], ",", "double", ",", "\"\< \>\"", ",", 
       "\"\<wing 2 position\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"xwfin", ",", 
       RowBox[{"xwfin0", " ", "mac"}], ",", "double", ",", "\"\<\>\"", ",", 
       "\"\<\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"xeng", ",", 
       RowBox[{"xeng0", "  ", "mac"}], ",", "double", ",", "\"\<m\>\"", ",", 
       "\"\<engines x-pos\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"yeng", ",", 
       RowBox[{"yeng0", "  ", "b1"}], ",", "double", ",", "\"\<m\>\"", ",", 
       "\"\<engines off. from center\>\""}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"betaM", ",", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{"1", "-", 
             SuperscriptBox[
              RowBox[{"(", 
               FractionBox["v", "vM"], ")"}], "2"]}], ")"}], "2"], "+", 
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{"epsM", " ", 
             FractionBox["v", "vM"]}], ")"}], "2"]}], ")"}], 
        RowBox[{"1", "/", "4"}]], ",", "double", ",", 
       "\"\<Mach effect on lift\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"CLalpha1e", ",", 
       RowBox[{"CLalpha1", "/", "betaM"}], ",", "double", ",", 
       "\"\<Effective lift sloop\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"CLalpha2e", ",", 
       RowBox[{"CLalpha2", "/", "betaM"}], ",", "double", ",", 
       "\"\<Effective lift sloop\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"e1e", ",", 
       RowBox[{"e1", " ", "-", 
        RowBox[{"e1", 
         RowBox[{"(", 
          RowBox[{"1", "-", 
           FractionBox["1", "AR1"]}], ")"}], " ", 
         RowBox[{"transM", "[", 
          RowBox[{"v", ",", "vM", ",", "dM"}], "]"}]}]}], ",", "double", ",", 
       "\"\<Effective oswald efficieny 1\>\""}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"e2e", ",", 
       RowBox[{"e2", " ", "-", 
        RowBox[{"e2", 
         RowBox[{"(", 
          RowBox[{"1", "-", 
           FractionBox["1", "AR2"]}], ")"}], " ", 
         RowBox[{"transM", "[", 
          RowBox[{"v", ",", "vM", ",", "dM"}], "]"}]}]}], ",", "double", ",", 
       "\"\<Effective oswald efficieny 2\>\""}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"v", ",", 
       SubscriptBox["v", "expr"], ",", "double", ",", 
       "\"\<Abs. value of speed\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"\[Alpha]", ",", 
       SubscriptBox["\[Alpha]", "expr"], ",", "double", ",", 
       "\"\<Angle of attack\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"q", ",", 
       SubscriptBox["q", "expr"], ",", "double", ",", 
       "\"\<Dynamic pressure\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"\[Beta]", ",", 
       SubscriptBox["\[Beta]", "expr"], ",", "double", ",", 
       "\"\<Slip angle\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "mass", " ", ",", "massexpr", ",", "double", ",", 
       "\"\<total AC-weight\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "xbcg", ",", " ", "xbcgexpr", ",", "double", ",", "\"\<AC-cg\>\""}], 
      "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "Dragl1", ",", "Dragl1expr", ",", "double", ",", 
       "\"\<Drag from wing 1\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "Dragr1", ",", "Dragr1expr", ",", "double", ",", 
       "\"\<Drag from wing 1\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "Liftl1", ",", "Liftl1expr", ",", "double", ",", 
       "\"\<Lift from wing 1\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "Liftr1", ",", "Liftr1expr", ",", "double", ",", 
       "\"\<Lift from wing 1\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "Dragl2", ",", "Dragl2expr", ",", "double", ",", 
       "\"\<Drag from wing 2\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "Dragr2", ",", "Dragr2expr", ",", "double", ",", 
       "\"\<Drag from wing 2\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "Liftl2", ",", "Liftl2expr", ",", "double", ",", 
       "\"\<Lift from wing 2\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "Liftr2", ",", "Liftr2expr", ",", "double", ",", 
       "\"\<Lift from wing 2\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "Liftb", ",", "Liftbexpr", ",", "double", ",", 
       "\"\<Lift from body\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "Dragb", ",", "Dragbexpr", ",", "double", ",", 
       "\"\<Drag from body\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "Cfin", ",", "Cfinexpr", ",", "double", ",", 
       "\"\<Force from vertical tail\>\""}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "Cl2", ",", "Cl2expr", ",", "double", ",", 
       "\"\<Side force from left canard\>\""}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "Cr2", ",", "Cr2expr", ",", "double", ",", 
       "\"\<Side force from right canard\>\""}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
      "Dragfin", ",", "Dragfinexpr", ",", "double", ",", 
       "\"\<Drag from body\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
       SubscriptBox["M", 
        OverscriptBox["\[Theta]", "."]], " ", ",", 
       SubscriptBox[
        SubscriptBox["M", 
         OverscriptBox["\[Theta]", 
          RowBox[{".", " "}]]], "expr"], ",", "double", ",", 
       "\"\<Damping term in pitch\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
       SubscriptBox["M", 
        OverscriptBox["\[Psi]", "."]], ",", 
       SubscriptBox[
        SubscriptBox["M", 
         OverscriptBox["\[Psi]", "."]], 
        RowBox[{" ", "expr"}]], ",", "double", ",", 
       "\"\<Damping term in yaw\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"Fx", ",", 
       RowBox[{
        RowBox[{"aircraft`F", "[", 
         RowBox[{"[", "1", "]"}], "]"}], "[", 
        RowBox[{"[", "1", "]"}], "]"}], ",", "double", ",", 
       "\"\<Force in x\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"Fy", ",", 
       RowBox[{
        RowBox[{"aircraft`F", "[", 
         RowBox[{"[", "2", "]"}], "]"}], "[", 
        RowBox[{"[", "1", "]"}], "]"}], ",", "double", ",", 
       "\"\<Force in y\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"Fz", ",", 
       RowBox[{
        RowBox[{"aircraft`F", "[", 
         RowBox[{"[", "3", "]"}], "]"}], "[", 
        RowBox[{"[", "1", "]"}], "]"}], ",", "double", ",", 
       "\"\<Force in z\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"Lb", ",", 
       RowBox[{
        RowBox[{"aircraft`T", "[", 
         RowBox[{"[", "1", "]"}], "]"}], "[", 
        RowBox[{"[", "1", "]"}], "]"}], ",", "double", ",", 
       "\"\<moment on x-axis\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"Mb", ",", 
       RowBox[{
        RowBox[{"aircraft`T", "[", 
         RowBox[{"[", "2", "]"}], "]"}], "[", 
        RowBox[{"[", "1", "]"}], "]"}], ",", "double", ",", 
       "\"\<moment on y-axis\>\""}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"Nb", ",", 
       RowBox[{
        RowBox[{"aircraft`T", "[", 
         RowBox[{"[", "3", "]"}], "]"}], "[", 
        RowBox[{"[", "1", "]"}], "]"}], ",", "double", ",", 
       "\"\<moment on z-axis\>\""}], "}"}]}], "}"}]}], ";"}]], "Input",
 CellChangeTimes->{
  3.446886190291441*^9, 3.518678404671875*^9, {3.518859856015625*^9, 
   3.518859872328125*^9}, {3.519443135515625*^9, 3.51944313903125*^9}, {
   3.522041884674801*^9, 3.5220418970179977`*^9}, {3.53448132590625*^9, 
   3.53448132765625*^9}, {3.534481426109375*^9, 3.53448142834375*^9}, {
   3.53448156546875*^9, 3.534481573671875*^9}, {3.534481665765625*^9, 
   3.53448166796875*^9}, {3.5344857835625*^9, 3.53448579675*^9}, {
   3.5382217251171556`*^9, 3.538221797569353*^9}, 3.538285079529546*^9, {
   3.5382882328492737`*^9, 3.538288239583606*^9}, 3.5547974635625*^9, {
   3.556274268232908*^9, 3.556274269732975*^9}, {3.556274362877773*^9, 
   3.556274364299711*^9}, {3.556296130140625*^9, 3.55629613940625*^9}, {
   3.5563351520294237`*^9, 3.5563351536386957`*^9}, {3.56802995458747*^9, 
   3.5680299590599174`*^9}, 3.5795823425412207`*^9, {3.5795824880617714`*^9, 
   3.579582488833849*^9}, {3.5840153974071803`*^9, 3.5840155285066786`*^9}, {
   3.5840156380089417`*^9, 3.5840156668645926`*^9}, 3.5840196864745007`*^9, {
   3.5840246108241577`*^9, 3.584024684242357*^9}, {3.5840247534843173`*^9, 
   3.5840248011400433`*^9}, {3.5840253028747406`*^9, 
   3.5840253052538767`*^9}, {3.5840253686585035`*^9, 3.584025370492608*^9}, {
   3.584025403639504*^9, 3.584025414189108*^9}, 3.584025453866377*^9, {
   3.5853716679487457`*^9, 3.58537173001336*^9}, {3.585371813974209*^9, 
   3.5853718795737686`*^9}, {3.5853719451133213`*^9, 
   3.5853719452133317`*^9}, {3.5853722332881365`*^9, 
   3.5853722367544827`*^9}, {3.5895277449014916`*^9, 3.58952788157575*^9}, {
   3.589527951387344*^9, 3.589527999608221*^9}, {3.5895288326556234`*^9, 
   3.5895288353855305`*^9}, {3.589528941420594*^9, 3.589528960435693*^9}, 
   3.608801991890525*^9, {3.612457641212614*^9, 3.61245769393563*^9}, 
   3.612458159049233*^9, {3.6125370908129425`*^9, 3.6125371030876446`*^9}, {
   3.6127045433744755`*^9, 3.612704599835555*^9}, {3.6131935232942677`*^9, 
   3.6131935859635334`*^9}, {3.613193792620189*^9, 3.613193816213189*^9}, 
   3.620709650116988*^9, 3.620709723078988*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"expressions", "=", 
   RowBox[{"Flatten", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"expression", ",", "expressionVE", ",", 
       RowBox[{"{", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"AlphaAttack", ",", "\[Alpha]"}], "}"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{"{", 
          RowBox[{"BetaSlip", ",", "\[Beta]"}], "}"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{"{", 
          RowBox[{"altitude", ",", 
           RowBox[{"-", "zcg"}]}], "}"}], ",", "\[IndentingNewLine]", 
         RowBox[{"{", 
          RowBox[{"\[Phi]", ",", 
           SubscriptBox["\[Phi]", "expr"]}], "}"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{"{", 
          RowBox[{"\[Theta]", ",", 
           SubscriptBox["\[Theta]", "expr"]}], "}"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{"{", 
          RowBox[{"\[Psi]", ",", 
           SubscriptBox["\[Psi]", "expr"]}], "}"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{"{", 
          RowBox[{"gfx", ",", 
           RowBox[{"Fx", "/", "mass"}]}], "}"}], ",", "\[IndentingNewLine]", 
         RowBox[{"{", 
          RowBox[{"gfy", ",", 
           RowBox[{"Fy", "/", "mass"}]}], "}"}], ",", "\[IndentingNewLine]", 
         RowBox[{"{", 
          RowBox[{"gfz", ",", 
           RowBox[{"Fz", "/", "mass"}]}], "}"}], ",", "\[IndentingNewLine]", 
         RowBox[{"{", 
          RowBox[{"Faz", ",", 
           RowBox[{
            RowBox[{"aircraft`aero`Fw", "[", 
             RowBox[{"[", "3", "]"}], "]"}], "[", 
            RowBox[{"[", "1", "]"}], "]"}]}], "}"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{"{", 
          RowBox[{"Fax", ",", 
           RowBox[{
            RowBox[{"aircraft`aero`Fw", "[", 
             RowBox[{"[", "1", "]"}], "]"}], "[", 
            RowBox[{"[", "1", "]"}], "]"}]}], "}"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{"{", 
          RowBox[{"CL1", ",", "CLtotexpr"}], "}"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{"{", 
          RowBox[{"Cd1", ",", "CDtotexpr"}], "}"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{"{", 
          RowBox[{"Zxfin", ",", 
           RowBox[{"kafin", " ", "mTimestep"}]}], "}"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{"{", 
          RowBox[{"Zxal1", ",", 
           RowBox[{"kal1", " ", "mTimestep"}]}], "}"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{"{", 
          RowBox[{"Zxar1", ",", 
           RowBox[{"kar1", " ", "mTimestep"}]}], "}"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{"{", 
          RowBox[{"Zxal12", ",", 
           RowBox[{"kal12", " ", "mTimestep"}]}], "}"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{"{", 
          RowBox[{"Zxar12", ",", 
           RowBox[{"kar12", " ", "mTimestep"}]}], "}"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{"{", 
          RowBox[{"Zxal2", ",", 
           RowBox[{"kal2", " ", "mTimestep"}]}], "}"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{"{", 
          RowBox[{"Zxar2", ",", 
           RowBox[{"kar2", " ", "mTimestep"}]}], "}"}]}], 
        "\[IndentingNewLine]", "}"}]}], "}"}], ",", "1"}], "]"}]}], 
  ";"}]], "Input",
 CellChangeTimes->{
  3.52203311498798*^9, {3.5220431780326705`*^9, 3.5220431826420155`*^9}, {
   3.5341584015*^9, 3.5341584655625*^9}, {3.534166960796875*^9, 
   3.53416696490625*^9}, {3.5341677573125*^9, 3.534167795625*^9}, {
   3.5341678845*^9, 3.534167886125*^9}, {3.53421817753125*^9, 
   3.534218197328125*^9}, {3.53421835015625*^9, 3.53421835984375*^9}, {
   3.534481328171875*^9, 3.534481328828125*^9}, {3.534481429359375*^9, 
   3.534481429890625*^9}, {3.534481575296875*^9, 3.5344815770625*^9}, {
   3.5344816690625*^9, 3.534481670578125*^9}, {3.534481776171875*^9, 
   3.53448177696875*^9}, {3.53448181796875*^9, 3.534481818453125*^9}, {
   3.5382218022255435`*^9, 3.5382218243502607`*^9}, {3.5382221810488195`*^9, 
   3.5382221828144217`*^9}, {3.5382224891341796`*^9, 3.538222493962459*^9}, 
   3.5382228682937813`*^9, {3.5382232291815233`*^9, 3.5382232292596474`*^9}, {
   3.5382235913018885`*^9, 3.538223594614346*^9}, {3.538236852375*^9, 
   3.538236859453125*^9}, {3.5382833251467714`*^9, 3.5382833341784835`*^9}, {
   3.5382841205825577`*^9, 3.538284126176272*^9}, {3.538284181754041*^9, 
   3.5382841900196133`*^9}, 3.538284304300132*^9, 3.5382882673646784`*^9, 
   3.5382899397912097`*^9, {3.5383951283415527`*^9, 3.538395132779024*^9}, 
   3.538395392714861*^9, {3.5383955818855247`*^9, 3.5383955903073463`*^9}, 
   3.538480304612813*^9, 3.538502019375*^9, {3.53850339046875*^9, 
   3.53850341325*^9}, {3.5385037604375*^9, 3.538503776453125*^9}, {
   3.5388060613230553`*^9, 3.538806062979295*^9}, {3.538806155056831*^9, 
   3.5388061645098953`*^9}, {3.5388295596132345`*^9, 3.538829589769677*^9}, {
   3.568148714015669*^9, 3.568148714106674*^9}, {3.5769934671326327`*^9, 
   3.576993487386633*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Compgen", "[", "file", "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"Part", "::", "partd"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"Part specification \[NoBreak]\\!\\(delayedPart \
\[LeftDoubleBracket] \\(\\(1, 1\\)\\) \[RightDoubleBracket]\\)\[NoBreak] is \
longer than depth of object. \\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", \
ButtonStyle->\\\"Link\\\", ButtonFrame->None, \
ButtonData:>\\\"paclet:ref/message/General/partd\\\", ButtonNote -> \
\\\"Part::partd\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.61319365102804*^9, 3.6131936894948854`*^9, 
  3.613193846340189*^9, 3.6207123570462527`*^9}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"Part", "::", "partd"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"Part specification \[NoBreak]\\!\\(xcg \
\[LeftDoubleBracket] 1 \[RightDoubleBracket]\\)\[NoBreak] is longer than \
depth of object. \\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", ButtonStyle->\\\
\"Link\\\", ButtonFrame->None, \
ButtonData:>\\\"paclet:ref/message/General/partd\\\", ButtonNote -> \
\\\"Part::partd\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.61319365102804*^9, 3.6131936894948854`*^9, 
  3.613193846340189*^9, 3.620712357048253*^9}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"Part", "::", "partd"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"Part specification \[NoBreak]\\!\\(0.` \
\[LeftDoubleBracket] 1 \[RightDoubleBracket]\\)\[NoBreak] is longer than \
depth of object. \\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", ButtonStyle->\\\
\"Link\\\", ButtonFrame->None, \
ButtonData:>\\\"paclet:ref/message/General/partd\\\", ButtonNote -> \
\\\"Part::partd\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.61319365102804*^9, 3.6131936894948854`*^9, 
  3.613193846340189*^9, 3.620712357050253*^9}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"General", "::", "stop"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"Further output of \[NoBreak]\\!\\(\\*StyleBox[\\(Part :: \
partd\\), \\\"MessageName\\\"]\\)\[NoBreak] will be suppressed during this \
calculation. \\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", \
ButtonStyle->\\\"Link\\\", ButtonFrame->None, \
ButtonData:>\\\"paclet:ref/message/General/stop\\\", ButtonNote -> \
\\\"General::stop\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.61319365102804*^9, 3.6131936894948854`*^9, 
  3.613193846340189*^9, 3.6207123570512533`*^9}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"Set", "::", "partw"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"Part \[NoBreak]\\!\\(101\\)\[NoBreak] of \
\[NoBreak]\\!\\({toral1, thetaal1, wal1, cal1, Zcal1, eqInertiaal1, torar1, \
thetaar1, war1, car1, Zcar1, eqInertiaar1, toral12, thetaal12, wal12, cal12, \
Zcal12, eqInertiaal12, torar12, thetaar12, war12, car12, Zcar12, \
eqInertiaar12, toral2, thetaal2, wal2, cal2, Zcal2, eqInertiaal2, torar2, \
thetaar2, war2, car2, Zcar2, eqInertiaar2, torfin, thetafin, wfin, cfin, \
Zcfin, eqInertiafin, xcg, ycg, zcg, vx, vy, vz, Psi, Thetao, \\(\\(\
\[LeftSkeleton] 50 \[RightSkeleton]\\)\\)}\\)\[NoBreak] does not exist. \
\\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", \
ButtonFrame->None, ButtonData:>\\\"paclet:ref/message/General/partw\\\", \
ButtonNote -> \\\"Set::partw\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.61319365102804*^9, 3.6131936894948854`*^9, 
  3.613193846340189*^9, 3.620712357227271*^9}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"Part", "::", "partw"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"Part \[NoBreak]\\!\\(101\\)\[NoBreak] of \
\[NoBreak]\\!\\({toral1, thetaal1, wal1, cal1, Zcal1, eqInertiaal1, torar1, \
thetaar1, war1, car1, Zcar1, eqInertiaar1, toral12, thetaal12, wal12, cal12, \
Zcal12, eqInertiaal12, torar12, thetaar12, war12, car12, Zcar12, \
eqInertiaar12, toral2, thetaal2, wal2, cal2, Zcal2, eqInertiaal2, torar2, \
thetaar2, war2, car2, Zcar2, eqInertiaar2, torfin, thetafin, wfin, cfin, \
Zcfin, eqInertiafin, xcg, ycg, zcg, vx, vy, vz, Psi, Thetao, \\(\\(\
\[LeftSkeleton] 50 \[RightSkeleton]\\)\\)}\\)\[NoBreak] does not exist. \
\\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", \
ButtonFrame->None, ButtonData:>\\\"paclet:ref/message/General/partw\\\", \
ButtonNote -> \\\"Part::partw\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.61319365102804*^9, 3.6131936894948854`*^9, 
  3.613193846340189*^9, 3.620712357229271*^9}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"Set", "::", "partw"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"Part \[NoBreak]\\!\\(102\\)\[NoBreak] of \
\[NoBreak]\\!\\({toral1, thetaal1, wal1, cal1, Zcal1, eqInertiaal1, torar1, \
thetaar1, war1, car1, Zcar1, eqInertiaar1, toral12, thetaal12, wal12, cal12, \
Zcal12, eqInertiaal12, torar12, thetaar12, war12, car12, Zcar12, \
eqInertiaar12, toral2, thetaal2, wal2, cal2, Zcal2, eqInertiaal2, torar2, \
thetaar2, war2, car2, Zcar2, eqInertiaar2, torfin, thetafin, wfin, cfin, \
Zcfin, eqInertiafin, xcg, ycg, zcg, vx, vy, vz, Psi, Thetao, \\(\\(\
\[LeftSkeleton] 50 \[RightSkeleton]\\)\\)}\\)\[NoBreak] does not exist. \
\\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", \
ButtonFrame->None, ButtonData:>\\\"paclet:ref/message/General/partw\\\", \
ButtonNote -> \\\"Set::partw\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.61319365102804*^9, 3.6131936894948854`*^9, 
  3.613193846340189*^9, 3.6207123572312713`*^9}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"Part", "::", "partw"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"Part \[NoBreak]\\!\\(101\\)\[NoBreak] of \
\[NoBreak]\\!\\({toral1, thetaal1, wal1, cal1, Zcal1, eqInertiaal1, torar1, \
thetaar1, war1, car1, Zcar1, eqInertiaar1, toral12, thetaal12, wal12, cal12, \
Zcal12, eqInertiaal12, torar12, thetaar12, war12, car12, Zcar12, \
eqInertiaar12, toral2, thetaal2, wal2, cal2, Zcal2, eqInertiaal2, torar2, \
thetaar2, war2, car2, Zcar2, eqInertiaar2, torfin, thetafin, wfin, cfin, \
Zcfin, eqInertiafin, xcg, ycg, zcg, vx, vy, vz, Psi, Thetao, \\(\\(\
\[LeftSkeleton] 50 \[RightSkeleton]\\)\\)}\\)\[NoBreak] does not exist. \
\\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", \
ButtonFrame->None, ButtonData:>\\\"paclet:ref/message/General/partw\\\", \
ButtonNote -> \\\"Part::partw\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.61319365102804*^9, 3.6131936894948854`*^9, 
  3.613193846340189*^9, 3.620712357232271*^9}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"Part", "::", "partw"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"Part \[NoBreak]\\!\\(102\\)\[NoBreak] of \
\[NoBreak]\\!\\({toral1, thetaal1, wal1, cal1, Zcal1, eqInertiaal1, torar1, \
thetaar1, war1, car1, Zcar1, eqInertiaar1, toral12, thetaal12, wal12, cal12, \
Zcal12, eqInertiaal12, torar12, thetaar12, war12, car12, Zcar12, \
eqInertiaar12, toral2, thetaal2, wal2, cal2, Zcal2, eqInertiaal2, torar2, \
thetaar2, war2, car2, Zcar2, eqInertiaar2, torfin, thetafin, wfin, cfin, \
Zcfin, eqInertiafin, xcg, ycg, zcg, vx, vy, vz, Psi, Thetao, \\(\\(\
\[LeftSkeleton] 50 \[RightSkeleton]\\)\\)}\\)\[NoBreak] does not exist. \
\\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", \
ButtonFrame->None, ButtonData:>\\\"paclet:ref/message/General/partw\\\", \
ButtonNote -> \\\"Part::partw\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.61319365102804*^9, 3.6131936894948854`*^9, 
  3.613193846340189*^9, 3.6207123572332716`*^9}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"General", "::", "stop"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"Further output of \[NoBreak]\\!\\(\\*StyleBox[\\(Part :: \
partw\\), \\\"MessageName\\\"]\\)\[NoBreak] will be suppressed during this \
calculation. \\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", \
ButtonStyle->\\\"Link\\\", ButtonFrame->None, \
ButtonData:>\\\"paclet:ref/message/General/stop\\\", ButtonNote -> \
\\\"General::stop\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.61319365102804*^9, 3.6131936894948854`*^9, 
  3.613193846340189*^9, 3.6207123572352715`*^9}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"Set", "::", "partw"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"Part \[NoBreak]\\!\\(103\\)\[NoBreak] of \
\[NoBreak]\\!\\({toral1, thetaal1, wal1, cal1, Zcal1, eqInertiaal1, torar1, \
thetaar1, war1, car1, Zcar1, eqInertiaar1, toral12, thetaal12, wal12, cal12, \
Zcal12, eqInertiaal12, torar12, thetaar12, war12, car12, Zcar12, \
eqInertiaar12, toral2, thetaal2, wal2, cal2, Zcal2, eqInertiaal2, torar2, \
thetaar2, war2, car2, Zcar2, eqInertiaar2, torfin, thetafin, wfin, cfin, \
Zcfin, eqInertiafin, xcg, ycg, zcg, vx, vy, vz, Psi, Thetao, \\(\\(\
\[LeftSkeleton] 50 \[RightSkeleton]\\)\\)}\\)\[NoBreak] does not exist. \
\\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", \
ButtonFrame->None, ButtonData:>\\\"paclet:ref/message/General/partw\\\", \
ButtonNote -> \\\"Set::partw\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.61319365102804*^9, 3.6131936894948854`*^9, 
  3.613193846340189*^9, 3.620712357236272*^9}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"General", "::", "stop"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"Further output of \[NoBreak]\\!\\(\\*StyleBox[\\(Set :: \
partw\\), \\\"MessageName\\\"]\\)\[NoBreak] will be suppressed during this \
calculation. \\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", \
ButtonStyle->\\\"Link\\\", ButtonFrame->None, \
ButtonData:>\\\"paclet:ref/message/General/stop\\\", ButtonNote -> \
\\\"General::stop\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.61319365102804*^9, 3.6131936894948854`*^9, 
  3.613193846340189*^9, 3.6207123572372723`*^9}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"XMLElement", "::", "cntsList"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"\[NoBreak]\\!\\(XMLElement[\\(\\(\\\"modelobject\\\", \
\\(\\({\\(\\(\\(\\(\\\"t\\\" \[Ellipsis] \\\"me\\\"\\)\\) \[Rule] \
\\(\\(\\\"\\\" \[Ellipsis] \\\"\\\"\\)\\)\\)\\), \\(\\(\[LeftSkeleton] 1 \
\[RightSkeleton]\\)\\)}\\)\\), \\(\\({\\(\\(XMLElement[\\(\\(\\\"icon\\\", \
\\(\\({\\(\\(\\\"isopath\\\" \[Rule] \\\"AeroAircraft6DOFSS.svg\\\"\\)\\), \
\\(\\(\\\"iconrotation\\\" \[Rule] \\\"ON\\\"\\)\\), \\(\\(\\\"userpath\\\" \
\[Rule] \\\"AeroAircraft6DOFSS.svg\\\"\\)\\)}\\)\\), \\(\\({}\\)\\)\\)\\)]\\)\
\\), \\(\\(XMLElement[\\(\\(\\\"portpositions\\\", \\(\\({}\\)\\), \\(\\({\\(\
\\(XMLElement[\\(\\(\\\"portpose\\\", \\(\\({\\(\\(\\\"x\\\" \[Rule] \
\\\"0\\\"\\)\\), \\(\\(\\\"y\\\" \[Rule] 0.125`\\)\\), \\(\\(\\\"a\\\" \
\[Rule] \\\"0\\\"\\)\\), \\(\\(\\\"name\\\" \[Rule] \
\\\"Pal1\\\"\\)\\)}\\)\\), \\(\\({}\\)\\)\\)\\)]\\)\\), \
\\(\\(XMLElement[\\(\\(\\\"portpose\\\", \\(\\({\\(\\(\\\"x\\\" \[Rule] \\\"0\
\\\"\\)\\), \\(\\(\\\"y\\\" \[Rule] 0.25`\\)\\), \\(\\(\\\"a\\\" \[Rule] \
\\\"0\\\"\\)\\), \\(\\(\\\"name\\\" \[Rule] \\\"Par1\\\"\\)\\)}\\)\\), \
\\(\\({}\\)\\)\\)\\)]\\)\\), \\(\\(\[LeftSkeleton] 47 \[RightSkeleton]\\)\\), \
\\(\\(XMLElement[\\(\\(\\\"portpose\\\", \\(\\({\\(\\(\\\"x\\\" \[Rule] \
\\\"0.9\\\"\\)\\), \\(\\(\\\"y\\\" \[Rule] \\\"1\\\"\\)\\), \\(\\(\\\"a\\\" \
\[Rule] \\\"90\\\"\\)\\), \\(\\(\\\"name\\\" \[Rule] \
\\\"Cd1\\\"\\)\\)}\\)\\), \\(\\({}\\)\\)\\)\\)]\\)\\), \\(\\(\[LeftSkeleton] \
2 \[RightSkeleton]\\)\\)}\\)\\)\\)\\)]\\)\\)}\\)\\)\\)\\)]\\)\[NoBreak] in \
\[NoBreak]\\!\\(XMLElement[\\(\\(\\\"hopsanobjectappearance\\\", \
\\(\\({\\(\\(\\\"\\\" \[Ellipsis] \\\"\\\"\\)\\) \[Rule] \
\\(\\(\[LeftSkeleton] 5 \[RightSkeleton]\\)\\)}\\)\\), \
\\(\\(XMLElement[\\(\\(\\\"modelobject\\\", \\(\\({\\(\\(\\\"typename\\\" \
\[Rule] \\\"AeroAircraft6DOFSS\\\"\\)\\), \\(\\(\\(\\(\\\"\\\" \[Ellipsis] \\\
\"e\\\"\\)\\) \[Rule] \\(\\(\\\"\\\" \[Ellipsis] \\\"\\\"\\)\\)\\)\\)}\\)\\), \
\\(\\({\\(\\(XMLElement[\\(\\(\\\"icon\\\", \\(\\({\\(\\(\\\"isopath\\\" \
\[Rule] \\\"AeroAircraft6DOFSS.svg\\\"\\)\\), \\(\\(\\\"iconrotation\\\" \
\[Rule] \\\"ON\\\"\\)\\), \\(\\(\\\"userpath\\\" \[Rule] \
\\\"AeroAircraft6DOFSS.svg\\\"\\)\\)}\\)\\), \\(\\({}\\)\\)\\)\\)]\\)\\), \\(\
\\(XMLElement[\\(\\(\\\"portpositions\\\", \\(\\({}\\)\\), \
\\(\\({\\(\\(XMLElement[\\(\\(\\\"portpose\\\", \\(\\({\\(\\(\\\"x\\\" \
\[Rule] \\\"0\\\"\\)\\), \\(\\(\\\"y\\\" \[Rule] 0.125`\\)\\), \
\\(\\(\\\"a\\\" \[Rule] \\\"0\\\"\\)\\), \\(\\(\\\"name\\\" \[Rule] \
\\\"Pal1\\\"\\)\\)}\\)\\), \\(\\({}\\)\\)\\)\\)]\\)\\), \
\\(\\(XMLElement[\\(\\(\\\"portpose\\\", \\(\\({\\(\\(\\\"x\\\" \[Rule] \\\"0\
\\\"\\)\\), \\(\\(\\\"y\\\" \[Rule] 0.25`\\)\\), \\(\\(\\\"a\\\" \[Rule] \
\\\"0\\\"\\)\\), \\(\\(\\\"name\\\" \[Rule] \\\"Par1\\\"\\)\\)}\\)\\), \
\\(\\({}\\)\\)\\)\\)]\\)\\), \\(\\(\[LeftSkeleton] 48 \[RightSkeleton]\\)\\), \
\\(\\(\[LeftSkeleton] 2 \
\[RightSkeleton]\\)\\)}\\)\\)\\)\\)]\\)\\)}\\)\\)\\)\\)]\\)\\)\\)\\)]\\)\
\[NoBreak] is not a list of contents. The third item in an XMLElement must be \
a list of contents, even if it is an empty list. \\!\\(\\*ButtonBox[\\\"\
\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", ButtonFrame->None, \
ButtonData:>\\\"paclet:ref/XMLElement\\\", ButtonNote -> \
\\\"XMLElement::cntsList\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.61319365102804*^9, 3.6131936894948854`*^9, 
  3.613193846340189*^9, 3.620712357396288*^9}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"Export", "::", "autofix"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"Malformed symbolic XML expression encountered. This may \
result in unexpected XML data. \\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", \
ButtonStyle->\\\"Link\\\", ButtonFrame->None, \
ButtonData:>\\\"paclet:ref/Export\\\", ButtonNote -> \\\"Export::autofix\\\"]\
\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.61319365102804*^9, 3.6131936894948854`*^9, 
  3.613193846340189*^9, 3.620712357400288*^9}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"XMLElement", "::", "attrhs"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"\[NoBreak]\\!\\(0.125`\\)\[NoBreak] in \
\[NoBreak]\\!\\(\\\"y\\\" \[Rule] 0.125`\\)\[NoBreak] is not a valid value \
for an attribute in an XMLElement. The value of the attribute must be a \
string. \\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", \
ButtonStyle->\\\"Link\\\", ButtonFrame->None, \
ButtonData:>\\\"paclet:ref/XMLElement\\\", ButtonNote -> \
\\\"XMLElement::attrhs\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.61319365102804*^9, 3.6131936894948854`*^9, 
  3.613193846340189*^9, 3.6207123574012885`*^9}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"Export", "::", "autofix"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"Malformed symbolic XML expression encountered. This may \
result in unexpected XML data. \\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", \
ButtonStyle->\\\"Link\\\", ButtonFrame->None, \
ButtonData:>\\\"paclet:ref/Export\\\", ButtonNote -> \\\"Export::autofix\\\"]\
\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.61319365102804*^9, 3.6131936894948854`*^9, 
  3.613193846340189*^9, 3.6207123574022884`*^9}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"XMLElement", "::", "attrhs"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"\[NoBreak]\\!\\(0.25`\\)\[NoBreak] in \
\[NoBreak]\\!\\(\\\"y\\\" \[Rule] 0.25`\\)\[NoBreak] is not a valid value for \
an attribute in an XMLElement. The value of the attribute must be a string. \
\\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", \
ButtonFrame->None, ButtonData:>\\\"paclet:ref/XMLElement\\\", ButtonNote -> \
\\\"XMLElement::attrhs\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.61319365102804*^9, 3.6131936894948854`*^9, 
  3.613193846340189*^9, 3.6207123574032884`*^9}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"Export", "::", "autofix"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"Malformed symbolic XML expression encountered. This may \
result in unexpected XML data. \\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", \
ButtonStyle->\\\"Link\\\", ButtonFrame->None, \
ButtonData:>\\\"paclet:ref/Export\\\", ButtonNote -> \\\"Export::autofix\\\"]\
\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.61319365102804*^9, 3.6131936894948854`*^9, 
  3.613193846340189*^9, 3.6207123574042883`*^9}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"General", "::", "stop"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"Further output of \[NoBreak]\\!\\(\\*StyleBox[\\(Export :: \
autofix\\), \\\"MessageName\\\"]\\)\[NoBreak] will be suppressed during this \
calculation. \\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", \
ButtonStyle->\\\"Link\\\", ButtonFrame->None, \
ButtonData:>\\\"paclet:ref/message/General/stop\\\", ButtonNote -> \
\\\"General::stop\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.61319365102804*^9, 3.6131936894948854`*^9, 
  3.613193846340189*^9, 3.6207123574052887`*^9}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"XMLElement", "::", "attrhs"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"\[NoBreak]\\!\\(0.375`\\)\[NoBreak] in \
\[NoBreak]\\!\\(\\\"y\\\" \[Rule] 0.375`\\)\[NoBreak] is not a valid value \
for an attribute in an XMLElement. The value of the attribute must be a \
string. \\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", \
ButtonStyle->\\\"Link\\\", ButtonFrame->None, \
ButtonData:>\\\"paclet:ref/XMLElement\\\", ButtonNote -> \
\\\"XMLElement::attrhs\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.61319365102804*^9, 3.6131936894948854`*^9, 
  3.613193846340189*^9, 3.620712357406289*^9}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"General", "::", "stop"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"Further output of \
\[NoBreak]\\!\\(\\*StyleBox[\\(XMLElement :: attrhs\\), \
\\\"MessageName\\\"]\\)\[NoBreak] will be suppressed during this calculation. \
\\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", \
ButtonFrame->None, ButtonData:>\\\"paclet:ref/message/General/stop\\\", \
ButtonNote -> \\\"General::stop\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.61319365102804*^9, 3.6131936894948854`*^9, 
  3.613193846340189*^9, 3.6207123574072886`*^9}]
}, Open  ]]
}, Open  ]]
}, Open  ]]
},
WindowToolbars->{},
CellGrouping->Manual,
WindowSize->{1088, 690},
WindowMargins->{{Automatic, 0}, {Automatic, 137}},
PrintingCopies->1,
PrintingPageRange->{Automatic, Automatic},
PrivateNotebookOptions->{"ColorPalette"->{RGBColor, 128}},
ShowCellLabel->True,
ShowCellTags->False,
RenderingOptions->{"ObjectDithering"->True,
"RasterDithering"->False},
Magnification->1.100000023841858,
FrontEndVersion->"8.0 for Microsoft Windows (64-bit) (October 6, 2011)",
StyleDefinitions->FrontEnd`FileName[{"Creative"}, "PastelColor.nb", 
  CharacterEncoding -> "WindowsANSI"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[CellGroupData[{
Cell[567, 22, 105, 1, 71, "Title"],
Cell[675, 25, 141, 3, 25, "Subtitle"],
Cell[819, 30, 248, 4, 43, "Input"],
Cell[1070, 36, 124, 3, 43, "Input"],
Cell[1197, 41, 476, 12, 43, "Input"],
Cell[1676, 55, 1074, 17, 104, "Abstract"],
Cell[2753, 74, 195659, 3239, 240, 175833, 2913, "CachedBoxData", "BoxData", \
"Output"],
Cell[198415, 3315, 2277, 44, 184, "Input"],
Cell[CellGroupData[{
Cell[200717, 3363, 42, 0, 83, "Section"],
Cell[200762, 3365, 2417, 52, 830, "Input"]
}, Closed]],
Cell[203194, 3420, 1259, 32, 140, "Input"],
Cell[CellGroupData[{
Cell[204478, 3456, 34, 0, 83, "Section"],
Cell[204515, 3458, 130, 2, 59, "Input"],
Cell[204648, 3462, 132, 2, 59, "Input"],
Cell[204783, 3466, 130, 2, 59, "Input"],
Cell[204916, 3470, 132, 2, 59, "Input"],
Cell[205051, 3474, 126, 2, 59, "Input"],
Cell[205180, 3478, 160, 3, 107, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[205377, 3486, 61, 1, 83, "Section",
 Evaluatable->False],
Cell[205441, 3489, 96, 7, 109, "Text",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[205562, 3500, 114, 2, 40, "Subsection",
 Evaluatable->False,
 PageBreakAbove->True],
Cell[CellGroupData[{
Cell[205701, 3506, 43, 0, 29, "Subsubsection"],
Cell[205747, 3508, 2365, 64, 2426, "Input"],
Cell[208115, 3574, 2251, 61, 2426, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[210403, 3640, 37, 0, 22, "Subsubsection"],
Cell[210443, 3642, 40, 0, 31, "Text"],
Cell[210486, 3644, 498, 15, 164, "Input"],
Cell[CellGroupData[{
Cell[211009, 3663, 56, 1, 43, "Input"],
Cell[211068, 3666, 4663, 67, 43, "Output"]
}, Open  ]],
Cell[215746, 3736, 150, 5, 36, "Text"],
Cell[215899, 3743, 371, 13, 95, "Input"],
Cell[216273, 3758, 344, 10, 124, "Input"],
Cell[216620, 3770, 26, 0, 43, "Input"],
Cell[CellGroupData[{
Cell[216671, 3774, 50, 0, 29, "Subsubsection"],
Cell[216724, 3776, 137, 4, 31, "Text",
 Evaluatable->False],
Cell[216864, 3782, 5554, 129, 664, "Input"],
Cell[222421, 3913, 159, 4, 31, "Text",
 Evaluatable->False],
Cell[222583, 3919, 3235, 73, 364, "Input"],
Cell[225821, 3994, 1873, 45, 184, "Input"],
Cell[227697, 4041, 131, 4, 31, "Text",
 Evaluatable->False],
Cell[227831, 4047, 216, 5, 31, "Text",
 Evaluatable->False],
Cell[228050, 4054, 17402, 378, 1584, "Input"],
Cell[245455, 4434, 87, 1, 31, "Text",
 Evaluatable->False],
Cell[245545, 4437, 54, 1, 43, "Input"],
Cell[245602, 4440, 137, 4, 31, "Text",
 Evaluatable->False]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[245800, 4451, 34, 0, 40, "Subsection"],
Cell[245837, 4453, 190, 6, 31, "Text"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[246076, 4465, 101, 2, 82, "Section",
 Evaluatable->False],
Cell[CellGroupData[{
Cell[246202, 4471, 77, 2, 40, "Subsection",
 Evaluatable->False,
 PageBreakAbove->True],
Cell[246282, 4475, 727, 23, 70, "Input"],
Cell[247012, 4500, 115, 3, 31, "Text"],
Cell[247130, 4505, 428, 15, 70, "Input"],
Cell[247561, 4522, 443, 15, 70, "Input"],
Cell[248007, 4539, 428, 15, 70, "Input"],
Cell[248438, 4556, 130, 3, 31, "Text"],
Cell[248571, 4561, 282, 9, 43, "Input"],
Cell[CellGroupData[{
Cell[248878, 4574, 91, 2, 43, "Input"],
Cell[248972, 4578, 7162, 137, 86, "Output"]
}, Open  ]],
Cell[256149, 4718, 1798, 54, 37, "DisplayFormula"],
Cell[257950, 4774, 60, 0, 31, "Text"],
Cell[258013, 4776, 1726, 56, 78, "Input"],
Cell[259742, 4834, 150, 3, 31, "Text"],
Cell[CellGroupData[{
Cell[259917, 4841, 2785, 82, 195, "Input"],
Cell[262705, 4925, 3633, 90, 166, "Output"]
}, Open  ]],
Cell[266353, 5018, 132, 3, 31, "Text"],
Cell[266488, 5023, 138, 4, 43, "Input"],
Cell[CellGroupData[{
Cell[266651, 5031, 91, 2, 43, "Input"],
Cell[266745, 5035, 6936, 135, 94, "Output"]
}, Open  ]],
Cell[273696, 5173, 183, 6, 31, "Text",
 Evaluatable->False],
Cell[273882, 5181, 284, 7, 24, "DisplayFormula"],
Cell[274169, 5190, 61, 0, 31, "Text"],
Cell[274233, 5192, 177, 4, 43, "Input"],
Cell[CellGroupData[{
Cell[274435, 5200, 229, 5, 43, "Input"],
Cell[274667, 5207, 4958, 73, 43, "Output"]
}, Open  ]],
Cell[279640, 5283, 157, 5, 31, "Text"],
Cell[279800, 5290, 326, 12, 70, "Input"],
Cell[280129, 5304, 444, 16, 84, "Input"],
Cell[280576, 5322, 268, 10, 70, "Input"],
Cell[280847, 5334, 51, 0, 31, "Text"],
Cell[280901, 5336, 128, 3, 43, "Input"],
Cell[281032, 5341, 52, 0, 31, "Text"],
Cell[281087, 5343, 135, 4, 43, "Input"],
Cell[281225, 5349, 53, 0, 31, "Text"],
Cell[281281, 5351, 222, 7, 43, "Input"],
Cell[281506, 5360, 52, 0, 31, "Text"],
Cell[281561, 5362, 142, 4, 43, "Input"],
Cell[281706, 5368, 75, 0, 31, "Text"],
Cell[281784, 5370, 521, 15, 64, "Input"],
Cell[282308, 5387, 46, 0, 31, "Text"],
Cell[282357, 5389, 159, 4, 24, "DisplayFormula"],
Cell[282519, 5395, 41, 0, 31, "Text"],
Cell[282563, 5397, 125, 3, 43, "Input"],
Cell[282691, 5402, 53, 0, 31, "Text"],
Cell[282747, 5404, 160, 5, 43, "Input"],
Cell[282910, 5411, 57, 0, 31, "Text"],
Cell[282970, 5413, 132, 4, 43, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[283139, 5422, 42, 0, 40, "Subsection"],
Cell[283184, 5424, 73, 1, 31, "Text"],
Cell[283260, 5427, 720, 21, 58, "Input"],
Cell[283983, 5450, 77, 1, 31, "Text"],
Cell[284063, 5453, 44, 0, 31, "Text"],
Cell[284110, 5455, 200, 5, 24, "DisplayFormula"],
Cell[284313, 5462, 43, 0, 31, "Text"],
Cell[284359, 5464, 292, 10, 58, "Input"],
Cell[284654, 5476, 74, 1, 31, "Text"],
Cell[284731, 5479, 462, 13, 43, "Input"],
Cell[285196, 5494, 79, 1, 31, "Text"],
Cell[285278, 5497, 191, 6, 43, "Input"],
Cell[285472, 5505, 226, 6, 31, "Text"],
Cell[285701, 5513, 1887, 57, 187, "Input"],
Cell[287591, 5572, 139, 3, 31, "Text"],
Cell[287733, 5577, 1189, 40, 88, "Input"],
Cell[288925, 5619, 89, 2, 31, "Text"]
}, Open  ]],
Cell[289029, 5624, 121, 3, 31, "Text"],
Cell[289153, 5629, 336, 10, 43, "Input"],
Cell[CellGroupData[{
Cell[289514, 5643, 56, 0, 40, "Subsection"],
Cell[CellGroupData[{
Cell[289595, 5647, 70, 1, 43, "Input"],
Cell[289668, 5650, 7191, 141, 115, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[296896, 5796, 71, 1, 43, "Input"],
Cell[296970, 5799, 7456, 146, 138, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[304463, 5950, 74, 1, 43, "Input"],
Cell[304540, 5953, 6624, 129, 140, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[311201, 6087, 75, 1, 43, "Input"],
Cell[311279, 6090, 7531, 152, 106, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[318847, 6247, 55, 0, 29, "Subsubsection"],
Cell[318905, 6249, 238, 6, 43, "Input"],
Cell[319146, 6257, 102, 3, 43, "Input"],
Cell[CellGroupData[{
Cell[319273, 6264, 74, 1, 43, "Input"],
Cell[319350, 6267, 12604, 307, 379, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[332015, 6581, 67, 0, 40, "Subsection"],
Cell[332085, 6583, 145, 3, 31, "Text"],
Cell[332233, 6588, 257, 6, 61, "Input"],
Cell[332493, 6596, 32, 0, 31, "Text"],
Cell[332528, 6598, 136, 2, 31, "Text"],
Cell[332667, 6602, 145, 4, 31, "Text",
 Evaluatable->False],
Cell[332815, 6608, 173, 4, 43, "Input"],
Cell[332991, 6614, 49, 0, 31, "Text"],
Cell[333043, 6616, 126, 3, 43, "Input"],
Cell[333172, 6621, 89, 2, 31, "Text"],
Cell[333264, 6625, 269, 9, 43, "Input"],
Cell[333536, 6636, 197, 4, 43, "Input"],
Cell[333736, 6642, 43, 0, 31, "Text"],
Cell[333782, 6644, 194, 4, 43, "Input"],
Cell[333979, 6650, 123, 2, 43, "Input"],
Cell[CellGroupData[{
Cell[334127, 6656, 51, 1, 29, "Subsubsection"],
Cell[334181, 6659, 51, 0, 31, "Text"],
Cell[334235, 6661, 449, 13, 70, "Input"],
Cell[334687, 6676, 2174, 60, 100, "Input"],
Cell[336864, 6738, 85, 2, 31, "Text"],
Cell[336952, 6742, 949, 22, 104, "Input"],
Cell[337904, 6766, 137, 1, 31, "Text"],
Cell[338044, 6769, 1381, 35, 124, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[339462, 6809, 62, 1, 29, "Subsubsection"],
Cell[339527, 6812, 62, 0, 31, "Text"],
Cell[339592, 6814, 22, 0, 31, "Text"],
Cell[339617, 6816, 587, 16, 70, "Input"],
Cell[340207, 6834, 23, 0, 31, "Text"],
Cell[340233, 6836, 2114, 57, 74, "Input"],
Cell[342350, 6895, 70, 3, 51, "Text"],
Cell[342423, 6900, 613, 13, 64, "Input"],
Cell[343039, 6915, 656, 19, 84, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[343732, 6939, 50, 1, 29, "Subsubsection"],
Cell[343785, 6942, 60, 0, 31, "Text"],
Cell[343848, 6944, 22, 0, 31, "Text"],
Cell[343873, 6946, 268, 10, 70, "Input"],
Cell[344144, 6958, 23, 0, 31, "Text"],
Cell[344170, 6960, 461, 16, 74, "Input"],
Cell[344634, 6978, 61, 0, 31, "Text"],
Cell[344698, 6980, 345, 8, 43, "Input"],
Cell[345046, 6990, 367, 10, 84, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[345450, 7005, 77, 1, 29, "Subsubsection"],
Cell[345530, 7008, 56, 0, 31, "Text"],
Cell[345589, 7010, 1993, 47, 124, "Input"],
Cell[347585, 7059, 1300, 31, 104, "Input"],
Cell[348888, 7092, 180, 4, 31, "Text"],
Cell[349071, 7098, 26, 0, 43, "Input"]
}, Open  ]]
}, Open  ]],
Cell[349124, 7102, 527, 13, 70, "Input"],
Cell[349654, 7117, 610, 15, 70, "Input"],
Cell[CellGroupData[{
Cell[350289, 7136, 48, 0, 40, "Subsection"],
Cell[350340, 7138, 49, 0, 31, "Text"],
Cell[CellGroupData[{
Cell[350414, 7142, 62, 1, 43, "Input"],
Cell[350479, 7145, 2183, 53, 121, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[352699, 7203, 70, 1, 43, "Input"],
Cell[352772, 7206, 1686, 36, 86, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[354495, 7247, 70, 1, 43, "Input"],
Cell[354568, 7250, 6151, 156, 158, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[360756, 7411, 70, 1, 43, "Input"],
Cell[360829, 7414, 905, 27, 86, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[361771, 7446, 70, 1, 43, "Input"],
Cell[361844, 7449, 3698, 94, 92, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[365579, 7548, 68, 1, 43, "Input"],
Cell[365650, 7551, 1620, 34, 86, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[367307, 7590, 68, 1, 43, "Input"],
Cell[367378, 7593, 2505, 60, 92, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[369920, 7658, 69, 1, 43, "Input"],
Cell[369992, 7661, 1763, 38, 86, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[371792, 7704, 69, 1, 43, "Input"],
Cell[371864, 7707, 1602, 34, 86, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[373503, 7746, 66, 1, 43, "Input"],
Cell[373572, 7749, 2570, 57, 86, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[376179, 7811, 66, 1, 43, "Input"],
Cell[376248, 7814, 2867, 65, 86, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[379152, 7884, 68, 1, 43, "Input"],
Cell[379223, 7887, 4789, 113, 86, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[384049, 8005, 68, 1, 43, "Input"],
Cell[384120, 8008, 10581, 276, 165, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[394738, 8289, 87, 1, 29, "Subsubsection"],
Cell[394828, 8292, 231, 7, 43, "Input"],
Cell[CellGroupData[{
Cell[395084, 8303, 72, 1, 43, "Input"],
Cell[395159, 8306, 2559, 56, 145, "Output"]
}, Open  ]],
Cell[397733, 8365, 52, 0, 31, "Text"],
Cell[397788, 8367, 152, 5, 34, "Text"],
Cell[397943, 8374, 19, 0, 31, "Text"],
Cell[397965, 8376, 152, 5, 34, "Text"],
Cell[398120, 8383, 33, 0, 31, "Text"],
Cell[398156, 8385, 615, 19, 62, "Input"],
Cell[398774, 8406, 590, 18, 62, "Input"],
Cell[399367, 8426, 510, 16, 62, "Input"],
Cell[399880, 8444, 515, 16, 62, "Input"],
Cell[400398, 8462, 34, 0, 31, "Text"],
Cell[400435, 8464, 463, 14, 61, "Input"],
Cell[400901, 8480, 434, 13, 61, "Input"],
Cell[401338, 8495, 32, 0, 31, "Text"],
Cell[401373, 8497, 370, 12, 61, "Input"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[401792, 8515, 55, 0, 40, "Subsection"],
Cell[401850, 8517, 4560, 118, 384, "Input"],
Cell[406413, 8637, 74, 1, 31, "Text",
 Evaluatable->False],
Cell[406490, 8640, 180, 6, 61, "Input"],
Cell[406673, 8648, 75, 2, 43, "Input"],
Cell[406751, 8652, 692, 20, 144, "Input"],
Cell[407446, 8674, 241, 8, 52, "Input"],
Cell[407690, 8684, 265, 7, 31, "Text"],
Cell[407958, 8693, 203, 6, 43, "Input"],
Cell[408164, 8701, 296, 10, 57, "Input"],
Cell[408463, 8713, 462, 14, 31, "Text"],
Cell[408928, 8729, 306, 9, 58, "Input"],
Cell[409237, 8740, 519, 13, 59, "Input"],
Cell[409759, 8755, 307, 7, 43, "Input"],
Cell[410069, 8764, 76, 1, 31, "Text",
 Evaluatable->False],
Cell[410148, 8767, 135, 4, 64, "Input"],
Cell[410286, 8773, 215, 5, 31, "Text"],
Cell[410504, 8780, 6651, 163, 351, "Input"]
}, Open  ]],
Cell[417170, 8946, 2102, 54, 204, "Input"],
Cell[CellGroupData[{
Cell[419297, 9004, 40, 0, 40, "Subsection"],
Cell[419340, 9006, 118, 3, 43, "Input"],
Cell[419461, 9011, 304, 9, 58, "Input"]
}, Open  ]],
Cell[419780, 9023, 59, 1, 40, "Subsection",
 Evaluatable->False],
Cell[419842, 9026, 8013, 165, 621, "Text"],
Cell[427858, 9193, 9066, 211, 738, "Text"],
Cell[436927, 9406, 16104, 370, 1441, "Input"],
Cell[453034, 9778, 5031, 112, 464, "Input"],
Cell[CellGroupData[{
Cell[458090, 9894, 63, 1, 43, "Input"],
Cell[458156, 9897, 604, 12, 23, "Message"],
Cell[458763, 9911, 579, 12, 23, "Message"],
Cell[459345, 9925, 579, 12, 23, "Message"],
Cell[459927, 9939, 600, 12, 23, "Message"],
Cell[460530, 9953, 1003, 17, 81, "Message"],
Cell[461536, 9972, 1005, 17, 81, "Message"],
Cell[462544, 9991, 1005, 17, 81, "Message"],
Cell[463552, 10010, 1005, 17, 81, "Message"],
Cell[464560, 10029, 1007, 17, 81, "Message"],
Cell[465570, 10048, 600, 12, 23, "Message"],
Cell[466173, 10062, 1003, 17, 81, "Message"],
Cell[467179, 10081, 599, 12, 23, "Message"],
Cell[467781, 10095, 3537, 53, 157, "Message"],
Cell[471321, 10150, 528, 11, 23, "Message"],
Cell[471852, 10163, 651, 13, 23, "Message"],
Cell[472506, 10178, 530, 11, 23, "Message"],
Cell[473039, 10191, 647, 12, 23, "Message"],
Cell[473689, 10205, 530, 11, 23, "Message"],
Cell[474222, 10218, 604, 12, 23, "Message"],
Cell[474829, 10232, 649, 13, 23, "Message"],
Cell[475481, 10247, 607, 12, 23, "Message"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}
]
*)

(* End of internal cache information *)
